(function($) {
    
    window.RAVEN = window.RAVEN || {};

    RAVEN.namespace("RAVEN.ExecuteTest");

    var ACTIVITY_SPAN_LIMIT = 10;

    RAVEN.ExecuteTest.ActivityStreamView = Backbone.View.extend({
        el: '#execution-activity-module',
        testRunId: null,

        events: {
            'click #execution-activity-module #activity-stream-show-more': '_onShowMoreClick'
        },

        initialize: function(options) {
            if (options) {
                _.extend(this, _.pick(options, "testRunId"));
            }

            this.render();
        },

        render: function() {
            this._loadActivityStream();
        },

        refresh: function() {
            if (this.isViewExpanded() === false) {
                return;
            }

            var activityEntries = $('#execution-activity-module .activity-item');
            if (activityEntries.length > 0) {
                var path = contextPath + '/rest/raven/1.0/testrun/' + this.testRunId + '/activity/refresh';
                var lastActivityEntryId = activityEntries.first().data('id');
                if (lastActivityEntryId) {
                    path += '?lastActivityEntry=' + lastActivityEntryId;
                }

                this._fetchActivityStream(path, this._refreshActivityCallback);
            }
        },

        isViewExpanded: function() {
            return !this.$el.hasClass('collapsed');
        },

        _loadActivityStream: function() {
            var path = contextPath + '/rest/raven/1.0/testrun/' + this.testRunId + '/activity?limit=' + ACTIVITY_SPAN_LIMIT;

            var activityEntries = $('#execution-activity-module .activity-item');
            var oldestActivityEntryId = activityEntries.last().data('id');
            if (oldestActivityEntryId) {
                path += '&untilActivityEntry=' + oldestActivityEntryId;
            }

            this._fetchActivityStream(path, this._loadActivityCallback);
        },

        _refreshActivityCallback: function(data) {
            if (data && data.length > 0) {
                $.each(data.reverse(), function(i, entry) {
                    var entryHtml = RAVEN.ExecuteTest.ActivityStream.activityEntry({
                        entry: entry,
                        contextPath: AJS.contextPath()
                    });
                    $('#activity-stream').prepend(entryHtml);
                });
            }
        },

        _loadActivityCallback: function(data) {
            if (data && data.length > 0) {
                $('#raven-no-activity').addClass('hidden');
                $.each(data, function(i, entry) {
                    var entryHtml = RAVEN.ExecuteTest.ActivityStream.activityEntry({
                        entry: entry,
                        contextPath: AJS.contextPath()
                    });
                    $('#activity-stream').append(entryHtml);
                });

                if (data.length >= ACTIVITY_SPAN_LIMIT) {
                    $('#activity-stream-show-more').removeClass('hidden');
                } else {
                    $('#activity-stream-show-more').addClass('hidden');
                }
            } else if ($('#execution-activity-module .activity-item').length === 0) {
                $('#raven-no-activity').removeClass('hidden');
            } else {
                // there is activity but no more data was returned
                $('#activity-stream-show-more').addClass('hidden');
            }
        },

        _fetchActivityStream: function(path, fnSuccess) {
            var self = this;

            $('#execution-activity-module-content').block();
            $.ajax({
                type: "GET",
                url: path,
                processData: true,
                async: true,
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() != '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }

                    self._message(AJS.I18n.getText('execute-test-action.activity.update.error'), errorThrown);
                },
                success: fnSuccess,
                complete: function() {
                    $('#execution-activity-module-content').unblock();
                }
            });
        },

        _message: function(title, message) {
            $('#raven-execute-test-activity-stream-message').empty();

            AJS.messages.error('#raven-execute-test-activity-stream-message', {
                title: title,
                body: message
            });
        },

        /****************************************************************
         *                        EVENT HANDLERS
         ****************************************************************/

        /**
         * Load more activity event handler
         */
        _onShowMoreClick: function(e) {
            this._loadActivityStream();

            e.preventDefault();
            e.stopPropagation();
        }

    });

})(AJS.$);