var RAVEN = window.RAVEN || {};
RAVEN.Component = RAVEN.Component || {};
RAVEN.Component.InlineDialogCarousel = (function($){
    this.LCAROUSELSIZE = 6;

    var carouselFixedSize = {};
    var carouselMaxSize = {};

    /**
     * Binds a LCarousel to an AUI inline dialog
     * @param inlineDialogID ID of the inline dialog element, should be unique
     * @param listContainerID ID of the inline dialog internal list
     * @param carouselName name to be given to the inner carousel, should be unique
     */
    function Carousel(inlineDialogID,listContainerID,carouselName){
        var self = this;

        /**
         * Applies the LCarousel and binds an handler to the LCarousel swipe event
         */
        self.bindLCarousel =  function(){
            var inlineDialog = getInlineDialogElement();
            var listContainer = getListContainerElement();

            if(inlineDialog && listContainer && listContainer.children("li").length > LCAROUSELSIZE){
                //on carousel swipe, fix the height if needed
                listContainer.on('carouselSwipe',function(e){
                    fixInlineCarouselHeight(listContainer,carouselName);
                });

                applyLCarousel(listContainer);
            }
        };

        /**
         * Unbinds the LCarousel off the inline dialog, including the swipe event
         */
        self.unbindLCarousel = function(){
            var inlineDialog = getInlineDialogElement();
            if(inlineDialog) {
                self.unbindLCarouselSwipeEvent();
                inlineDialog.removeEventListener("aui-layer-show", self.bindLCarousel);
                inlineDialog.removeEventListener("aui-layer-hide", self.unbindLCarouselSwipeEvent);
            }
        };

        /**
         * Unbinds the LCarousel swipe event
         */
        self.unbindLCarouselSwipeEvent = function(){
            var listContainer = getListContainerElement();
            if (listContainer) {
                listContainer.unbind('carouselSwipe');
            }
        };

        var getInlineDialogElement = function(){
            return document.querySelector(inlineDialogID);
        };

        var getListContainerElement = function () {
            return $(inlineDialogID).find(listContainerID);
        };

        var applyLCarousel = function(listContainer){
            listContainer.find(".inline-dots-holder").remove();

            RAVEN.Component.LCarousel.apply(carouselName, listContainer, LCAROUSELSIZE, false); //apply the carousel

            //adapt LCarousel dots holder
            var dotsHolder = listContainer.find('.dots-holder');

            var inlineDotsHolder = $("<li class='inline-dots-holder' style='display:inline-block'></li>");

            inlineDotsHolder.appendTo(listContainer)
                .append(dotsHolder);

            listContainer.css({
                height: "auto"
            });

            listContainer.closest('.aui-inline-dialog-contents')
                .css({
                    position: "relative"
                });

            inlineDotsHolder.css({
                position: "absolute",
                bottom: "0px",
                left: "0px",
                right:"0px"
            });

            dotsHolder.css({
                position: "relative",
                "padding-bottom": "0px"
            });
        };

        /**
         * Fixes the height for the carousel
         */
        var fixInlineCarouselHeight = function(){
            var listContainer = getListContainerElement();

            if(!listContainer){
                return;
            }

            var carouselHeight = listContainer.height();
            var currentSize = listContainer.children('li:visible:not(.inline-dots-holder)').length;
            var unitHeight = listContainer.children('li:visible:not(.inline-dots-holder)').first().outerHeight(true);

            if(currentSize === 0 && !carouselFixedSize[carouselName]){// cannot resize
                return;
            }

            if(currentSize === 0 && carouselFixedSize[carouselName]){ //Force resize
                listContainer.height(carouselFixedSize[carouselName]);
                return;
            }

            if(carouselMaxSize[carouselName] && carouselHeight >= carouselMaxSize[carouselName]) { //Current carousel window has already been resized
                return;
            }

            //Carousel needs to be resized
            carouselFixedSize[carouselName] = (LCAROUSELSIZE * unitHeight) + carouselHeight - (currentSize * unitHeight);
            carouselMaxSize[carouselName] = carouselFixedSize[carouselName];
            listContainer.height(carouselFixedSize[carouselName]);
        };

        /**
         * Add event handlers of show and hide the AUI inline dialog element
         */
        var inlineDialog = getInlineDialogElement();
        inlineDialog.addEventListener("aui-layer-show",self.bindLCarousel);
        inlineDialog.addEventListener("aui-layer-hide", self.unbindLCarouselSwipeEvent);

        return self;
    }

    /**
     * Creates an Inline Dialog Carousel for the AUI Inline Dialog
     * @param inlineDialogID Inline dialog ID
     * @param listContainerID List element container ID
     * @param carouselName Carousel name to define group of carousels
     */
    this.createInlineDialogCarousel = function(inlineDialogID,listContainerID,carouselName){
        var inlineDialog = document.querySelector(inlineDialogID);
        if(inlineDialogID) {
            var listContainer = $(inlineDialog).find(listContainerID);
            if(listContainer && listContainer.children("li").length > LCAROUSELSIZE) {
                inlineDialog.RavenLCarousel = new Carousel(inlineDialogID, listContainerID, carouselName);
            }
        }
    }

    /**
     * Removes the Inline Dialog Carousel events from the AUI Inline Dialog
     * @param inlineDialogID Inline dialog ID
     */
    this.removeInlineDialogCarousel = function(inlineDialogID){
        var inlineDialog = document.querySelector(inlineDialogID);
        if(inlineDialog && inlineDialog.RavenLCarousel) {
            inlineDialog.RavenLCarousel.unbindLCarousel();
            delete inlineDialog.RavenLCarousel;
        }
    }

    return this;
})(AJS.$);