(function($) {

    window.RAVEN = window.RAVEN || {};

    RAVEN.namespace("RAVEN.ExecuteTest");

    var stepImageInfoHash = {};

    var imageLoadedInfoHash = {};

    var elementIds =[];

    var stepImageInfoHash = {};

    var imageLoadedInfoHash = {};

    function getTestRunId() {
        return $('#test-run-id').val();
    }

    function limitTestTableHeight() {
        return $('#limited-test-cell-height').val() === 'true';
    }

    function getRequiresExperimentalDialog2() {
    	return $("#require-experimental-dialog").val() === 'true';
    }

    function getApplicationLinks() {
    	if($("#application-links").val()){
    		return JSON.parse($("#application-links").val())
    	}
    	return null;
    }

    function getTestExecKey() {
        return $('#test-exec-key').val();
    }

    function getReadonly() {
    	return $("#test-run-readonly").val() == 'true';
    }

    var addEvent = function(object, type, callback) {
        if (object == null || typeof(object) == 'undefined') return;
        if (object.addEventListener) {
            object.addEventListener(type, callback, false);
        } else if (object.attachEvent) {
            object.attachEvent("on" + type, callback);
        } else {
            object["on"+type] = callback;
        }
    };

    addEvent(window, "resize", fixExtenders);

    var fancyboxDefaultOpts = {
            type: "image",
            centerOnScroll: true,

            // disable JIRA's keyboard shortcuts when displaying a lightbox
            onStart : AJS.disableKeyboardShortcuts,
            onClosed : AJS.enableKeyboardShortcuts
    };

    var imageRegex = /([^\s]+(\.(jp(e|g|eg)|gif|png|bmp|jfif|svg|ico))$)/;

	function applyFancybox() {
		$(".attachment-title a").each(
			function(key, value){
				if(imageRegex.test(value.href.toLowerCase())){
					$(value).fancybox(fancyboxDefaultOpts);
				}
		});
	}

    function message(title, message, type) {
        clearMessage();

        if (type === "warning") {
			AJS.messages.warning('#raven-execute-test-message-bar', {
				title : title,
				body : message
			});
			return;
		}

        AJS.messages.error('#raven-execute-test-message-bar', {
            title: title,
            body: message
        });
    }

    function clearMessage() {
        $('#raven-execute-test-message-bar').empty();
    }

    function refreshActivityStream() {
    	$("#execution-activity-module").trigger("test-step-refresh-activity");
    }

    /******************************
    /*********** MODELS ***********
    /******************************/

    var TestStepCollection = Backbone.Collection.extend({
        url: function() {
            return AJS.contextPath() + '/rest/raven/1.0/testrun/' + getTestRunId() + '/steps';
        }
    });

    function fixExtenders() {
        var length = elementIds.length;
        for (var i = 0; i < length; i++) {
            fixAUIExtender(elementIds[i]);
        }
    }
    const SCREEN_CHANGE_WIDTH = 960;
    const COMMENT_MAX_HEIGHT_WIDE = 800;
    const COMMENT_MAX_HEIGHT_NARROW = 200;

    function bottomExtraMargin(ele){
        var parent = {};
        parent["outerHeigth"] = ele.outerHeight();
        parent["offset"] = ele.offset().top;
        parent["margin"] = parseInt(ele.css("marginBottom").slice(0,-2));
        if(ele.children().length == 0){
            return parent;
        }
        var child = bottomExtraMargin(ele.children().last());
        return (parent["outerHeigth"] + parent["offset"] + parent["margin"]) > (child["outerHeigth"] + child["offset"] + child["margin"]) ? parent : child;
    }

    /*this method calculates the scrollbar size so we can add an extra height to the element
    * this is an alternative to the use of a magic number like 17 or 20*/
    function getScrollBarWidth () {
        var inner = document.createElement('p');
        inner.style.width = "100%";
        inner.style.height = "200px";

        var outer = document.createElement('div');
        outer.style.position = "absolute";
        outer.style.top = "0px";
        outer.style.left = "0px";
        outer.style.visibility = "hidden";
        outer.style.width = "200px";
        outer.style.height = "150px";
        outer.style.overflow = "hidden";
        outer.appendChild (inner);

        document.body.appendChild (outer);
        var w1 = inner.offsetWidth;
        outer.style.overflow = 'scroll';
        var w2 = inner.offsetWidth;
        if (w1 == w2) w2 = outer.clientWidth;

        document.body.removeChild (outer);

        return (w1 - w2);
    };

    function bottomExtraMargin(ele){
        var parent = {};
        parent["outerHeigth"] = ele.outerHeight();
        parent["offset"] = ele.offset().top;
        parent["margin"] = parseInt(ele.css("marginBottom").slice(0,-2));
        if(ele.children().length == 0){
            return parent;
        }
        var child = bottomExtraMargin(ele.children().last());
        return (parent["outerHeigth"] + parent["offset"] + parent["margin"]) > (child["outerHeigth"] + child["offset"] + child["margin"]) ? parent : child;
    }

    function fixAUIExtenderEle(id, ele) {
        if (!ele || typeof ele.css("min-height") !== 'string') {
            return;
        }

        var maxHeight = screen.width > SCREEN_CHANGE_WIDTH ? COMMENT_MAX_HEIGHT_WIDE : COMMENT_MAX_HEIGHT_NARROW;
        var height = 10;
        /*besides of the outer height we also need to count space between elements*/
        if($(ele).children().length > 0){
            var first = $(ele).children().first().offset().top;
            var last = $(ele).children().last().offset().top;
            var lastElemHeight = $(ele).children().last().outerHeight(true);
            height = Math.abs(parseInt(first) - parseInt(last)) + lastElemHeight;
            /*also have to take in account the needed margin of last child*/
            var extra = bottomExtraMargin($(ele).children().last());
            var value1 = $(ele).offset().top + height;
            var value2 = extra["outerHeigth"] + extra["offset"] + extra["margin"];
            var calc = value2 - value1;
            if(value2 > value1 && height + calc <= maxHeight){
                height += Math.ceil(calc);
            }
        }

        /*Adding an extra so the horizontal scrollbar does not stay over the element*/
        height = height + getScrollBarWidth();

        ele.css({"min-height": ''});
        if(limitTestTableHeight() && height > maxHeight) {
            ele.parent().find("#reveal-text-trigger-"+id).show();
        } else {
            ele.css("min-height", height);
            ele.parent().find("#reveal-text-trigger-"+id).hide();
        }
    }

    function containsValue(arr, value) {
        for(var i = 0; i < arr.length; i++){
            if(arr[i] == value){
                return true;
            }
        }
        return false;
    }

    function filterUniqueImgsSrc( arr ){
        var res = [];
        for(var i = 0; i < arr.length; i++){
            if(!containsValue(res, arr[i])){
                if(!imageLoadedInfoHash.hasOwnProperty(arr[i])){
                    imageLoadedInfoHash[arr[i]] = false;
                }
                res.push(arr[i]);
            }
        }
        return res;
    }

    function getImgsList(id){
        var imageSources = $("#step-content-step-" + id +" img").map(function(){
            return $(this).attr('src');
        }).get();

        var temp = imageSources.concat($("#data-content-step-" + id +" img").map(function(){
            return $(this).attr('src');
        }).get());

        var completeList = temp.concat($("#result-content-step-" + id +" img").map(function(){
            return $(this).attr('src');
        }).get());
        return filterUniqueImgsSrc(completeList);
    }

    /*images may not be all loaded when executing fixAUIExtenderEle, this would result in wrong min-heigth calculation*/
    function checkForImagesInTestSteps(id){
        if(!stepImageInfoHash.hasOwnProperty(id.toString())){
            stepImageInfoHash[id.toString()] = {"srcs": getImgsList(id), "count": 0};
        }
        stepImageInfoHash[id.toString()]["srcs"].forEach(function(item, index){
            if(stepImageInfoHash[id.toString()]["count"] < stepImageInfoHash[id.toString()]["srcs"].length){
                $("img[src='"+item+"']").load(function(){
                    stepImageInfoHash[id.toString()]["count"] ++;
                    if(stepImageInfoHash[id.toString()]["count"] == stepImageInfoHash[id.toString()]["srcs"].length && !imageLoadedInfoHash[item]){
                        imageLoadedInfoHash[item]=true;
                        fixAUIExtender(id);
                    }
                });
            }
        });
    }

    function fixAUIExtender(id) {
        fixAUIExtenderEle(id, $("#step-content-step-" + id));
        fixAUIExtenderEle(id, $("#data-content-step-" + id));
        fixAUIExtenderEle(id, $("#result-content-step-" + id));

    }


    /**
     * Initializes a list of evidences for the test step, building its carousel in the inline dialog
     * @param testStepId Test step ID
     */
    function initializeStepEvidencesList(testStepId){
        RAVEN.Component
            .InlineDialogCarousel
            .createInlineDialogCarousel("#ts-evidences-"+testStepId,"#step-attachments-"+testStepId, "step-evidences");
    }

    function removeStepEvidencesList(testStepId){
        RAVEN.Component
            .InlineDialogCarousel
            .removeInlineDialogCarousel("#ts-evidences-"+testStepId);
    }

    function resetStepEvidencesList(testStepId){
        removeStepEvidencesList(testStepId);
        initializeStepEvidencesList(testStepId);
    }

    /**
     * Initializes a list of attachments for the test step, building its carousel in the inline dialog
     * @param testStepId Test step ID
     */
    function initializeStepAttachmentsList(testStepId){
        RAVEN.Component
            .InlineDialogCarousel
            .createInlineDialogCarousel("#ts-attachment-"+testStepId,"#step-attachments-"+testStepId,"step-attachments");
    }

    /**
     * Initializes a list of defects for the test step, building its carousel in the inline dialog
     * @param testStepId Test step ID
     */
    function initializeStepDefectsList(testStepId){
        RAVEN.Component
            .InlineDialogCarousel
            .createInlineDialogCarousel("#ts-defects-"+testStepId,"#step-defects-"+testStepId,"step-defects");
    }

    function removeStepDefectsList(testStepId){
        RAVEN.Component
            .InlineDialogCarousel
            .removeInlineDialogCarousel("#ts-defects-"+testStepId);
    }

    function resetStepDefectsList(testStepId){
        removeStepDefectsList(testStepId);
        initializeStepDefectsList(testStepId);
    }

    function resetList(list){
        if(list){//reset content and size
            list.html("");
            list.css("height","auto");
        }
    }

    /******************************
    /************ VIEWS ***********
    /******************************/

    RAVEN.ExecuteTest.TestStepView = Backbone.View.extend({
        collection: new TestStepCollection(),
        el: '#execution-steps-content',
        activityStreamView: null,
        isStatusEditable: false,

        currentTestStepView: null,
        selectedStepRow: null,
        events: {
            'step-update' : '_refresh'
        },

        /**
         * Unscoped Events - these events must be registered elsewhere as they will not be
         * fired by backbone because they are not within the view element. Refer to "_delegateUnscopedEvents".
         */
        unscopedEvents: {
        	'click.step.raven.execute .step-attachments li .attachment-remove-command' : '_removeStepEvidences',
        	'click.step.raven.execute #raven-step-add-defects-global-context' : '_issueDefectPicker',
        	'click.step.raven.execute #raven-step-create-subtask-link-global-context' : '_createSubTaskStep',
        	'click.step.raven.execute .test-step-defects .remove-command' : '_removeStepDefect',
            'click.step.raven.execute #create-subtask-dialog .cancel' : '_createSubTaskCancel',
            'click.step.raven.execute #raven-parent-issue-selector-dialog .cancel' : '_createSubTaskCancel',
            'click.step.raven.execute #create-subtask-dialog .cancel' : '_createSubTaskCancel'
        },

        initialize: function(options) {
            var self = this;

                if (options) {
                _.extend(this, _.pick(options, "allowEdit"));
            }
            this.collection.on('reset', this.render, this);
            this._delegateUnscopedEvents();

            this.$el.block();
            this.collection.fetch({
                success: function() {
                    self.$el.unblock();
                    self.render();
                },
                error: function(model, response) {
                    AJS.messages.error(self.$el, {
                        title: AJS.I18n.getText('execute-test-action.requirement-status.fetch-error'),
                        body: response.responseText,
                        closeable: false
                    });
                    self.$el.unblock();
                }
            });

            JIRA.bind(JIRA.Events.NEW_CONTENT_ADDED, function(e, context, reason) {
                var dialog = $('#create-issue-dialog.raven-create-defect-dialog.jira-dialog-open, \
                    #create-issue-dialog.raven-create-defect-dialog.aui-dialog-open, \
                    #create-subtask-dialog.jira-dialog-open, \
                    #create-subtask-dialog.aui-dialog-open');
                if (dialog.length > 0 && self.selectedStepRow) {
                    self.getDefectFieldsDefaults(dialog, self.selectedStepRow);
                }
            });

            $(AJS).bind("QuickCreateIssue.issueCreated", function(event, issueData) {
                if (self.selectedStepRow) {
                	self.addDefectsToStepExecution([issueData.issueKey],self.selectedStepRow);
                	self.selectedStepRow = null;
                }
            });

            // bind AJS create sub-task event
            $(AJS).bind("QuickCreateSubtask.sessionComplete", function (e, issues) {
            	if (self.selectedStepRow) {
                	self.addDefectsToStepExecution([issues[0].issueKey],self.selectedStepRow);
                	self.selectedStepRow = null;

                    require('aui/flag')({
                        type: 'success',
                        body: AJS.I18n.getText('execute-test-action.create-subtask-dialog.success', contextPath, issues[0].issueKey, issues[0].summary),
                        close:'auto'
                    });
                }
            });
        },

        render: function() {
            var self = this;

            // clear view
            this.$el.empty();

            // empty table
            if(this.collection) {
            	$("#execution-steps-module-heading .size").html("("+ self.collection.length+")")
	            if (this.collection.length == 0) {
	                this.$el.append('<span>'+AJS.I18n.getText('execute-test-action.stepstable.nosteps')+'</span>');
	            } else {
	            	var $testSteps = "";
	            	// clean old
	            	$(".step-inline-dialog").remove();

                    $("#execution-steps-module-heading .test-steps-loading").show();
                    $("#execution-steps-module-heading .test-steps-loading").spin();

                    $('#execution-steps-content').block();

                    var testExecKey = $("#test-exec-key").val();
                    var experimentalDialog2 = getRequiresExperimentalDialog2();
                    var readOnly = getReadonly();
                    var defectIssueType = $("#defectIssueType").val();

                    this.processTestStepsAsync(this.collection.models, function(result, index) {
                        //store element ids for later use
                        elementIds.push(result.attributes.id);

                        self.$el.append(RAVEN.ExecuteTest.renderTestStep({
                            step: result.attributes,
                            testexec: testExecKey,
                            context: AJS.contextPath(),
                            readonly: readOnly,
                            requiresExperimentalDialog2: experimentalDialog2,
                            defectIssueType: defectIssueType
                        }));
                        self.initializeCommentsWikiField(result.attributes.id);
                        self.initializeDefects(result.attributes.id);

                        initializeStepDefectsList(result.attributes.id);
                        initializeStepEvidencesList(result.attributes.id);
                        initializeStepAttachmentsList(result.attributes.id);

                        checkForImagesInTestSteps(result.attributes.id);
                        fixAUIExtender(result.attributes.id);

                        var stepStatus = new RAVEN.ExecuteTest.StatusView({
                            "el" : "#status-action-" + result.attributes.id,
                            "stepId" : result.attributes.id,
                            "mainView" : result
                        });

                        // last element
                        if (index === self.collection.length - 1) {
                            $("#execution-steps-module-heading .test-steps-loading").spinStop();
                            $("#execution-steps-module-heading .test-steps-loading").hide();

                            $('#execution-steps-content').unblock();
                        }
                    }, 1);

	            }
            }

            AJS.messages.setup();
            this.initializeStepAttachmentsDialog();
            applyFancybox();
            return this;
        },

        // last two args are optional
        processTestStepsAsync: function(array, fn, chunk, context) {
            context = context || window;
            chunk = chunk || 100;
            var index = 0;
            function doChunk() {
                var cnt = chunk;
                while (cnt-- && index < array.length) {
                    // callback called with args (value, index, array)
                    fn.call(context, array[index], index, array);
                    ++index;
                }
                if (index < array.length) {
                    // set Timeout for async iteration
                    setTimeout(doChunk, 1);
                }
            }
            doChunk();
        },

        initializeStepAttachmentsDialog: function() {
        	var self = this;
            var attachDialog = new JIRA.FormDialog({
                id: "raven-attach-file-dialog",
                trigger: ".add-step-attachments",
                ajaxOptions: JIRA.Dialogs.getDefaultAjaxOptions,
                onSuccessfulSubmit: function() {
                	var stepId = $(this.$activeTrigger).attr("data-step-id");
                    self.refreshStepAttachmentsFromServer(stepId);

                    refreshActivityStream();

                },
                onContentRefresh: function() {
                    jQuery(".overflow-ellipsis").textOverflow();
                },
                onDialogFinished: function() {},
                autoClose: true
            });
        },

        initializeCommentsWikiField: function(id) {
        	this.createWikiField($('#step-comment-field-' + id),id);
        	this.createWikiField($('#step-comment-field-' + id + "-add-new"),id);
        },

        renderCommentWikiField: function( dialog, id, raw, rendered, extra) {
        	var html = RAVEN.ExecuteTest.renderStepComment({
        		context: contextPath,
        		raw: raw,
        		rendered: rendered,
        		id: id,
        		readonly: getReadonly(),
        		extra: extra
        	});

        	dialog.html(html);
        	this.createWikiField($('#step-comment-field-' + id + extra),id);
        },

        createWikiField: function (ele, id) {
        	var self = this;
        	new RAVEN.InlineEdit.WikiTextField(
        			ele, {
                        success: function(data) {
                        	if(data.rendered === "") {
                        		var ele = $("#raven-comment-"+id);
                        		if (!ele.hasClass("disabled")) {
	                        		ele.addClass("disabled");
	                        		ele.attr("aria-controls","ts-comments-"+id+"-disable");
	                        		$("#add-comment-action-"+id).css("visibility","visible");
	                        		self.renderCommentWikiField( $("#ts-comments-"+id+"-add-new .aui-inline-dialog-contents"), id, data.raw, data.rendered, "-add-new");
                        		}
                        	} else {
                        		var ele = $("#raven-comment-"+id);
                        		if (ele.hasClass("disabled")) {
	                        		ele.removeClass("disabled");
	                        		ele.attr("aria-controls","ts-comments-"+id);
	                        		$("#add-comment-action-"+id).css("visibility","hidden");
	                        		self.renderCommentWikiField( $("#ts-comments-"+id+" .aui-inline-dialog-contents"), id, data.raw, data.rendered, "");
                        		}
                        	}
                        	refreshActivityStream();
                        	$(this.el).closest("aui-inline-dialog").open = false;
                        },
                        error: function(field, error) {
                            message(AJS.I18n.getText('execute-test-action.comment.update.error'), error);
                        }
                    });
        },

        initializeDefects: function(testStepId) {
        	var self = this;
        	if (JIRA.Forms && JIRA.Forms.createCreateIssueForm) {
                createForm = JIRA.Forms.createCreateIssueForm({
                    issueType: $("#defectIssueType").val()
                });

                // select trigger for create issue in global context
                var createIssueDialogGlobalContext = createForm.asDialog({
                    trigger: $('#raven-step-create-issue-link-global-context-'+testStepId ),
                    id: 'create-issue-dialog'
                });

                createIssueDialogGlobalContext.bind('Dialog.show', function(e, data, d2) {
                    createIssueDialogGlobalContext.get$popup().addClass('raven-create-defect-dialog');

                    self.selectedStepRow = testStepId;
                });

                createIssueDialogGlobalContext.bind('Dialog.hide', function(e) {
                    self.selectedStepRow = null;
                });
            }
        },
        _issueDefectPicker : function(e) {
        	var self = this;
        	var stepId = $(e.target).attr("data-step-id");
        	 new RAVEN.Dialogs.createIssueSelectorDialog('DEFAULT', null, null, getTestExecKey(), null, function(issues) {
                 if (issues && issues.length > 0) {
                     var appId = $("#jira-app-link").val();
                     appId ? self.addDefectsToStepExecution(issues,stepId,appId) : self.addDefectsToStepExecution(issues,stepId);
                 }
             }, null, getApplicationLinks());
        },

        _createSubTaskStep: function(e) {
        	e.preventDefault();
            e.stopPropagation();
            clearMessage();

            this.selectedStepRow = $(e.target).attr("data-step-id");
            new RAVEN.Dialogs.createParentIssueSelectorDialog('DEFAULT', null, getTestExecKey(), null, null, null, null, getApplicationLinks(), null);

        },

        addDefectsToStepExecution : function(issueKeys, stepResultId, appId) {
        	var self = this;
        	$('#execution-steps-content').block();
            clearMessage();

            var path = contextPath + '/rest/raven/1.0/testrun/steps/' + stepResultId + '/defect' + (appId ? '?appId='+appId : '');
            
            var value = JSON.stringify(issueKeys);

            $.ajax({
                type: "PUT",
                url: path,
                processData: true,
                async: true,
                data: value,
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() != '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }
                    message(AJS.I18n.getText('execute-test-action.defects.add.title'), AJS.I18n.getText('execute-test-action.defects.add.message'));
                },
                success: function(data, textStatus, jqXHR) {
                    var inlineDialog = $("#ts-defects-"+stepResultId+"-container");
                	var defectsList = inlineDialog.find("#step-defects-"+stepResultId);

                	$(".testStep[data-step-id="+stepResultId+"] #defects-actions .count").html("("+data.length+")");

                    defectsList.html("");
                	$.each( data , function(index, value ) {
          
                        var innerHtml = RAVEN.ExecuteTest.defectLink({
                        	appId : value.appId ? value.appId : '',
                			id : value.id,
                			key : value.key,
                			summary : value.summary,
                			iconUrl : value.iconUrl,
                			status : value.status,
                			readonly : getReadonly(),
                			issueURL : value.url
                        });

                        defectsList.append(innerHtml);
                        
                	});

                    // This enables the dialog
                    if(data.length > 0) {
                        $(inlineDialog).closest(".step-inline-dialog").attr("id","ts-defects-" + stepResultId);
                        $("a[aria-controls='ts-defects-"+stepResultId+"']").removeClass("disabled");
                        resetStepDefectsList(stepResultId);
                    } else {
	                   	 $(inlineDialog).closest(".step-inline-dialog").attr("id","ts-defects-" + stepResultId + "-disabled");
	                   	 $("a[aria-controls='ts-defects-"+stepResultId+"']").addClass("disabled");
                         removeStepDefectsList(stepResultId);
                    }
                },
                complete: function() {
                	refreshActivityStream();
                	$('#execution-steps-content').unblock();
                }
            });
        },

        refreshStepAttachmentsFromServer: function(id) {
        	 $('#execution-steps-content').block();

        	 var self = this;
             clearMessage();

             var stepResultId = id;
             var path = contextPath + '/rest/raven/1.0/testrun/' + getTestRunId() + '/steps/' + stepResultId + '/attachment';

             $.ajax({
                 type: "GET",
                 url: path,
                 processData: true,
                 async: true,
                 contentType: "application/json",
                 error: function(qXHR, textStatus, errorThrown) {
                     if (errorThrown && errorThrown.trim() != '') {
                         errorThrown = errorThrown + ': ' + qXHR.responseText;
                     } else {
                         errorThrown = qXHR.responseText;
                     }

                     message(AJS.I18n.getText('execute-test-action.attachments.refresh.error'), errorThrown)
                 },
                 success: function(data, textStatus, jqXHR) {
                	 var inlineDialog = $("#ts-evidences-"+stepResultId+"-container");
                	 $(".testStep[data-step-id="+stepResultId+"] #evidences-actions .count").html("("+data.length+")");

                     var innerHtml = RAVEN.Attachments.createStepAttachments({
                         entityType: 'TEST_STEP_RESULT',
                         stepId: stepResultId,
                         issueKey: $('#test-exec-key').val(),
                         attachments: data,
                         contextPath: AJS.contextPath(),
                         hideAdd: true
                     });

                     inlineDialog.html(innerHtml);

                     // This enables the dialog
                     if(data.length > 0) {
                    	 $(inlineDialog).closest(".step-inline-dialog").attr("id","ts-evidences-" + stepResultId);
                    	 $("a[aria-controls='ts-evidences-"+stepResultId+"']").removeClass("disabled");
                         resetStepEvidencesList(stepResultId);
                     } else {
                    	 $(inlineDialog).closest(".step-inline-dialog").attr("id","ts-evidences-" + stepResultId + "-disabled");
                    	 $("a[aria-controls='ts-evidences-"+stepResultId+"']").addClass("disabled");
                    	 removeStepEvidencesList(stepResultId);
                     }

                 },
                 complete: function() {
                 	 applyFancybox()
                     $('#execution-steps-content').unblock();
                 }
             });
        },

        getDefectFieldsDefaults : function(dialog, tsrId) {
            var path = contextPath + '/rest/raven/1.0/testrun/' + getTestRunId() + '/steps/' + tsrId + '/defect/defaults';

            $.ajax({
                type: "GET",
                url: path,
                processData: true,
                async: false,
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() != '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }
                    message(AJS.I18n.getText('execute-test-action.defects.defaults.error.title'), AJS.I18n.getText('execute-test-action.defects.defaults.error.message'));
                },
                success: function(data, textStatus, jqXHR) {
                    if (data) {
                        if (data.environment) {
                            dialog.find('#environment').val(data.environment);
                        }

                        if (data.stepDescription) {
                            if ($.trim(dialog.find("#description").val()) === ''){
                                dialog.find('#description').val(
                                    "*" + AJS.I18n.getText('execute-test-action.steps.module.label') + "*\n"
                                    + data.stepDescription
                                    + "\\\\");
                                dialog.find("div[field-id=description] rich-editor").html(
                                    "<p><strong>" + AJS.I18n.getText('execute-test-action.steps.module.label')+ "</strong></p>"
                                    + data.stepDescriptionRendered
                                    + "<p><br class=\"atl-forced-newline\"></p>");
                            }
                        }

                        if (data.affectVersion) {
                            data.affectVersion.forEach(function(element) {
                                AJS.$('select[id="versions"] option').each(function()
                                {
                                    //If the name of version is the same then we choose it.
                                    //this is only using names because identical versions in differents projects have not the same ID
                                    if(jQuery.trim($(this).text()) == element.label){
                                         AJS.$('select[id="versions"]').trigger('selectOption', [{value: $(this).val(), title: element.label, label: element.label }]);
                                    }
                                });

                            }, this);
                        }

                        if (data.label) {
                        	data.label.forEach(function(element) {
                                AJS.$('select[id="labels"]').trigger('selectOption', [{value: element}]);
                            }, this);
                        }
                    }
                }
            });
        },

        _refresh: function() {
            this._delegateUnscopedEvents();
        },

        /****************************************************************
         *                        EVENT HANDLERS
         ****************************************************************/

        _delegateUnscopedEvents: function() {
            var self = this;
            $(document).off('.step.raven.execute');

            $.each(this.unscopedEvents, function(key, val) {
                var eventName = key.substring(0, key.indexOf(' '));
                var eventTrigger = key.substring(key.indexOf(' ') + 1);

                $(document).on(eventName, eventTrigger, $.proxy(self[val], self));
            });
        },

         /**
         * Event handler for the click event any requirement status option.
         */
        _removeStepEvidences : function(e) {
            var self = this;

            var el = $(e.target).closest('li');
            var id = el.val();
            var fileName = el.data('name');

            var options = {};
            options.$el = el;
            options.stepResultId = $(el).closest("div[data-step-id]").attr("data-step-id");
            options.refreshFn = this.refreshStepAttachmentsFromServer;

            this.deleteAttachmentConfirmationDialog(id, fileName, this.deleteAttachmentFromStepResult, options)

            e.preventDefault();
            e.stopPropagation();
        },

        _createSubTaskCancel: function(e) {
            this.selectedStepRow = null;
        },

        _removeStepDefect: function(e) {
            var el = $(e.target).closest('li');

            var key = e.target.parentElement.id;
            var appId = e.target.parentElement.dataset.appId;

            var options = {
                $el: el,
                stepResultId: $(el).closest("div[data-step-id]").attr("data-step-id"),
                refreshFn: this.refreshStepAttachmentsFromServer,
                key: key
            };

            if(appId){
                options.appId = appId;
                options.defectId = e.target.parentElement.dataset.defectId;
            }

            this.deleteDefectConfirmationDialog(key, this.deleteDefectFromStepResult, options);

            e.preventDefault();
            e.stopPropagation();
        },

        deleteDefectFromStepResult: function(options) {
            $('#execution-steps-content').block();
            clearMessage();
            var path = contextPath + '/rest/raven/1.0/testrun/steps/' + options.stepResultId + '/defect?defectKey=' + options.key;
            var stepResultId = options.stepResultId;
            var remoteJiraIssueDetails = (options.appId && options.defectId) ? options.appId+"&"+options.defectId : '';
            $.ajax({
                type: "DELETE",
                url: path,
                data: remoteJiraIssueDetails,
                processData: false,
                async: true,
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() != '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }

                    message(AJS.I18n.getText('execute-test-action.attachments.delete.error'), errorThrown)
                },
                success: function(data, textStatus, jqXHR) {
                	var inlineDialog = $("#ts-defects-"+stepResultId+"-container");
                    var defectsList = inlineDialog.find("#step-defects-"+stepResultId);
                	$(".testStep[data-step-id="+stepResultId+"] #defects-actions .count").html("("+data.length+")");

                    resetList(defectsList);
                	$.each( data , function(index, value ) {
                        var innerHtml = RAVEN.ExecuteTest.defectLink({
                        	appId : value.appId ? value.appId : '',
                			id : value.id,
                			key : value.key,
                			summary : value.summary,
                			iconUrl : value.iconUrl,
                			status : value.status,
                			readonly : getReadonly(),
                			issueURL : value.url
                        });

                        defectsList.append(innerHtml);
                	});

                    // This enables the dialog
                    if(data.length > 0) {
                         inlineDialog.closest(".step-inline-dialog").attr("id","ts-defects-" + stepResultId);
	                   	 $("a[aria-controls='ts-defects-"+stepResultId+"']").removeClass("disabled");
                         resetStepDefectsList(stepResultId);
                    } else {
                         inlineDialog.closest(".step-inline-dialog").attr("id","ts-defects-" + stepResultId + "-disabled");
	                   	 $("a[aria-controls='ts-defects-"+stepResultId+"']").addClass("disabled");
	                   	 removeStepDefectsList(stepResultId);
                    }

                },
                complete: function() {
                	refreshActivityStream();
                    $('#execution-steps-content').unblock();
                }
            });
        },

        deleteAttachmentConfirmationDialog: function(attachmentId, attachmentName, fnDeleteAttachment, options) {
            var dialog = new JIRA.FormDialog({
                id: 'delete-attachment-dialog-' + attachmentId,
                content: function(callback) {
                    var innerHtml = RAVEN.Attachments.deleteAttachmentConfirmationDialog({
                        attachmentName: attachmentName
                    });
                    callback(innerHtml);
                },
                submitHandler: function(e) {
                    fnDeleteAttachment(options);
                    dialog.hide();
                    e.preventDefault();
                },
                autoClose: true
            });

            dialog.show();
        },

        deleteDefectConfirmationDialog: function(key, fnDeleteEvidence, options) {
            var dialog = new JIRA.FormDialog({
                id: 'delete-defect-dialog-' + key,
                content: function(callback) {
                    var innerHtml = RAVEN.ExecuteTest.deleteDefectConfirmationDialog({
                        defectKey: key
                    });
                    callback(innerHtml);
                },
                submitHandler: function(e) {
                	fnDeleteEvidence(options);
                    dialog.hide();
                    e.preventDefault();
                },
                autoClose: true
            });

            dialog.show();
        },

        deleteAttachmentFromStepResult: function(options) {
            $('#execution-steps-content').block();
            var self = this;
            clearMessage();

            var id = options.$el.val();
            var path = contextPath + '/rest/raven/1.0/testrun/' + getTestRunId() + '/steps/' + options.stepResultId + '/attachment/' + id;

            $.ajax({
                type: "DELETE",
                url: path,
                processData: false,
                async: true,
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() != '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }

                    message(AJS.I18n.getText('execute-test-action.attachments.delete.error'), errorThrown)
                },
                success: function(data, textStatus, jqXHR) {
                	options.refreshFn(options.stepResultId);

                    refreshActivityStream();
                },
                complete: function() {
                    $('#execution-steps-content').unblock();
                }
            });
        }

    });

})(AJS.$);
