(function ($) {

    $( document ).ready(function() {

        var currentProject = null;

        if(JIRA.ProjectConfig && JIRA.ProjectConfig.getKey) {
            currentProject = JIRA.ProjectConfig.getKey();
        } else {
            return; // disable if there is no project context
        }

        var pageURL = decodeURIComponent(window.location.search.substring(1));
        var urlVariables = pageURL.split('&');

        for (var i = 0; i < urlVariables.length; i++) {
            var parameter = urlVariables[i].split('=');

            if (parameter[0] === "versionsCopied") {
                $('#project-config-panel-summary').before('<div class="message"></div>');

                if(parameter[1] === "true") {
                    AJS.messages.success('.message', {
                        body: AJS.I18n.getText('raven-admin-config.copy-project-versions-dialog.message.success', currentProject)
                    });
                } else if (parameter[1] === "false") {
                    AJS.messages.error('.message', {
                        body: AJS.I18n.getText('raven-admin-config.copy-project-versions-dialog.message.error', currentProject)
                    });
                }

                $('.message').delay(5000).fadeOut('slow');
            }
        }

        function getUrlAction() {
            var url = contextPath + "/XrayCopyProjectVersionsConfiguration.jspa?xrayProject=" + currentProject;

            var selectedProjects = AJS.$("#select2-projects").auiSelect2('val');
            $(selectedProjects).each(function() {
                url += "&project=" + this;
            });

            return url;
        }

        function getProjects() {
            var projects = [];
            $.ajax({
                type: "GET",
                url: contextPath + "/rest/api/2/project",
                async: false,
                contentType: "application/json",
                success: function (data) {
                    for(var i = 0; i < data.length; i++) {
                        if(data[i].key !== currentProject) {
                            projects[projects.length] = data[i];
                        }
                    }
                }
            });
            return projects;
        }

        var $projects = getProjects();

        function createProjectsForm() {
            var form = '<form class="aui projects" id="copy-versions-selector-form">' +
                '<div class="copy-project-versions-container-main">' + AJS.I18n.getText('dialogs.copy-project-versions-dialog.description', currentProject) + '</div>' +
                '<div class="copy-project-versions-container"><div id="select2-projects"></div></div></form>';
            return form;
        }

        function formatSelection(o) {
            if (!o.id){
                return o.text;
            } else {
                var span = '<a href="'+ AJS.contextPath() + '/projects/' + o.key + '"><div style="padding: 2px;"><img style="padding: 0;" src="'+ o.icon +'" width="16" height="16"><div style="float: right; padding: 2px 4px; color:#3b73af;">' + o.text + '</div></div></a>';
                return span;
            }
        }

        function formatResult(o) {
            if (!o.id){
                return o.text;
            } else {
                var span = '<div style="padding: 2px;"><img style="float: left; padding: 4px;" src="'+ o.icon +'" width="20" height="20"><div style="float: left;padding: 4px;">' + o.text + '</div></div>';
                return span;
            }
        }

        // try to fill the window
        var innerWidth = window.innerWidth || document.documentElement.clientWidth || document.body.clientWidth;
        var innerHeight = window.innerHeight || document.documentElement.clientHeight || document.body.clientHeight;

        var width = Math.max(Math.round(innerWidth * 0.30), 600);
        var height = Math.max(Math.round(innerHeight * 0.30), 250);

        var dialog = new AJS.Dialog({
            width: width,
            height: height,
            id:"raven-copy-project-versions-dialog"
        });

        title = AJS.I18n.getText('dialogs.copy-project-versions-dialog.title', currentProject);

        dialog.addHeader(title);

        dialog.addPanel(AJS.I18n.getText('dialogs.choose-requirement-projects'), createProjectsForm(), "panel-body");

        dialog.addSubmit("OK",function () {
            window.location.href = getUrlAction();
        });

        dialog.addLink(AJS.I18n.getText('common.forms.cancel'), function (dialog) {
            $('#select2-projects').auiSelect2('data', null);
            dialog.hide();
        });

        AJS.$("#raven-copy-project-versions").click(function() {
            AJS.$("#select2-projects").auiSelect2({
                data: $.map($projects, function (obj) {
                    obj.id = obj.key;
                    obj.text = obj.name;
                    obj.icon = obj.avatarUrls['16x16'];
                    obj.url = obj.key;
                    obj.key = obj.key;
                    return obj;
                }),
                multiple: true,
                formatResult: formatResult,
                formatSelection: formatSelection
            });
            dialog.show();
        });
    });

})(AJS.$);
