(function($) {

	var CREATE_ISSUE_DIALOG_ID = 'create-issue-dialog';
	var EDIT_ISSUE_DIALOG_ID = 'edit-issue-dialog';

    var initContext = null;

	var __$ = function(elem) {
		return getContext().find(elem);
	}

	function getContext() {
        if (initContext) {
            return $(initContext);
        } else {
            var $context = $('#create-issue-dialog.jira-dialog-open, #create-issue-dialog.aui-dialog-open, \
                    #edit-issue-dialog.jira-dialog-open, #edit-issue-dialog.aui-dialog-open, \
                    #create-test-issue-dialog.jira-dialog-open, #create-test-issue-dialog.aui-dialog-open, \
                    #create-requirement-test-issue-dialog.jira-dialog-open, #create-requirement-test-issue-dialog.aui-dialog-open, \
    				#prefillable-create-issue-dialog.jira-dialog-open');
    		
    		if ($context.length > 0) {
    			return $($context);	
    		} else {
    			return $('#page');
    		}
        }
	}

	String.prototype.bool = function() {
		return (/^true$/i).test(this);
	};

	function getTestIssueKey() {
		return __$('#raven-test-field-type-issue-key').val();
	}

    function hasPrecondition() {
        return __$('#raven-has-precondition').val() === 'true';
    }

    function getPersistedTestTypeValue() {
        return __$('#raven-test-field-type-value').val();
    }

    function isManualTestTypeOption(value) {
        var manualOptions = __$("#manual-options").val();
        if (manualOptions) {
            var ids = JSON.parse(manualOptions);
            return _.contains(ids, parseInt(value, 10));
        }

        return false;
    }

    function isCucumberTestTypeOption(value) {
        var cucumberOptions = __$("#cucumber-options").val();
        if (cucumberOptions) {
            var ids = JSON.parse(cucumberOptions);
            return _.contains(ids, parseInt(value, 10));
        }

        return false;
    }

    function isOtherTestTypeOption(value) {
        return isCucumberTestTypeOption(value) == false && isManualTestTypeOption(value) == false;
    }

	function deleteTestInformation() {
		return $('#delete-test-definition').val() === "true";
	}

    function compareTypeOptions(value1, value2) {
        return (isManualTestTypeOption(value1) === true && isManualTestTypeOption(value2) === true)
            || (isCucumberTestTypeOption(value1) === true && isCucumberTestTypeOption(value2) === true)
            || (isOtherTestTypeOption(value1) === true && isOtherTestTypeOption(value2) === true);
    }

    function compareTypeOptionValues(value1, value2) {
        return (isManualTestTypeOption(value1) === true && isManualTestTypeOption(value2) === true)
            || (isCucumberTestTypeOption(value1) === true && isCucumberTestTypeOption(value2) === true)
            || (value1 == value2);
    }

	function changeTestType() {
		var typeFieldId = __$('#raven-test-field-type').val();

		var typeField = __$('#' + typeFieldId);

		var currentValue = typeField.val();
		var previousValue = typeField[0].previousValue;
		var currentType = $.trim(typeField.children('option[value=' + currentValue + ']').text());
		var value = previousValue;
		

        var isFieldEmpty = true;

		// Check if the current field is empty
		if (isManualTestTypeOption(value) === true) {
			if(__$('#manual-test-steps-table tbody tr').length > 1) {
				isFieldEmpty = false;
			}
		} else if (isCucumberTestTypeOption(value) === true) {
			var testStepsAceEditor = getCurrentAceEditor();
			if(testStepsAceEditor && testStepsAceEditor.getSession().getValue() != '') {
				isFieldEmpty = false;
			}
		} else{
			if(__$('#test-path-create-field').val() != '') {
				isFieldEmpty = false;
			}
		}

        var isToShowMessage = !isFieldEmpty && !compareTypeOptions(previousValue, currentValue) && deleteTestInformation();
		
		if(isToShowMessage) {
			var cancelFn = function() {
				typeField.val(previousValue);
			};
			
			var title = AJS.I18n.getText('view-issue-test-section.dialogs.changetype.title',currentType);
			var msg = AJS.I18n.getText('view-issue-test-section.dialogs.changetype.generic.message');
			confirmChangeTestType(title, msg, cancelFn);
		} else {
			performChangeTestType();
		}
	}

	function confirmChangeTestType(title, message, cancelFn) {
		var dialog = new JIRA.FormDialog({
			id : 'raven-change-test-type',
			content : function(callback) {
				var innerHtml = RAVEN.Dialog.confirmationDialog({
					title : title,
					message : message
				});
				callback(innerHtml);
			},
			submitHandler : function(e) {
				dialog.hide();
				e.preventDefault();
				performChangeTestType();
			},
			autoClose : true
		});
		dialog.bind('Dialog.hide', function(e, popup, hideReason) {
			if (hideReason === JIRA.Dialog.HIDE_REASON.cancel || hideReason === JIRA.Dialog.HIDE_REASON.escape) {
				cancelFn();
			}
		});

		dialog.show();
	}

	function resetFields(fields) {
		var props = {};
		$.each(fields, function(i, field) {
			var fieldId = $(field).data('field-id');
			props[fieldId] = null;
		});

		var value = {
			"fields" : props
		}
		var pack = JSON.stringify(value);
		$.ajax({
			type : "PUT",
			url : contextPath + '/rest/api/2/issue/' + getTestIssueKey(),
			processData : false,
			async : true,
			contentType : "application/json",
			data : pack,
			error : function(qXHR, textStatus, errorThrown) {
				__$('#raven-error-message').remove(); // clear previous
				// message
				if (errorThrown && errorThrown.trim() != '') {
					errorThrown = errorThrown + ': ' + qXHR.responseText;
				} else {
					errorThrown = qXHR.responseText;
				}
				AJS.messages.error({
					title : AJS.I18n.getText('view-issue-test-section.steps.delete.error'),
					body : errorThrown,
					id : "raven-error-message"
				});
			}
		});
	}
	
	function resetManualGeneral() {
		// TODO: empty manual steps table
	}

	function resetCucumberScenario() {
		var testStepsAceEditor = getCurrentAceEditor();
		if(testStepsAceEditor) {
            testStepsAceEditor.getSession().setValue("");
            testStepsAceEditor.setValue("");
            __$('#test-steps-hidden-value').val('');
            __$('#test-steps-hidden-new-value').val('');
			__$('#test-steps-ace-editor').css('display', 'none');
			__$('#test-steps-empty').removeAttr('style');
		}
	}
	
	function resetPath() {
		__$('#test-path-create-field').val('');
	}

	function initTestType() {
		var typeFieldId = __$('#raven-test-field-type').val();
		var typeField = __$('#' + typeFieldId);
		var name = typeField.attr('name');
		var id = typeField.attr('id');

		var stepsField = __$('#test-steps-create-ace-editor');
		var testPath=__$('#test-path-create-field');
		var automatedTestType=__$('#automated-test-type');

		typeField.off('change.raven-edit-test');
		typeField.on('change.raven-edit-test', function() {
			changeTestType();
		});

		var value = typeField.val();
		var currentType = $.trim(typeField.children('option[value=' + value + ']').text());
		if (isManualTestTypeOption(value) === true) {
			stepsField.parent().hide();
			__$('#manual-test-steps').parent().show();
			testPath.parent().hide();
			automatedTestType.parent().hide();
			var fields = __$('.raven-editable-field');
			__$('.ace_cursor').css('left', '4px');
		} else if (isCucumberTestTypeOption(value) === true) {
			stepsField.parent().show();
			stepsField.parent().show();
			testPath.parent().hide();
			automatedTestType.parent().show();
			__$('#manual-test-steps').parent().hide();
		} else {
			testPath.parent().show();
			stepsField.parent().hide();
			automatedTestType.parent().hide();
			__$('#manual-test-steps').parent().hide();
		}
		__$('#test-precondition-match-type').val(currentType);
		typeField[0].previousValue = typeField.val();
	}

	function performChangeTestType() {
		var typeFieldId = __$('#raven-test-field-type').val();

		var typeField = __$('#' + typeFieldId);
		var stepsField = __$('#test-steps-create-ace-editor');
		var testPath=__$('#test-path-create-field');
		var automatedTestType=__$('#automated-test-type');

		var value = typeField.val();
		var currentType = $.trim(typeField.children('option[value=' + value + ']').text());
		if (isManualTestTypeOption(value) === true) {
            stepsField.parent().hide();
            __$('#manual-test-steps').parent().show();
            testPath.parent().hide();
            automatedTestType.parent().hide();
            var fields = __$('.raven-editable-field');
            __$('.ace_cursor').css('left', '4px');

			if(deleteTestInformation()) {
				resetCucumberScenario();
				resetPath();
			}
		} else if (isCucumberTestTypeOption(value) === true) {
			stepsField.parent().show();

			if(deleteTestInformation()) {
				resetManualGeneral();
				resetPath();
			}

			stepsField.parent().show();

            // reset value after editor is visible because the editor was not updated when it was reseted
            var testStepsAceEditor = getCurrentAceEditor();
            testStepsAceEditor.getSession().setValue(__$('#test-steps-create-ace-editor-hidden').val());

			testPath.parent().hide();
			automatedTestType.parent().show();
			__$('#manual-test-steps').parent().hide();
			if(deleteTestInformation()) {
				// reset manual steps table
				RAVEN.Component.TestStepsTable.reset();
			}
		} else {
            testPath.parent().show();
            stepsField.parent().hide();

			if(deleteTestInformation()) {
				resetManualGeneral();
				resetCucumberScenario();
			}

            automatedTestType.parent().hide();
            __$('#manual-test-steps').parent().hide();
			if(deleteTestInformation()) {
				RAVEN.Component.TestStepsTable.reset(); // reset manual steps table
			}
		}
		__$('#test-precondition-match-type').val(currentType);
		typeField[0].previousValue = typeField.val();

        // show Test -> PreCondition type mismatch warning
        var typeMatches = compareTypeOptionValues(getPersistedTestTypeValue(), typeField.val());
        if (hasPrecondition() && typeMatches === true) {
            __$('#raven-edit-test-type-precondition-mismatch-message').hide();
        } else {
            __$('#raven-edit-test-type-precondition-mismatch-message').show();
        }
	}

	function getCurrentAceEditor() {
		return __$('#test-steps-create-ace-editor').data('ace-editor');
	}

	function initScreen() {
		__$('#raven-edit-test-type-message').empty();

		var typeFieldId = __$('#raven-test-field-type').val();

		if (typeFieldId) {
			var select = __$('#' + typeFieldId)[0];
			if (select == undefined)
				return;
			select.previousValue = select.value;

			initTestType();
		}

		// init context test set
		if (window.createTestForTestSet) {
			var $select = __$('#' + createTestForTestSet.testSetsCustomFieldId);
			$select.trigger('selectOption', {
				value: createTestForTestSet.testSetKey
			});
		}

		// init ace gherkin editor
		var editorElem = __$('#test-steps-create-ace-editor');
		if (editorElem.length > 0) {
			require(['ace/ace'], function(ace) {
				var testStepsAceEditor = ace.edit(editorElem[0]);
				var theme = require("ace/theme/textmate");
				testStepsAceEditor.setTheme(theme);
				testStepsAceEditor.setFontSize(14);
				testStepsAceEditor.getSession().setMode("ace/mode/gherkin");
				testStepsAceEditor.renderer.setShowGutter(false);
				testStepsAceEditor.renderer.setShowPrintMargin(false);
				
				testStepsAceEditor.getSession().setValue(__$('#test-steps-create-ace-editor-hidden').val());
				
				testStepsAceEditor.getSession().on('change', function() {
					var testStepsAceEditor = getCurrentAceEditor();
					if (testStepsAceEditor) {
						__$('#test-steps-create-ace-editor-hidden').val(testStepsAceEditor.getSession().getValue());
					}
				});
				
				// register dom reference
				editorElem.data('ace-editor', testStepsAceEditor);
			});
		}
	}

	function clearScreen() {
		var typeFieldId = __$('#raven-test-field-type').val();
		__$('#' + typeFieldId).attr('disabled', 'disabled');
		__$('#manual-test-steps-table .edit-command').css('display', 'none');
		__$('#manual-test-steps-table .remove-command').css('display', 'none');
		__$('input[id="add-step"]').css('display', 'none');
		__$('textarea[id="step-textarea"]').css('display', 'none');
		__$('textarea[id="data-textarea"]').css('display', 'none');
		__$('textarea[id="result-textarea"]').css('display', 'none');
	}
	
	// register for NEW_CONTENT_ADDED
	JIRA.bind(JIRA.Events.NEW_CONTENT_ADDED, function(e, context, reason) {
        if (context) {
            initContext = context.parent();
		    initScreen();
            initContext = null;
        } else {
            initScreen();
        }
	});

	/* Sometimes, the JIRA.CONTENT_ADDED_REASON.pageLoad event fires before the 
	 * page is completly loaded and the ace global object is not yet available. 
	 * Because of this, we init the screen (with the ace editor) only on dom ready. 
	 */
	$(document).ready(function() {
		initScreen();
	});

})(AJS.$);
