(function ($) {

    window.RAVEN = window.RAVEN || {};

    RAVEN.Filters = RAVEN.Filters || {};

    RAVEN.Filters.KEY_EMPTY = 'empty';
    RAVEN.Filters.KEY_TEST_TYPE = 'testtype';
    RAVEN.Filters.KEY_LABEL = 'label';
    RAVEN.Filters.KEY_PRIORITY = 'priority';
    RAVEN.Filters.KEY_COMPONENT = 'component';
    RAVEN.Filters.KEY_SAVED_FILTER = 'savedfilter';
    RAVEN.Filters.KEY_TESTSET = 'testset';
    RAVEN.Filters.KEY_WORKFLOW_STATUS = 'workflowstatus';
    RAVEN.Filters.KEY_CONTAINS = 'contains';
    RAVEN.Filters.KEY_REQUIREMENT = 'requirement';
    RAVEN.Filters.KEY_REQUIREMENTVERSION = 'requirementversion';
    RAVEN.Filters.KEY_TESTSET_VERSION = 'testsetversion';
    RAVEN.Filters.KEY_PROJECT = 'project';

    RAVEN.Filters.MODE_SIMPLE = 1;
    RAVEN.Filters.MODE_ADVANCED = 2;

    RAVEN.Filters.FilterDialog = Backbone.View.extend({

        groups: null,
        colCount: null,
        filters: null,
        projectKey: null,
        requiresExperimentalDialog2: null,
        domContainer: null,
        applyCallback: null,
        customFieldTestTypeId: null,
        jqlConstant: null,

        filterContent: [],
        pickerData: [],
        renderedContent: null,
        currentJQL: '',
        testIssueTypeId: null,

        unscopedEvents: {
            'click.raven.filter #raven-dialog-clear': 'clearDialog',
            'click.raven.filter #raven-dialog-apply': 'applyDialog',
            'click.raven.filter #raven-test-dialog-cancel': 'cancelDialog',
            'click.raven.filter .filter-title': 'filterTabClick'
        },

        initialize: function (options) {
            if (options) {
                _.extend(this, _.pick(options, "groups"));
                _.extend(this, _.pick(options, "colCount"));
                _.extend(this, _.pick(options, "filters"));
                _.extend(this, _.pick(options, "projectKey"));
                _.extend(this, _.pick(options, "requiresExperimentalDialog2"));
                _.extend(this, _.pick(options, "domContainer"));
                _.extend(this, _.pick(options, "customFieldTestTypeId"));
                _.extend(this, _.pick(options, "applyCallback"));
                _.extend(this, _.pick(options, "jqlConstant"));
            } else {
                console.error('Error: unable to init filter dialog');
                return;
            }

            this.initTestType();
            this.delegateUnscopedEvents(this.unscopedEvents);
            this.processFilters();
        },

        //todo: if the filter-dialog evolves in the future, an action on the backend and a vm template should be used to replace this request
        initTestType: function() {
            var self = this;
            $.ajax({
                type: "GET",
                url: contextPath + "/rest/raven/1.0/filterDialog/testIssueType",
                async: false,
                contentType: "application/json",
                success: function (data) {
                    self.testIssueTypeId = data.id;
                },
                error: function () {
                    console.error('filter-dialog: error fetching Test issue type name');
                    //in the unlikely case of error, fall back to using 'Test' -> id or name is indifferent for this purpose
                    self.testIssueTypeId = 'Test';
                }
            });
        },

        processFilters: function () {
            var self = this;
            var filterData = this.filters.map(function (filter) {
                return self.initFilter(filter);
            });

            this.processSimpleFilters(this.filterSimpleFilters(filterData));
            this.processRequestableFilters(this.filterRequestableFilters(filterData));
        },

        filterSimpleFilters: function (filterData) {
            return filterData.filter(function (filter) {
                return filter.request === null
            });
        },

        filterRequestableFilters: function (filterData) {
            return filterData.filter(function (filter) {
                return filter.request !== null;
            });
        },

        processSimpleFilters: function (filterData) {
            var self = this;
            filterData.forEach(function (simpleFilter) {
                self.addFilterData(simpleFilter.key, null);
            })
        },

        processRequestableFilters: function (filterData) {
            var filterRequests = filterData.map(function (filter) {
                return filter.request;
            });
            var self = this;
            $.when.apply($, filterRequests)
                .then(function () {
                    self.postProcessFilters();
                });
        },

        initFilter: function (filter) {
            var request = this.getKeyRequest(filter.key);
            return {key: filter.key, group: filter.group, request: request};
        },

        getKeyRequest: function (key) {
            switch (key) {
                case RAVEN.Filters.KEY_TEST_TYPE:
                    return this.requestTestTypes();
                case RAVEN.Filters.KEY_PRIORITY:
                    return this.requestPriority();
                case RAVEN.Filters.KEY_COMPONENT:
                    return this.requestComponents();
                case RAVEN.Filters.KEY_SAVED_FILTER:
                    return this.requestSavedFilters();
                case RAVEN.Filters.KEY_TESTSET:
                case RAVEN.Filters.KEY_REQUIREMENT:
                    return this.requestContextPath(key);
                case RAVEN.Filters.KEY_REQUIREMENTVERSION:
                case RAVEN.Filters.KEY_TESTSET_VERSION:
                    return this.requestVersions(key);
                case RAVEN.Filters.KEY_PROJECT:
                case RAVEN.Filters.KEY_WORKFLOW_STATUS:
                case RAVEN.Filters.KEY_CONTAINS:
                case RAVEN.Filters.KEY_LABEL:
                default:
                    return null;
            }
        },

        requestTestTypes: function () {
            var url = contextPath + '/rest/raven/1.0/customField/TEST_TYPE';
            return this.requestGET(url, RAVEN.Filters.KEY_TEST_TYPE);
        },

        requestVersions: function (key) {
            var url = contextPath + '/rest/api/2/project/' + this.projectKey + '/versions';
            return this.requestGET(url, key);
        },

        requestPriority: function () {
            var url = contextPath + '/rest/api/2/priority';
            return this.requestGET(url, RAVEN.Filters.KEY_PRIORITY);
        },

        requestComponents: function () {
            var url = contextPath + "/rest/api/2/project/" + this.projectKey + "/components";
            return this.requestGET(url, RAVEN.Filters.KEY_COMPONENT);
        },

        requestSavedFilters: function () {
            var url = contextPath + "/rest/gadget/1.0/filtersAndProjects";
            return this.requestGET(url, RAVEN.Filters.KEY_SAVED_FILTER,
                function (data) {
                    return data.options[0].group.options;
                });
        },

        requestGET: function (url, key, processData) {
            var self = this;
            return $.ajax({
                type: "GET",
                url: url,
                processData: true,
                async: true,
                contentType: "application/json",
                success: function (data) {
                    if (processData != null) {
                        data = processData.call(self, data);
                    }
                    self.addFilterData(key, data);
                },
                error: function () {
                    console.error('error getting filter' + key);
                }
            });
        },

        requestContextPath: function (key) {
            var self = this;
            return $.ajax({
                contentType: "application/json",
                async: false,
                complete: function () {
                    self.addFilterData(key, contextPath);
                },
            });
        },

        addFilterData: function (key, data) {
            this.filterContent.push({type: key, filterContent: data});
        },

        postProcessFilters: function () {
            var self = this;

            var processedGroups = this.groups.map(function (group) {
                return self.createGroupJSON(group);
            });
            this.renderFiltersSoy(processedGroups);
        },

        createGroupJSON: function (group) {
            var self = this;
            var filtersInGroup = this.filters
                .filter(function (filter) {
                    return filter.group === group;
                })
                .map(function (filter) {
                    return filter.key;
                });

            var filterDataInGroup = filtersInGroup.map(function (key) {
                for(var i = 0; i < self.filterContent.length; i++) {
                    var filterContent = self.filterContent[i];
                    if(filterContent.type === key) {
                        return filterContent;
                    }
                }
            });
            return {label: group, groupArray: filterDataInGroup};
        },

        renderFiltersSoy: function (filters) {
            var self = this;
            this.renderedContent = RAVEN.Filters.renderFilters({
                'filterGroups': filters,
                'colCount': 3,
                'requiresExperimentalDialog2': self.requiresExperimentalDialog2
            });

            $(this.domContainer).html(this.renderedContent);

            this.postRenderFilters();
            this.initializeJQLSearch();
        },

        postRenderFilters: function () {
            var self = this;
            this.pickerData = this.filterContent.map(function (filterContent) {
                return self.createPicker(filterContent);
            }).filter(function (element) {
                return element !== null;
            });

            this.setUnscopedEvents();
        },

        createPicker: function (filterContent) {
            switch (filterContent.type) {
                case RAVEN.Filters.KEY_TEST_TYPE:
                    return this.createGenericPicker('raven-test-filter-testtype', AJS.I18n.getText('custom-field.name.test-type'), 'cf[' + this.customFieldTestTypeId + ']');
                case RAVEN.Filters.KEY_PRIORITY:
                    return this.createGenericPicker('raven-test-filter-priority', AJS.I18n.getText('common.label.priority'), 'priority');
                case RAVEN.Filters.KEY_COMPONENT:
                    return this.createGenericPicker('raven-test-filter-component', AJS.I18n.getText('common.label.components'), 'component');
                case RAVEN.Filters.KEY_LABEL:
                    return this.createPickerData(this.createLabelPicker('raven-test-filter-label'), 'raven-test-filter-label', AJS.I18n.getText("common.label.label"), 'labels', this.buildJQLMultiSelect);
                case RAVEN.Filters.KEY_SAVED_FILTER:
                    return this.createPickerData(this.createSinglePicker('raven-test-filter-saved-filter'), 'raven-test-filter-saved-filter', AJS.I18n.getText("common.label.savedfilter"), 'savedfilter', this.buildJQLSavedFilter);
                case RAVEN.Filters.KEY_TESTSET:
                    return this.createPickerData(this.createIssuePicker('raven-test-filter-testset'), 'raven-test-filter-testset', AJS.I18n.getText('test-sets-custom-field.title'), 'null', this.buildJQLTestSet);
                case RAVEN.Filters.KEY_WORKFLOW_STATUS:
                    return this.createPickerData(this.createIssueStatusPicker('raven-test-filter-workflow-status'), 'raven-test-filter-workflow-status', AJS.I18n.getText('raven-admin-config.general.label.status'), 'status', this.buildJQLMultiSelect);
                case RAVEN.Filters.KEY_CONTAINS:
                    return this.createPickerData(this.createTextInput('raven-test-filter-search-text'), 'raven-test-filter-search-text', AJS.I18n.getText('common.label.contains-text'), 'null', this.buildJQLContainsText);
                case RAVEN.Filters.KEY_REQUIREMENT:
                    return this.createPickerData(this.createIssuePicker('raven-test-filter-requirement'), 'raven-test-filter-requirement', AJS.I18n.getText('raven-admin-config.general.label.requirement'), 'null', this.buildJQLRequirement);
                case RAVEN.Filters.KEY_REQUIREMENTVERSION:
                    return this.createPickerData(this.createVersionPicker('raven-test-filter-requirement-version'), 'raven-test-filter-requirement-version', AJS.I18n.getText('dialogs.issueselector.requirement-fixversion'), 'null', this.buildJQLRequirementVersion);
                case RAVEN.Filters.KEY_TESTSET_VERSION:
                    return this.createPickerData(this.createMultiSelectVersion('raven-test-filter-testsetversion'), 'raven-test-filter-testsetversion', AJS.I18n.getText('view-issue-search-filter.label.testset-fixversion'), 'null', this.buildJQLTestSetVersion);
                case RAVEN.Filters.KEY_PROJECT:
                    return this.createPickerData(this.createProjectPicker('raven-test-filter-project'), 'raven-test-filter-project', AJS.I18n.getText('common.label.project'), 'null', this.buildJQLProject);
                case RAVEN.Filters.KEY_EMPTY:
                default:
                    return null;
            }
        },

        createGenericPicker: function (elementId, description, jqlClause) {
            return this.createPickerData(this.renderPicker(elementId), elementId, description, jqlClause, this.buildJQLMultiSelect);
        },

        createPickerData: function (picker, elementId, description, jqlClause, jqlRenderer) {
            var self = this;
            var pData = {
                picker: picker,
                filterDescription: description,
                elementId: elementId,
                jqlClause: jqlClause,
                projectKey: self.projectKey
            };

            pData.jqlRenderer = $.proxy(jqlRenderer, pData);
            return pData;
        },

        /*******************VISUAL INITS*******************************/
        renderPicker: function (elementId) {
            return new AJS.MultiSelect({
                element: $('#' + elementId),
                itemAttrDisplayed: "label",
                showDropdownButton: true,
                removeOnUnSelect: false,
                submitInputVal: false,
                errorMessage: AJS.I18n.getText("jira.ajax.autocomplete.error"),
                maxInlineResultsDisplayed: 8
            });
        },

        createLabelPicker: function (elementId) {
            return new JIRA.LabelPicker({
                element: $('#' + elementId),
                ajaxOptions: {
                    url: contextPath + '/rest/api/1.0/labels/suggest',
                    data: {}
                }
            });
        },

        createIssuePicker: function (elementId) {
            var picker = new RAVEN.IssuePicker({
                element: $('#' + elementId),
                userEnteredOptionsMsg: AJS.I18n.getText('dialogs.issueselector.default.enter'),
                uppercaseUserEnteredOnSelect: true
            });

            picker.$container.next('.issue-picker-popup').remove();
            picker.$container.removeClass('hasIcon');

            return picker;
        },

        createMultiSelectVersion: function (elementId) {
            return new AJS.MultiSelect({
                element: $('#' + elementId),
                itemAttrDisplayed: "label",
                showDropdownButton: true,
                removeOnUnSelect: false,
                submitInputVal: false,
                errorMessage: AJS.I18n.getText("jira.ajax.autocomplete.error"),
                maxInlineResultsDisplayed: 8
            });
        },

        createVersionPicker: function (elementId) {
            return new AJS.MultiSelect({
                element: $('#' + elementId),
                itemAttrDisplayed: "label",
                removeOnUnSelect: false,
                submitInputVal: true,
                errorMessage: AJS.I18n.getText("jira.ajax.autocomplete.versions.error"),
                maxInlineResultsDisplayed: 8
            });
        },

        createIssueStatusPicker: function (elementId) {
            return new AJS.MultiSelect({
                element: $('#' + elementId),
                showDropdownButton: true,
                submitInputVal: true,
                noQueryNoRequest: false,
                revertOnInvalid: true,
                loadOnInit: true,
                ajaxOptions: {
                    query: true,
                    url: contextPath + '/rest/api/2/status/',
                    formatResponse: function (data) {
                        var optgroup = new AJS.GroupDescriptor({
                            weight: 0,
                            id: "all-filter-opt-group",
                            replace: true
                        });

                        $(data).each(function () {
                            this.label = this.name;
                            var item = new AJS.ItemDescriptor({
                                value: this.name,
                                label: this.name,
                                title: this.description,

                                highlighted: true
                            });
                            optgroup.addItem(item);
                        });

                        return [optgroup];
                    }
                }
            });
        },

        createSinglePicker: function (elementId) {
            return new AJS.SingleSelect({
                element: $('#' + elementId),
                itemAttrDisplayed: "label",
                removeOnUnSelect: false,
                submitInputVal: false,
                maxInlineResultsDisplayed: 8
            });
        },

        createTextInput: function (elementId) {
            return $('#' + elementId);
        },

        createProjectPicker: function (elementId) {
            var self = this;
            return new AJS.SingleSelect({
                element: $('#' + elementId),
                showDropdownButton: true,
                submitInputVal: true,
                noQueryNoRequest: false,
                revertOnInvalid: true,
                loadOnInit: true,
                ajaxOptions: {
                    query: true,
                    url: contextPath + '/rest/api/2/project',
                    formatResponse: function (data) {
                        var optgroup = new AJS.GroupDescriptor({
                            weight: 0,
                            id: "all-projects-opt-group",
                            replace: true,
                            label: "All Projects"
                        });

                        $(data).each(function (i, project) {
                            var item = new AJS.ItemDescriptor({
                                value: project.key,
                                label: project.name,
                                icon: project.avatarUrls['16x16'],
                                allowDuplicate: false,
                                selected: project.key === self.projectKey ? 'selected' : null
                            });
                            optgroup.addItem(item);
                        });
                        return [optgroup];
                    }
                }
            });
        },


        /******************FILTER JQL PROCESSING***************************/

        buildFilterJQL: function () {
            if (this.getSearchMode() === RAVEN.Filters.MODE_ADVANCED) {
                return this.buildAdvancedSearchJQL();
            } else {
                return this.buildSimpleSearchJQL();
            }
        },

        getSearchMode: function () {
            var $selected = $('.filter-title.selected');
            return $selected.data('mode');
        },

        buildAdvancedSearchJQL: function () {
            if (!this.isAdvancedSearchValid()) {
                return '';
            }
            var jql = $('#filter-jql-text').val();
            if (jql === '') {
                return '';
            }
            return this.getJqlPrefix() + " and " + jql;
        },

        getJqlPrefix: function () {
            if (this.jqlConstant.length === 0) {
                return 'issuetype = ' + this.testIssueTypeId + ' ';
            }
            return 'issuetype = ' + this.testIssueTypeId + ' and ' + this.jqlConstant;
        },

        isAdvancedSearchValid: function () {
            var $error = $('#filter-jql-error');
            return !$error.hasClass('jqlerror');
        },

        buildSimpleSearchJQL: function () {
            var prefix = this.getJqlPrefix();
            var renderedJQL = this.pickerData
                .map(function (element) {return element.jqlRenderer();})
                .filter(function(element) {return element.trim().length > 0;})
                .map(function(element) {return '(' + element + ')';})
                .join(' and ');

            return renderedJQL.trim().length > 0 ?
                prefix + ' and ' + renderedJQL :
                prefix;
        },

        buildJQLMultiSelect: function () {
            var elements = $('#' + this.elementId + ' option:selected');
            if (!elements.length) {
                return '';
            }

            var joinedElements = elements.map(function () {
                return '"' + $(this).text() + '"';
            }).get().join();

            return this.jqlClause + ' in (' + joinedElements + ') ';
        },

        buildJQLSavedFilter: function () {
            var savedFilter = String($('#' + this.elementId + '').val());

            if (!savedFilter.length || !savedFilter.match('^filter-')) {
                return '';
            }
            savedFilter = savedFilter.substr('filter-'.length);
            return 'savedfilter = ' + savedFilter + ' ';
        },

        buildJQLTestSet: function () {
            var testSets = $('#' + this.elementId + ' option:selected');
            if (!testSets.length) {
                return '';
            }

            return testSets.map(function () {
                return 'issue in testSetTests("' + $(this).val() + '")';
            }).get().join(' or ');
        },

        buildJQLRequirement: function () {
            var requirements = $('#' + this.elementId + ' option:selected');
            if (!requirements.length) {
                return '';
            }

            return requirements.map(function () {
                return 'issue in requirementTests("' + $(this).val() + '")';
            }).get().join(' or ');
        },

        buildJQLRequirementVersion: function () {
            var versions = $('#' + this.elementId + ' option:selected');
            if (!versions.length) {
                return '';
            }

            var joinedVersions = versions.map(function () {
                return '"' + $(this).val() + '"'
            }).get().join();

            return ' issue in testsWithReqVersion("' + this.projectKey + '",' + joinedVersions + ')';
        },

        buildJQLContainsText: function () {
            var searchText = $('#' + this.elementId).val();
            if (!searchText) {
                return '';
            }
            searchText = searchText.trim();
            if (!searchText.length) {
                return '';
            }

            return '(summary ~ "' + searchText + '" OR description ~ "' + searchText + '" OR comment ~ "' + searchText + '")';
        },

        buildJQLTestSetVersion: function () {
            var versions = $('#' + this.elementId + ' option:selected');
            if (!versions.length) {
                return '';
            }

            var joinedVersions = versions.map(function () {
                return '"' + $(this).val() + '"'
            }).get().join();

            return 'issue in testsWithTestSetVersion("' + this.projectKey + '",' + joinedVersions + ')';
        },

        buildJQLProject: function() {
            var project = $('#' + this.elementId + ' option:selected').attr('value');
            if (!project) {
                return '';
            }

            return ' project = "' + project +'" ';
        },

        initializeJQLSearch: function () {
            var self = this;
            var autoCompletePromise = this.getJqlAutoCompleteData();
            var $jqlSearchText = $("#filter-jql-text");
            if ($jqlSearchText.attr("filter-jql-initialized")) {
                return;
            }
            if (!$jqlSearchText.attr("filter-jql-initialized")) {
                //setAutoCompleteLoadingIconVisible(true, $dialog);
                autoCompletePromise.done(function (smartAjaxResult) {
                    if (smartAjaxResult.successful) {
                        // Enable JQL AutoComplete
                        IssueLinkJQLAutoComplete.initialize({
                            fieldID: "filter-jql-text",
                            errorID: "filter-jql-error",
                            autoCompleteData: smartAjaxResult.data,
                            formSubmitFunction: function () {
                                self.applyDialog();
                            }
                        });
                    } else {
                        self.message('JQL filter', 'Unable to initialize JQL filter', 'error');
                    }
                    $jqlSearchText.attr('filter-jql-initialized', 1);
                });
            }
        },

        getJqlAutoCompleteData: function () {
            var deferred = $.Deferred();
            var url = contextPath + "/rest/api/2/jql/autocompletedata";

            JIRA.SmartAjax.makeRequest({
                url: url,
                complete: function (xhr, textStatus, smartAjaxResult) {
                    deferred.resolve(smartAjaxResult);
                }
            });
            return deferred.promise();
        },

        /*************************UNSCOPED EVENTS********************************/

        setUnscopedEvents: function () {
            var eventList = {};
            var self = this;
            this.pickerData.forEach(function (picker) {
                eventList[self.createEventEntryKey(picker)] = 'filterPreventDefault';
            });

            eventList['click.raven.filter .item-delete'] = 'filterPreventDefault';

            this.delegateUnscopedEvents(eventList);
        },

        createEventEntryKey: function (pickerData) {
            return 'click.raven.filter #' + pickerData.elementId + '-suggestions';
        },

        delegateUnscopedEvents: function (unscopedEvents) {
            var self = this;

            $.each(unscopedEvents, function (key, val) {
                var eventName = key.substring(0, key.indexOf(' '));
                var eventTrigger = key.substring(key.indexOf(' ') + 1);

                $(document).on(eventName, eventTrigger, $.proxy(self[val], self));
            });
        },

        filterPreventDefault: function (e) {
            e.stopPropagation();
            e.preventDefault();
        },

        /*************** PUBLIC ACCESS METHODS*******************/
        clearDialog: function () {
            //clear simple filters
            this.pickerData.forEach(function (element) {
                if (element.picker.clearLozenges) {
                    element.picker.clearLozenges();
                } else if (element.picker.clear) {
                    element.picker.clear();
                } else {
                    element.picker.val('');
                }
            });

            //clear advanced jql
            $('#filter-jql-text').val('');
            $('#jql-search-error').removeClass('jqlerror')
                .addClass('jqlgood');
        },

        applyDialog: function () {
            this.currentJQL = this.buildFilterJQL();
            this.renderFilterSummary();
            this.closeFilters();

            this.applyCallback.call(this, this.currentJQL);
        },

        cancelDialog: function () {
            this.closeFilters();
        },

        filterTabClick: function (e) {
            e.stopPropagation();
            var $trigger = $(e.currentTarget);
            $('.filter-title.selected').removeClass('selected');

            $trigger.addClass('selected');

            if ($trigger.data('mode') === RAVEN.Filters.MODE_ADVANCED) {
                $('#raven-test-filters-form').hide();
                $('#raven-test-advanced-search-form').show();
            } else {
                $('#raven-test-filters-form').show();
                $('#raven-test-advanced-search-form').hide();
            }
        },

        isFilterActive: function () {
            /*we consider 'inactive' jql to be an empty string or a string equal to the prefix since
             having only a prefix in the JQL means no filter is used.
             It is safer to keep the jql with a prefix than to keep the jql completely empty when no filter is used*/
            var prefix = this.getJqlPrefix().trim();
            var trimmedJQL = this.currentJQL.trim();
            return trimmedJQL.length > 0  && trimmedJQL !== prefix;
        },

        getCurrentJQL: function () {
            return this.currentJQL.trim();
        },

        /*************FILTER SUMMARY****************/
        renderFilterSummary: function () {
            var filterRepresentation;
            if (this.getSearchMode() === RAVEN.Filters.MODE_ADVANCED) {
                filterRepresentation = this.buildAdvancedFilterSummary();
            } else {
                filterRepresentation = this.buildSimpleFilterSummary();
            }

            if (filterRepresentation.length > 0) {
                $('#raven-text-filter-trigger')
                    .html(filterRepresentation)
                    .attr('data-tooltip', filterRepresentation)
                    .attr('original-title', '');
            } else {
                $('#raven-text-filter-trigger')
                    .html(AJS.I18n.getText('common.label.filter'))
                    .removeAttr('data-tooltip')
                    .removeAttr('original-title');
            }
        },

        buildAdvancedFilterSummary: function () {
            if (!this.isAdvancedSearchValid()) {
                return AJS.I18n.getText('common.label.filter');
            }
            return $('#filter-jql-text').val();
        },

        buildSimpleFilterSummary: function () {
            var self = this;

            var filterRepresentations = this.pickerData.map(function (element) {
                return self.getDescriptionFilter(element.elementId, element.filterDescription);
            });

            return filterRepresentations.filter(function (element) {
                return element.length > 0;
            }).join('; ');
        },

        getDescriptionFilter: function (filterId, filterTitle) {
            var element = $('#' + filterId);
            var nodeName = element.prop('nodeName');
            var filterValues;

            if (nodeName === 'INPUT') {
                filterValues = this.getDescriptionInput(filterId);
            } else {
                filterValues = this.getDescriptionGeneric(filterId);
            }

            return filterValues.length === 0 ? '' : '<b>' + filterTitle + '</b>  ' + filterValues;
        },

        getDescriptionInput: function (filterId) {
            return $('#' + filterId).val();
        },

        getDescriptionGeneric: function (filterId) {
            var values = $('#' + filterId + ' option:selected');
            if (!values.length || (values.length === 1 && values.val() === '-1')) {
                return '';
            }

            return values.map(function () {
                return $(this).html()
            }).get().join(', ');
        },

        closeFilters: function () {
            var $testFilterInlineDialog = $('#raven-test-filter-inline-dialog');
            if ($testFilterInlineDialog.attr('open') === 'open') {
                $testFilterInlineDialog.removeAttr('open');
            } else if ($('#raven-test-filter-inline-dialog').is(':visible')) {
                $('#raven-test-filter-inline-dialog').hide();
            }
        }
    });
})(AJS.$);