(function($) {

	RAVEN.namespace("RAVEN.Gadget");

	var OK_STATUS = 'OK';
	var NOK_STATUS = 'NOK';
	var NOTRUN_STATUS = 'Not Run';
    var UNKNOWN_STATUS = 'Unknown';
	var UNCOVERED_STATUS = 'Uncovered';

    var NONE = 'none';

	var overallChart = null;

	function message(title, message, type) {
		clearMessage();
		if (!type) {
			type = 'error';
		}
		AJS.messages[type]($('#gadget-message'), {
			title: title,
			body: message
		});
	}

	function serverError(gadget, message) {
        var obj = null;
        try {
            obj = JSON.parse(message);
        } catch(e) {}

        var obj = JSON.parse(message);

        var msg = null;
        if (obj && obj.message) {
            msg = obj.message;
        } else {
            msg = gadget.getMsg('gadget.requirement-coverage.common.error.data');
        }

		$('.gadget .view').append('<div id="gadget-message" style="margin:10px;"></div>');
		AJS.messages['error']($('#gadget-message'), {
			body: msg
		});
	}

	function clearMessage() {
		$('#gadget-message').empty();
	}

	function getStatusFromName(statusName) {
		if (statusName) {
			if (statusName === OK_STATUS) {
				return 'ok';
			} else if (statusName === NOK_STATUS) {
				return 'nok';
			} else if (statusName === NOTRUN_STATUS) {
				return 'notrun';
            } else if (statusName === UNKNOWN_STATUS) {
                return 'unknown';
			} else if (statusName === UNCOVERED_STATUS) {
				return 'uncovered';
			}
		}
		return null;
	}

	function createOverallCoverageChart(gadget, data) {
		var content = $('<div id="gadget-content" style="margin: 10px;"><div id="gadget-message"></div><div id="overall-requirement-coverage-chart"></div></div>');
		gadget.getView().html(content);

		if (data && data.resultset && data.resultset.length > 0) {

			var height = 0;

            //The size of the gadget will be calculated according to the number of rows it has
            if(data.rows && data.rows > 0){
                height = 40 * data.rows;
            }

			if(height < 300){
				height = 300;
			}else if(height > 1000){
				height = 1000;
			}

			clearMessage();

			overallChart = new pvc.NormalizedBarChart({
				canvas: "overall-requirement-coverage-chart",
				width: gadgets.window.getViewportDimensions().width - 10,
				height: height,
				orientation: 'horizontal',
				stacked: true,
				animate: true,
				legend:  true,
				interactive: true,
				selectable: false,
				clickable: true,
				hoverable: true,
				legendPosition: 'top',
				legendAlign:    'right',
				legendFont : '12px sans-serif',
				valuesFont : 'bold 12px sans-serif',
				orthoAxisLabelSpacingMin: 6,
				valuesVisible: true,
				valuesOptimizeLegibility: true,
				clickAction: function(scene) {
					sceneClickHandler(gadget, scene);
				},
				colors: ['#95C160', '#D45D52', '#F1E069', '#9AA1A8', '#3671A3'],
				extensionPoints: {
					axisLabel_font: '12px sans-serif',
					barLabel_text: function() {
						// don't show percentages less than 7% because the label is too big...
						var percent = this.scene.vars.value.percent.value;
						if (percent > 0.18) {
							return this.scene.vars.value.percent.label + ' (' + this.scene.vars.value.value + ')';
						} else {
							return null;
						}
					}
				},
                dimensions: {
                    // Shown in the axis, but not in the tooltip
                    categoryCopy: {isHidden: true}
                },
                calculations: [
                {
                    names: "categoryCopy",
                    calculation: function(datum, out) {
                        var category = datum.atoms.category.value;
                        out.categoryCopy = category.substr(0, 30) + (category.length > 30 ? "..." : "");
                    }
                }
                ],
                // Use the categoryCopy instead of the default category dimension in the base axis
                categoryRole: "categoryCopy"
            });

			overallChart.setData(data, {crosstabMode: false });
			overallChart.render();
		} else {
			message(null, gadget.getMsg("gadget.requirement-coverage.common.empty"), 'info');
		}
	}

	function sceneClickHandler(gadget, scene) {
		var datum = scene.datums().first();

		if (datum) {
			var group = datum.atoms.category.value;
			var status = datum.atoms.series.value;

			var link = createViewIssuesLink(gadget, group, status);
			window.parent.location.href = link;
		}
	}

    // TODO: Handle this link on the server
    function createViewIssuesLink(gadget, group, status) {
		var jql = "";

        var _filterId = gadget.getPref("filterId");
        var project = gadget.getPref("project");
        var projectFilter = gadget.getPref("projectFilter");
        var execScope = gadget.getPref("execScope");

        var filterByProject = project && project != '__ALL__' && ((_filterId && projectFilter == 'true') || !_filterId);

        // project
        var project = gadget.getPref("project");
        if (filterByProject) {
            jql = jql + ' and project = ' + project;
        } else {
            project = '';
        }

        // saved filter
        if (_filterId) {
            var filterId = "";
            if (_filterId) {
                filterId = _filterId.substring(7);
                jql = jql + ' and filter = ' + filterId;
            }
        }

        // grouping
        var grouping = gadget.getPref("grouping");
        if (grouping) {
            var groupCondition = createJqlGroupCondition(grouping, group);
            jql = jql + ' and ' + groupCondition;
        }

        // requirements JQL function
        var statusEnum = getStatusFromName(status).toUpperCase();
        var version = gadget.getPref("version");
        if (version == -1) {
            version = '';
        }

        var flattenRequirements = gadget.getPref("flattenRequirements");

        var testPlanKey = gadget.getPref("testPlanKey");

        if (execScope === 'version') {
        	jql = jql + ' and issue in requirements(' + statusEnum + ', "' + project + '", "' + version + '", "", "' + flattenRequirements + '")';
        } else if (execScope === 'testPlan') {
        	jql = jql + ' and issue in requirementsWithStatusByTestPlan(' + statusEnum + ', "' + testPlanKey + '", "", "' + flattenRequirements + '")';
        }

        // remove first " and "
        jql = jql.substring(5);

		var jql = encodeURIComponent(jql);
		return gadget.getBaseUrl() + '/secure/IssueNavigator.jspa?reset=true&mode=hide&jqlQuery=' + jql;
	}

    function createJqlGroupCondition(grouping, group) {
        if (!grouping) {
            return null;
        }

        if (group) {
            group = group.toLowerCase();
        }

        if (grouping === 'priority') {
            if (group !== NONE) {
                return 'priority = "' + group + '"';
            } else {
                return 'priority is EMPTY';
            }
        } else if (grouping === 'components') {
            if (group !== NONE) {
                var groupValue = group;
                var regex = /.*? \((\d*)\)/g;
                if (group.match(regex)) {
                    var match = regex.exec(group);
                    groupValue = match[1];
                }
                return 'component = "' + groupValue + '"';
            } else {
                return 'component is EMPTY';
            }
        } else if (grouping === 'status') {
            if (group !== NONE) {
                return 'status = "' + group + '"';
            } else {
                return 'status is EMPTY';
            }
        } else if (grouping === 'resolution') {
            if (group !== NONE) {
                return 'resolution = "' + group + '"';
            } else {
                return 'resolution is EMPTY';
            }
        } else {
            var cfId = grouping.replace(/customfield_/, '');
            if (group !== NONE) {
                return 'cf[' + cfId + '] = "' + group + '"';
            } else {
                return 'cf[' + cfId + '] is EMPTY';
            }
        }
    }

	function setTitle(gadget) {
		var execScope = gadget.getPref("execScope");
        var filterId = gadget.getPref("filterId");
        var project = gadget.getPref("project");
        var projectFilter = gadget.getPref("projectFilter");
        var testPlan = gadget.getPref("testPlanKey");
        var version = gadget.getPref("version");

        var params = '?execScope=' + execScope;

        if (project && project != '__ALL__') {
            params = params + '&project=' + project;
        }

        if (projectFilter) {
            params = params + '&projectFilter=' + projectFilter;
        }

        if (version && version != -1) {
            params = params + '&version=' + version;
        }

        if (testPlan && testPlan != -1) {
            params = params + '&testPlan=' + testPlan;
        }

        if (filterId) {
            // remove prefix "filter-"
            params = params + "&savedFilter=" + filterId.substring(7);
        }

		$.ajax({
			type: "GET",
			url: gadget.getBaseUrl() + '/rest/raven/1.0/chart/requirement/title' + params,
			contentType: "application/json",
			success: function(data) {
				if (data && data.titleParams) {
					var title = AJS.format(gadget.getMsg("gadget.requirement-coverage.overall.title"), data.titleParams);
					gadgets.window.setTitle(title);
				} else {
					var title = gadget.getMsg("gadget.requirement-coverage.overall.title.default");
					gadgets.window.setTitle(title);
				}
			}
		});
	}

	RAVEN.Gadget.overallRequirementCoverage = function(contextPath) {
		return AJS.Gadget({
			baseUrl: contextPath, /* Used to make base url available to gadget */
			useOauth: "/rest/gadget/1.0/currentUser",
			config: {
    			descriptor: function(args) {
					var gadget = this;

					$(document).off('change.raven');
					// register event handler for refreshing project versions
					$(document).on('change.raven.project', '#project', function() {
						var versionField = gadget.versionField;
                        var groupingField = gadget.groupingField;
						var selectedProject = $("#project option:selected").val();

                        groupingField.initProjectGroupings(gadget, selectedProject);
						versionField.initProjectVersions(gadget, selectedProject);
					});

					// register event handler for toggling execution scope fields
					$(document).on('change.raven.execscope', '#execScope', function() {
						var selectedScope = $("#execScope option:selected").val();
						if (selectedScope === "version") {
							$("#testPlanKey").prop('disabled', true);
							$("#version").prop('disabled', false);
						} else {
							$("#testPlanKey").prop('disabled', false);
							$("#version").prop('disabled', true);
						}
					});

					var versionDisabled = false;
					var testPlanKeyDisabled = false;
					var selectedScope = gadget.getPref("execScope");
					if (selectedScope === "version") {
						testPlanKeyDisabled = true;
					} else {
						versionDisabled = true;
					}

					// get project keys because the versions rest service only
					// accepts keys for project in JIRA prior to v6.1
					var projects = $.map(args.projects, function(p) {
						return {
							label: p.name,
							value: p.key
						};
					});

                    projects = [{
                        label: gadget.getMsg("common.label.all"),
                        value: "__ALL__"
                    }].concat(projects);

                    gadget.groupingField = RAVEN.Gadget.fields.GroupingSelect(gadget, "grouping", function() {
                        return $("#project option:selected").val(); //gadget.getPref("project");
                    });

					gadget.versionField = RAVEN.Gadget.fields.VersionSelect(gadget, "version", versionDisabled, true, function() {
						return $("#project option:selected").val(); //gadget.getPref("project");
					});

					var filters = null;
					$.each(args.filters.options, function() {
						if(this.group.label === "Filters") {
							filters = this.group;
						}
					});

                    if (filters) {
                        var filterId = gadget.getPref("filterId");
                        if (filterId) {
                            var filter = _.find(filters.options, function(f) {
                                return f.value == filterId;
                            });
                            if (filter) {
                                gadget.projectOrFilterName = filter.label;
                            }
                        } else {
                            gadget.projectOrFilterName = null;
                        }
                    } else {
                        gadget.projectOrFilterName = null;
                    }

                    var searchParam;
                    if (/^jql-/.test(this.getPref("filterId")) || this.getPref("isPopup") === "true") {
                        searchParam =
                            {
                                userpref: "filterId",
                                type: "hidden",
                                value: gadgets.util.unescapeString(gadget.getPref("filterId"))
                            };
                    }
                    else {
                        searchParam = RAVEN.gadget.fields.filterPicker(gadget, "filterId");
                    }

					return {
					    action: '/rest/raven/1.0/chart/requirement/overall/validate',
					    theme : function(){
					        if (gadgets.window.getViewportDimensions().width < 450){
					            return "gdt top-label";
					        } else{
					            return "gdt";
					        }
					    }(),
					    fields: [
                            searchParam,
                            {
                                userpref: "projectFilter",
                                type: "checkbox",
                                options:[
                                    {
                                        id: "filter-project-issues",
                                        label: gadget.getMsg("gadget.requirement-coverage.common.field.project-filter.desc"),
                                        value: "true",
                                        selected : gadget.getPref("projectFilter") == 'true'
                                    }
                                ]
                            },
                            {
								id: "execScope",
        						userpref: "execScope",
        						label: "Execution Scope",
        						description: gadget.getMsg("gadget.requirement-coverage.common.field.exec-scope.desc"),
        						type: "select",
        						selected: gadget.getPref("execScope"),
        						options:[
						            {
						                label: "Version",
						                value: "version"
						            },
						            {
						                label: "Test Plan",
						                value: "testPlan"
						            }
						        ]
							},
					    	{
								userpref: "project",
								label: "Project",
								description: gadget.getMsg("gadget.requirement-coverage.common.field.project.desc"),
								type: "select",
								selected: gadget.getPref("project"),
								options: projects
							},
							{
                                id: "testPlanKey",
                                userpref: "testPlanKey",
                                label: "Test Plan Key",
                                description: gadget.getMsg("gadget.requirement-coverage.common.field.test-plan-key.desc"),
                                type: "custom",
                                value: gadget.getPref("testPlanKey"),
                                template: function() {
                                	var disabled = "";
                                	if (testPlanKeyDisabled === true) {
                                		disabled = " disabled"
                                	}
						            return '<input type="text" class="text" value="' + gadget.getPref("testPlanKey") + '" id="testPlanKey" name="testPlanKey"' + disabled + '>';
						        }
                            },
					        gadget.versionField,
                            gadget.groupingField,
                            {
                                userpref: "flattenRequirements",
                                type: "checkbox",
                                options:[
                                    {
                                        id: "flat-requirement-presentation",
                                        label: gadget.getMsg("gadget.requirement-coverage.common.field.flat-requirement-presentation.desc"),
                                        value: "true",
                                        selected : gadget.getPref("flattenRequirements") == 'true'
                                    }
                                ]
                            },
    						AJS.gadget.fields.nowConfigured()
					    ]
					};
				},
    			args: [
    			    {
        				key: "settings",
						ajaxOptions: "/rest/raven/1.0/chart/requirement/settings"
    				},
    				{
        				key: "projects",
						ajaxOptions: {
							url: "/rest/api/2/project",
							contentType: "application/json"
						}
    				},
                    {
                        key: "filters",
                        ajaxOptions: {
                            url: "/rest/gadget/1.0/filtersAndProjects",
                            contentType: "application/json"
                        }
                    }
				]
            },
		   	view: {
				enableReload: true,
				onResizeReload: true,
				onResizeAdjustHeight: false,
				template: function (args) {
					var gadget = this;

					// set gadget title
					setTitle(this);

					// server errors and timeout errors are not catched by the error handler of the REST date request !!!
					if (!args.data) {
						message(null, gadget.getMsg("gadget.requirement-coverage.common.error.data"));
					} else {
						createOverallCoverageChart(gadget, args.data);
					}
				},
				args: [{
					key: "data",
					ajaxOptions: function() {
                        var gadget = this;

                        var project = this.getPref("project");
                        var projectFilter = this.getPref("projectFilter");
                        var execScope = this.getPref("execScope");
                        var version = this.getPref("version");
                        var testPlan = this.getPref("testPlanKey");
                        var filterId = gadgets.util.unescapeString(this.getPref("filterId"));

                        var params = '';
						if (execScope) {
                            params = params + '&execScope=' + execScope;
                        }

                        if (project && project != '__ALL__') {
                            params = params + '&project=' + project;
                        }

                        if (projectFilter) {
                            params = params + '&projectFilter=' + projectFilter;
                        }

                        if (version && version != -1) {
                            params = params + '&version=' + version;
                        }

                        if (testPlan && testPlan != -1) {
                            params = params + '&testPlan=' + testPlan;
                        }

                        if (filterId) {
                            // remove prefix "filter-"
                            params = params + "&savedFilter=" + filterId.substring(7);
                        }

						return {
		    				type: "GET",
		    				url: contextPath + '/rest/raven/1.0/chart/requirement/coverage?groupingField=' + this.getPref("grouping") + '&flattenRequirements=' + this.getPref("flattenRequirements") + params,
		    				processData: false,
		    				async: true,
							timeout: 120000,
		    				contentType: "application/json",
							error: function(qXHR, textStatus, errorThrown) {
								if (errorThrown && errorThrown.trim() != '') {
									errorThrown = errorThrown + ': ' + qXHR.responseText;
								} else {
									errorThrown = qXHR.responseText;
								}
								serverError(gadget, errorThrown);
								gadgets.window.adjustHeight();

								/*
								var errors = {
									errors: {
										error: errorThrown
									}
								};
								qXHR.status = 500;
								qXHR.responseText = JSON.stringify(errors);
								*/
							}
			    		}
			    	}
    			}]
		   	}
		});
	}
})(AJS.$);
