(function($) {

	
	RAVEN.namespace("RAVEN.Project.TestExec");
	
    /**
     * This Model defines the column layout for the given user
     */
    RAVEN.Project.TestExec.TestRunsTableMetaModel = Backbone.Model.extend({
        urlRoot: function() {
            return contextPath + '/rest/raven/1.0/testexec/columns';
        },

        defaults: function() {
            return [];
        }
    });
    
    /**
     * This is the main view for the Test Runs table on a Test Execution issue
     */
    RAVEN.Project.TestExec.TestExecsView = Backbone.View.extend({
        el: '#raven-testexec',
    	oTable:null,
        versionPicker: null,
        revisionPicker: null,
        statusPicker:null,
        testEnvironmentsMultiSelect: null,
        
        allItem: new AJS.ItemDescriptor({
		      value: '-1',
		      label: 'All',
		      icon: null,
		      allowDuplicate: false,
		      selected: true
		  }),
    
	message: function(title, message, type) {
		clearMessage();
		AJS.messages.error('#raven-testexec-message-bar', {
			title : title,
			body : message
		});
	},
	
	clearMessage: function() {
		$('#raven-testexec-message-bar').empty();
	},

	refresh: function(){
		this._cleanView();
		this.initialize();
		//this.refreshDataTable();
		
	},
	
  prepareRequirementLabels:function() {
	  var ids = $('#test-environment-labels option:selected').map(function() {
	      return $(this).val();
	  }).get();
	
	  $('#test-environment-labels-ids').val(JSON.stringify(ids));
	},
	
	initializefilters: function() {
		  if(!this.versionPicker){
			  this.versionPicker = this.createSingleSelectPicker($('#raven-testexec-filter-version'));
		  }
		  if(!this.statusPicker){
			  this.statusPicker = this.createSingleSelectPicker($('#raven-testexec-filter-status'));
		  }
		  
		  if(!this.revisionPicker){
			  this.revisionPicker = this.createSingleSelectPicker($('#raven-testexec-filter-revision'));
		  }
		  
		 if(!this.testEnvironmentsMultiSelect){ 
			 this.testEnvironmentsMultiSelect = this.createTestEnvironmentsLabelPicker($('#test-environment-labels'));
		  }
		 
		if(!this.filterComponentPicker) {
			this.filterComponentPicker = this.createComponentPicker($("#raven-testexec-filter-component"));
		}

		$('#raven-testexec-panel-form').show();
	},
	
	
	createTestEnvironmentsLabelPicker: function ($selectField) {
		var testEnvironmentsCFID = $("#raven-testexec-testenvironments-cf-id").val();
	    var getDefaultOptionLabelPicker = AJS.LabelPicker.extend({
	        _getDefaultOptions: function () {
	            return jQuery.extend(true, this._super(), {
	                ajaxOptions: {
	                    url: contextPath + "/rest/api/1.0/labels/suggest?customFieldId="+testEnvironmentsCFID,
	                    query: true,
	                    formatResponse: this._formatResponse
	                },
	                removeDuplicates: true,
	                removeOnUnSelect: true
	            });
	        },
	    });
	    
	    return new getDefaultOptionLabelPicker({
	        element: $selectField,
	        itemAttrDisplayed: "label",
	        showDropdownButton: true,
	        removeOnUnSelect: false,
	        submitInputVal: false,
	        errorMessage: AJS.I18n.getText("raven-admin-config.requirement-coverage.ignore-requirements-in-labels.error"),
	        maxInlineResultsDisplayed: 15
	    });
	},
	
	createSingleSelectPicker:function($selectField) {
	  $selectField.on('initialized', function(e, field) {            
	      var version = $('body').data('version');
	      if (!version || version.toString().charAt(0) === '5') {
	          $('input', $selectField.parent()).height('23px');
	      }
	  });
	  
	  return new AJS.SingleSelect({
	      element: $selectField,
	      showDropdownButton: true,
	      revertOnInvalid: true,
	  });
	},
	
	initialize: function (){
		var self = this;
		
		this.model = new RAVEN.Project.TestExec.TestRunsTableMetaModel();
		
		AJS.messages.setup();
		
        this.model.fetch({
            success: function() {
                if (self.$el.closest(document.documentElement).length > 0) {
                    self.render();
                }
            },
            error: function(model, response) {
                self.message(AJS.I18n.getText('projectpanels-section.column-layout.error'), null, 'fatal');
            }
        });
        
        $(document).off('.raven-testexec-panel');
		
		// bind AJS create issue event
		$(AJS).bind("QuickCreateIssue.issueCreated", function(event, issueData) {
            self.refreshDataTable();
		});
		
	   	 // render test runs toolbar view
  //      this._renderTestRunsToolbarView();
		
//        this.initializefilters();
//		this.createDataTable();

	},
	
    render: function() {
    	var self = this;

		$(document).on('click.raven-testexec-panel', '#raven-testexec-panel-clear-filters', function(e) {
		      if (self.versionPicker) {
		    	  self.versionPicker._setDescriptorWithValue("-1");
		      }
		      if (self.revisionPicker) {
		    	  self.revisionPicker._setDescriptorWithValue("-1");
		      }
		      if (self.statusPicker) {
		    	  self.statusPicker._setDescriptorWithValue("-1");
		      }
		      
		      if(self.testEnvironmentsMultiSelect) {
		    	  self.testEnvironmentsMultiSelect.clearLozenges();
		      }

		      if(self.filterComponentPicker) {
		    	  self.filterComponentPicker.clearLozenges();
		      }
		      
		      $('#raven-testexec-filter-contains-text').val('');
		
		      self.refreshDataTable();
	  });
	
	  $(document).on('submit.raven-testexec-panel', '#raven-testexec-panel-form', function(e) {
	      	self.refreshDataTable();
	
	      	return false;
	  });
    	
        this.initializefilters();
        this.createDataTable();
        
        // render test runs toolbar view
        this._renderTestRunsToolbarView();

        this.ready = true;

        return this;
    },
	
	createDataTable: function () {
		var self = this;
		var projectKey = $('#raven-testexec-projectKey').val();
	
		var path = contextPath + '/rest/raven/1.0/project/' + projectKey + "/testexec";
	
		var $table = RAVEN.Project.renderTestExecTable({'columns': _.values(this.model.attributes)});
		
		var $tableWrapper = $('#testexecs-section');
		$tableWrapper.empty();
		$tableWrapper.html($table);
		
		oTable = $('#testexec-entries-table').xrayDataTable({
		        "bServerSide" : true,
		        "fnServerData": function( sSource, aoData, fnCallback, oSettings){
                    self._fnServerData( sSource, aoData, fnCallback, oSettings );
                },
		        "sAjaxSource": path,
		        "sAjaxDataProp" : "entries",
		        "bProcessing": false,
		        "bAutoWidth" : false,
		        "bDestroy": true,
		        "bDefaultFontSize": true,
		        "dataType": "json",
		        "type":"GET",
		        "sPaginationType" : "full_numbers",
		        "bFilter": false,
		        "bSort" : true,
		        "oLanguage": {
		            "sEmptyTable": AJS.I18n.getText('projectpanels.test.datatable.empty'),
		            "oPaginate": {
		                "sFirst": AJS.I18n.getText('common.label.first'),
		                "sLast": AJS.I18n.getText('common.label.last'),
		                "sNext": AJS.I18n.getText('common.label.next'),
		                "sPrevious": AJS.I18n.getText('common.label.previous')
		            },
		            "sInfoEmpty": AJS.I18n.getText('common.table.label.no-entries-to-show'),
		            "sEmptyTable": AJS.I18n.getText('projectpanels.testexec.datatable.empty'),
		            "sInfo": AJS.I18n.getText('common.table.label.showing-records'),
		            "sZeroRecords": AJS.I18n.getText('common.table.label.no-records-found'),
		            "sInfoFiltered": AJS.I18n.getText('common.table.label.table-filtered-from'),
		            "sSearch":  AJS.I18n.getText('common.table.label.search'),
		            "sLengthMenu": AJS.I18n.getText('common.table.label.show-menu-entries')
		        },
		        "aaSorting" : [],
				
				"aoColumns" : this._createDataTableUserColumn(),
		        "fnInitComplete": function(oSettings, json) {
		           
		        },
			});
		
	        this._initSortableColumns();
	    },
	
	    _initSortableColumns: function() {
	        var self = this;
	
	        $('#testexec-entries-table thead tr').sortable({
	            items: 'th:not(.fixed)',
	            placeholder: "dragging-placeholder",
	            update: function (event, ui) {
	                var ancorNode = ui.item.prev();
	                var columnId = $(ui.item).data('column-id');
	                if (columnId) {
	                    var anchorColumnId = null;
	                    if (!$(ancorNode).hasClass('fixed')) {
	                        anchorColumnId = $(ancorNode).data('column-id');
	                    }
	
	                    self.columnLayoutView.moveColumn(columnId, anchorColumnId);
	                }
	            }
	        });
	    },
	/**
	    * Creates the DataTable column def descriptors dynamically based on the user column layout
	    */
	   _createTestRunTableColumnDefs:function() {
	       var self = this;
	       var userColumnDefs = this._createDataTableUserColumnDefs();

	       var lastColumnDefs = [ {
				"mData" : "status",
				"bSortable" : true,
				"sWidth" : "110px",
				"mRender" : function(data, type, row) {
					if (data) {
						return "<span data-status-id="
								+ data.id
								+ " class=\"aui-lozenge testentry-progress-bar\" style=\"background-color:"
								+ data.color + "\" title=\"" + data.description
								+ "\">" + data.name + "</span>";
					} else {
						return "-";
					}
				},
				"aTargets" : [_.values(this.model.attributes).length]
			},
			{
				"mData" : "status",
				"bSortable" : true,
				"sWidth" : "110px",
				"mRender" : function(data, type, row) {
					if (data) {
						return "<span data-status-id="
								+ data.id
								+ " class=\"aui-lozenge testentry-progress-bar\" style=\"background-color:"
								+ data.color + "\" title=\"" + data.description
								+ "\">" + data.name + "</span>";
					} else {
						return "-";
					}
				},
				"aTargets" : [_.values(this.model.attributes).length + 1]
			}
			];
	       
	       return userColumnDefs.concat(lastColumnDefs);
	   },
	   
	    /**
	     * Creates the DataTable column def for user columns
	     */
		_createDataTableUserColumnDefs:function() {
			return _.map(_.values(this.model.attributes), function(column, index) {
					return {
						"mData" : 'userColumns.' + column.id,
						"bSortable" : column.sortable,
						"bSearchable" : false,
						"aTargets" : [ index ]
					};
				});
		},
		
		 /**
	     * Creates the DataTable column descriptors dynamically based on the user column layout
	     */
		_createDataTableUserColumn:function () {
		   var userColumnDefs = this._createTestRunUserColumns();
		   
		   var lastColumnDefs = [{
						"sWidth":"25%",
						"mDataProp" : "statusBar",
						"bSortable" : false,
						"bSearchable" : false,
						"mRender" : function(data, type, row) {
	                        var statusBar = row["statusBar"];
	
	                        var htmlProgressBar='';
	                        htmlProgressBar=htmlProgressBar+ '<ul class="testexec-progress-bar">'
	                        
	                        if(statusBar.count === 0) {
	                             htmlProgressBar=htmlProgressBar+'<li class="pb-todo" style="width:100%;" title="'+AJS.I18n.getText('view-issue-testexec-tests-section.progressbar.todo')+' 100%" data-status="TODO">'+AJS.I18n.getText('view-issue-testexec-tests-section.progressbar.todo')+'</li>';
	                        } else {
	                            for (var i = 0; i < statusBar.statuses.length; i++) {
	                                var status = statusBar.statuses[i];
	                                htmlProgressBar = htmlProgressBar + '<li style="background-color:' + status.color + ';width:'+ status.statusPercent +'%;" title="' + status.name + ':' + status.statusCount + ' (' + status.statusPercent + '%)" data-status-id="' + status.id + '">' + status.name + '</li>';
	                            }
	                        }
	
	                        htmlProgressBar=htmlProgressBar+'</ul>';
	                
	                        return htmlProgressBar;
						}
					},
					{
						"sWidth":"5%",
						"bSortable" : false,
						"bSearchable" : false,
						"mDataProp" : null,
						"sClass" : "nav issue_actions",
						"sDefaultContent" : '',
						"mRender" : function(data, type, row) {
							return '<a href="#'
									+ row["key"]
									+ '" aria-owns="'
									+ row["key"]
									+ '" aria-haspopup="true" class="aui-dropdown2-trigger aui-dropdown2-trigger-arrowless aui-style-default icon-tools-small"></a>'
									+ '<div id="'
									+ row["key"]
									+ '" class="aui-dropdown2 aui-style-default">'
									+ '<ul class="aui-list-truncate">'
									+ '<li><a id="'
									+ row["id"]
									+ '" href="' + contextPath + '/secure/EditIssue!default.jspa?id='
									+ row["id"]
									+ '&amp;returnUrl='
									+ window.location
									+ '" class="edittestexecAction aui-list-item-link">'+AJS.I18n.getText('common.action.edit')+'</a></li>'
									+ '<li><a href="' + contextPath + '/secure/DeleteIssue!default.jspa?id='
									+ row["id"]
									+ '&amp;returnUrl='
									+ window.location
									+ '">'+AJS.I18n.getText('common.action.delete')+'</a></li>'
									+ '</ul></div>';
	
						}
					}];
		   
		   return userColumnDefs.concat(lastColumnDefs);
	    },
	    
	    _createTestRunUserColumns: function() {
	    	return _.map(_.values(this.model.attributes), function(column, index) {
				   return {
		               "mData": 'userColumns.' + column.id
		           };
				 });
		},
	   
	   _fnServerData: function( sSource, aoData, fnCallback, oSettings ) {
	     $('#raven-testexec').block();
	
	     // set filter
	     this.populateFilterQueryParams(aoData);
	
	     oSettings.jqXHR = $.ajax( {
	         "dataType": 'json',
	         "type": "GET",
	         "url": sSource,
	         "data": aoData,
	         "error": function(qXHR, textStatus, errorThrown) {
	             if (errorThrown && errorThrown.trim() != '') {
	                 errorThrown = errorThrown + ': ' + qXHR.responseText;
	             } else {
	                 errorThrown = qXHR.responseText;
	             }
	             message(AJS.I18n.getText('projectpanels.testexec.datatable.processingerror'), errorThrown)
	         },
	         "complete": function() {
	             $('#raven-testexec').unblock();
	         },
	         "success": fnCallback
	     });
	 },
	 
	 populateFilterQueryParams: function (aoData) {
	   var versionValue = $('#raven-testexec-filter-version').val();
	   if (versionValue && versionValue != -1) {
	       aoData.push({ "name": "version", "value": versionValue});
	   }
	
	   var revisionValue = $('#raven-testexec-filter-revision').val();
	   if (revisionValue && revisionValue != -1) {
	       aoData.push({ "name": "revision", "value": revisionValue});
	   }
	
	   var statusValue = $('#raven-testexec-filter-status').val();
	   if (statusValue && statusValue != -1) {
	       aoData.push({ "name": "status", "value": statusValue});
	   }
	
	   var testEnvironments = $('#test-environment-labels').val();
	   if (testEnvironments) {
	       aoData.push({ "name": "testEnvironments", "value": testEnvironments});
	   }
	   
	   var searchValue = $('#raven-testexec-filter-contains-text').val();
	   if (searchValue) {
	       aoData.push({ "name": "contains", "value": searchValue});
	   }
	   
       var componentValue = $('#raven-testexec-filter-component').val();
       if(componentValue) {
       	aoData.push({ "name": "components", "value": componentValue});
       }
	},

	refreshDataTable: function () {
	   if ($('#testexec-entries-table').length > 0) {
	       var table = $('#testexec-entries-table').xrayDataTable();
	       table.fnDraw(false);
	   }
	},
	
    
    _cleanView: function() {
        if (this.dataTable) {
            this.dataTable.fnDestroy(true);
            this.dataTable = null;
        }

        this.model = null;

        var $tableWrapper = this.$el.find('#testexecs-section');

        $tableWrapper.empty();
    },
	
   _renderTestRunsToolbarView: function() {
	  var $wrapper = $('.xrayDataTables_wrapper');
	  // if filter already exists then return
	  if ($wrapper.find('#toolbar-wrapper').length > 0) {
	      return;
	  }
	
	  var toolbarHtml = RAVEN.Project.renderTestRunsToolBar();
	  $wrapper.find('.xrayDataTables_length').after(toolbarHtml);
	
	  this.columnLayoutView = new RAVEN.Project.TestExecsColumnLayoutView({
	      mainView: this
	  });
	},
	
	createComponentPicker: function($selectField) {
    	return new AJS.MultiSelect({
			element: $selectField,
			itemAttrDisplayed: "label",
			showDropdownButton: true,
			submitInputVal: false,
			noQueryNoRequest: false,
			revertOnInvalid: true,
			loadOnInit: true,
			errorMessage: "",
			ajaxOptions: {
				query: true,
				url: contextPath + '/rest/api/2/project/' + $('#raven-testexec-projectKey').val() + '/components',
				formatResponse: function (data) {
					var optgroup = new AJS.GroupDescriptor({
						weight: 0,
						id: "all-filter-opt-group",
						replace: true
					});

					$(data).each(function(i, status) {
						this.label = this.name;
						var item = new AJS.ItemDescriptor({
							value: this.name,
							label: this.name,
							title: this.description,
							
							highlighted: true,
						});
						optgroup.addItem(item);
					});

					return [optgroup];
				}
			}
		});
    }
	
        
	});

    RAVEN.Project.TestExec.TestExecsView.REFRESH_VIEW = "refreshTestView";

})(AJS.$);
