(function($) {

	
	RAVEN.namespace("RAVEN.Project.TestPlan");
	
    /**
     * This Model defines the column layout for the given user
     */
    RAVEN.Project.TestPlan.TestPlansTableMetaModel = Backbone.Model.extend({
        urlRoot: function() {
            return contextPath + '/rest/raven/1.0/testplan/columns';
        },

        defaults: function() {
            return [];
        }
    });
	
	 /**
     * This is the main view for the Test Runs table on a Test Execution issue
     */
    RAVEN.Project.TestPlan.TestPlansView = Backbone.View.extend({
    	el: '#raven-testplan',
    	
        /**
         * DataTable Instance
         */
    	testPlanTable: null,
    	
    	userColumns: [],
    	
    	filterVersionPicker: null,
    	
        statusPicker:null,
        
        filterComponentPicker: null,
        
    	message: function(title, message, type) {
    		clearMessage();
    		AJS.messages.error('#raven-testplan-message-bar', {
    			title : title,
    			body : message
    		});
    	},
    	
    	clearMessage: function() {
    		$('#raven-testplan-message-bar').empty();
    	},

    	refresh: function(){
    		this._cleanView();
    		this.initialize();
    		//this.refreshDataTable();
    	},
    	
    	initialize: function (){
    		var self = this;
    		
    		this.model = new RAVEN.Project.TestPlan.TestPlansTableMetaModel();
    
	        this.model.fetch({
	            success: function() {
	                if (self.$el.closest(document.documentElement).length > 0) {
	                    self.render();
	                }
	            },
	            error: function(model, response) {
	                self.message(AJS.I18n.getText('projectpanels-section.column-layout.error'), null, 'fatal');
	            }
	        });
    	},
    	
        render: function() {
        	var self = this;
        	$(document).on('submit.raven-testplan-panel', '#raven-testplan-panel-form', function(e) {
                self.refreshDataTable();
                return false;
            });
        		
            $(document).on('click.raven-testplan-panel', '#raven-testplan-panel-clear-filters', function(e) {
	            if (self.filterVersionPicker) {
	                self.filterVersionPicker._setDescriptorWithValue("-1");
	            }
	            
			    if (self.statusPicker) {
			    	self.statusPicker._setDescriptorWithValue("-1");
			    }

				if(self.filterComponentPicker) {
					self.filterComponentPicker.clearLozenges();
				}
			      
	            $('#raven-testplan-filter-contains-text').val('');
	
	            self.refreshDataTable();
	        });
        	
	        this.initializefilters();
	        this.createDataTable();
	        
            // render test runs toolbar view
            this._renderTestPlansToolbarView();

            this.ready = true;

            return this;
        },
    	
    	createDataTable: function() {
    		var self = this;
    		
    		var projectKey = $('#raven-testplan-projectKey').val();

    		var path = contextPath + '/rest/raven/1.0/project/' + projectKey + "/testplan";

    		var $table = RAVEN.Project.renderTestPlanTable({'columns': _.values(this.model.attributes)});
    		
    		var $tableWrapper = $('#testplans-section');
    		$tableWrapper.empty();
    		$tableWrapper.html($table);
    		
    		testPlanTable = $('#testplan-entries-table').xrayDataTable({
                "oLanguage": {
                    "sEmptyTable": AJS.I18n.getText('projectpanels.testplan.datatable.empty'),
                    "oPaginate": {
                        "sFirst": AJS.I18n.getText('common.label.first'),
                        "sLast": AJS.I18n.getText('common.label.last'),
                        "sNext": AJS.I18n.getText('common.label.next'),
                        "sPrevious": AJS.I18n.getText('common.label.previous')
                    },
                    "sInfoEmpty": AJS.I18n.getText('common.table.label.no-entries-to-show'),
                    "sEmptyTable": AJS.I18n.getText('projectpanels.testplan.datatable.empty'),
                    "sInfo": AJS.I18n.getText('common.table.label.showing-records'),
                    "sZeroRecords": AJS.I18n.getText('common.table.label.no-records-found'),
                    "sInfoFiltered": AJS.I18n.getText('common.table.label.table-filtered-from'),
                    "sSearch":  AJS.I18n.getText('common.table.label.search'),
                    "sLengthMenu": AJS.I18n.getText('common.table.label.show-menu-entries')
                },
                "bServerSide" : true,
                "fnServerData": function( sSource, aoData, fnCallback, oSettings){
                    self._fnServerData( sSource, aoData, fnCallback, oSettings );
                },
                "sAjaxSource": path,
    			"sAjaxDataProp" : "entries",
    			"bProcessing": false,
    			"bAutoWidth" : false,
    			"bDestroy": true,
                "bDefaultFontSize": true,
    			"dataType": "json",
    			"type":"GET",
    			"sPaginationType" : "full_numbers",
                "bFilter": false,
    			"bSort" : true,
    			"aaSorting" : [],
    			"aoColumnDefs" : this._createTestPlanTableColumnDefs(),
    			"aoColumns" : this._createTestPlanTableColumns(),
                "fnInitComplete": function(oSettings, json) {
                    
                },
    		});
    	},
    	
    	 /**
         * Creates the DataTable column descriptors dynamically based on the user column layout
         */
    	_createTestPlanTableColumns: function() {
            var userColumnDefs = this._createTestPlanUserColumns();

            var lastColumnDefs = [{
            	"sWidth":"25%",
        		"mDataProp" : "statusBar",
        		"bSortable" : false,
        		"bSearchable" : false,
        		"mRender" : function(data, type, row) {
        			var statusBar = row["statusBar"];
        	
        	        var htmlProgressBar='';
        	        htmlProgressBar=htmlProgressBar+ '<ul class="testplan-progress-bar">'
        	                      
        	        if(statusBar.count === 0) {
        	        	htmlProgressBar=htmlProgressBar+'<li class="pb-todo" style="width:100%;" title="'+AJS.I18n.getText('view-issue-testplan-tests-section.progressbar.todo')+' 100%" data-status="TODO">'+AJS.I18n.getText('view-issue-testplan-tests-section.progressbar.todo')+'</li>';
        	        } else {
        	            for (var i = 0; i < statusBar.statuses.length; i++) {
        	            	var status = statusBar.statuses[i];
        	                htmlProgressBar = htmlProgressBar + '<li style="background-color:' + status.color + ';width:'+ status.statusPercent +'%;" title="' + status.name + ':' + status.statusCount + ' (' + status.statusPercent + '%)" data-status-id="' + status.id + '">' + status.name + '</li>';
        	            }
        	        }
        	
        	        htmlProgressBar=htmlProgressBar+'</ul>';
        	                
        	        return htmlProgressBar;
        		}
        	},{
        		"sWidth":"5%",
    			"bSortable" : false,
    			"bSearchable" : false,
    			"mDataProp" : null,
    			"sClass" : "nav issue_actions",
    			"sDefaultContent" : '',
    			"mRender" : function(data, type, row) {
    				return '<a href="#'
    						+ row["key"]
    						+ '" aria-owns="'
    						+ row["key"]
    						+ '" aria-haspopup="true" class="aui-dropdown2-trigger aui-dropdown2-trigger-arrowless aui-style-default icon-tools-small"></a>'
    						+ '<div id="'
    						+ row["key"]
    						+ '" class="aui-dropdown2 aui-style-default">'
    						+ '<ul class="aui-list-truncate">'
    						+ '<li><a id="'
    						+ row["id"]
    						+ '" href="' + contextPath + '/secure/EditIssue!default.jspa?id='
    						+ row["id"]
    						+ '&amp;returnUrl='
    						+ window.location
    						+ '" class="editTestAction aui-list-item-link">'+AJS.I18n.getText('common.action.edit')+'</a></li>'
    						+ '<li><a href="' + contextPath + '/secure/DeleteIssue!default.jspa?id='
    						+ row["id"]
    						+ '&amp;returnUrl='
    						+ window.location
    						+ '">'+AJS.I18n.getText('common.action.delete')+'</a></li>'
    						+ '</ul></div>';

    			}
    		}];

            return userColumnDefs.concat(lastColumnDefs);
        },
        
        /**
         * Creates the DataTable column def descriptors dynamically based on the user column layout
         */
        _createTestPlanTableColumnDefs: function() {
            var self = this;
            var userColumnDefs = this._createDataTableUserColumnDefs();

            var lastColumnDefs = [ {
    			"mData" : "status",
    			"bSortable" : true,
    			"sWidth" : "110px",
    			"mRender" : function(data, type, row) {
    				if (data) {
    					return "<span data-status-id="
    							+ data.id
    							+ " class=\"aui-lozenge testentry-progress-bar\" style=\"background-color:"
    							+ data.color + "\" title=\"" + data.description
    							+ "\">" + data.name + "</span>";
    				} else {
    					return "-";
    				}
    			},
    			"aTargets" : [_.values(this.model.attributes).length]
    		},
    		{
    			"mData" : "status",
    			"bSortable" : true,
    			"sWidth" : "110px",
    			"mRender" : function(data, type, row) {
    				if (data) {
    					return "<span data-status-id="
    							+ data.id
    							+ " class=\"aui-lozenge testentry-progress-bar\" style=\"background-color:"
    							+ data.color + "\" title=\"" + data.description
    							+ "\">" + data.name + "</span>";
    				} else {
    					return "-";
    				}
    			},
    			"aTargets" : [_.values(this.model.attributes).length + 1]
    		}
    		];
    		
    		
    		return userColumnDefs.concat(lastColumnDefs);
        },
        
        /**
         * Creates the DataTable column def for user columns
         */
        _createDataTableUserColumnDefs: function() {
			
    		return _.map(_.values(this.model.attributes), function(column, index) {
    			console.log('index: ' + index);
				return {
					"mData" : 'userColumns.' + column.id,
					"bSortable" : column.sortable,
					"bSearchable" : false,
					"aTargets" : [ index ]
				};
			});
    	},
    	
    	_createTestPlanUserColumns:function() {
		   
		   return _.map(_.values(this.model.attributes), function(column, index) {
			   return {
	               "mData": 'userColumns.' + column.id
	           };
			 });
		   
	   },
	   
	   _fnServerData: function( sSource, aoData, fnCallback, oSettings ) {
	        $('#raven-testplan').block();

	        // set filter
	        this.populateFilterQueryParams(aoData);

	        oSettings.jqXHR = $.ajax( {
	            "dataType": 'json',
	            "type": "GET",
	            "url": sSource,
	            "data": aoData,
	            "error": function(qXHR, textStatus, errorThrown) {
	                if (errorThrown && errorThrown.trim() != '') {
	                    errorThrown = errorThrown + ': ' + qXHR.responseText;
	                } else {
	                    errorThrown = qXHR.responseText;
	                }
	                message(AJS.I18n.getText('projectpanels.test.datatable.processingerror'), errorThrown)
	            },
	            "complete": function() {
	            	$('#raven-testplan').unblock();
	            },
	            "success": fnCallback
	        });
	    },
	    
	    populateFilterQueryParams: function(aoData) {
	        var versionValue = $('#raven-testplan-filter-version').val();
	        if (versionValue && versionValue != -1) {
	            aoData.push({ "name": "version", "value": versionValue});
	        }
	        
	 	   var statusValue = $('#raven-testplan-filter-status').val();
		   if (statusValue && statusValue != -1) {
		       aoData.push({ "name": "status", "value": statusValue});
		   }

	        var searchValue = $('#raven-testplan-filter-contains-text').val();
	        if (searchValue) {
	            aoData.push({ "name": "contains", "value": searchValue});
	        }
	        
	        var componentValue = $('#raven-testplan-filter-component').val();
	        if(componentValue) {
	        	aoData.push({ "name": "components", "value": componentValue});
	        }
	    },

	    refreshDataTable: function() {
	        if ($('#testplan-entries-table').length > 0) {
	            var table = $('#testplan-entries-table').xrayDataTable();
	            table.fnDraw(false);
	        }
	    },
	    
        _cleanView: function() {
            if (this.dataTable) {
                this.dataTable.fnDestroy(true);
                this.dataTable = null;
            }

            this.model = null;

            var $tableWrapper = this.$el.find('#testplans-section');

            $tableWrapper.empty();
        },
	    
	    initializefilters: function(){
			var projectKey = $('#raven-testplan-projectKey').val();
			var self = this;
			
			if(!this.filterVersionPicker){
				this.filterVersionPicker = this.createVersionPicker($('#raven-testplan-filter-version'));
			}
			if(!this.statusPicker){
				this.statusPicker = this.createSingleSelectPicker($('#raven-testplan-filter-status'));
			}
			
			if(!this.filterComponentPicker) {
				this.filterComponentPicker = this.createComponentPicker($("#raven-testplan-filter-component"));
			}

			$('#raven-testplan-panel-form').show();
		},
		
		createVersionPicker: function($selectField) {
	        $selectField.on('initialized', function(e, field) {            
	            var version = $('body').data('version');
	            if (!version || version.toString().charAt(0) === '5') {
	                $('input', $selectField.parent()).height('23px');
	            }
	        });
	        
	        return new AJS.SingleSelect({
	            element: $selectField,
	            showDropdownButton: true,
	            revertOnInvalid: true,
	        });
	    },
	    
		createSingleSelectPicker:function($selectField) {
			$selectField.on('initialized', function(e, field) {            
				var version = $('body').data('version');
			    if (!version || version.toString().charAt(0) === '5') {
			    	$('input', $selectField.parent()).height('23px');
			    }
			});
			  
			return new AJS.SingleSelect({
				element: $selectField,
			    showDropdownButton: true,
			    revertOnInvalid: true,
			});
		},
	    
	    _renderTestPlansToolbarView: function() {
	        var $wrapper = $('.xrayDataTables_wrapper');
	        // if filter already exists then return
	        if ($wrapper.find('#toolbar-wrapper').length > 0) {
	            return;
	        }

	        var toolbarHtml = RAVEN.Project.renderTestRunsToolBar();
	        $wrapper.find('.xrayDataTables_length').after(toolbarHtml);

            this.columnLayoutView = new RAVEN.Project.TestPlansColumnLayoutView({
                mainView: this
            });
        },
        
    	createComponentPicker: function($selectField) {
        	return new AJS.MultiSelect({
				element: $selectField,
				itemAttrDisplayed: "label",
				showDropdownButton: true,
				submitInputVal: false,
				noQueryNoRequest: false,
				revertOnInvalid: true,
				loadOnInit: true,
				errorMessage: "",
    			ajaxOptions: {
    				query: true,
    				url: contextPath + '/rest/api/2/project/' + $('#raven-testplan-projectKey').val() + '/components',
    				formatResponse: function (data) {
    					var optgroup = new AJS.GroupDescriptor({
    						weight: 0,
    						id: "all-filter-opt-group",
    						replace: true
    					});

    					$(data).each(function(i, status) {
    						this.label = this.name;
    						var item = new AJS.ItemDescriptor({
    							value: this.name,
    							label: this.name,
    							title: this.description,
    							
    							highlighted: true,
    						});
    						optgroup.addItem(item);
    					});

    					return [optgroup];
    				}
    			}
    		});
        }
    });   
	    
    RAVEN.Project.TestPlan.TestPlansView.REFRESH_VIEW = "refreshTestView";
})(AJS.$);
