require(['aui/inline-dialog2']);
(function($) {

    var filterValues = {
        "savedFilter": null,
        "fixVersion": null,
        "includePreviousVersions": null,
        "flattenRequirements": null,
        "statusId": null,
        "resolution": null,
        "components": null,
        "assignee": null,
        "contains": null,
        "environment": null
    };

    /*we are sending status ids instead of status name (because of translations), so we'll keep
     * this object to map ids with names*/
    var statusesObject = {};

    var Requirements = Backbone.Collection.extend({
        url: function() {
            return contextPath + '/rest/raven/1.0/report/requirementCoverage/overall';
        }
    });

    var Filters = Backbone.Model.extend({
        urlRoot: function() {
            return contextPath + '/rest/raven/1.0/report/requirementCoverage/overall/filters';
        }
    });

    RAVEN.Report.OverallRequirementCoverageChartReportView = Backbone.View.extend({
        el: "#report-wrapper",
        filterView: null,
        render: function(requiresExperimentalDialog2) {
            var $htmlReport = RAVEN.Report.renderOverallRequirementCoverageReport({ 'requiresExperimentalDialog2': requiresExperimentalDialog2 });
            this.$el.empty();
            this.$el.html($htmlReport);
            this.filterView = new RAVEN.Report.OverallRequirementCoverageChartFilterView();
        },
        destroy: function() {
            if (this.filterView) {
                this.filterView.destroy();
                this.filterView = null;
            }
            $('#show-tooltip').remove();
        }
    });

    RAVEN.Report.OverallRequirementCoverageChartFilterView = Backbone.View.extend({
        el: "#overall-requirement-coverage-filter-content-place-holder",
        projectId: null,
        overallRequirementCoverageChartView: null,
        dialog: null,
        render: function() {
            var filters = new Filters();
            var self = this;
            var $filterWrapper = self.$el;
            var params = $.param({ projectId: this.projectId });
            filters.fetch({
                data: params,
                success: function(data, textStatus, jqXHR) {
                    if (data && data.attributes) {
                        var savedFilter = null;

                        if (JSON.parse(data.attributes.prefsJson).savedFilter != null) {
                            savedFilter = JSON.parse(data.attributes.prefsJson).savedFilter;
                        }

                        filterValues.projectId = $("#selected-project-id").val();
                        filterValues.filterscope = data.attributes.filterscopepref;
                        if (data.attributes.versionpref != null) {
                            filterValues.version = data.attributes.versionpref;
                        }
                        if (data.attributes.testPlanKeypref != null) {
                            filterValues.testplan = data.attributes.testPlanKeypref;
                        }
                        if (data.attributes.environmentpref != null) {
                            filterValues.environment = data.attributes.environmentpref;
                        }
                        if (data.attributes.savedFilterpref != null) {
                            filterValues.savedfilter = data.attributes.savedFilterpref;
                        }
                        if (data.attributes.components != null) {
                            filterValues.components = data.attributes.components;
                        }
                        self.dialog = RAVEN.Report.renderOverallRequirementCoverageChartReportFilters({
                            'versions': data.attributes.versions,
                            'testEnvironments': data.attributes.testEnvironments,
                            'nativeGroupingFields': data.attributes.nativeGroupingFields,
                            'customGroupingFields': data.attributes.customGroupingFields,
                            'statuses': data.attributes.statuses,
                            'components': data.attributes.components,
                            'resolutions': data.attributes.resolutions,
                            'requiresExperimentalDialog2': data.attributes.requiresExperimentalDialog2,
                            'userPreferences': JSON.parse(data.attributes.prefsJson),
                            'assigneeUserPref': JSON.parse(data.attributes.prefsJson).assignee,
                            'savedFilter': savedFilter,
                            'contextPath': contextPath
                        });

                        statusesObject = data.attributes.statuses;

                        $filterWrapper.empty();
                        $filterWrapper.html(self.dialog);

                        self.overallRequirementCoverageChartView = new RAVEN.Report.OverallRequirementCoverageChartView();
                    }

                },
                error: function(model, response) {
                    self.clearMessage();
                    AJS.messages.error($("#message-row-wrapper"), {
                        title: 'Error retrieving overall requirement coverage chart report filters',
                        body: response.responseText,
                        closeable: false
                    });
                }
            });
        },
        initialize: function() {
            this.projectId = $("#selected-project-id").val();
            this.render();
        },
        destroy: function() {
            if (this.overallRequirementCoverageChartView) {
                this.overallRequirementCoverageChartView.destroy();
                this.overallRequirementCoverageChartView = null;
            }

            if (this.dialog) {
                $('#raven-filter-inline-dialog').remove();
            }
            this.remove();
        }
    });

    RAVEN.Report.OverallRequirementCoverageChartView = Backbone.View.extend({
        el: "#requirement-coverage-wrapper",

        version: null,

        projectId: null,

        execScope: null,

        testPlan: null,

        environment: null,

        groupingField: null,

        /** Scope and Filters Field Pickers */
        execScopeFieldPicker: null,

        versionPicker: null,

        testPlanPicker: null,

        environmentPicker: null,

        groupingFieldPicker: null,

        /** Filters Field Pickers */
        fixVersionFieldPicker: null,

        statusFieldPicker: null,

        componentFieldPicker: null,

        resolutionFieldPicker: null,

        assigneeFieldPicker: null,

        /** Chart */
        chartDetailsData: null,

        overallChart: null,

        /** STATUSES */
        OK_STATUS: 'OK',
        NOK_STATUS: 'NOK',
        NOTRUN_STATUS: 'Not Run',
        UNKNOWN_STATUS: 'Unknown',
        UNCOVERED_STATUS: 'Uncovered',

        /** DataTable */

        requirementsDataTable: null,

        FILTERS_MODE: 1,
        SAVED_FILTER_MODE: 2,
        NO_ANALISYS_VERSION: 0,

        /**
         * Unscoped Events - these events must be registered elsewhere as they will not be
         * fired by backbone because they are not within the view element. Refer to "_delegateUnscopedEvents".
         */
        unscopedEvents: {
            'submit.raven.report.overallcoverage #raven-requirement-coverage-filters-form': 'resetFilterValues',
            'click.raven.report.overallcoverage #raven-dialog-clear': 'resetFilterValues',
            'submit.raven.report.overallcoverage #raven-requirement-savedfilter-form': 'filtersSubmit',
            'click.raven.report.overallcoverage #raven-overall-dialog-apply': 'filtersSubmit',
            'click.raven.report.overallcoverage #requirement-presentation-option a': 'showFlattenRequirements',
            'click.raven.report.overallcoverage .filter-title': 'clickTitleNotSelected',
            'submitraven.report.overallcoverage #raven-overall-dialog-cancel': 'closeDialog',
            'click.raven.report.overallcoverage #raven-requirement-filter-fixversion-suggestions': 'preventDefault',
            'click.raven.report.overallcoverage #raven-requirement-filter-status-suggestions': 'preventDefault',
            'click.raven.report.overallcoverage #raven-requirement-filter-resolution-suggestions': 'preventDefault',
            'click.raven.report.overallcoverage #raven-overall-requirement-coverage-filter-assignee-suggestions': 'preventDefault',
            'click.raven.report.overallcoverage #raven-requirement-filter-component-suggestions': 'preventDefault',
            'click.raven.report.overallcoverage #raven-requirement-filter-saved-suggestions': 'preventDefault',
            'submitraven.report.overallcoverage #raven-requirement-coverage-form': 'preventDefault',
            'click.raven.report.overallcoverage .item-delete': 'preventDefault',
            'click.raven.report.overallcoverage .toogle-sub-requirements': 'toogleSubRequirements',
            'click.raven.report.overallcoverage #dismiss-how-to-read-msg': 'disableHowToRead',
            'click.raven.report.overallcoverage #enable-how-to-read-msg': 'enableHowToRead',
            'change.raven.report.overallcoverage #raven-overall-requirement-coverage-filter-assignee' : 'removeAssigneeIcon'
        },

        render: function() {
            var self = this;

            $('#report-wrapper').block();
            $('.raven-requirement-header').block();

            self.clearMessage();
            var $tableWrapper = self.$el;
            $tableWrapper.empty();

            $('#raven-overall-requirement-coverage-content').spin("large");

            var requirements = new Requirements();
            var params = $.param(this._createFilterParams());
            params = params + "&withDetails=true";

            requirements.fetch({
                data: params,
                success: function(data, textStatus, jqXHR) {
                    self._checkShowHowToRead();

                    if (data.models.length > 0) {
                        self._createOverallCoverageChart(data.models["0"].attributes);
                    } else {
                        AJS.messages.generic($("#message-row-wrapper"), {
                            body: AJS.I18n.getText('projectpanels.requirement-traceability.resultset.empty'),
                            closeable: false
                        });
                    }
                    $('#raven-overall-requirement-coverage-content').spinStop();
                    $('#report-wrapper').unblock();
                    $('.raven-requirement-header').unblock();
                },
                error: function(model, response) {
                    self.clearMessage();
                    AJS.messages.error($("#message-row-wrapper"), {
                        title: AJS.I18n.getText('execute-test-action.requirement-status.fetch-error'),
                        body: response.statusText + ": " + response.responseText,
                        closeable: false
                    });
                    $('#raven-overall-requirement-coverage-content').spinStop();
                    $('#report-wrapper').unblock();
                    $('.raven-requirement-header').unblock();
                }

            });
        },

        initialize: function() {

            this.projectId = $("#selected-project-id").val();

            this._initSelectPickers();

            this._initFilterValues();

            this._renderFilterSummary();

            this._delegateUnscopedEvents();

            if (this.versionPicker.getSelectedItem()) {
                var version = this.versionPicker.getSelectedItem().id;
                if (version == "0") {
                    this.version = null;
                } else {
                    this.version = version;
                }
            }

            this.testPlan = $('#raven-report-analysis-test-plan-select').val();

            self.execScope = $('#raven-requirement-coverage-parameters').find("#report-exec-scope-picker").val()[0];

            if (this.environmentPicker.getSelectedItem()) {
                var environment = this.environmentPicker.getSelectedItem().id;
                if (environment == "-1") {
                    this.environment = null;
                } else {
                    this.environment = environment;
                }
            }

            if (this.groupingFieldPicker.getSelectedDescriptor()) {
                this.groupingField = this.groupingFieldPicker.getSelectedDescriptor().properties.value
            }

            this._toogleExecScopeValuePicker(false);

            if (filterValues && filterValues.savedFilter) {
                this._toogleFilters($('.filter-title[data-mode=2]'));
            }

            this.render();
        },

        _createFilterParams: function() {
            var filters = {};

            if (this.version != null && this.version != "0") {
                filters["version"] = this.version;
            }
            if (this.environment != null) {
                filters["environment"] = this.environment;
            }
            if (this.testPlan != null) {
                filters["testPlan"] = this.testPlan;
            }
            filters["projectId"] = this.projectId;
            filters["execScope"] = this.execScope;
            filters["groupingField"] = this.groupingField;
            filters["flattenRequirements"] = filterValues.flattenRequirements;

            if ($('.filter-title.selected').data('mode') == this.SAVED_FILTER_MODE) {
                var savedFilter = filterValues.savedFilter;
                if (savedFilter) {
                    filters["savedFilter"] = savedFilter;
                }
            } else {
                var fixVersion = filterValues.fixVersion;
                if (fixVersion && fixVersion != -1) {
                    filters["fixVersion"] = fixVersion;
                    filters["previousVersions"] = filterValues.includePreviousVersions;
                }
                var status = filterValues.statusId;
                if (status) {
                    var len = status.length;
                    filters["statusId"] = "";
                    $.each(status, function(index) {
                        filters["statusId"] = filters["statusId"] + this;
                        if (index < len - 1) {
                            filters["statusId"] = filters["statusId"] + ",";
                        }
                    });
                }
                var resolution = filterValues.resolution;
                if (resolution && resolution != -1) {
                    filters["resolution"] = resolution;
                }
                var components = filterValues.components;
                if (components) {
                    var len = components.length;
                    filters["components"] = "";
                    $.each(components, function(index) {
                        filters["components"] = filters["components"] + this;
                        if (index < len - 1) {
                            filters["components"] = filters["components"] + ",";
                        }
                    });
                }
                var assignee = filterValues.assignee;
                if (assignee && assignee != -1) {
                    filters["assignee"] = assignee;
                }
                var contains = filterValues.contains;
                if (contains) {
                    filters["contains"] = contains;
                }
            }

            return filters;
        },

        _initSelectPickers: function() {
            var self = this;
            $('#raven-requirement-coverage-form').on("submit", function(e) {
                e.preventDefault();
            });

            this.execScopeFieldPicker = this._renderExecScopePicker();

            this.versionPicker = this._createSubtleSingleSelect($('#raven-report-analysis-version-select'));
            $(this.versionPicker).on('selectedItemChanged', function(e) {
                self.version = e.currentTarget.selectedItem.id;

                self.render();
            });

            this.testPlanPicker = this._createTestPlanSingleSelectPicker($('#raven-report-analysis-test-plan-select'));
            $('#raven-report-analysis-test-plan-select').on("change", function() {
                self.testPlan = $('#raven-report-analysis-test-plan-select').val();
                self.render();
            });
            $("#raven-report-analysis-test-plan-select-field").attr("placeholder", AJS.I18n.getText("view-issue-requirement-test-coverage-section.placeholer.strategy.test-plan"));

            this.environmentPicker = this._createSubtleSingleSelect($('#raven-report-analysis-environment-select'));
            $(this.environmentPicker).on('selectedItemChanged', function(e) {
                self.environment = e.currentTarget.selectedItem.id;

                if (self.environment == "-1") {
                    self.environment = null;
                }

                self.render();
            });

            this.groupingFieldPicker = this._createSubtleSelect($('#raven-grouping-field-select'));
            $('#raven-grouping-field-select').on('change', function() {
                self.groupingField = $('#raven-grouping-field-select').val();
                self.render();
            });

            this.execScope = $('#raven-requirement-coverage-parameters').find("#report-exec-scope-picker").val()[0];
            this._toogleExecScopeValuePicker(false);

            // Filter pickers
            this.fixVersionFieldPicker = this._createSingleSelectPicker($('#raven-filter-inline-dialog #raven-requirement-filter-fixversion'));
            this.statusFieldPicker = this._createStatusPicker($('#raven-filter-inline-dialog #raven-requirement-filter-status'));
            this.componentFieldPicker = this._createComponentPicker($('#raven-filter-inline-dialog #raven-requirement-filter-component'));
            this.resolutionFieldPicker = this._createSingleSelectPicker($('#raven-filter-inline-dialog #raven-requirement-filter-resolution'));
            this.assigneeFieldPicker = this._initAssigneeFilterPicker($('#raven-overall-requirement-coverage-filter-assignee'));
            this._createSingleFilterPickers();
        },

        _toogleExecScopeValuePicker: function(refresh) {
            switch (this.execScope) {
                case "version":
                    this.versionPicker.$container.show();
                    this.testPlanPicker.$container.hide();
                    break;
                case "testPlan":
                    this.versionPicker.$container.hide();
                    this.testPlanPicker.$container.show();
                    break;
            }

            if (refresh === true && this.execScope == "version" || (this.execScope == "testPlan" && this.testPlan != null && this.testPlan.length > 0)) {
                this.render();
            }
        },

        _toogleFilters: function(trigger) {
            $('.filter-title.selected').removeClass('selected');

            $(trigger).addClass('selected');

            if ($(trigger).data('mode') == this.SAVED_FILTER_MODE) {
                $('#raven-requirement-coverage-filters-form').hide();
                $('#raven-requirement-savedfilter-form').show();
            } else {
                $('#raven-requirement-coverage-filters-form').show();
                $('#raven-requirement-savedfilter-form').hide();
            }
        },

        _createSubtleSelect: function($selectField) {
            return new RAVEN.SubtleSelect({
                element: $selectField,
                showDropdownButton: false,
                revertOnInvalid: true
            });
        },

        _includePreviousVersions: function() {
            return $('#include-previous-versions-option').is(':checked');
        },

        _flattenRequirements: function() {
            return !$('#hierarchical-requirements-option').hasClass("checked");
        },

        destroy: function() {
            $("#show-tooltip").remove();
            // COMPLETELY UNBIND THE VIEW
            this.undelegateEvents();

            this._undelegateUnscopedEvents();
        },

        _createOverallCoverageChart: function(data) {
            var self = this;
            if (data && data.resultset && data.resultset.length > 0) {
                var renderOverallRequirementCoverageChart = RAVEN.Report.renderOverallRequirementCoverageChart();
                self.$el.html(renderOverallRequirementCoverageChart);

                self.chartDetailsData = data.details;

                var height = 0;

                // The size of the gadget will be calculated according to the number of rows it has
                if (data.rows && data.rows > 0) {
                    height = 40 * data.rows;
                }

                if (height < 300) {
                    height = 300;
                } else if (height > 1000) {
                    height = 1000;
                }

                self.clearMessage();

                self.overallChart = new pvc.NormalizedBarChart({
                    canvas: "overall-coverage-chart",
                    width: self._calculateGraphicWidth(),
                    height: height,
                    orientation: 'horizontal',
                    stacked: true,
                    animate: true,
                    legend: true,
                    interactive: true,
                    selectable: false,
                    clickable: true,
                    hoverable: true,
                    legendPosition: 'top',
                    legendAlign: 'right',
                    legendFont: '12px sans-serif',
                    valuesFont: 'bold 12px sans-serif',
                    orthoAxisLabelSpacingMin: 6,
                    valuesVisible: true,
                    valuesOptimizeLegibility: true,
                    colors: ['#95C160', '#D45D52', '#F1E069', '#9AA1A8', '#3671A3'],
                    clickAction: function(scene) {
                        if (scene.datums().first().isSelected) {
                            this.chart.data.replaceSelected([]);
                        } else {
                            this.chart.data.replaceSelected(scene.datums());
                        }

                        // Re-renders if any changed
                        this.chart.updateSelections();
                    },
                    selectionChangedAction: function(selectedDatums, selectedChangedDatums) {
                        if (selectedDatums && selectedDatums.length > 0) {
                            var datum = selectedDatums[0];
                            var group = datum.atoms.category.value;
                            var status = datum.atoms.series.value;
                            self._showOverallChartDetailsDataTable(group, status);
                        } else {
                            self._hideOverallChartDetailsDataTable();
                        }
                    },
                    extensionPoints: {
                        axisLabel_font: '12px sans-serif',
                        barLabel_text: function() {
                            // don't show percentages less than 8% because the label is too big...
                            var percent = this.scene.vars.value.percent.value;
                            if (percent > 0.07) {
                                return this.scene.vars.value.percent.label + ' (' + this.scene.vars.value.value + ')';
                            } else {
                                return null;
                            }
                        }
                    }
                });
                self.overallChart.setData(data, {
                    crosstabMode: false
                });
                self.overallChart.render();

            } else {
                self.message(null, AJS.I18n.getText('projectpanels.requirement-coverage.resultset.empty'), 'info')
            }
        },

        _calculateGraphicWidth: function() {
            var graphContentRatio = 0.75;
            var size = $('.aui-page-panel').width();
            if (size < 1200) {
                return size;
            } else {
                return size * graphContentRatio;
            }
        },

        _showOverallChartDetailsDataTable: function(group, status) {
            var self = this;
            // set details section width
            $('#overall-coverage-chart-details').css({ "max-width": self._calculateGraphicWidth() });

            // dispose previous datatable
            self._disposeOverallCoverageChartDetails();

            var version = self.version;
            if (version == self.NO_ANALISYS_VERSION) {
                version = null;
            }
            var groupingField = $('#raven-grouping-field-select').val();
            if (version != -1 && groupingField != -1) {
                $('#overall-coverage-chart-details').show();
                var statusCode = self._getStatusFromName(status);

                // create header
                var groupingName = $('#raven-grouping-field-select').find(":selected").text();

                var requirementsTitle;
                if (groupingName !== 'None') {
                    requirementsTitle = '<h3 class="requirements-title">' + AJS.I18n.getText('common.label.requirements-with') + ' ' + groupingName + ' "' + group + '" ' + AJS.I18n.getText('common.label.and') + ' ' + AJS.I18n.getText('common.label.status') + ' "' + status + '"</h3>';
                } else {
                    requirementsTitle = '<h3 class="requirements-title">' + AJS.I18n.getText('common.label.requirements-with') + ' ' + AJS.I18n.getText('common.label.status') + ' "' + status + '"</h3>';
                }

                $('#overall-coverage-chart-details-title')
                    .append(requirementsTitle)
                    .append('<p><a id="view-issues-link" style="display: none;" href="">' + AJS.I18n.getText('common.label.view-issues') + '</a></p>');

                self._createOverallChartDetailsDataTable(self.projectId, version, groupingField, group, statusCode);
            }
        },

        _createOverallChartDetailsDataTable: function(projectKey, version, groupingField, group, status) {
            var self = this;
            if (!self.chartDetailsData) {
                self.message(null, AJS.I18n.getText('projectpanels.requirement-coverage.details.error.server'), 'error');
                return;
            }

            var data = _.find(self.chartDetailsData, function(details) {
                return details.group.name === group && details.status.toUpperCase() === status.toUpperCase();
            });

            if (!data) {
                self.message(null, AJS.I18n.getText('projectpanels.requirement-coverage.details.error.server'), 'error');
                return;
            }

            if (data.jqlQuery) {
                var viewIssuesLink = self._createViewIssuesLink(data.jqlQuery);
                $('#view-issues-link').attr('href', viewIssuesLink).show();
            }

            self.requirementsDataTable = $('#requirements-datatable').xrayDataTable({
                "bServerSide": false,
                "bProcessing": false,
                "bAutoWidth": false,
                "iDisplayLength": 10,
                "sPaginationType": "full_numbers",
                "bSort": true,
                "oLanguage": {
                    "oPaginate": {
                        "sFirst": AJS.I18n.getText('common.label.first'),
                        "sLast": AJS.I18n.getText('common.label.last'),
                        "sNext": AJS.I18n.getText('common.label.next'),
                        "sPrevious": AJS.I18n.getText('common.label.previous')
                    },
                    "sInfoEmpty": AJS.I18n.getText('common.table.label.no-entries-to-show'),
                    "sEmptyTable": AJS.I18n.getText('projectpanels.requirement-coverage.details.empty'),
                    "sInfo": AJS.I18n.getText('common.table.label.showing-records'),
                    "sZeroRecords": AJS.I18n.getText('common.table.label.no-records-found'),
                    "sInfoFiltered": AJS.I18n.getText('common.table.label.table-filtered-from'),
                    "sSearch": AJS.I18n.getText('common.table.label.search'),
                    "sLengthMenu": AJS.I18n.getText('common.table.label.show-menu-entries')
                },
                "aaSorting": [],
                "aoColumns": [{
                    "mData": "subRequirements"
                }, {
                    "mData": "key"
                }, {
                    "mData": "summary"
                }, {
                    "mData": "total"
                }, {
                    "mData": "ok"
                }, {
                    "mData": "nok"
                }, {
                    "mData": "notRun"
                }, {
                    "mData": "completeness"
                }],
                "aoColumnDefs": [{
                    "bSearchable": false,
                    "bSortable": false,
                    "mRender": function(data, type, row) {
                        if (data && data.length > 0) {
                            return '<span class="toogle-sub-requirements expand-icon disabled"></span>';
                        } else {
                            return '';
                        }
                    },
                    "aTargets": [0]
                }, {
                    "mRender": function(data, type, row) {
                        return '<a href="' + contextPath + '/browse/' + row["key"] + '" class="link-title">' + $('<span></span>').text(data).html() + '</a>';
                    },
                    "aTargets": [1, 2]
                }, {
                    "bSearchable": false,
                    "mRender": function(data, type, row) {
                        return '<span class="tests-value">' + data + '</span>';
                    },
                    "aTargets": [3, 4, 5, 6]
                }, {
                    "mRender": function(data, type, row) {
                        return self._createCompletenessBar(data);
                    },
                    "aTargets": [7]
                }],
                "aaData": data.requirements
            });

            self.$el.css("padding-bottom", "35px");
        },

        _createViewIssuesLink: function(jql) {
            var jql = encodeURIComponent(jql);
            return contextPath + '/secure/IssueNavigator.jspa?reset=true&mode=hide&jqlQuery=' + jql;
        },

        _createCompletenessBar: function(completeness) {
            return '<ul class="completeness-bar" title="' + completeness + '%"><li style="width:' + completeness + '%;"><span>' + completeness + '%</span></li><li></li></ul>';
        },

        _getStatusFromName: function(statusName) {
            var self = this;
            if (statusName) {
                if (statusName === self.OK_STATUS) {
                    return 'OK';
                } else if (statusName === self.NOK_STATUS) {
                    return 'NOK';
                } else if (statusName === self.NOTRUN_STATUS) {
                    return 'NOTRUN';
                } else if (statusName === self.UNKNOWN_STATUS) {
                    return 'UNKNOWN';
                } else if (statusName === self.UNCOVERED_STATUS) {
                    return 'UNCOVERED';
                }
            }
            return null;
        },

        _hideOverallChartDetailsDataTable: function() {
            var self = this;
            // dispose previous datatable
            self._disposeOverallCoverageChartDetails();

            $('#overall-coverage-chart-details').hide();
        },

        _disposeOverallCoverageChartDetails: function() {
            if (this.requirementsDataTable) {
                this.requirementsDataTable.fnClearTable(false);
                this.requirementsDataTable.fnDestroy();
                this.requirementsDataTable = null;
            }

            $('#overall-coverage-chart-details-title').empty();
            this.$el.css("padding-bottom", "0px");
        },

        _createSubtleSingleSelect: function($selectField) {
            var items = _.map($selectField.find('option'), function(item) {
                return {
                    'id': item.value,
                    'name': item.text,
                };
            });

            var selectedItem = _.first(_.map($selectField.find('option:selected'), function(item) {
                return {
                    'id': item.value,
                    'name': item.text,
                };
            }));

            var subtlePicker = new RAVEN.Field.SubtleSingleSelect(items, selectedItem, $selectField.closest('#raven-subtle-container'));
            subtlePicker.render();

            // remove select element. This is only for retrieving the initial items.
            $selectField.remove();

            return subtlePicker;
        },

        _createTestPlanSingleSelectPicker: function($element) {
            return new AJS.SingleSelect({
                element: $element,
                showDropdownButton: true,
                submitInputVal: true,
                noQueryNoRequest: false,
                revertOnInvalid: true,
                loadOnInit: true,
                ajaxOptions: {
                    query: true,
                    url: contextPath + '/rest/raven/1.0/issue/picker?issueType=TESTPLAN',
                    formatResponse: function(data) {
                        var section = data.sections[0];
                        var optgroup = new AJS.GroupDescriptor({
                            weight: 0,
                            id: section.id + "-opt-group",
                            replace: true,
                            label: section.label + "(" + section.sub + ")"
                        });

                        $(section.issues).each(function(i, issue) {
                            var item = new AJS.ItemDescriptor({
                                value: issue.key,
                                label: issue.key + ' - ' + issue.summaryText,
                                title: issue.summary,
                                icon: contextPath + issue.img,
                                allowDuplicate: false,
                                //selected: issue.key === strategyValue[0] ? 'selected' : null
                            });
                            optgroup.addItem(item);
                        });
                        return [optgroup];
                    }
                }
            });
        },

        _renderExecScopePicker: function() {
            var self = this;
            if ($("#report-exec-scope-picker-field").html() === null) {
                var $element = $('#raven-requirement-coverage-parameters').find("#report-exec-scope-picker");
                $element.on("change", function(e, field) {
                    self.execScope = $('#raven-requirement-coverage-parameters').find("#report-exec-scope-picker").val()[0];

                    self._toogleExecScopeValuePicker(true);
                });

                return new RAVEN.SubtleSelect({
                    element: $element,
                    showDropdownButton: false,
                    revertOnInvalid: true
                });
            }

            return null;
        },

        _initFilterValues: function() {
            filterValues.savedFilter = $('#raven-filter-inline-dialog #raven-requirement-filter-saved').val();
            filterValues.fixVersion = $('#raven-filter-inline-dialog #raven-requirement-filter-fixversion').val();
            filterValues.includePreviousVersions = this._includePreviousVersions();
            filterValues.flattenRequirements = this._flattenRequirements();
            var statusesIds = [];
            var arr = $('#raven-filter-inline-dialog #raven-requirement-filter-status').val();
            if (arr != null) {
                for (var i = 0; i < arr.length; i++) {
                    for (var j = 0; j < statusesObject.length; j++) {
                        if (arr[i] == statusesObject[j].name) {
                            statusesIds.push(statusesObject[j].id);
                        }
                    }
                }
            }
            filterValues.statusId = (statusesIds.length > 0) ? statusesIds : null;
            filterValues.resolution = $('#raven-filter-inline-dialog #raven-requirement-filter-resolution').val();
            filterValues.assignee = $('#raven-filter-inline-dialog #raven-overall-requirement-coverage-filter-assignee').val();
            filterValues.contains = $('#raven-filter-inline-dialog #raven-requirement-filter-contains').val();
            filterValues.components = $('#raven-filter-inline-dialog #raven-requirement-filter-component').val();
        },

        _renderFilterSummary: function() {
            var summary = '';
            if (filterValues.savedFilter && filterValues.savedFilter != "-1") {
                summary = '<b>' + AJS.I18n.getText('projectpanels.requirement-coverage.label.saved-filter') + '</b>  ' + $('#raven-filter-inline-dialog #raven-requirement-filter-saved option[value="' + filterValues.savedFilter + '"]').text();
            } else {
                if (filterValues.fixVersion && filterValues.fixVersion != "-1") {
                    var previousVersions = filterValues.includePreviousVersions;
                    summary += '<b>' + AJS.I18n.getText('projectpanels.requirement-coverage.label.fix-version') + '</b>  ';
                    if (previousVersions === true) {
                        summary += '<= ';
                    }
                    summary += $('#raven-filter-inline-dialog #raven-requirement-filter-fixversion option[value="' + filterValues.fixVersion + '"]').text();
                }
                if (filterValues.statusId) {
                    if (summary) {
                        summary += '; ';
                    }
                    var statusesNames = "";
                    for (var i = 0; i < filterValues.statusId.length; i++) {
                        for (var j = 0; j < statusesObject.length; j++) {
                            if (filterValues.statusId[i] == statusesObject[j].id) {
                                if (i > 0 && i < filterValues.statusId.length) {
                                    statusesNames = statusesNames + ",";
                                }
                                statusesNames = statusesNames + statusesObject[j].name;
                            }
                        }
                    }
                    summary += '<b>' + AJS.I18n.getText('projectpanels.requirement-coverage.label.status') + '</b>  ' + statusesNames;
                }
                if (filterValues.resolution && filterValues.resolution != "-1") {
                    if (summary) {
                        summary += '; ';
                    }
                    summary += '<b>' + AJS.I18n.getText('projectpanels.requirement-coverage.label.resolution') + '</b>  ' + $('#raven-filter-inline-dialog #raven-requirement-filter-resolution option[value="' + filterValues.resolution + '"]').text();
                }
                if (filterValues.assignee && filterValues.assignee != "-1") {
                    if (summary) {
                        summary += '; ';
                    }
                    summary += '<b>' + AJS.I18n.getText('projectpanels.requirement-coverage.label.assignee') + '</b>  ' + $('#raven-filter-inline-dialog #raven-overall-requirement-coverage-filter-assignee option[value="' + filterValues.assignee + '"]').text();
                }
                if (filterValues.contains) {
                    if (summary) {
                        summary += '; ';
                    }
                    summary += '<b>' + AJS.I18n.getText('projectpanels.requirement-coverage.label.contains') + '</b> ' + filterValues.contains;
                }
                if (filterValues.components) {
                    if (summary) {
                        summary += '; ';
                    }
                    summary += '<b>' + AJS.I18n.getText('projectpanels.requirement-coverage.label.component') + '</b>  ' + filterValues.components;
                }
            }

            if (summary) {
                $('#raven-filter-trigger').html(summary);
            } else {
                $('#raven-filter-trigger').text(AJS.I18n.getText('projectpanels.requirement-coverage.label.filter-button'));
            }
        },

        _createComponentPicker: function($selectField) {
            return new AJS.MultiSelect({
                element: $selectField,
                showDropdownButton: true,
                submitInputVal: true,
                noQueryNoRequest: false,
                revertOnInvalid: true,
                loadOnInit: true,
                ajaxOptions: {
                    query: true,
                    url: contextPath + '/rest/api/2/project/' + this.projectId + '/components',
                    formatResponse: function(data) {
                        var optgroup = new AJS.GroupDescriptor({
                            weight: 0,
                            id: "all-filter-opt-group",
                            replace: true
                        });

                        $(data).each(function(i, status) {
                            this.label = this.name;
                            var item = new AJS.ItemDescriptor({
                                value: this.name,
                                label: this.name,
                                title: this.description,
                                highlighted: true,
                            });
                            optgroup.addItem(item);
                        });

                        return [optgroup];
                    }
                }
            });
        },

        _createSingleSelectPicker: function($selectField) {
            $selectField.on('initialized', function(e, field) {
                var version = $('body').data('version');
                if (!version || version.toString().charAt(0) === '5') {
                    $('input', $selectField.parent()).height('23px');
                }
            });

            return new AJS.SingleSelect({
                element: $selectField,
                showDropdownButton: true,
                revertOnInvalid: true
            });
        },

        _initAssigneeFilterPicker: function(e) {
            var $assineePicker = e;

            if ($assineePicker.data("aui-ss")) return;

            var restPath = "/rest/api/1.0/users/picker";

            var data = {
                    showAvatar: true
                },
                inputText = $assineePicker.data('inputValue');

            return new AJS.SingleSelect({
                element: $assineePicker,
                submitInputVal: true,
                revertOnInvalid: true,
                showDropdownButton: !!$assineePicker.data('show-dropdown-button'),
                errorMessage: AJS.I18n.getText("user.picker.invalid.user", "'{0}'"),
                ajaxOptions: {
                    url: contextPath + restPath,
                    query: true, // keep going back to the sever for each keystroke
                    data: data,
                    formatResponse: function(data) {
                        var res = [];
                        var defaultGroupDescriptor = new AJS.GroupDescriptor({
                            weight: 0 // order of groups in suggestions dropdown
                        });
                        defaultGroupDescriptor.addItem(new AJS.ItemDescriptor({
                            value: "-1",
                            label: AJS.I18n.getText("raven-admin-config.general.label.all")
                        }));
                        res.push(defaultGroupDescriptor);

                        jQuery(data).each(function(i, suggestions) {
                            var groupDescriptor = new AJS.GroupDescriptor({
                                weight: 1, // order or groups in suggestions dropdown
                                label: suggestions.footer
                            });

                            jQuery(suggestions.users).each(function(){
                                groupDescriptor.addItem(new AJS.ItemDescriptor({
                                    value: this.name, // value of item added to select
                                    label: this.displayName, // title of lozenge
                                    html: this.html,
                                    icon: this.avatarUrl,
                                    allowDuplicate: false,
                                    highlighted: true
                                }));
                            });
                            res.push(groupDescriptor);
                        });
                        return res;
                    }
                },
                inputText: inputText
            });
        },

        _createStatusPicker: function($selectField) {
            return new AJS.MultiSelect({
                element: $selectField,
                showDropdownButton: true,
                revertOnInvalid: true,
            });
        },

        _createSingleFilterPickers: function(ctx) {

            var restPath = "/rest/gadget/1.0/pickers/filters";

            $(".raven-default-filter-picker", ctx).each(function() {
                var $this = $(this);
                if ($this.data("aui-ss")) return;
                var data = { showAvatar: true },
                    inputText = $this.data('inputValue');

                var field = new AJS.SingleSelect({
                    element: $this,
                    submitInputVal: true,
                    showDropdownButton: !$this.data('show-dropdown-button'),
                    errorMessage: AJS.format("filters.picker.invalid.filter", "'{0}'"),
                    ajaxOptions: {
                        url: contextPath + restPath,
                        query: true, // keep going back to the sever for each keystroke
                        data: data,
                        formatResponse: function(data) {
                            var savedFilters = [];
                            if (data.filters) {
                                $.each($(data.filters), function(index, filter) {
                                    savedFilters[index] = new AJS.ItemDescriptor({
                                        value: filter.id,
                                        label: filter.name
                                    });
                                });
                            }
                            return savedFilters;
                        }
                    },
                    inputText: inputText
                });

                $this.data('field-support-obj', field);
            });
        },

        _checkShowHowToRead: function() {
            var self = this;
            $.ajax({
                type: "GET",
                url: contextPath + "/rest/raven/1.0/report/requirementCoverage/overall/isHowToRead",

                success: function(data, textStatus, jqXHR) {
                    if (data) {
                        self._showHowToRead();
                    } else {
                        self._hideHowToRead();
                    }
                },
                error: function(qXHR, textStatus, errorThrown) {
                    console.log(qXHR);
                }
            });
        },

        _setHowToReadServer: function(value) {
            $.ajax({
                type: "POST",
                data: value,
                url: contextPath + "/rest/raven/1.0/report/requirementCoverage/overall/setHowToRead",
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    console.log(qXHR);
                }
            });
        },

        _showHowToRead: function() {
            $(".raven-how-to-read-chart").show("fast");
            $("#raven-notifications-icon").hide("fast");
            $("#enable-how-to-read-msg").hide("fast");
        },

        _hideHowToRead: function() {
            $(".raven-how-to-read-chart").hide("fast");
            $("#raven-notifications-icon").show("fast");
            $("#enable-how-to-read-msg").show("fast");
        },

        clearMessage: function() {
            $('#message-row-wrapper').empty();
        },

        message: function(title, message, type) {
            type = (type === undefined) ? "error" : type;
            this.clearMessage();

            if (type === "error") {
                AJS.messages.error('#message-row-wrapper', {
                    title: title,
                    body: message,
                    closeable: false
                });
            } else if (type === "info") {
                AJS.messages.info('#message-row-wrapper', {
                    title: title,
                    body: message,
                    closeable: false
                });
            } else if (type === "warning") {
                AJS.messages.warning('#message-row-wrapper', {
                    title: title,
                    body: message,
                    closeable: false
                });
            } else if (type === "success") {
                AJS.messages.success('#message-row-wrapper', {
                    title: title,
                    body: message,
                    closeable: false
                });
            }
        },

        /****************************************************************
         *                        EVENT HANDLERS
         ****************************************************************/

        _delegateUnscopedEvents: function() {
            var self = this;
            $(document).off('.raven.report.overallcoverage');

            $.each(this.unscopedEvents, function(key, val) {
                var eventName = key.substring(0, key.indexOf(' '));
                var eventTrigger = key.substring(key.indexOf(' ') + 1);

                $(document).on(eventName, eventTrigger, $.proxy(self[val], self));
            });
        },

        _undelegateUnscopedEvents: function() {
            $(document).off('.raven.report.overallcoverage');
        },

        toogleSubRequirements: function(e) {
            var self = this;
            self._toogleExpandRow(e.target);
        },

        _toogleExpandRow: function(trigger) {
            var self = this;
            var row = $(trigger).parents('tr')[0];
            if (row) {
                if (self.requirementsDataTable.fnIsOpen(row)) {
                    // This row is already open - close it
                    $(trigger).removeClass('enabled').addClass('disabled');
                    self.requirementsDataTable.fnClose(row);
                } else {
                    // Open this row
                    $(trigger).removeClass('disabled').addClass('enabled');
                    self.requirementsDataTable.fnOpen(row, self._fillSubRequirementsDetails(row), 'details');
                }
            }
        },

        _fillSubRequirementsDetails: function(row) {
            var self = this;
            var data = self.requirementsDataTable.fnGetData(row);
            if (data && data.subRequirements && data.subRequirements.length > 0) {
                var table = $('<table class="detail aui data-table">\
                    <colgroup>\
                    <col span="1" style="width: 40px;">\
                    <col span="1" style="width: 120px;">\
                    <col span="1">\
                    <col span="1" style="width: 80px;">\
                    <col span="1" style="width: 80px">\
                    <col span="1" style="width: 80px;">\
                    <col span="1" style="width: 100px;">\
                    <col span="1" style="width: 130px;">\
                    </colgroup>\
                    <tbody>\
                    </tbody>\
                    </table>');
                $.each(data.subRequirements, function() {
                    table.append(self._renderSubRequirementRow(this));
                });

                return table.prop('outerHTML');
            }
            return '';
        },

        _renderSubRequirementRow: function(req) {
            var self = this;
            var row = '<tr>\
                <td></td>\
                <td><a href="' + contextPath + '/browse/' + req.key + '" style="padding-left:10px;" class="link-title" >' + req.key + '</a></td>\
                <td><a href="' + contextPath + '/browse/' + req.key + '" class="link-title">' + req.summary + '</a></td>\
                <td><span class="tests-value">' + req.total + '</span></td>\
                <td><span class="tests-value">' + req.ok + '</span></td>\
                <td><span class="tests-value">' + req.nok + '</span></td>\
                <td><span class="tests-value">' + req.notRun + '</span></td>\
                <td>' + self._createCompletenessBar(req.completeness) + '</td>\
                </tr>';
            return row;
        },

        filtersSubmit: function() {
            this._saveFilterValues();
            this.render();

            this._renderFilterSummary();

            if ($('#raven-filter-inline-dialog').attr('open') == "open") {
                $('#raven-filter-inline-dialog').removeAttr('open');
            } else {
                document.querySelector('#raven-filter-inline-dialog').hide();
            }

            return false;
        },

        _saveFilterValues: function() {
            if ($('.filter-title.selected').data('mode') == this.SAVED_FILTER_MODE) {
                filterValues.savedFilter = $('#raven-filter-inline-dialog #raven-requirement-filter-saved').val();
                filterValues.fixVersion = null;
                filterValues.includePreviousVersions = null;
                filterValues.statusId = null;
                filterValues.resolution = null;
                filterValues.assignee = null;
                filterValues.contains = null;
                filterValues.components = null;
            } else {
                filterValues.savedFilter = null;
                filterValues.fixVersion = $('#raven-filter-inline-dialog #raven-requirement-filter-fixversion').val();
                filterValues.includePreviousVersions = this._includePreviousVersions();

                var statusesIds = [];
                var arr = $('#raven-filter-inline-dialog #raven-requirement-filter-status').val();
                if (arr != null) {
                    for (var i = 0; i < arr.length; i++) {
                        for (var j = 0; j < statusesObject.length; j++) {
                            if (arr[i] == statusesObject[j].name) {
                                statusesIds.push(statusesObject[j].id);
                            }
                        }
                    }
                }
                filterValues.statusId = (statusesIds.length > 0) ? statusesIds : null;
                filterValues.resolution = $('#raven-filter-inline-dialog #raven-requirement-filter-resolution').val();
                filterValues.assignee = $('#raven-filter-inline-dialog #raven-overall-requirement-coverage-filter-assignee').val();
                filterValues.contains = $('#raven-filter-inline-dialog #raven-requirement-filter-contains').val();
                filterValues.components = $('#raven-filter-inline-dialog #raven-requirement-filter-component').val();
            }
        },

        removeAssigneeIcon: function(){
            var self = this;
            if (!self.assigneeFieldPicker._hasIcon()){
                if(self.assigneeFieldPicker.$entityIcon !== undefined){
                    self.assigneeFieldPicker.$entityIcon.remove();
                }
                self.assigneeFieldPicker.$container.removeClass("aui-ss-has-entity-icon");
            }
        },

        resetFilterValues: function() {
            var self = this;
            if (self.fixVersionFieldPicker) {
                self.fixVersionFieldPicker._setDescriptorWithValue("-1");
            }
            if (self.statusFieldPicker) {
                self.statusFieldPicker.clearLozenges();
            }
            if (self.resolutionFieldPicker) {
                self.resolutionFieldPicker._setDescriptorWithValue("-1");
            }
            if (self.assigneeFieldPicker) {
                self.assigneeFieldPicker.clear();
                self.assigneeFieldPicker._setDescriptorWithValue("-1");
            }
            if (self.componentFieldPicker) {
                self.componentFieldPicker.clearLozenges();
            }

            $('#raven-filter-inline-dialog #raven-requirement-filter-contains').val("");

            var fieldPicker = $('#raven-requirement-filter-saved').data('field-support-obj');
            if (fieldPicker) {
                fieldPicker.clear();
            }
        },

        showFlattenRequirements: function(e) {
            if (e.target.id == 'hierarchical-requirements-option') {
                filterValues.flattenRequirements = false;
            } else {
                filterValues.flattenRequirements = true;
            }

            this.render();

            this._renderFilterSummary();

            if ($('#raven-filter-inline-dialog').attr('open') == "open") {
                $('#raven-filter-inline-dialog').removeAttr('open');
            } else {
                $('#raven-filter-inline-dialog').hide();
            }

            return false;
        },

        preventDefault: function(e) {
            e.stopPropagation();
            e.preventDefault();
        },

        closeDialog: function() {
            if ($('#raven-filter-inline-dialog').attr('open') == "open") {
                $('#raven-filter-inline-dialog').removeAttr('open');
            } else {
                $('#raven-filter-inline-dialog').hide();
            }
        },

        clickTitleNotSelected: function(e) {
            this._toogleFilters(e.currentTarget);
            e.stopPropagation();
        },

        disableHowToRead: function() {
            $('.header-filter').attr('data-initialized', false);
            this._hideHowToRead();
            this._setHowToReadServer("disabled");
        },

        enableHowToRead: function() {
            $('.header-filter').attr('data-initialized', false);
            this._showHowToRead();
            this._setHowToReadServer("enabled");
        }
    });
})(AJS.$);