require(['aui/inline-dialog2']);
(function($) {

    /*to prevent undefined values on page load, I am saving values in this var*/
    var filtervalues = {
        "projectId": null,
        "filterscope": null
    };

    var statusesObject = {};

    var requireExperimentalDialog = false;

    var TestExecutionReportModel = Backbone.Model.extend({
        key: null,
        summary: null,
        testEnvironements: null,
        startDate: null,
        endDate: null,
        totalTests: null,
        statuses: null,
        manual: null,
        cucumber: null,
        generic: null,
        progress: null,
        success: null,
        elapsedTime: null,
        openDefects: null,
        closedDefects: null
    });

    var Testexecutions = Backbone.Collection.extend({
        model: TestExecutionReportModel,
        url: function() {
            return contextPath + '/rest/raven/1.0/report/testexecutionreport/testexecutions/';
        }
    });

    var TestStatusModel = Backbone.Model.extend({
        name: null,
        color: null
    });

    var TestStatuses = Backbone.Collection.extend({
        model: TestStatusModel,
        url: function() {
            return contextPath + '/rest/raven/1.0/report/testexecutionreport/teststatuses';
        }
    });

    var Filters = Backbone.Model.extend({
        urlRoot: function() {
            return contextPath + '/rest/raven/1.0/report/testexecutionreport/filterValues';
        }
    });

    RAVEN.Report.TestExecutionReportView = Backbone.View.extend({
        el: "#report-wrapper",
        filterView: null,
        render: function(requiresExperimentalDialog2) {
            var $htmlReport = RAVEN.Report.renderTestExecutionReport({ 'requiresExperimentalDialog2': requiresExperimentalDialog2 });
            this.$el.empty();
            this.$el.html($htmlReport);
            this.filterView = new RAVEN.Report.TestExecutionFilterView();
        },
        destroy: function() {
            if (this.filterView) {
                this.filterView.destroy();
                this.filterView = null;
            }
            $('#show-tooltip').remove();
            $('#load-all-warning').remove();
            $('#elapsed-time-tip-dialog').remove();
            this.$el.empty();
            this.undelegateEvents();
        }
    });

    RAVEN.Report.TestExecutionFilterView = Backbone.View.extend({
        el: "#testexecution-filter-content-place-holder",
        projectId: null,
        testExecutionView: null,
        dialog: null,
        render: function() {
            var filters = new Filters();
            var self = this;
            var $filterWrapper = self.$el;
            var params = $.param({ projectId: this.projectId });
            filters.fetch({
                data: params,
                success: function(data, textStatus, jqXHR) {

                    filtervalues.projectId = $("#selected-project-id").val();

                    filtervalues.filterscope = "filter";

                    if (data.attributes.filterscopepref) {
                        filtervalues.filterscope = data.attributes.filterscopepref;
                    }

                    requireExperimentalDialog = data.attributes.requiresExperimentalDialog2;

                    statusesObject = data.attributes.statuses;

                    if (data.attributes.versionpref != null) { filtervalues.version = data.attributes.versionpref; }
                    if (data.attributes.testPlanKeypref != null) { filtervalues.testplan = data.attributes.testPlanKeypref; }
                    if (data.attributes.environmentpref != null) { filtervalues.environment = data.attributes.environmentpref; }
                    if (data.attributes.savedFilterpref != null) { filtervalues.savedfilter = data.attributes.savedFilterpref; }
                    if (data.attributes.assigneepref != null) { filtervalues.assignee = data.attributes.assigneepref.name; }
                    if (data.attributes.statuspref != null) { filtervalues.statusId = data.attributes.statuspref; }
                    if (data.attributes.resolutionpref != null) { filtervalues.resolution = data.attributes.resolutionpref; }
                    if (data.attributes.contains != null) { filtervalues.contains = data.attributes.contains; }
                    self.dialog = RAVEN.Report.renderTestExecutionReportFilter({
                        'versions': data.attributes.versions,
                        'environments': data.attributes.environments,
                        'requiresExperimentalDialog2': data.attributes.requiresExperimentalDialog2,
                        'filterscopepref': filtervalues.filterscope,
                        'testPlanKeypref': data.attributes.testPlanKeypref,
                        'environmentpref': data.attributes.environmentpref,
                        'versionpref': data.attributes.versionpref,
                        'savedFilterpref': data.attributes.savedFilterpref,
                        'statuses': data.attributes.statuses,
                        'statuspref': data.attributes.statuspref,
                        'assignees': data.attributes.assignees,
                        'assigneepref': data.attributes.assigneepref,
                        'resolutions': data.attributes.resolutions,
                        'resolutionpref': data.attributes.resolutionpref,
                        'assigneepref': data.attributes.assigneepref,
                        'contains': data.attributes.contains,
                        'testPlanIssueIcon': data.attributes.testPlanIssueIcon,
                        'contextPath': contextPath


                    });
                    $filterWrapper.empty();
                    $filterWrapper.html(self.dialog);

                    self.testExecutionView = new RAVEN.Report.TestExecutionView();

                },
                error: function(model, response) {
                    AJS.messages.error($("#message-row-wrapper"), {
                        title: AJS.I18n.getText('testexecution-report-view-error-filters'),
                        body: response.responseText,
                        closeable: false
                    });
                }
            });
        },
        initialize: function() {
            this.projectId = $("#selected-project-id").val();
            this.render();
        },
        destroy: function() {
            if (this.testExecutionView) {
                this.testExecutionView.destroy();
                this.testExecutionView = null;
            }

            if (this.dialog) {
                $('#raven-testexec-filter-inline-dialog').remove();
            }
            this.remove();
        }
    });

    RAVEN.Report.TestExecutionView = Backbone.View.extend({
        el: "#testexecs-row-wrapper",
        version: null,
        testplan: null,
        environment: null,
        assignee: null,
        status: null,
        resolution: null,
        contains: null,
        filterscope: null,
        savedfilter: null,
        projectId: null,
        start: 0,
        total: 0,
        pageSize: 20,
        prevValues: null,
        filterMode: true,

        /**
         * Unscoped Events - these events must be registered elsewhere as they will not be
         * fired by backbone because they are not within the view element. Refer to "_delegateUnscopedEvents".
         */
        unscopedEvents: {
            'click.raven.report.testexecution #raven-testexec-dialog-apply': 'filtersSubmit',
            'click.raven.report.testexecution #raven-testexec-dialog-clear': 'filterClear',
            'click.raven.report.testexecution #raven-testexec-filter-dialog .filter-title': 'clickTitleNotSelected',
            'submit.raven.report.testexecution #raven-testexec-report-filters-form': 'preventDefault',
            'submit.raven.report.testexecution #raven-testexec-savedfilter-form': 'preventDefault',
            'click.raven.report.testexecution #raven-testexec-filter-saved-suggestions': 'preventDefault',
            'click.raven.report.testexecution #raven-testexec-report-filter-environment-suggestions': 'preventDefault',
            'click.raven.report.testexecution #raven-testexec-report-filter-testplan-suggestions': 'preventDefault',
            'click.raven.report.testexecution #raven-testexec-report-filter-fixversion-suggestions': 'preventDefault',
            'click.raven.report.testexecution #raven-testexec-report-filter-status-suggestions': 'preventDefault',
            'click.raven.report.testexecution #raven-testexec-report-filter-resolution-suggestions': 'preventDefault',
            'click.raven.report.testexecution #raven-testexec-report-filter-assignee-suggestions': 'preventDefault',
            'click.raven.report.testexecution #raven-testexec-dialog-cancel': 'closeDialog',
            'click.raven.report.testexecution #dismiss-how-to-read-msg': 'disableHowToRead',
            'click.raven.report.testexecution #enable-how-to-read-msg': 'enableHowToRead',
            'click.raven.report.testexecution .item-delete': 'preventDefault',
            'click.raven.report.testexecution #load-more': 'loadMore',
            'click.raven.report.testexecution #load-all-bt': 'loadAll',
            'change.raven.report.testexecution #raven-testexec-report-filter-assignee' : 'removeIcon',
            'change.raven.report.testexecution #raven-testexec-report-filter-testplan' : 'clearTestPlanIconOnChange'
        },
        loadAll: function() {
            this.prevValues = null;
            this.pageSize = -1;
            this.render();
        },

        loadMore: function() {
            //load next 20 items
            this.start = this.start + 20;
            if (this.total > this.pageSize) {
                $("#load-more").addClass("lastRowSpin");
                this.render();
            }
        },
        setPaginationInfo: function(dataModelsLength) {
            var entriesMsg = "";
            if (dataModelsLength == -1) {
                $("#load-more").css("display", "none");
                $(".load-all-entries").css("display", "none");
                entriesMsg = AJS.I18n.getText('common.table.label.no-entries-to-show');
            } else {
                var totalShowing = this.start + dataModelsLength;
                if (totalShowing >= this.total || this.total == 0) {
                    $("#load-more").css("display", "none");
                } else {
                    $("#load-more").css("display", "block");
                }
                if (totalShowing >= this.total) {
                    totalShowing = this.total;
                    $(".load-all-entries").css("display", "none");
                } else {
                    $(".load-all-entries").css("display", "block");
                }
                entriesMsg = AJS.I18n.getText('common.label.showing-records', totalShowing, this.total);
            }
            $(".info-entries").empty();
            $(".info-entries").append(entriesMsg);
        },
        render: function() {
            $('#report-wrapper').block();
            $('.raven-requirement-header').block();
            if ($('.lastRowSpin').length) {
                $(".lastRowSpin").spin("medium");
                $("#load-more").css("color", "#eee");
            } else {
                $('#report-wrapper').spin("large");
            }

            this.clearMessage();
            
            var self = this;
            var $tableWrapper = self.$el;
            var project = $("#selected-project-id").val();
            var teststatuses = new TestStatuses();
            var testexecutions = new Testexecutions();
            var params = filtervalues;

            teststatuses.fetch({
                success: function(data1, textStatus, jqXHR) {
                    self.checkShowHowToRead();

                    params = $.param({
                        start: self.start,
                        pageSize: self.pageSize
                    }) + "&" + $.param(params);

                    testexecutions.fetch({
                        data: params,
                        success: function(data2, textStatus, jqXHR) {
                            if (data2.models && data2.models.length > 0) {
                                self.total = data2.models[0].attributes.total;
                                self.pageSize = data2.models[0].attributes.pageSize;


                                if (self.prevValues) {
                                    $.merge(self.prevValues, data2.models);
                                    var $table = RAVEN.Report.renderTestExecutionRows({
                                        'issues': data2.models,
                                        'contextPath': contextPath,
                                        'statuses': data1.models
                                    });
                                    var position = $("tr:last").offset().top - 80;
                                    $("#test-execution-report-table tbody").append($table);
                                    $("html, body").animate({ scrollTop: position }, 1000);
                                } else {

                                    $('#elapsed-time-tip-dialog').remove();

                                    self.prevValues = data2.models;

                                    var $table = RAVEN.Report.renderTestExecutionsTable({
                                        'issues': data2.models,
                                        'contextPath': contextPath,
                                        'statuses': data1.models,
                                        'requiresExperimentalDialog2': requireExperimentalDialog
                                    });
                                    $tableWrapper.html($table);
                                }

                                self.setPaginationInfo(data2.models.length);
                                self.stopLoading();
                            } else {
                                if ($tableWrapper) {
                                    $tableWrapper.empty();

                                    self.setPaginationInfo(-1);
                                    self.stopLoading();
                                }
                                AJS.messages.generic($("#message-row-wrapper"), {
                                    body: AJS.I18n.getText('common.table.label.no-records-found'),
                                    closeable: false
                                });
                            }

                            var exportParams = params;
                            $("#raven-report-comon-to-csv-link").attr("href", contextPath + "/plugins/servlet/raven/report/testexecution/export?" + exportParams);

                        },
                        error: function(model, response) {
                            if ($tableWrapper) {
                                $tableWrapper.empty();

                                self.setPaginationInfo(-1);
                                self.stopLoading();
                            }
                            AJS.messages.error($("#message-row-wrapper"), {
                                title: AJS.I18n.getText('testexecution-report-view-error-table'),
                                body: response.responseText,
                                closeable: false
                            });
                            self.stopLoading();

                        }
                    });

                },
                error: function(model, response) {
                    self.setPaginationInfo(-1);
                    AJS.messages.error($("#message-row-wrapper"), {
                        title: AJS.I18n.getText('testexecution-report-view-error-table-statuses'),
                        body: response.responseText,
                        closeable: false
                    });
                    self.stopLoading();
                }
            });
        },
        stopLoading: function() {
            $('#report-wrapper').unblock();
            $('.raven-requirement-header').unblock();
            $('#report-wrapper').spinStop();

            $(".lastRowSpin").spinStop();
            $(".lastRowSpin").removeClass("lastRowSpin");
            $("#load-more").css("color", "#999");
            $(".lastRowSpin").unblock();
        },
        initialize: function() {

            $(".load-all-entries").css("display", "block");
            var self = this;
            this.projectId = $("#selected-project-id").val();
            this.initSelectPickers();
            this.renderFilterSummary();
            this._delegateUnscopedEvents();

            if (filtervalues.filterscope != "filter") {
                this.filterMode = false;
                this.toogleFilters(($('.filter-title[data-mode=2]')));
            }

            this.render();

        },
        renderFilterSummary: function() {
            var summary = '';
            if (filtervalues.savedfilter && filtervalues.savedfilter != "-1") {
                summary = '<b>' + AJS.I18n.getText('projectpanels.testexec-coverage.label.saved-filter') + '</b>  ' + filtervalues.savedfilter;
            } else {
                if (filtervalues.version && filtervalues.version != "-1") {
                    summary += '<b>' + AJS.I18n.getText('projectpanels.testexec-coverage.label.fix-version') + '</b>  ' + filtervalues.version;
                }
                if (filtervalues.assignee && filtervalues.assignee != "-1") {
                    if (summary) {
                        summary += '; ';
                    }
                    summary += '<b>' + AJS.I18n.getText('projectpanels.requirement-coverage.label.assignee-user') + '</b>  ' + filtervalues.assignee;
                }
                if (filtervalues.contains) {
                    if (summary) {
                        summary += '; ';
                    }
                    summary += '<b>' + AJS.I18n.getText('projectpanels.testexec-coverage.label.contains') + '</b>  ' + filtervalues.contains;
                }
                if (filtervalues.statusId) {
                    if (summary) {
                        summary += '; ';
                    }
                    var statusesNames = [];
                    var selectedStatuses = filtervalues.statusId.split(",");
                    for (var i = 0; i < selectedStatuses.length; i++){
                        statusesNames.push(statusesObject[selectedStatuses[i]]);
                    }
                    summary += '<b>' + AJS.I18n.getText('projectpanels.testexec-coverage.label.status') + '</b>  ' + statusesNames;
                }
                if (filtervalues.resolution) {
                    if (summary) {
                        summary += '; ';
                    }
                    summary += '<b>' + AJS.I18n.getText('projectpanels.testexec-coverage.label.resolution') + '</b>  ' + filtervalues.resolution;
                }
                if (filtervalues.testplan) {
                    if (summary) {
                        summary += '; ';
                    }
                    summary += '<b>' + AJS.I18n.getText('projectpanels.testexec-coverage.label.testplan') + '</b>  ' + filtervalues.testplan;
                }
                if (filtervalues.environment) {
                    if (summary) {
                        summary += '; ';
                    }
                    summary += '<b>' + AJS.I18n.getText('projectpanels.testexec-coverage.label.environment') + '</b>  ' + filtervalues.environment;
                }
            }

            if (summary) {
                $('#raven-testexec-report-filter-trigger').html(summary);
            } else {
                $('#raven-testexec-report-filter-trigger').text(AJS.I18n.getText('projectpanels.testexec-coverage.label.filters'));
            }
        },

        checkShowHowToRead: function() {
            var self = this;
            $.ajax({
                type: "GET",
                url: contextPath + "/rest/raven/1.0/report/testexecutionreport/isHowToRead",

                success: function(data, textStatus, jqXHR) {
                    if (data) {
                        self.showHowToRead();
                    } else {
                        self.hideHowToRead();
                    }
                },
                error: function(qXHR, textStatus, errorThrown) {
                    console.log(qXHR);
                }
            });
        },
        setHowToReadServer: function(value) {
            $.ajax({
                type: "POST",
                data: value,
                url: contextPath + "/rest/raven/1.0/report/testexecutionreport/setHowToRead",
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    console.log(qXHR);
                }
            });
        },

        initSelectPickers: function() {
            testPlanPicker = this.createTestPlanSingleSelectPicker($('#raven-testexec-report-filter-testplan'));
            fixVersionFieldPicker = this.createSingleSelectPicker($('#raven-testexec-report-filter-fixversion'));
            environmentPicker = this.createSingleSelectPicker($('#raven-testexec-report-filter-environment'));
            statusPicker = this.createStatusPicker($('#raven-testexec-report-filter-status'));
            resolutionPicker = this.createSingleSelectPicker($('#raven-testexec-report-filter-resolution'));
            assigneeFieldPicker = this.initAssigneeFilterPicker($('#raven-testexec-report-filter-assignee'));
            this.createSingleFilterPickers();
        },

        createSingleSelectPicker: function($selectField) {
            $selectField.on('initialized', function(e, field) {
                var version = $('body').data('version');
                if (!version || version.toString().charAt(0) === '5') {
                    $('input', $selectField.parent()).height('23px');
                }
            });
            return new AJS.SingleSelect({
                element: $selectField,
                showDropdownButton: true,
                revertOnInvalid: true,
            });
        },

        createStatusPicker: function($selectField) {
            return new AJS.MultiSelect({
                element: $selectField,
                showDropdownButton: true,
                revertOnInvalid: true,
            });
        },

        createTestPlanSingleSelectPicker: function($element) {
            return new AJS.SingleSelect({
                element: $element,
                showDropdownButton: true,
                submitInputVal: true,
                noQueryNoRequest: false,
                revertOnInvalid: true,
                loadOnInit: true,
                ajaxOptions: {
                    query: true,
                    url: contextPath + '/rest/raven/1.0/issue/picker?issueType=TESTPLAN',
                    formatResponse: function(data) {
                        var section = data.sections[0];

                        var defaultOptGroup = new AJS.GroupDescriptor({
                            weight: 0,
                            id: section.id + "-default-opt-group",
                            replace: true
                        });
                        var defaultItem = new AJS.ItemDescriptor({
                            value: "-1",
                            label: AJS.I18n.getText("raven-admin-config.general.label.all"),
                            allowDuplicate:false
                        });
                        defaultOptGroup.addItem(defaultItem);

                        var optgroup = new AJS.GroupDescriptor({
                            weight: 1,
                            id: section.id + "-opt-group",
                            replace: true,
                            label: section.label + "(" + section.sub + ")"
                        });

                        $(section.issues).each(function(i, issue) {
                            var item = new AJS.ItemDescriptor({
                                value: issue.key,
                                label: issue.key + ' - ' + issue.summaryText,
                                title: issue.summary,
                                icon: contextPath + issue.img,
                                allowDuplicate: false,
                                //selected: issue.key === strategyValue[0] ? 'selected' : null
                            });
                            optgroup.addItem(item);
                        });
                        return [defaultOptGroup, optgroup];
                    }
                }
            });
        },
        initAssigneeFilterPicker: function(e) {
            var $assineePicker = e;

            if ($assineePicker.data("aui-ss")) return;

            var restPath = "/rest/api/1.0/users/picker";

            var data = {
                    showAvatar: true
                },
                inputText = $assineePicker.data('inputValue');

            return new AJS.SingleSelect({
                element: $assineePicker,
                submitInputVal: true,
                revertOnInvalid: true,
                showDropdownButton: !!$assineePicker.data('show-dropdown-button'),
                errorMessage: AJS.I18n.getText("user.picker.invalid.user", "'{0}'"),
                ajaxOptions: {
                    url: contextPath + restPath,
                    query: true, // keep going back to the sever for each keystroke
                    data: data,
                    formatResponse: function(data) {
                        var res = [];
                        var defaultGroupDescriptor = new AJS.GroupDescriptor({
                            weight: 0 // order of groups in suggestions dropdown
                        });
                        defaultGroupDescriptor.addItem(new AJS.ItemDescriptor({
                            value: "-1",
                            label: AJS.I18n.getText("raven-admin-config.general.label.all")
                        }));
                        res.push(defaultGroupDescriptor);

                        jQuery(data).each(function(i, suggestions) {
                            var groupDescriptor = new AJS.GroupDescriptor({
                                weight: 1, // order or groups in suggestions dropdown
                                label: suggestions.footer
                            });

                            jQuery(suggestions.users).each(function(){
                                groupDescriptor.addItem(new AJS.ItemDescriptor({
                                    value: this.name, // value of item added to select
                                    label: this.displayName, // title of lozenge
                                    html: this.html,
                                    icon: this.avatarUrl,
                                    allowDuplicate: false,
                                    highlighted: true
                                }));
                            });
                            res.push(groupDescriptor);
                        });
                        return res;
                    }
                },
                inputText: inputText
            });
        },
        createSingleFilterPickers: function(ctx) {
            var restPath = "/rest/gadget/1.0/pickers/filters";
            $(".raven-default-filter-picker", ctx).each(function() {
                var $this = $(this);
                if ($this.data("aui-ss")) return;
                var data = { showAvatar: true },
                    inputText = $this.data('inputValue');
                var field = new AJS.SingleSelect({
                    element: $this,
                    submitInputVal: true,
                    //revertOnInvalid: true,
                    showDropdownButton: !!$this.data('show-dropdown-button'),
                    errorMessage: AJS.format("filters.picker.invalid.filter", "'{0}'"),
                    ajaxOptions: {
                        url: contextPath + restPath,
                        query: true, // keep going back to the sever for each keystroke
                        data: data,
                        formatResponse: function(data) {
                            var savedFilters = [];
                            if (data.filters) {
                                $.each($(data.filters), function(index, filter) {
                                    savedFilters[index] = new AJS.ItemDescriptor({
                                        value: filter.id,
                                        label: filter.name
                                    });
                                });
                            }
                            return savedFilters;
                        }
                    },
                    inputText: inputText
                });
                $this.data('field-support-obj', field);
            });
        },

        /****************************************************************
         *                        EVENT HANDLERS
         ****************************************************************/

        destroy: function() {
            $("#show-tooltip").remove();
            $('#load-all-warning').remove();
            // COMPLETELY UNBIND THE VIEW
            this.undelegateEvents();

            this._undelegateUnscopedEvents();
        },

        _delegateUnscopedEvents: function() {
            var self = this;
            $(document).off('.raven.report.testexecution');
            $.each(this.unscopedEvents, function(key, val) {
                var eventName = key.substring(0, key.indexOf(' '));
                var eventTrigger = key.substring(key.indexOf(' ') + 1);
                $(document).on(eventName, eventTrigger, $.proxy(self[val], self));
            });
        },

        _undelegateUnscopedEvents: function() {
            $(document).off('.raven.report.testexecution');
        },

        preventDefault: function(e) {
            e.stopPropagation();
            e.preventDefault();
        },

        closeDialog: function() {
            if ($('#raven-testexec-filter-inline-dialog').attr('open') == "open") {
                $('#raven-testexec-filter-inline-dialog').removeAttr('open');
            } else {
                document.querySelector('#raven-testexec-filter-inline-dialog').hide();
            }
        },

        clearTestPlanIconOnChange: function(){
            if($('#raven-testexec-report-filter-testplan').val() == "-1"){
                $("#raven-testexec-report-filter-testplan-single-select").removeClass("aui-ss-has-entity-icon");
            }
        },

        filtersSubmit: function() {
            this.prevValues = null;
            this.start = 0;
            this.pageSize = 20;
            filtervalues = {};
            filtervalues.projectId = $("#selected-project-id").val();
            statusesObject;
            if (this.filterMode) {
                filtervalues.filterscope = "filter";
                if ($('#raven-testexec-report-filter-fixversion').val() != "-1") {
                    filtervalues.version = $('#raven-testexec-report-filter-fixversion').val();
                }
                if ($('#raven-testexec-report-filter-testplan').val() != "-1") {
                    filtervalues.testplan = $('#raven-testexec-report-filter-testplan').val();
                }
                if ($('#raven-testexec-report-filter-environment').val() != "-1") {
                    filtervalues.environment = $('#raven-testexec-report-filter-environment').val();
                }
                if ($('#raven-testexec-report-filter-status').val() != null) {
                    var selectedStatuses = $('#raven-testexec-report-filter-status').val();
                    var statusesIds = [];
                    for(var i = 0; i < selectedStatuses.length; i++){
                        for(var key in statusesObject){
                            if(selectedStatuses[i] == statusesObject[key]){
                                statusesIds.push(key);
                            }
                        }
                    }
                    filtervalues.statusId = statusesIds.toString();
                }
                if ($('#raven-testexec-report-filter-resolution').val() != "-1") {
                    filtervalues.resolution = $('#raven-testexec-report-filter-resolution').val();
                }
                if ($('#raven-testexec-report-filter-assignee').val() != "-1") {
                    filtervalues.assignee = $('#raven-testexec-report-filter-assignee').val();
                }
                if ($('#raven-testexec-report-filter-contains').val() != "") {
                    filtervalues.contains = $('#raven-testexec-report-filter-contains').val();
                }
            } else {
                filtervalues.filterscope = "saved_filter";
                if ($('#raven-testexec-filter-saved').val() == null) {
                    filtervalues.savedfilter = "-1";
                } else {
                    filtervalues.savedfilter = $("#raven-testexec-filter-saved option:selected").text();
                }
            }
            this.renderFilterSummary();
            this.render();
            if ($('#raven-testexec-filter-inline-dialog').attr('open') == "open") {
                $('#raven-testexec-filter-inline-dialog').removeAttr('open');
            } else {
                document.querySelector('#raven-testexec-filter-inline-dialog').hide();
            }
        },

        removeIcon: function(){
            if (!assigneeFieldPicker._hasIcon()){
                if(assigneeFieldPicker.$entityIcon !== undefined){
                    assigneeFieldPicker.$entityIcon.remove();

                }
                assigneeFieldPicker.$container.removeClass("aui-ss-has-entity-icon");
            }
        },

        filterClear: function() {
            if (fixVersionFieldPicker) {
                fixVersionFieldPicker._setDescriptorWithValue("-1");
            }
            if (testPlanPicker) {
                testPlanPicker._setDescriptorWithValue("-1");
                $("#raven-testexec-report-filter-testplan-single-select").removeClass("aui-ss-has-entity-icon");
            }

            if (environmentPicker) {
                environmentPicker._setDescriptorWithValue("-1");
            }

            var fieldPicker = $('#raven-testexec-filter-saved').data('field-support-obj');
            if (fieldPicker) {
                fieldPicker.clear();
            }
            if (statusPicker) {
                statusPicker.clearLozenges();
            }
            if (resolutionPicker) {
                resolutionPicker._setDescriptorWithValue("-1");
            }
            if (assigneeFieldPicker) {
                assigneeFieldPicker.clear();
                assigneeFieldPicker._setDescriptorWithValue("-1");
            }
            $('#raven-testexec-report-filter-contains').val("");
        },

        toogleFilters: function(trigger) {
            $('.filter-title.selected').removeClass('selected');
            $(trigger).addClass('selected');
            if ($(trigger).data('mode') == 1) {
                this.filterMode = true;
                $('#raven-testexec-report-filters-form').show();
                $('#raven-testexec-savedfilter-form').hide();
            } else {
                this.filterMode = false;
                $('#raven-testexec-report-filters-form').hide();
                $('#raven-testexec-savedfilter-form').show();
            }
        },

        disableHowToRead: function() {
            $('.header-filter').attr('data-initialized', false);
            this.hideHowToRead();
            this.setHowToReadServer("disabled");
        },

        enableHowToRead: function() {
            $('.header-filter').attr('data-initialized', false);
            this.showHowToRead();
            this.setHowToReadServer("enabled");
        },

        showHowToRead: function() {
            $(".raven-how-to-read-chart").show("fast");
            $("#raven-notifications-icon").hide("fast");
            $("#enable-how-to-read-msg").hide("fast");
        },

        hideHowToRead: function() {
            $(".raven-how-to-read-chart").hide("fast");
            $("#raven-notifications-icon").show("fast");
            $("#enable-how-to-read-msg").show("fast");

        },

        clickTitleNotSelected: function(e) {
            this.toogleFilters(e.currentTarget);
        },

        clearMessage: function() {
            $('#message-row-wrapper').empty();
        }

    });
})(AJS.$);