require(['aui/inline-dialog2']);
(function($) {

    var filtervalues = {
        "projectId": null,
        "filterScope": null
    };

    var requiresDialog2 = false;

    var TestRunsReportModel = Backbone.Model.extend({
       testKey: null,
       testExecutionKey:null,
       testSummary: null,
       components: null,
       testPriority: null,
       testEnvironements: null,
       testPlanKey: null,
       testRunStatus: null,
       executedBy: null,
       fixVersions: null,
       testExecutionRevision: null,
       startDate: null,
       endDate: null,
       elapsedTime: null,
       openDefects: null,
       openDefectsLink: null,
       closedDefects: null,
       closedDefectsLinks: null,
       requiresExperimentalDialog2: null,
       start: null,
       total: null,
       pageSize: null
   });

   var TestRuns = Backbone.Collection.extend({
       model: TestRunsReportModel,
       url: function() {
           return contextPath + '/rest/raven/1.0/report/testrunsreport/testruns/';
       }
   });

    var Filters = Backbone.Model.extend({
        urlRoot: function() {
            return contextPath + '/rest/raven/1.0/report/testrunsreport/filtervalues';
        }
    });

    RAVEN.Report.TestRunsReportView = Backbone.View.extend({
        el: "#report-wrapper",
        filterView: null,
        render: function(requiresExperimentalDialog2) {
            var $htmlReport = RAVEN.Report.renderTestRunsReport({ 'requiresExperimentalDialog2': requiresExperimentalDialog2 });
            this.$el.empty();
            this.$el.html($htmlReport);
            this.filterView = new RAVEN.Report.TestRunsFilterView();
        },
        destroy: function() {
            if (this.filterView) {
                this.filterView.destroy();
                this.filterView = null;
            }
            $('#show-tooltip').remove();
            $('#load-all-warning').remove();
            $("#show-outdated-report-tooltip").remove();
            this.$el.empty();
            this.undelegateEvents();
        }
    });

    RAVEN.Report.TestRunsFilterView = Backbone.View.extend({
        el: "#testruns-filter-content-place-holder",
        projectId: null,
        testRunsTableView: null,
        dialog: null,
        render: function(){
            var filters = new Filters();
            var self = this;
            var $filterWrapper = self.$el;
            var params = $.param({projectId: this.projectId});
            filters.fetch({
                data: params,
                success: function (data, textStatus,jqXHR) {
                    filtervalues.projectId = $("#selected-project-id").val();
                    filtervalues.filterScope = data.attributes.filterScopePref;
                    requiresDialog2 = data.attributes.requiresExperimentalDialog2;
                    requireExperimentalDialog = data.attributes.requiresExperimentalDialog2;
                    if (data.attributes.savedFilterPref != null) { filtervalues.savedFilter = data.attributes.savedFilterPref; }
                    if(data.attributes.testPlanPref != null) {filtervalues.testPlan = data.attributes.testPlanPref;}
                    if(data.attributes.fixVersionPref != null) {filtervalues.fixVersion = data.attributes.fixVersionPref;}
                    if(data.attributes.revisionPref != null) {filtervalues.revision = data.attributes.revisionPref;}
                    if(data.attributes.testContainsPref != null) {filtervalues.testContains = data.attributes.testContainsPref;}
                    if(data.attributes.testExecutionContainsPref != null) {filtervalues.testExecutionContains = data.attributes.testExecutionContainsPref;}
                    if(data.attributes.testRunAssigneePref != null) {filtervalues.testRunAssignee = data.attributes.testRunAssigneePref.name;}
                    if(data.attributes.testRunStatusPref != null) {filtervalues.testRunStatus = data.attributes.testRunStatusPref;}
                    if(data.attributes.testPriorityPref != null) {filtervalues.testPriority = data.attributes.testPriorityPref;}
                    if(data.attributes.testComponentPref != null) {filtervalues.testComponent = data.attributes.testComponentPref;}
                    if(data.attributes.testEnvironmentPref != null) {filtervalues.testEnvironment = data.attributes.testEnvironmentPref;}
                    if(data.attributes.executedByPref !=null) {filtervalues.executedBy = data.attributes.executedByPref.name;}
                    self.dialog = RAVEN.Report.renderTestRunReportFilter({
                        'filterscopepref': data.attributes.filterScopePref,
                        'savedfilterpref': data.attributes.savedFilterPref,
                        'testplanpref': data.attributes.testPlanPref,
                        'fixversionpref': data.attributes.fixVersionPref,
                        'revisionpref': data.attributes.revisionPref,
                        'testContainsPref': data.attributes.testContainsPref,
                        'testExecutionContainsPref': data.attributes.testExecutionContainsPref,
                        'testrunstatuspref': data.attributes.testRunStatusPref,
                        'testprioritypref': data.attributes.testPriorityPref,
                        'testcomponentpref': data.attributes.testComponentPref,
                        'testenvironmentpref': data.attributes.testEnvironmentPref,
                        'testrunassigneepref': data.attributes.testRunAssigneePref,
                        'testrunstatuses': data.attributes.testRunStatuses,
                        'testpriorities': data.attributes.testPriorities,
                        'testcomponents': data.attributes.testComponents,
                        'testenvironments': data.attributes.testEnvironments,
                        'fixversions': data.attributes.fixVersions,
                        'requiresExperimentalDialog2': data.attributes.requiresExperimentalDialog2,
                        'executedbypref': data.attributes.executedByPref,
                        'testPlanIssueIcon': data.attributes.testPlanIssueIcon,
                        'contextPath': contextPath,
                    });
                    $filterWrapper.empty();
                    $filterWrapper.html(self.dialog);
                    self.testRunsTableView = new RAVEN.Report.TestRunsTableView();
                },
                error: function(model, response){
                    AJS.messages.error($("#message-row-wrapper"),{
                        title: AJS.I18n.getText('testruns-report-view-error-filters'),
                        body: response.responseText,
                        closeable: false
                    });
                }
            });
        },

        initialize: function(){
            this.projectId = $("#selected-project-id").val();
            this.render();
        },

        destroy: function(){
            if (this.testRunsTableView) {
                this.testRunsTableView.destroy();
                this.testRunsTableView = null;
            }
            if (this.dialog) {
                $('#raven-testruns-filter-inline-dialog').remove();
            }
            this.remove();
        }
    });



    RAVEN.Report.TestRunsTableView = Backbone.View.extend({
        el: "#testruns-row-wrapper",
        projectId: null,
        start: 0,
        total: 0,
        pageSize: 20,
        prevValues: null,
        filterMode: true,
        firstID: null,
        /**
         * Unscoped Events - these events must be registered elsewhere as they will not be
         * fired by backbone because they are not within the view element. Refer to "_delegateUnscopedEvents".
         */
        unscopedEvents: {
            'click.raven.report.testruns #raven-testruns-dialog-apply': 'filtersSubmit',
            'click.raven.report.testruns #raven-testruns-filter-dialog .filter-title': 'clickTitleNotSelected',
            'click.raven.report.testruns #raven-testruns-dialog-cancel': 'closeDialog',
            'click.raven.report.testruns #raven-testrun-report-filter-testplan-suggestions': 'preventDefault',
            'click.raven.report.testruns #raven-testruns-report-filter-testrunstatuses-suggestions': 'preventDefault',
            'click.raven.report.testruns #raven-testruns-report-filter-fixversion-suggestions': 'preventDefault',
            'click.raven.report.testruns #raven-testruns-report-filter-testpriorities-suggestions': 'preventDefault',
            'click.raven.report.testruns #raven-testruns-report-filter-testcomponents-suggestions': 'preventDefault',
            'click.raven.report.testruns #raven-testruns-report-filter-testenvironments-suggestions': 'preventDefault',
            'click.raven.report.testruns #raven-testrun-report-filter-testrunassignee-suggestions': 'preventDefault',
            'click.raven.report.testruns #raven-testrun-report-filter-executedby-suggestions': 'preventDefault',
            'click.raven.report.testruns #raven-testruns-filter-saved-suggestions': 'preventDefault',
            'click.raven.report.testruns .item-delete': 'preventDefault',
            'click.raven.report.testruns #raven-testruns-dialog-clear': 'filterClear',
            'submit.raven.report.testruns #raven-testruns-report-filters-form': 'preventDefault',
            'submit.raven.report.testruns #raven-testruns-savedfilter-form': 'preventDefault',
            'click.raven.report.testruns #dismiss-how-to-read-msg': 'disableHowToRead',
            'click.raven.report.testruns #enable-how-to-read-msg': 'enableHowToRead',
            'click.raven.report.testruns #load-more': 'loadMore',
            'click.raven.report.testruns #load-all-bt': 'loadAll',
            'change.raven.report.testruns #raven-testrun-report-filter-testrunassignee' : 'removeTestRunAssigneeIcon',
            'change.raven.report.testruns #raven-testrun-report-filter-executedby' : 'removeExecutedByIcon',
            'change.raven.report.testruns #raven-testrun-report-filter-testplan' : 'clearTestPlanIconOnChange'
        },
        render: function(){
            $('#report-wrapper').block();
            $('.raven-requirement-header').block();
            if ($('.lastRowSpin').length) {
                $(".lastRowSpin").spin("medium");
                $("#load-more").css("color", "#eee");
            } else {
                $('#report-wrapper').spin("large");
            }
            this.clearMessage();
            var self = this;
            var $tableWrapper = self.$el;
            this.projectId = $("#selected-project-id").val();
            var params = filtervalues;
            var testRuns = new TestRuns();
            params = $.param({
                    start: self.start,
                    pageSize: self.pageSize,
                    projectId: self.projectId
                }) + "&" + $.param(params);
            testRuns.fetch({
                data: params,
                timeout: 300000,
                success: function(data, textStatus, jqXHR){
                    self.checkShowHowToRead();
                    if(data.models && data.models.length > 0){
                        self.total = data.models[0].attributes.total;
                        self.pageSize = data.models[0].attributes.pageSize;
                        if(self.prevValues){
                            if(self.firstID != data.models[0].attributes.firstID){
                                $("#raven-outdated-report-icon").show("fast");
                            }
                            $.merge(self.prevValues, data.models);
                            var $table = RAVEN.Report.renderTestRunsRows({
                                'issues': data.models,
                                'contextPath': contextPath
                            });
                            var position = $("tr:last").offset().top - 80;
                            $("#test-runs-report-table tbody").append($table);
                            $("html, body").animate({ scrollTop: position }, 1000);
                        }else{
                            $("#raven-outdated-report-icon").hide("fast");
                            if(data.models[0].attributes.firstID){
                                self.firstID = data.models[0].attributes.firstID;
                            }
                            self.prevValues = data.models;
                            var $table = RAVEN.Report.renderTestRunsTable({
                                'issues': data.models,
                                'contextPath': contextPath
                            });
                            $tableWrapper.html($table);
                        }
                        self.setPaginationInfo(data.models.length);
                        self.stopLoading();
                    }else{
                        if ($tableWrapper) {
                            $tableWrapper.empty();
                            self.setPaginationInfo(-1);
                            self.stopLoading();
                        }
                        AJS.messages.generic($("#message-row-wrapper"), {
                            body: AJS.I18n.getText('common.table.label.no-records-found'),
                            closeable: false
                        });
                    }

                    var exportParams = params;

                    if(self.total > 10000){
                        $("#raven-report-comon-to-csv-link").attr("href", "#");
                        if(requiresDialog2 === true){
                            var dialog = self.getDeprecatedWarningDialog();
                            document.getElementById("raven-report-comon-to-csv-link").onclick = function(){
                                dialog.show();
                            };
                        } else {
                            document.getElementById("raven-report-comon-to-csv-link").onclick = function(){
                                AJS.dialog2("#report-testruns-longreport-warning").show();
                            };
                            AJS.$("#report-testruns-longreport-warning-button-ok").click(function() {
                                AJS.dialog2("#report-testruns-longreport-warning").hide();
                            });
                        }

                    } else{
                        $("#raven-report-comon-to-csv-link").attr("href", contextPath + "/plugins/servlet/raven/report/testruns/export?" + exportParams);
                        document.getElementById("raven-report-comon-to-csv-link").onclick = null;
                    }
                },
                error: function(model, response){
                    if ($tableWrapper) {
                        $tableWrapper.empty();

                        self.setPaginationInfo(-1);
                        self.stopLoading();
                    }
                    AJS.messages.error($("#message-row-wrapper"), {
                        title: AJS.I18n.getText('testruns-report-view-error-table'),
                        body: response.responseText,
                        closeable: false
                    });
                }
            });
        },

        setPaginationInfo: function(dataModelsLength){
            var entriesMsg = "";
            if (dataModelsLength == -1) {
                $("#load-more").css("display", "none");
                $(".load-all-entries").css("display", "none");
                entriesMsg = AJS.I18n.getText('common.table.label.no-entries-to-show');
            } else {
                var totalShowing = this.start + dataModelsLength;
                if (totalShowing >= this.total || this.total == 0) {
                    $("#load-more").css("display", "none");
                } else {
                    $("#load-more").css("display", "block");
                }
                if (totalShowing >= this.total) {
                    totalShowing = this.total;
                    $(".load-all-entries").css("display", "none");
                } else {
                    $(".load-all-entries").css("display", "block");
                }
                entriesMsg = AJS.I18n.getText('common.label.showing-records', totalShowing, this.total);
            }
            $(".info-entries").empty();
            $(".info-entries").append(entriesMsg);
        },

        stopLoading: function() {
            $('#report-wrapper').unblock();
            $('.raven-requirement-header').unblock();
            $('#report-wrapper').spinStop();

            $(".lastRowSpin").spinStop();
            $(".lastRowSpin").removeClass("lastRowSpin");
            $("#load-more").css("color", "#999");
            $(".lastRowSpin").unblock();
        },

        destroy: function(){
            $("#show-tooltip").remove();
            $('#load-all-warning').remove();
            $("#show-outdated-report-tooltip").remove();
            document.getElementById("raven-report-comon-to-csv-link").onclick = null;
            this.undelegateEvents();
            this._undelegateUnscopedEvents();
        },

        initialize: function(){
            $(".load-all-entries").css("display", "block");
            this.projectId = $("#selected-project-id").val();
            this.initSelectPickers();
            this.renderFilterSummary();
            this._delegateUnscopedEvents();
            if (filtervalues.filterScope != "filter") {
                this.filterMode = false;
                this.toogleFilters(($('.filter-title[data-mode=2]')));
            }
            this.render();
        },

        renderFilterSummary: function () {
            var summary = '';
            if(filtervalues.savedFilter && filtervalues.savedFilter != "-1"){
                summary = AJS.I18n.getText('projectpanels.testruns-coverage.label.saved-filter') + ": "+filtervalues.savedFilter;
            }
            else{
                if(filtervalues.fixVersion && filtervalues.fixVersion != "-1"){
                    if(summary){
                        summary += ", ";
                    }
                    summary += AJS.I18n.getText('projectpanels.testruns-coverage.label.fixversion') + ": " + filtervalues.fixVersion;
                }
                if(filtervalues.revision){
                    if(summary){
                        summary += ", ";
                    }
                    summary += AJS.I18n.getText('projectpanels.testruns-coverage.label.revision') + ": " + filtervalues.revision;
                }
                if(filtervalues.testEnvironment && filtervalues.testEnvironment != "-1"){
                    if(summary){
                        summary += ", ";
                    }
                    summary += AJS.I18n.getText('projectpanels.testruns-coverage.label.testenvironments') + ": " + filtervalues.testEnvironment;
                }
                if(filtervalues.testPlan && filtervalues.testPlan != "-1"){
                    if(summary){
                        summary += ", ";
                    }
                    summary += AJS.I18n.getText('projectpanels.testruns-coverage.label.testplan') + ": " + filtervalues.testPlan;
                }
                if(filtervalues.testPriority && filtervalues.testPriority != "-1"){
                    if(summary){
                        summary += ", ";
                    }
                    summary += AJS.I18n.getText('projectpanels.testruns-coverage.label.testpriority') + ": " + filtervalues.testPriority;
                }
                if(filtervalues.testComponent && filtervalues.testComponent != "-1"){
                    if(summary){
                        summary += ", ";
                    }
                    summary += AJS.I18n.getText('projectpanels.testruns-coverage.label.testcomponents') + ": " + filtervalues.testComponent;
                }
                if(filtervalues.testRunAssignee && filtervalues.testRunAssignee != "-1"){
                    if(summary){
                        summary += ", ";
                    }
                    summary += AJS.I18n.getText('projectpanels.testruns-coverage.label.tesrunassignee') + ": " + filtervalues.testRunAssignee;
                }
                if(filtervalues.executedBy && filtervalues.executedBy != "-1"){
                    if(summary){
                        summary += ", ";
                    }
                    summary += AJS.I18n.getText('projectpanels.testruns-coverage.label.executedby') + ": " + filtervalues.executedBy;
                }
                if(filtervalues.testRunStatus){
                    if(summary){
                        summary += ", ";
                    }
                    summary += AJS.I18n.getText('projectpanels.testruns-coverage.label.testrunstatus') + ": " + filtervalues.testRunStatus;
                }
                if(filtervalues.testContains){
                    if(summary){
                        summary += ", ";
                    }
                    summary += AJS.I18n.getText('projectpanels.testruns-coverage.label.contains') +": " + filtervalues.testContains;
                }
                if(filtervalues.testExecutionContains){
                    if(summary){
                        summary += ", ";
                    }
                    summary += AJS.I18n.getText('projectpanels.testruns-coverage.label.contains') +": " + filtervalues.testExecutionContains;
                }
            }
            if(summary){
                $("#raven-testruns-report-filter-trigger").html(summary);
            } else{
                $("#raven-testruns-report-filter-trigger").html(AJS.I18n.getText('projectpanels.testruns-coverage.label.filter'));
            }
        },

        checkShowHowToRead: function() {
            var self = this;
            $.ajax({
                type: "GET",
                url: contextPath + "/rest/raven/1.0/report/testrunsreport/isHowToRead",
                success: function(data, textStatus, jqXHR) {
                    if (data) {
                        self.showHowToRead();
                    } else {
                        self.hideHowToRead();
                    }
                },
                error: function(qXHR, textStatus, errorThrown) {
                    console.log(qXHR);
                }
            });
        },

        setHowToReadServer: function(value){
            $.ajax({
                type: "POST",
                data: value,
                url: contextPath + "/rest/raven/1.0/report/testrunsreport/setHowToRead?value="+value,
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    console.log(qXHR);
                }
            });
        },

        _delegateUnscopedEvents: function(){
            var self = this;
            $(document).off('.raven.report.testruns');
            $.each(this.unscopedEvents, function(key, val) {
                var eventName = key.substring(0, key.indexOf(' '));
                var eventTrigger = key.substring(key.indexOf(' ') + 1);
                $(document).on(eventName, eventTrigger, $.proxy(self[val], self));
            });
        },

        _undelegateUnscopedEvents: function() {
            $(document).off('.raven.report.testruns');
        },

        getDeprecatedWarningDialog: function(){
            var title = AJS.I18n.getText('testruns-report-view-warning-longreport-header');
            var body = "<p>"+AJS.I18n.getText('testruns-report-view-warning-longreport')+"<p>";

            var dialog = new AJS.Dialog({
                width: 400,
                height: 230,
                id: "long-report-warning",
                closeOnOutsideClick: true
            });
            dialog.addHeader(title);
            dialog.addPanel("Panel 1", body, "panel-body");
            dialog.addButton("OK", function (dialog) {
                dialog.hide();
            });
            return dialog;
        },

        /****************************************************************
         *                        EVENT HANDLERS
         ****************************************************************/

        loadAll: function(){
            if(this.total > 10000){
                if(requiresDialog2 === true){
                    var dialog = this.getDeprecatedWarningDialog();
                    dialog.show();
                } else {
                    AJS.dialog2("#report-testruns-longreport-warning").show();
                }
            } else{
                this.prevValues = null;
                this.pageSize = -1;
                this.render();
            }
        },

        loadMore: function(){
            this.start = this.start + 20;
            if(this.total > this.pageSize){
                $("#load-more").addClass("lastRowSpin");
                this.render();
            }
        },

        filtersSubmit: function(){
          this.prevValues = null;
          this.start = 0;
          this.pageSize = 20;
          filtervalues = {};
          if(this.filterMode){
              filtervalues.filterScope = "filter";
              if($("#raven-testruns-report-filter-fixversion").val() != "-1"){
                  filtervalues.fixVersion = $("#raven-testruns-report-filter-fixversion").val();
              }
              if($("#raven-testruns-report-filter-testrunstatuses").val() != null){
                  filtervalues.testRunStatus = $("#raven-testruns-report-filter-testrunstatuses").val().toString();
              }
              if($("#raven-testruns-report-filter-testpriorities").val() != "-1"){
                  filtervalues.testPriority = $("#raven-testruns-report-filter-testpriorities").val();
              }
              if($("#raven-testruns-report-filter-testcomponents").val() != "-1"){
                  filtervalues.testComponent = $("#raven-testruns-report-filter-testcomponents").val();
              }
              if($("#raven-testruns-report-filter-testenvironments").val() != "-1"){
                  filtervalues.testEnvironment = $("#raven-testruns-report-filter-testenvironments").val();
              }
              if($("#raven-testrun-report-filter-testplan").val() != "-1"){
                  filtervalues.testPlan = $("#raven-testrun-report-filter-testplan").val();
              }
              if($("#raven-testrun-report-filter-testrunassignee").val() != "-1"){
                  filtervalues.testRunAssignee = $("#raven-testrun-report-filter-testrunassignee").val();
              }
              if($("#raven-testrun-report-filter-test-contains").val() != ""){
                  filtervalues.testContains = $("#raven-testrun-report-filter-test-contains").val();
              }
              if($("#raven-testrun-report-filter-testexecution-contains").val() != ""){
                  filtervalues.testExecutionContains = $("#raven-testrun-report-filter-testexecution-contains").val();
              }
              if($("#raven-testrun-report-filter-revision").val() != ""){
                  filtervalues.revision = $("#raven-testrun-report-filter-revision").val();
              }
              if($("#raven-testruns-report-filter-testrunstatuses").val() != null){
                  filtervalues.testRunStatus = $("#raven-testruns-report-filter-testrunstatuses").val().toString();
              }
              if($("#raven-testrun-report-filter-executedby").val() != "-1"){
                  filtervalues.executedBy = $("#raven-testrun-report-filter-executedby").val();
              }
          }
          else{
              filtervalues.filterScope = "saved_filter";
              if($("#raven-testruns-filter-saved").val() == null){
                  filtervalues.savedFilter = "-1";
              }
              else{
                  filtervalues.savedFilter = $("#raven-testruns-filter-saved option:selected").text();
              }
          }
          this.renderFilterSummary();
          this.render();
          if($("#raven-testruns-filter-inline-dialog").attr('open') == "open"){
              $("#raven-testruns-filter-inline-dialog").removeAttr('open');
          }
          else{
              document.querySelector('#raven-testruns-filter-inline-dialog').hide();
          }
        },

        removeTestRunAssigneeIcon: function() {
            if (!testRunAssigneeFieldPicker._hasIcon()){
                if(testRunAssigneeFieldPicker.$entityIcon !== undefined){
                    testRunAssigneeFieldPicker.$entityIcon.remove();
                }
                testRunAssigneeFieldPicker.$container.removeClass("aui-ss-has-entity-icon");
            }
        },

        removeExecutedByIcon: function(){
            if (!executedByFieldPicker._hasIcon()){
                if(executedByFieldPicker.$entityIcon !== undefined){
                    executedByFieldPicker.$entityIcon.remove();
                }
                executedByFieldPicker.$container.removeClass("aui-ss-has-entity-icon");
            }
        },

        filterClear: function () {
          if(testPlanPicker){
              testPlanPicker._setDescriptorWithValue("-1");
              $("#raven-testrun-report-filter-testplan-single-select").removeClass("aui-ss-has-entity-icon");
          }
          if(testRunStatusPicker){
              testRunStatusPicker.clearLozenges();
          }
          if(fixVersionPicker){
              fixVersionPicker._setDescriptorWithValue("-1");
          }
          if(testPriorityPicker){
              testPriorityPicker._setDescriptorWithValue("-1");
          }
          if(testComponentsPicker){
              testComponentsPicker._setDescriptorWithValue("-1");
          }
          if(testEnvironmentsPicker){
              testEnvironmentsPicker._setDescriptorWithValue("-1");
          }
          if(testRunAssigneeFieldPicker){
              testRunAssigneeFieldPicker.clear();
              testRunAssigneeFieldPicker._setDescriptorWithValue("-1");
          }
          if(executedByFieldPicker){
              executedByFieldPicker.clear();
              executedByFieldPicker._setDescriptorWithValue("-1");
          }
          $("#raven-testrun-report-filter-test-contains").val("");
          $("#raven-testrun-report-filter-testexecution-contains").val("");
          $("#raven-testrun-report-filter-revision").val("");
          var fieldPicker = $('#raven-testruns-filter-saved').data('field-support-obj');
          if (fieldPicker) {
              fieldPicker.clear();
          }
        },

        clearTestPlanIconOnChange: function(){
            if($('#raven-testrun-report-filter-testplan').val() == "-1"){
                $("#raven-testrun-report-filter-testplan-single-select").removeClass("aui-ss-has-entity-icon");
            }
        },

        initSelectPickers: function(){
            testPlanPicker = this.createTestPlanSingleSelectPicker($('#raven-testrun-report-filter-testplan'));
            testRunStatusPicker = this.createStatusPicker($('#raven-testruns-report-filter-testrunstatuses'));
            fixVersionPicker = this.createSingleSelectPicker($('#raven-testruns-report-filter-fixversion'));
            testPriorityPicker = this.createSingleSelectPicker($('#raven-testruns-report-filter-testpriorities'));
            testComponentsPicker = this.createSingleSelectPicker($('#raven-testruns-report-filter-testcomponents'));
            testEnvironmentsPicker = this.createSingleSelectPicker($('#raven-testruns-report-filter-testenvironments'));
            testRunAssigneeFieldPicker = this.initAssigneeFilterPicker($("#raven-testrun-report-filter-testrunassignee"));
            executedByFieldPicker = this.initAssigneeFilterPicker($("#raven-testrun-report-filter-executedby"));
            this.createSingleFilterPickers();
        },

        toogleFilters: function(trigger) {
            $('.filter-title.selected').removeClass('selected');
            $(trigger).addClass('selected');
            if ($(trigger).data('mode') == 1) {
                this.filterMode = true;
                $('#raven-testruns-report-filters-form').show();
                $('#raven-testruns-savedfilter-form').hide();
            } else {
                this.filterMode = false;
                $('#raven-testruns-report-filters-form').hide();
                $('#raven-testruns-savedfilter-form').show();
            }
        },

        clickTitleNotSelected: function(e) {
            this.toogleFilters(e.currentTarget);
        },

        closeDialog: function() {
            if ($('#raven-testruns-filter-inline-dialog').attr('open') == "open") {
                $('#raven-testruns-filter-inline-dialog').removeAttr('open');
            } else {
                document.querySelector('#raven-testruns-filter-inline-dialog').hide();
            }
        },

        preventDefault: function(e) {
            e.stopPropagation();
            e.preventDefault();
        },

        clearMessage: function() {
            $('#message-row-wrapper').empty();
        },

        disableHowToRead: function() {
            $('.header-filter').attr('data-initialized', false);
            this.hideHowToRead();
            this.setHowToReadServer("disabled");
        },

        enableHowToRead: function() {
            $('.header-filter').attr('data-initialized', false);
            this.showHowToRead();
            this.setHowToReadServer("enabled");
        },

        showHowToRead: function() {
            $(".raven-how-to-read-chart").show("fast");
            $("#raven-notifications-icon").hide("fast");
            $("#enable-how-to-read-msg").hide("fast");
        },

        hideHowToRead: function() {
            $(".raven-how-to-read-chart").hide("fast");
            $("#raven-notifications-icon").show("fast");
            $("#enable-how-to-read-msg").show("fast");

        },


        /****************************************************************
         *                        FIELD PICKERS
         ****************************************************************/

        createTestPlanSingleSelectPicker: function($element) {
            return new AJS.SingleSelect({
                element: $element,
                showDropdownButton: true,
                submitInputVal: true,
                noQueryNoRequest: false,
                revertOnInvalid: true,
                loadOnInit: true,
                ajaxOptions: {
                    query: true,
                    url: contextPath + '/rest/raven/1.0/issue/picker?issueType=TESTPLAN',
                    formatResponse: function(data) {
                        var section = data.sections[0];

                        var defaultOptGroup = new AJS.GroupDescriptor({
                            weight: 0,
                            id: section.id + "-default-opt-group",
                            replace: true
                        });
                        var defaultItem = new AJS.ItemDescriptor({
                            value: "-1",
                            label: AJS.I18n.getText("raven-admin-config.general.label.all"),
                            allowDuplicate:false
                        });
                        defaultOptGroup.addItem(defaultItem);

                        var optgroup = new AJS.GroupDescriptor({
                            weight: 1,
                            id: section.id + "-opt-group",
                            replace: true,
                            label: section.label + "(" + section.sub + ")"
                        });
                        $(section.issues).each(function(i, issue) {
                            var item = new AJS.ItemDescriptor({
                                value: issue.key,
                                label: issue.key + ' - ' + issue.summaryText,
                                title: issue.summary,
                                fallbackIcon: "none",
                                icon: contextPath + issue.img,
                                allowDuplicate: false
                            });
                            optgroup.addItem(item);
                        });
                        return [defaultOptGroup, optgroup];
                    }
                }
            });
        },

        createStatusPicker: function($selectField) {
            return new AJS.MultiSelect({
                element: $selectField,
                showDropdownButton: true,
                revertOnInvalid: true,
            });
        },

        createSingleSelectPicker: function($selectField) {
            $selectField.on('initialized', function(e, field) {
                var version = $('body').data('version');
                if (!version || version.toString().charAt(0) === '5') {
                    $('input', $selectField.parent()).height('23px');
                }
            });
            return new AJS.SingleSelect({
                element: $selectField,
                showDropdownButton: true,
                revertOnInvalid: true,
            });
        },

        initAssigneeFilterPicker: function(e) {
            var $assineePicker = e;

            if ($assineePicker.data("aui-ss")) return;

            var restPath = "/rest/api/1.0/users/picker";

            var data = {
                    showAvatar: true
                },
                inputText = $assineePicker.data('inputValue');

            return new AJS.SingleSelect({
                element: $assineePicker,
                submitInputVal: true,
                revertOnInvalid: true,
                showDropdownButton: !!$assineePicker.data('show-dropdown-button'),
                errorMessage: AJS.I18n.getText("user.picker.invalid.user", "'{0}'"),
                ajaxOptions: {
                    url: contextPath + restPath,
                    query: true, // keep going back to the sever for each keystroke
                    data: data,
                    formatResponse: function(data) {
                        var res = [];
                        var defaultGroupDescriptor = new AJS.GroupDescriptor({
                            weight: 0 // order of groups in suggestions dropdown
                        });
                        defaultGroupDescriptor.addItem(new AJS.ItemDescriptor({
                            value: "-1",
                            label: AJS.I18n.getText("raven-admin-config.general.label.all")
                        }));
                        res.push(defaultGroupDescriptor);

                        jQuery(data).each(function(i, suggestions) {
                            var groupDescriptor = new AJS.GroupDescriptor({
                                weight: 1, // order or groups in suggestions dropdown
                                label: suggestions.footer
                            });

                            jQuery(suggestions.users).each(function(){
                                groupDescriptor.addItem(new AJS.ItemDescriptor({
                                    value: this.name, // value of item added to select
                                    label: this.displayName, // title of lozenge
                                    html: this.html,
                                    icon: this.avatarUrl,
                                    allowDuplicate: false,
                                    highlighted: true
                                }));
                            });
                            res.push(groupDescriptor);
                        });
                        return res;
                    }
                    //formatResponse: JIRA.UserPickerUtil.formatResponse
                },
                inputText: inputText
            });
        },

        createSingleFilterPickers: function(ctx) {
            var restPath = "/rest/gadget/1.0/pickers/filters";
            $(".raven-default-filter-picker", ctx).each(function() {
                var $this = $(this);
                if ($this.data("aui-ss")) return;
                var data = { showAvatar: true },
                    inputText = $this.data('inputValue');
                var field = new AJS.SingleSelect({
                    element: $this,
                    submitInputVal: true,
                    //revertOnInvalid: true,
                    showDropdownButton: !!$this.data('show-dropdown-button'),
                    errorMessage: AJS.format("filters.picker.invalid.filter", "'{0}'"),
                    ajaxOptions: {
                        url: contextPath + restPath,
                        query: true, // keep going back to the sever for each keystroke
                        data: data,
                        formatResponse: function(data) {
                            var savedFilters = [];
                            if (data.filters) {
                                $.each($(data.filters), function(index, filter) {
                                    savedFilters[index] = new AJS.ItemDescriptor({
                                        value: filter.id,
                                        label: filter.name
                                    });
                                });
                            }
                            return savedFilters;
                        }
                    },
                    inputText: inputText
                });
                $this.data('field-support-obj', field);
            });
        }

    });


})(AJS.$);