require(['aui/inline-dialog2']);
(function($) {

    /*RAVEN.namespace("RAVEN.Report");*/

    var filterValues = {
        "savedFilter": null,
        "fixVersion": null,
        "includePreviousVersions": null,
        "statusId": null,
        "resolution": null,
        "assignee": null,
        "contains": null
    }

    /*we are sending status ids instead of status name (because of translations), so we'll keep
    * this object to map ids with names*/
    var statusesObject = {};

    /**
     * This Model defines the column layout for the given user
     */
    var traceabilityModel = Backbone.Model.extend({
        version: null,
        sprint: null,
        testPlan: null,
        projectId: 1,
        environment: -1
    });

    var Requirements = Backbone.Collection.extend({
        model: traceabilityModel,
        url: function() {
            return contextPath + '/rest/raven/1.0/report/traceability/requirements';
        }
    });

    RAVEN.Report.TraceabilityReportView = Backbone.View.extend({
        el: "#report-wrapper",
        filterView: null,
        render: function(requiresExperimentalDialog2) {

            var $htmlReport = RAVEN.Report.renderTraceability({ 'requiresExperimentalDialog2': requiresExperimentalDialog2 });

            this.$el.empty();
            this.$el.html($htmlReport);

            this.filterView = new RAVEN.Report.TraceabilityReportFiltersView();
        },

        destroy: function() {
            if (this.filterView) {
                this.filterView.destroy();
                this.filterView = null;
            }
            $('#show-tooltip').remove();
            $('#load-all-warning').remove();
        }
    });

    var Filters = Backbone.Model.extend({
        urlRoot: function() {
            return contextPath + '/rest/raven/1.0/report/traceability/filterValues';
        }
    });

    RAVEN.Report.TraceabilityReportFiltersView = Backbone.View.extend({
        el: "#filter-content-place-holder",
        projectId: null,
        requirementList: null,

        render: function() {
            var self = this;
            var filters = new Filters();
            var params = $.param({ projectId: this.projectId });
            filters.fetch({
                data: params,
                success: function(data, textStatus, jqXHR) {

                    if (data && data.attributes) {
                        var savedFilter = null;

                        if (JSON.parse(data.attributes.prefsJson).savedFilter != null) {
                            savedFilter = JSON.parse(data.attributes.prefsJson).savedFilter;
                        }
                        var $htmlFilters = RAVEN.Report.renderFilterCoverage({
                            'versions': data.attributes.versions,
                            'testEnvironments': data.attributes.testEnvironments,
                            'statuses': data.attributes.statuses,
                            'resolutions': data.attributes.resolutions,
                            'requiresExperimentalDialog2': data.attributes.requiresExperimentalDialog2,
                            'userPreferences': JSON.parse(data.attributes.prefsJson),
                            'assigneeUserPref': JSON.parse(data.attributes.prefsJson).assignee,
                            'savedFilter': savedFilter,
                            'contextPath': contextPath,
                            'hideTestRuns': JSON.parse(data.attributes.prefsJson).hideTestRuns
                        });

                        statusesObject = data.attributes.statuses;

                        self.$el.empty();
                        self.$el.html($htmlFilters);

                        self.requirementList = new RAVEN.Report.RequirementView();
                    }
                },
                error: function(model, response) {
                    AJS.messages.error($("#message-row-wrapper"), {
                        title: AJS.I18n.getText('execute-test-action.requirement-status.fetch-error'),
                        body: response.statusText,
                        closeable: false
                    });
                }

            });
        },

        destroy: function() {
            if (this.requirementList) {
                this.requirementList.destroy();
                this.requirementList = null;
            }

            $('#raven-filter-inline-dialog').remove();
        },

        initialize: function() {
            this.projectId = $("#selected-project-id").val();
            this.render();
        }
    });

    RAVEN.Report.RequirementView = Backbone.View.extend({
        el: "#tests-row-wrapper",

        version: null,
        projectId: null,
        execScope: null,
        testPlan: null,
        environment: null,
        versionPicker: null,
        hideTestRunsValue: null,
        start: 0,
        pageSize: 20,
        prevValues: null,

        FILTERS_MODE: 1,
        SAVED_FILTER_MODE: 2,

        events: {},

        /**
         * Unscoped Events - these events must be registered elsewhere as they will not be
         * fired by backbone because they are not within the view element. Refer to "_delegateUnscopedEvents".
         */
        unscopedEvents: {
            'submit.raven.report.traceability #raven-requirement-filters-form': 'resetFilterValues',
            'click.raven.report.traceability #raven-dialog-clear': 'resetFilterValues',
            'submit.raven.report.traceability #raven-requirement-savedfilter-form': 'filtersSubmit',
            'click.raven.report.traceability #raven-traceability-dialog-apply': 'filtersSubmit',
            'change.raven.report.traceability #status-filter': 'dialogApply',
            'click.raven.report.traceability #raven-traceability-dialog-cancel': 'closeDialog',
            'click.raven.report.traceability #raven-requirement-filter-fixversion-suggestions': 'preventDefault',
            'click.raven.report.traceability #raven-requirement-filter-status-suggestions': 'preventDefault',
            'click.raven.report.traceability #raven-requirement-filter-resolution-suggestions': 'preventDefault',
            'click.raven.report.traceability #raven-requirement-traceability-filter-assignee-suggestions': 'preventDefault',
            'click.raven.report.traceability #raven-requirement-filter-saved-suggestions': 'preventDefault',
            'click.raven.report.traceability #raven-requirement-filter-component-suggestions': 'preventDefault',
            'submit.raven.report.traceability #raven-requirement-coverage-form': 'preventDefault',
            'click.raven.report.traceability .item-delete': 'preventDefault',
            'click.raven.report.traceability .filter-title': 'clickTitleNotSelected',
            'click.raven.report.traceability #hide_testruns': 'hideTestRuns',
            'change.raven.report.traceability #report-entries-table_length': 'lengthChange',
            'click.raven.report.traceability #dismiss-how-to-read-msg': 'disableHowToRead',
            'click.raven.report.traceability #enable-how-to-read-msg': 'enableHowToRead',
            'click.raven.report.traceability #load-more': 'loadMore',
            'click.raven.report.traceability #load-all-bt': 'loadAll',
            'submit.raven.report.traceability #raven-testplans-coverage-form': 'preventDefault',
            'change.raven.report.traceability #raven-requirement-traceability-filter-assignee' : 'removeAssigneeIcon'
        },
        loadAll: function() {
            this.prevValues = null;
            this.pageSize = -1;
            this.render();
        },

        loadMore: function() {
            //load next 20 items
            this.start = this.start + 20;
            if (this.total > this.pageSize) {
                $("#load-more").addClass("lastRowSpin");
                this.render();
            }
        },
        setPaginationInfo: function(dataModelsLength) {
            var entriesMsg = "";
            if (dataModelsLength == -1) {
                $("#load-more").css("display", "none");
                $(".load-all-entries").css("display", "none");
                entriesMsg = AJS.I18n.getText('common.table.label.no-entries-to-show');
            } else {
                var totalShowing = this.start + dataModelsLength;
                if (totalShowing >= this.total || this.total == 0) {
                    $("#load-more").css("display", "none");
                } else {
                    $("#load-more").css("display", "block");
                }
                if (totalShowing >= this.total) {
                    totalShowing = this.total;
                    $(".load-all-entries").css("display", "none");
                } else {
                    $(".load-all-entries").css("display", "block");
                }
                entriesMsg = AJS.I18n.getText('common.label.showing-records', totalShowing, this.total);
            }
            $(".info-entries").empty();
            $(".info-entries").append(entriesMsg);
        },
        render: function() {
            $('#report-wrapper').block();
            $('.raven-requirement-header').block();
            if ($('.lastRowSpin').length) {
                $(".lastRowSpin").spin("medium");
                $("#load-more").css("color", "#eee");
            } else {
                $('#raven-requirement-content-panel').spin("large");
            }

            this.clearMessage();

            var self = this;
            var $tableWrapper = self.$el;

            $('#raven-requirement-content-panel').css("border-left", "none");
            $('#raven-requirement-content-panel').css("border-right", "none");

            var requirements = new Requirements();
            var params = $.param(this.createFilterParams());
            params = $.param({ version: this.version, projectId: this.projectId, execScope: this.execScope, testPlan: this.testPlan, environment: this.environment, pageSize: this.pageSize, start: this.start, hideTestRuns: this.hideTestRunsValue }) + "&" + params;
            requirements.fetch({
                data: params,
                success: function(data, textStatus, jqXHR) {
                    self.checkShowHowToRead();

                    if (data.models.length > 0) {

                        self.total = data.models[0].attributes.total;
                        self.pageSize = data.models[0].attributes.pageSize;

                        if (self.prevValues) {
                            $.merge(self.prevValues, data.models);
                            var $table = RAVEN.Report.renderRow({ 'issues': data.models, 'contextPath': contextPath });

                            var position = $(".requirement-row:last").offset().top - 80;
                            $tableWrapper.append($table);
                            $("html, body").animate({ scrollTop: position }, 1000);
                        } else {
                            $tableWrapper.empty();
                            self.prevValues = data.models;

                            var $table = RAVEN.Report.renderTable({ 'issues': self.prevValues, 'contextPath': contextPath });
                            $tableWrapper.html($table);

                        }
                        self.setPaginationInfo(data.models.length);
                        self.stopLoading();

                        if ($("#hide_testruns")) {
                            $("#hide_testruns").prop("checked", self.hideTestRunsValue);
                            self.hideTestRuns();
                        }
                        $('#tests-row-wrapper').css("cssText", "border-top-width: 1px !important;");
                        $('#raven-requirement-content-panel').css("border-left", "solid 1px lightgray");
                        $('#raven-requirement-content-panel').css("border-right", "solid 1px lightgray");
                    } else if (!self.prevValues || self.prevValues.length == 0) {
                        $('#tests-row-wrapper').css("cssText", "border-top-width: 0px !important;");
                        if ($tableWrapper) {
                            $tableWrapper.empty();

                            self.setPaginationInfo(-1);
                            self.stopLoading();
                        }
                        AJS.messages.generic($("#message-row-wrapper"), {
                            body: AJS.I18n.getText('projectpanels.requirement-traceability.resultset.empty'),
                            closeable: false
                        });
                    }
                },
                error: function(model, response) {
                    $('#tests-row-wrapper').css("cssText", "border-top-width: 0px !important;");
                    if ($tableWrapper) {
                        $tableWrapper.empty();

                        self.setPaginationInfo(-1);
                        self.stopLoading();
                    }
                    AJS.messages.error($("#message-row-wrapper"), {
                        title: AJS.I18n.getText('execute-test-action.requirement-status.fetch-error'),
                        body: response.statusText + ": " + response.responseText,
                        closeable: false
                    });
                    self.stopLoading();
                }

            });
        },

        stopLoading: function() {
            $('#raven-requirement-content-panel').spinStop();
            $('#report-wrapper').unblock();
            $('.raven-requirement-header').unblock();
            $('.raven-requirement-header').spinStop();
            $(".lastRowSpin").unblock();

            $(".lastRowSpin").spinStop();
            $(".lastRowSpin").removeClass("lastRowSpin");
            $("#load-more").css("color", "#999");
        },
        initialize: function() {

            // bind to window
            var self = this;

            $(".load-all-entries").css("display", "block");

            this.initSelectPickers();
            // init filter values and render filter summary
            this.initFilterValues();

            this.renderFilterSummary();

            this._delegateUnscopedEvents();

            this.hideTestRunsValue = $("#hideTestRuns").val() == "true" || $("#hideTestRuns").val() == true;

            if (this.versionPicker.getSelectedItem()) {
                this.version = this.versionPicker.getSelectedItem().id;
            }

            this.testPlan = $('#raven-report-analysis-test-plan-select').val();

            self.execScope = $('#raven-report-requirement-coverage-parameters').find("#report-exec-scope-picker").val()[0];

            this.projectId = $("#selected-project-id").val();

            if (this.environmentPicker.getSelectedItem()) {
                this.environment = this.environmentPicker.getSelectedItem().id;
            }

            this.toogleExecScopeValuePicker(false);

            if (filterValues && filterValues.savedFilter) {
                this.toogleFilters($('.filter-title[data-mode=2]'));
            }

            this.render();
        },

        /****************************************************************
         *                        EVENT HANDLERS
         ****************************************************************/

        _delegateUnscopedEvents: function() {
            var self = this;
            $(document).off('.raven.report.traceability');

            $.each(this.unscopedEvents, function(key, val) {
                var eventName = key.substring(0, key.indexOf(' '));
                var eventTrigger = key.substring(key.indexOf(' ') + 1);

                $(document).on(eventName, eventTrigger, $.proxy(self[val], self));
            });
        },
        _undelegateUnscopedEvents: function() {
            $(document).off('.raven.report.traceability');
        },
        createFilterParams: function() {
            var filters = {};

            if ($('.filter-title.selected').data('mode') == this.SAVED_FILTER_MODE) {
                var savedFilter = filterValues.savedFilter;
                if (savedFilter) {
                    filters["savedFilter"] = savedFilter;
                }
            } else {
                var fixVersion = filterValues.fixVersion;
                if (fixVersion && fixVersion != -1) {
                    filters["fixVersion"] = fixVersion;
                    filters["previousVersions"] = filterValues.includePreviousVersions;
                }
                var status = filterValues.statusId;
                if (status) {
                    var len = status.length;
                    filters["statusId"] = "";
                    $.each(status, function(index) {
                        filters["statusId"] = filters["statusId"] + this;
                        if (index < len - 1) {
                            filters["statusId"] = filters["statusId"] + ",";
                        }
                    });
                }
                var resolution = filterValues.resolution;
                if (resolution && resolution != -1) {
                    filters["resolution"] = resolution;
                }
                var assignee = filterValues.assignee;
                if (assignee && assignee != -1) {
                    filters["assignee"] = assignee;
                }
                var contains = filterValues.contains;
                if (contains) {
                    filters["contains"] = contains;
                }

                var components = filterValues.components;
                if (components) {
                    filters["components"] = components;
                }
            }

            return filters;
        },

        initSelectPickers: function() {
            // Parameter pickers
            var self = this;
            $('#raven-requirement-coverage-form').on("submit", function(e) {
                e.preventDefault();
            });
            
            this.execScopeFieldPicker = this.renderExecScopePicker();

            this.versionPicker = this.createSubtleSingleSelect($('#raven-report-analysis-version-select'));

            $(this.versionPicker).on('selectedItemChanged', function(e) {
                self.version = e.currentTarget.selectedItem.id;
                self.prevValues = null;
                self.render();
            });

            this.testPlanPicker = this.createTestPlanSingleSelectPicker($('#raven-report-analysis-test-plan-select'));
            $('#raven-report-analysis-test-plan-select').on("change", function(e, field) {
                self.testPlan = $('#raven-report-analysis-test-plan-select').val();
                self.prevValues = null;
                self.render();
            });

            $("#raven-report-analysis-test-plan-select-field").attr("placeholder", AJS.I18n.getText("view-issue-requirement-test-coverage-section.placeholer.strategy.test-plan"));

            this.environmentPicker = this.createSubtleSingleSelect($('#raven-report-analysis-environment-select'));
            $(this.environmentPicker).on('selectedItemChanged', function(e) {
                self.environment = e.currentTarget.selectedItem.id;
                self.prevValues = null;
                self.render();
            });

            groupingFieldPicker = this.createSubtleSelect($('#raven-grouping-field-select'));
            $(document).on('change.grouping-field.raven', '#raven-grouping-field-select-select', function(e) {
                $('#raven-grouping-field-select-single-select input').removeClass('option-hint');
                $('#raven-grouping-field-select option[value="-1"]').remove();
                self.refreshCoverageCharts();
            });

            this.execScope = $('#raven-report-requirement-coverage-parameters').find("#report-exec-scope-picker").val()[0];
            this.toogleExecScopeValuePicker(false);


            // Filter pickers
            fixVersionFieldPicker = this.createSingleSelectPicker($('#raven-filter-inline-dialog #raven-requirement-filter-fixversion'));
            statusFieldPicker = this.createStatusPicker($('#raven-filter-inline-dialog #raven-requirement-filter-status'));
            resolutionFieldPicker = this.createSingleSelectPicker($('#raven-filter-inline-dialog #raven-requirement-filter-resolution'));
            assigneeFieldPicker = this.initAssigneeFilterPicker($('#raven-requirement-traceability-filter-assignee'));
            this.createSingleFilterPickers();
        },

        toogleExecScopeValuePicker: function(refresh) {
            this.prevValues = null;
            switch (this.execScope) {
                case "version":
                    this.versionPicker.$container.show();
                    this.testPlanPicker.$container.hide();
                    break;
                case "testPlan":
                    this.versionPicker.$container.hide();
                    this.testPlanPicker.$container.show();
                    break;
            }

            if (refresh === true && (this.execScope == "version" || (this.execScope == "testPlan" && this.testPlan != null && this.testPlan.length > 0))) {
                this.render();
            }
        },

        toogleFilters: function(trigger) {
            $('.filter-title.selected').removeClass('selected');

            $(trigger).addClass('selected');

            if ($(trigger).data('mode') == this.SAVED_FILTER_MODE) {
                $('#raven-requirement-filters-form').hide();
                $('#raven-requirement-savedfilter-form').show();
            } else {
                $('#raven-requirement-filters-form').show();
                $('#raven-requirement-savedfilter-form').hide();
            }
        },

        checkShowHowToRead: function() {
            var self = this;
            $.ajax({
                type: "GET",
                url: contextPath + "/rest/raven/1.0/report/traceability/isHowToRead",

                success: function(data, textStatus, jqXHR) {
                    if (data) {
                        self.showHowToRead();
                    } else {
                        self.hideHowToRead();
                    }
                },
                error: function(qXHR, textStatus, errorThrown) {
                    console.log(qXHR);
                }
            });
        },
        setHowToReadServer: function(value) {
            $.ajax({
                type: "POST",
                data: value,
                url: contextPath + "/rest/raven/1.0/report/traceability/setHowToRead",
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    console.log(qXHR);
                }
            });
        },

        disableHowToRead: function() {
            $('.header-filter').attr('data-initialized', false);
            this.hideHowToRead();
            this.setHowToReadServer("disabled");
        },
        enableHowToRead: function() {
            $('.header-filter').attr('data-initialized', false);
            this.showHowToRead();
            this.setHowToReadServer("enabled");
        },

        showHowToRead: function() {
            $(".raven-how-to-read-chart").show("fast");
            $("#raven-notifications-icon").hide("fast");
            $("#enable-how-to-read-msg").hide("fast");
        },
        hideHowToRead: function() {
            $(".raven-how-to-read-chart").hide("fast");
            $("#raven-notifications-icon").show("fast");
            $("#enable-how-to-read-msg").show("fast");

        },
        createSubtleSelect: function($selectField) {
            return new RAVEN.SubtleSelect({
                element: $selectField,
                showDropdownButton: false,
                revertOnInvalid: true,
            });
        },

        includePreviousVersions: function() {
            return $('#include-previous-versions-option').is(':checked');
        },
        destroy: function() {
            $("#show-tooltip").remove();
            $('#load-all-warning').remove();
            // COMPLETELY UNBIND THE VIEW
            this.undelegateEvents();

            this._undelegateUnscopedEvents();
            this.remove();
            this.unbind();
        },
        createSubtleSingleSelect: function($selectField) {
            var items = _.map($selectField.find('option'), function(item) {
                return {
                    'id': item.value,
                    'name': item.text,
                };
            });

            var selectedItem = _.first(_.map($selectField.find('option:selected'), function(item) {
                return {
                    'id': item.value,
                    'name': item.text,
                };
            }));

            var subtlePicker = new RAVEN.Field.SubtleSingleSelect(items, selectedItem, $selectField.closest('#raven-subtle-container'));
            subtlePicker.render();

            // remove select element. This is only for retrieving the initial items.
            $selectField.remove();

            return subtlePicker;
        },

        createTestPlanSingleSelectPicker: function($element) {
            return new AJS.SingleSelect({
                element: $element,
                showDropdownButton: true,
                submitInputVal: true,
                noQueryNoRequest: false,
                revertOnInvalid: true,
                loadOnInit: true,
                ajaxOptions: {
                    query: true,
                    url: contextPath + '/rest/raven/1.0/issue/picker?issueType=TESTPLAN',
                    formatResponse: function(data) {
                        var section = data.sections[0];
                        var optgroup = new AJS.GroupDescriptor({
                            weight: 0,
                            id: section.id + "-opt-group",
                            replace: true,
                            label: section.label + "(" + section.sub + ")"
                        });

                        $(section.issues).each(function(i, issue) {
                            var item = new AJS.ItemDescriptor({
                                value: issue.key,
                                label: issue.key + ' - ' + issue.summaryText,
                                title: issue.summary,
                                icon: contextPath + issue.img,
                                allowDuplicate: false,
                                //selected: issue.key === strategyValue[0] ? 'selected' : null
                            });
                            optgroup.addItem(item);
                        });
                        return [optgroup];
                    }
                }
            });
        },

        lengthChange: function(e) {
            this.pageSize = e.target.value;
            this.render();
        },

        renderExecScopePicker: function() {
            var self = this;
            if ($("#report-exec-scope-picker-field").html() === null) {
                var $element = $('#raven-report-requirement-coverage-parameters').find("#report-exec-scope-picker");
                $element.on("change", function(e, field) {
                    self.execScope = $('#raven-report-requirement-coverage-parameters').find("#report-exec-scope-picker").val()[0];

                    self.toogleExecScopeValuePicker(true);
                });

                return new RAVEN.SubtleSelect({
                    element: $element,
                    showDropdownButton: false,
                    revertOnInvalid: true,
                });
            }

            return null;
        },

        initFilterValues: function() {
            filterValues.savedFilter = $('#raven-filter-inline-dialog #raven-requirement-filter-saved').val();
            filterValues.fixVersion = $('#raven-filter-inline-dialog #raven-requirement-filter-fixversion').val();
            filterValues.includePreviousVersions = this.includePreviousVersions();
            var statusesIds = [];
            var arr = $('#raven-filter-inline-dialog #raven-requirement-filter-status').val();
            if(arr != null){
                for(var i = 0; i < arr.length; i++){
                    for(var j =0; j < statusesObject.length; j++ ){
                        if(arr[i] == statusesObject[j].name){
                            statusesIds.push(statusesObject[j].id);
                        }
                    }
                }
            }
            filterValues.statusId = (statusesIds.length > 0)?statusesIds:null;
            filterValues.resolution = $('#raven-filter-inline-dialog #raven-requirement-filter-resolution').val();
            filterValues.assignee = $('#raven-filter-inline-dialog #raven-requirement-traceability-filter-assignee').val();
            filterValues.contains = $('#raven-filter-inline-dialog #raven-requirement-filter-contains').val();
            filterValues.components = $('#raven-filter-inline-dialog #raven-requirement-filter-component').val();
        },

        saveFilterValues: function() {
            if ($('.filter-title.selected').data('mode') == this.SAVED_FILTER_MODE) {
                filterValues.savedFilter = $('#raven-filter-inline-dialog #raven-requirement-filter-saved').val();
                filterValues.fixVersion = null;
                filterValues.includePreviousVersions = null;
                filterValues.statusId = null;
                filterValues.resolution = null;
                filterValues.assignee = null;
                filterValues.contains = null;
                filterValues.components = null;
            } else {
                var statusesIds = [];
                var arr = $('#raven-filter-inline-dialog #raven-requirement-filter-status').val();
                if(arr != null){
                    for(var i = 0; i < arr.length; i++){
                        for(var j =0; j < statusesObject.length; j++ ){
                            if(arr[i] == statusesObject[j].name){

                                statusesIds.push(statusesObject[j].id);

                            }
                        }
                    }
                }
                filterValues.savedFilter = null;
                filterValues.fixVersion = $('#raven-filter-inline-dialog #raven-requirement-filter-fixversion').val();
                filterValues.includePreviousVersions = this.includePreviousVersions();
                filterValues.statusId = (statusesIds.length > 0)?statusesIds:null;
                filterValues.resolution = $('#raven-filter-inline-dialog #raven-requirement-filter-resolution').val();
                filterValues.assignee = $('#raven-filter-inline-dialog #raven-requirement-traceability-filter-assignee').val();
                filterValues.contains = $('#raven-filter-inline-dialog #raven-requirement-filter-contains').val();
                filterValues.components = $('#raven-filter-inline-dialog #raven-requirement-filter-component').val();
            }
        },

        removeAssigneeIcon: function(){
            if (!assigneeFieldPicker._hasIcon()){
                if(assigneeFieldPicker.$entityIcon !== undefined){
                    assigneeFieldPicker.$entityIcon.remove();
                }
                assigneeFieldPicker.$container.removeClass("aui-ss-has-entity-icon");
            }
        },

        resetFilterValues: function() {
            if (fixVersionFieldPicker) {
                fixVersionFieldPicker._setDescriptorWithValue("-1");
            }
            if (statusFieldPicker) {
                statusFieldPicker.clearLozenges();
            }
            if (resolutionFieldPicker) {
                resolutionFieldPicker._setDescriptorWithValue("-1");
            }
            if (assigneeFieldPicker) {
                assigneeFieldPicker.clear();
                assigneeFieldPicker._setDescriptorWithValue("-1");
            }

            $('#raven-filter-inline-dialog #raven-requirement-filter-contains').val("");

            var fieldPicker = $('#raven-requirement-filter-saved').data('field-support-obj');
            if (fieldPicker) {
                fieldPicker.clear();
            }
        },

        renderFilterSummary: function() {
            var summary = '';
            if (filterValues.savedFilter && filterValues.savedFilter != "-1") {
                summary = '<b>' + AJS.I18n.getText('projectpanels.requirement-coverage.label.saved-filter') + '</b>  ' + $('#raven-filter-inline-dialog #raven-requirement-filter-saved option[value="' + filterValues.savedFilter + '"]').text();
            } else {
                if (filterValues.fixVersion && filterValues.fixVersion != "-1") {
                    var previousVersions = filterValues.includePreviousVersions;
                    summary += '<b>' + AJS.I18n.getText('projectpanels.requirement-coverage.label.fix-version') + '</b>  ';
                    if (previousVersions === true) {
                        summary += '<= ';
                    }
                    summary += $('#raven-filter-inline-dialog #raven-requirement-filter-fixversion option[value="' + filterValues.fixVersion + '"]').text();
                }
                if (filterValues.statusId) {
                    if (summary) {
                        summary += '; ';
                    }
                    var statusesNames = "";
                    for(var i = 0; i < filterValues.statusId.length; i++){
                        for(var j = 0; j < statusesObject.length; j++){
                            if(filterValues.statusId[i] == statusesObject[j].id){
                                if(i > 0 && i < filterValues.statusId.length){
                                    statusesNames = statusesNames + ",";
                                }
                                statusesNames = statusesNames + statusesObject[j].name;
                            }
                        }
                    }
                    summary += '<b>' + AJS.I18n.getText('projectpanels.requirement-coverage.label.status') + '</b>  ' + statusesNames;
                }
                if (filterValues.resolution && filterValues.resolution != "-1") {
                    if (summary) {
                        summary += '; ';
                    }
                    summary += '<b>' + AJS.I18n.getText('projectpanels.requirement-coverage.label.resolution') + '</b>  ' + $('#raven-filter-inline-dialog #raven-requirement-filter-resolution option[value="' + filterValues.resolution + '"]').text();
                }
                if (filterValues.assignee && filterValues.assignee != "-1") {
                    if (summary) {
                        summary += '; ';
                    }
                    summary += '<b>' + AJS.I18n.getText('projectpanels.requirement-coverage.label.assignee') + '</b>  ' + $('#raven-filter-inline-dialog #raven-requirement-traceability-filter-assignee option[value="' + filterValues.assignee + '"]').text();
                }
                if (filterValues.contains) {
                    if (summary) {
                        summary += '; ';
                    }
                    summary += '<b>' + AJS.I18n.getText('projectpanels.requirement-coverage.label.contains') + '</b>  ' + filterValues.contains;
                }
                if (filterValues.components) {
                    if (summary) {
                        summary += '; ';
                    }
                    summary += '<b>' + AJS.I18n.getText('projectpanels.requirement-coverage.label.component') + '</b>  ' + filterValues.components;
                }
            }

            if (summary) {
                $('#raven-filter-trigger').html(summary);
            } else {
                $('#raven-filter-trigger').text(AJS.I18n.getText('projectpanels.requirement-coverage.label.filter-button'));
            }
        },

        createSingleSelectPicker: function($selectField) {
            $selectField.on('initialized', function(e, field) {
                var version = $('body').data('version');
                if (!version || version.toString().charAt(0) === '5') {
                    $('input', $selectField.parent()).height('23px');
                }
            });

            return new AJS.SingleSelect({
                element: $selectField,
                showDropdownButton: true,
                revertOnInvalid: true,
            });
        },

        initAssigneeFilterPicker: function(e) {
            var $assineePicker = e;

            if ($assineePicker.data("aui-ss")) return;

            var restPath = "/rest/api/1.0/users/picker";

            var data = {
                    showAvatar: true
                },
                inputText = $assineePicker.data('inputValue');

            return new AJS.SingleSelect({
                element: $assineePicker,
                submitInputVal: true,
                revertOnInvalid: true,
                showDropdownButton: !!$assineePicker.data('show-dropdown-button'),
                errorMessage: AJS.I18n.getText("user.picker.invalid.user", "'{0}'"),
                ajaxOptions: {
                    url: contextPath + restPath,
                    query: true, // keep going back to the sever for each keystroke
                    data: data,
                    formatResponse: function(data) {
                        var res = [];
                        var defaultGroupDescriptor = new AJS.GroupDescriptor({
                            weight: 0 // order of groups in suggestions dropdown
                        });
                        defaultGroupDescriptor.addItem(new AJS.ItemDescriptor({
                            value: "-1",
                            label: AJS.I18n.getText("raven-admin-config.general.label.all")
                        }));
                        res.push(defaultGroupDescriptor);

                        jQuery(data).each(function(i, suggestions) {
                            var groupDescriptor = new AJS.GroupDescriptor({
                                weight: 1, // order or groups in suggestions dropdown
                                label: suggestions.footer
                            });

                            jQuery(suggestions.users).each(function(){
                                groupDescriptor.addItem(new AJS.ItemDescriptor({
                                    value: this.name, // value of item added to select
                                    label: this.displayName, // title of lozenge
                                    html: this.html,
                                    icon: this.avatarUrl,
                                    allowDuplicate: false,
                                    highlighted: true
                                }));
                            });
                            res.push(groupDescriptor);
                        });
                        return res;
                    }
                },
                inputText: inputText
            });
        },

        getProjectKey: function() {
            return $('#raven-project-key').val();
        },

        createStatusPicker: function($selectField) {
            return new AJS.MultiSelect({
                element: $selectField,
                showDropdownButton: true,
                revertOnInvalid: true,
            });
        },

        filtersSubmit: function() {
            this.prevValues = null;
            this.start = 0;
            this.pageSize = 20;
            this.saveFilterValues();
            this.render();

            this.renderFilterSummary();

            if ($('#raven-filter-inline-dialog').attr('open') == "open") {
                $('#raven-filter-inline-dialog').removeAttr('open');
            } else {
                document.querySelector('#raven-filter-inline-dialog').hide();
            }

            return false;
        },

        dialogApply: function() {
            this.saveFilterValues();
            this.render();
            this.renderFilterSummary();

            if ($('#raven-filter-inline-dialog').attr('open') == "open") {
                $('#raven-filter-inline-dialog').removeAttr('open');
            } else {
                document.querySelector('#raven-filter-inline-dialog').hide();
            }
        },

        closeDialog: function() {
            // Cancel button inline Dialog
            if ($('#raven-filter-inline-dialog').attr('open') == "open") {
                $('#raven-filter-inline-dialog').removeAttr('open');
            } else {
                document.querySelector('#raven-filter-inline-dialog').hide();
            }
        },

        preventDefault: function(e) {
            console.log("preventDefault");
            e.stopPropagation();
            e.preventDefault();
        },

        clickTitleNotSelected: function(e) {
            this.toogleFilters(e.currentTarget);
            e.stopPropagation();
        },

        hideTestRuns: function() {
            if ($('#hide_testruns').is(":checked")) {
                $(".hideable").css("display", "none");
                $(".four-columns").addClass("three-columns");
                $(".four-columns").removeClass("four-columns");
                $(".test-column-four-columns").addClass("test-column-three-columns");
                $(".test-column-four-columns").removeClass("test-column-four-columns");

                this.hideTestRunsValue = true;
            } else {
                $(".hideable").css("display", "block");
                $(".three-columns").addClass("four-columns");
                $(".three-columns").removeClass("three-columns");
                $(".test-column-three-columns").addClass("test-column-four-columns");
                $(".test-column-three-columns").removeClass("test-column-three-columns");

                this.hideTestRunsValue = false;
            }
        },

        createSingleFilterPickers: function(ctx) {

            var restPath = "/rest/gadget/1.0/pickers/filters";

            $(".raven-default-filter-picker", ctx).each(function() {
                var $this = $(this);
                if ($this.data("aui-ss")) return;
                var data = { showAvatar: true },
                    inputText = $this.data('inputValue');

                var field = new AJS.SingleSelect({
                    element: $this,
                    submitInputVal: true,
                    //revertOnInvalid: true,
                    showDropdownButton: !!$this.data('show-dropdown-button'),
                    errorMessage: AJS.format("filters.picker.invalid.filter", "'{0}'"),
                    ajaxOptions: {
                        url: contextPath + restPath,
                        query: true, // keep going back to the sever for each keystroke
                        data: data,
                        formatResponse: function(data) {
                            var savedFilters = [];
                            if (data.filters) {
                                $.each($(data.filters), function(index, filter) {
                                    savedFilters[index] = new AJS.ItemDescriptor({
                                        value: filter.id,
                                        label: filter.name
                                    });
                                });
                            }
                            return savedFilters;
                        }
                    },
                    inputText: inputText
                });

                $this.data('field-support-obj', field);
            });
        },

        clearMessage: function() {
            $('#message-row-wrapper').empty();
        }

    });
})(AJS.$);