(function($) {
    RAVEN.namespace("RAVEN.Folder");

    RAVEN.Folder.FolderModel = Backbone.Model.extend({
        repositoryPath: null,

        entityKey: null,

        initialize: function(options) {
            if (options) {
                _.extend(this, _.pick(options, "repositoryPath"));
                _.extend(this, _.pick(options, "entityKey"));
            }
        },

        urlRoot: function() {
            return contextPath + '/rest/raven/1.0/' + this.repositoryPath +'/rootFolder?entityKey=' + this.entityKey;
        },

        defaults: function() {
            return [];
        }
    });

    /**
     * @PublicEvent folderInitializedEvt
     * @PublicEvent messageEvt
     * @PublicEvent blockElementEvt
     * @PublicEvent unblockElementEvt
     * @PublicEvent folderSelectEvt
     * @PublicEvent folderActiveEvt
     * @PublicEvent folderCreateEvt
     * @PublicEvent folderCollapseEvt
     * @PublicEvent folderEditEvt
     */
    RAVEN.Folder.FolderView = Backbone.View.extend({
        el: null,

        ALL_TESTS: 'all-tests',

        ALL_ORPHAN_TESTS: 'all-orphan-tests',

        FOLDER_ROOT: 'folder-root',

        TEMPORARY: 'temporary',

        COLLAPSE: 'collapse',

        EXPAND: 'expand',

        COLLAPSE_ALL: 'collapse_all',

        EXPAND_ALL: 'expand_all',

        ACTIVATE: 'activate',

        repositoryPath: null,

        entityKey: null,

        hasAllFolder: null,

        hasOrphansFolder: null,

        rootDescription: null,

        isEditable: null,

        hasFolderActions: null,

        events: {
            'click.raven.test-repository .raven-folder-create': 'createFolder',
            'click.raven.test-repository .raven-folder-edit': '_editFolder',
            'dblclick.raven.test-repository .raven-folder-name': '_editFolder',
            'click.raven.test-repository .raven-folder-delete': '_deleteFolder',

            'click.raven.test-repository .raven-folders-view-mode': 'folderItemClick',
            'click.raven.test-repository .raven-global-folder': 'globalFolderItemClick',

            'keyup.raven.test-repository .raven-folder-name-input': '_keyupNameChange',
            'blur.raven.test-repository .raven-folder-name-input': '_blurNameChange'
        },

        initialize: function(options) {
            if (options) {
                _.extend(this, _.pick(options, "el"));
                _.extend(this, _.pick(options, "repositoryPath"));
                _.extend(this, _.pick(options, "entityKey"));
                _.extend(this, _.pick(options, "hasAllFolder"));
                _.extend(this, _.pick(options, "hasOrphansFolder"));
                _.extend(this, _.pick(options, "rootDescription"));
                _.extend(this, _.pick(options, "isEditable"));
                _.extend(this, _.pick(options, "hasFolderActions"));
            }

            this.model = new RAVEN.Folder.FolderModel({
                repositoryPath: this.repositoryPath,
                entityKey: this.entityKey
            });

            var self = this;
            this.model.fetch({
                success: function() {
                    self.render();
                },
                error: function(model, response) {
                    self.$el.trigger('messageEvt', ['Error', AJS.I18n.getText('raven.test-repository.error.fetch-root'), 'error']);
                }
            });
        },

        render: function() {
            var content = RAVEN.Folder.renderFolderView({
                'children': this.model.attributes.children,
                'allTestsCount': this.model.attributes.allTestsCount,
                'allOrphanTestsCount': this.model.attributes.allOrphanTestsCount,
                'hasAllFolder': this.hasAllFolder,
                'hasOrphansFolder': this.hasOrphansFolder,
                'rootDescription': this.rootDescription,
                'isEditable':  this.isEditable,
                'hasFolderActions': this.hasFolderActions
            });

            this.$el.html(content);

            if (!this.isEditable) {
                this._disableEditFolder();
            }

            this._initTreeDotLineStructure();

            this.$el.trigger('folderInitializedEvt');
        },

        destroy: function() {
            this.$el.removeData().unbind();
        },

        _removeActiveFolderStyle: function() {
            // remove active style
            this.$el.find('.raven-folder-global-name.active').removeClass('active');
            this.$el.find('.raven-folder-global-number-of-tests.active').removeClass('active');
            this.$el.find('.raven-folder-name.active').removeClass('active');
            this.$el.find('.raven-folder-number-of-tests.active').removeClass('active');
        },

        addActiveFolderStyle: function($elem) {
            // add active style
            $elem.find('.raven-folder-name')
                .first()
                .addClass('active');
            $elem.find('.raven-folder-number-of-tests')
                .first()
                .addClass('active');
        },

        addActiveFolderGlobalStyle: function($elem) {
            // add active style
            $elem.find('.raven-folder-global-name')
                .first()
                .addClass('active');
            $elem.find('.raven-folder-global-number-of-tests')
                .first()
                .addClass('active');
        },

        activateAllFolder: function() {
            var $allTestsElement = this.$el.find('#' + this.ALL_TESTS);

            this.addActiveFolderGlobalStyle($allTestsElement);
            this.setActiveFolder($allTestsElement);
            this._disableActions();
        },

        activateAllFolderAndResetPath: function() {
            var $allTestsElement = this.$el.find('#' + this.ALL_TESTS);

            // unhighlight folder-open-b that aren't element parents
            this.unhighlightOutOfPathFolders(this.$el.find('li'));

            // clean the tree dot line of the tree
            this._clearTreeDotLineStructure();

            this._removeActiveFolderStyle();

            this.addActiveFolderGlobalStyle($allTestsElement);

            // change all elements with status open-b to open-c
            this.$el.find('.raven-folder-open-b')
                .removeClass('raven-folder-open-b')
                .addClass('raven-folder-open-c');

            // change all elements with status closed-b to closed-c
            this.$el.find('.raven-folder-closed-b')
                .removeClass('raven-folder-closed-b')
                .addClass('raven-folder-closed-c');

            this.setActiveFolder($allTestsElement);

            this._disableActions();
        },

        activateFolderRoot: function() {
            var $folderRootElement = this.$el.find('#' + this.FOLDER_ROOT);

            // unhighlight folder-open-b that aren't element parents
            this.unhighlightOutOfPathFolders(this.$el.find('li'));

            // clean the tree dot line of the tree
            this._clearTreeDotLineStructure();

            this._removeActiveFolderStyle();

            this.addActiveFolderGlobalStyle($folderRootElement);

            // change all elements with status open-b to open-c
            this.$el.find('.raven-folder-open-b')
                .removeClass('raven-folder-open-b')
                .addClass('raven-folder-open-c');

            // change all elements with status closed-b to closed-c
            this.$el.find('.raven-folder-closed-b')
                .removeClass('raven-folder-closed-b')
                .addClass('raven-folder-closed-c');

            this._disableActions();

            this.setActiveFolder($folderRootElement);
        },

        getMaxTreeLevel: function() {
            var maxTreeLevelIds = [];
            var maxTreeLevel = 0;
            this.$el.find('.raven-folder-tree-level:visible').each(function() {
                var treeDotLineExtension = $(this).find('.raven-folder-item-structure').first().children('.raven-tree-dot-line-extension').length;
                var treeDotLine = $(this).find('.raven-folder-item-structure').first().children('.raven-tree-dot-line').length;

                var level = treeDotLineExtension + treeDotLine;
                var elemId = $(this).parents('li').first().attr('id');
                if(level > maxTreeLevel) {
                    maxTreeLevelIds = [];
                    maxTreeLevelIds.push(elemId);
                    maxTreeLevel = level;
                } else if (level === maxTreeLevel) {
                    maxTreeLevelIds.push(elemId);
                }
            });

            return {'level': maxTreeLevel, 'ids': maxTreeLevelIds};
        },

        /****************************************************************
         *                   TREE STRUCTURE FUNCTIONS
         ****************************************************************/
        _initTreeDotLineStructure: function() {
            // for each element with class folders-view-mode
            // calculate what's its current level in the tree
            // and get its first li parent.
            // Check if there are li uncles next to its parent
            // and check if it has li brothers
            // if true, then change the tree-dot-line-extension to visible
            this.$el.find('.raven-folders-view-mode').each(function() {

                var level = $(this).closest('li').find('div')
                    .first()
                    .children('.raven-tree-dot-line-extension')
                    .length;

                var parent = $(this).parents('li:not("#folder-root")').first();

                if (parent.nextAll('li').length > 0) {
                    if ($(this).closest('li').nextAll('li').length > 0) {
                        $(this).closest('li')
                            .find('ul.raven-folder-tree-level')
                            .first()
                            .find('li.raven-folder-item')
                            .each(function() {
                                $(this).children('div')
                                    .children('.raven-tree-dot-line-extension:eq(' + level + ')')
                                    .css('visibility', 'visible');
                            });
                    }
                }
            });
        },

        _clearTreeDotLineStructure: function() {
            var $elem = this.$el.find('ul.raven-folder-tree-level');

            // change all tree-dot-line from selected to normal
            $elem.find('.raven-tree-dot-line')
                .removeClass("raven-tree-dot-line-selected")
                .addClass("raven-tree-dot-line-normal");

            // change all tree-dot-line from breadcrumb to normal
            $elem.find('.raven-tree-dot-line')
                .removeClass("raven-tree-dot-line-breadcrumb")
                .addClass("raven-tree-dot-line-normal");

            // change all tree-dot-line-extension from selected to normal
            $elem.find('.raven-tree-dot-line-extension')
                .removeClass("raven-tree-dot-line-extension-selected")
                .addClass("raven-tree-dot-line-extension-normal");
        },

        _removeTreeDotLineExtension: function($elem) {
            var level = $elem.find('div')
                .first()
                .children('.raven-tree-dot-line-extension')
                .length;

            if ($elem.nextAll('li').length === 0) {
                $elem.prevAll('li')
                    .each(function() {
                        $(this).find('li')
                            .each(function() {
                                $(this).find('.raven-tree-dot-line-extension:eq(' + level + ')')
                                    .css('visibility', 'hidden');
                            });
                    });
            }
        },

        updateTreeDotLineStructure: function($elem, type) {
            if (type === this.EXPAND) {
                this._expandTreeDotLine($elem);
            } else if (type === this.COLLAPSE) {
                this._collapseTreeDotLine($elem);
            } else if (type === this.EXPAND_ALL) {
                this._expandAllTreeDotLine($elem);
            } else if (type === this.ACTIVATE) {
                this._activateTreeDotLine($elem);
            }
        },

        _expandTreeDotLine: function($elem) {
            $elem.parents('li:not("#folder-root")')
                .each(function() {
                    // change tree-dot-line from normal to selected
                    $(this).find('.raven-tree-dot-line')
                        .first()
                        .removeClass('raven-tree-dot-line-normal')
                        .addClass('raven-tree-dot-line-selected');

                    // change folders-view-mode from closed-b to open-b
                    $(this).find('.raven-folders-view-mode')
                        .first()
                        .children('.raven-folder-closed-b')
                        .first()
                        .removeClass('raven-folder-closed-b')
                        .addClass('raven-folder-open-b');

                    // change folders-view-mode from open-c to open-b
                    $(this).find('.raven-folders-view-mode')
                        .first()
                        .children('.raven-folder-open-c')
                        .first()
                        .removeClass('raven-folder-open-c')
                        .addClass('raven-folder-open-b');

                    // change folders-view-mode from closed-c to open-b
                    $(this).find('.raven-folders-view-mode')
                        .first()
                        .children('.raven-folder-closed-c')
                        .first()
                        .removeClass('raven-folder-closed-c')
                        .addClass('raven-folder-open-b');

                    var level = $(this).find('div')
                        .first()
                        .children('.raven-tree-dot-line-extension')
                        .length;

                    $(this).prevAll('li')
                        .each(function() {

                            // change tree-dot-line from normal to breadcrumb in all previous items
                            $(this).find('.raven-tree-dot-line')
                                .first()
                                .removeClass('raven-tree-dot-line-normal')
                                .addClass('raven-tree-dot-line-breadcrumb');

                            // for all "li" children of each previous item,
                            // find the tree-dot-line-extension at a given level
                            // and change it from normal to selected
                            $(this).find('li')
                                .each(function() {
                                    $(this).find('.raven-tree-dot-line-extension:eq(' + level + ')')
                                        .removeClass('raven-tree-dot-line-extension-normal')
                                        .addClass('raven-tree-dot-line-extension-selected');
                                });
                        });
                });
        },

        _collapseTreeDotLine: function($elem) {
            // change tree-dot-line from breadcrumb to normal in all previous items
            $elem.closest('li')
                .prevAll('li')
                .each(function() {
                    $(this)
                        .find('.raven-tree-dot-line')
                        .first()
                        .removeClass('raven-tree-dot-line-breadcrumb')
                        .addClass('raven-tree-dot-line-normal');
                });

            var level = $elem.closest('li')
                .find('div')
                .first()
                .children('.raven-tree-dot-line-extension')
                .length;

            $elem.closest('li')
                .prevAll('li')
                .each(function() {

                    // for all "li" children of each previous item,
                    // find the tree-dot-line-extension at a given level
                    // and change it from selected to normal
                    $(this).find('li')
                        .each(function() {
                            $(this).find('.raven-tree-dot-line-extension:eq(' + level + ')')
                                .removeClass('raven-tree-dot-line-extension-selected')
                                .addClass('raven-tree-dot-line-extension-normal');
                        });
                });
        },

        _expandAllTreeDotLine: function($elem) {
            // change folders-view-mode from closed-b to open-b
            $(this).find('.raven-folders-view-mode')
                .first()
                .children('.raven-folder-closed-b')
                .first()
                .removeClass('raven-folder-closed-b')
                .addClass('raven-folder-open-b');

            $elem.find('li')
                .each(function() {
                    // change folders-view-mode from open-b to open-c
                    $(this).find('.raven-folders-view-mode')
                        .first()
                        .children('.raven-folder-open-b')
                        .first()
                        .removeClass('raven-folder-open-b')
                        .addClass('raven-folder-open-c');

                    // change folders-view-mode from closed-c to open-c
                    $(this).find('.raven-folders-view-mode')
                        .first()
                        .children('.raven-folder-closed-c')
                        .first()
                        .removeClass('raven-folder-closed-c')
                        .addClass('raven-folder-open-c');

                    // change tree-dot-line from breadcrumb to normal
                    $(this).find('.raven-tree-dot-line')
                        .first()
                        .removeClass('raven-tree-dot-line-breadcrumb')
                        .addClass('raven-tree-dot-line-normal');

                    // change tree-dot-line from selected to normal
                    $(this).find('.raven-tree-dot-line')
                        .first()
                        .removeClass('raven-tree-dot-line-selected')
                        .addClass('raven-tree-dot-line-normal');

                    var level = $(this).find('div')
                        .first()
                        .children('.raven-tree-dot-line-extension')
                        .length;

                    $(this).prevAll('li')
                        .each(function() {
                            // for all "li" children of each previous item,
                            // find the tree-dot-line-extension at a given level
                            // and change it from selected to normal
                            $(this).find('li')
                                .each(function() {
                                    $(this).find('.raven-tree-dot-line-extension:eq(' + level + ')')
                                        .removeClass('raven-tree-dot-line-extension-selected')
                                        .addClass('raven-tree-dot-line-extension-normal');
                                });
                        });
                });
        },

        _activateTreeDotLine: function($elem) {
            this._activateTreeDotLineFolder($elem);

            var self = this;
            $elem.parents('li:not("#folder-root")')
                .each(function() {
                    self._expandTreeDotLineFolder($(this));
                });
        },

        _activateTreeDotLineFolder: function($elem) {
            // change tree-dot-line from normal to selected
            $elem.find('.raven-tree-dot-line')
                .first()
                .removeClass('raven-tree-dot-line-normal')
                .addClass('raven-tree-dot-line-selected');

            // change folders-view-mode from open-c to open-b
            $elem.find('.raven-folders-view-mode')
                .first()
                .children('.raven-folder-open-c')
                .first()
                .removeClass('raven-folder-open-c')
                .addClass('raven-folder-open-b');

            // change folders-view-mode from closed-c to closed-b
            $elem.find('.raven-folders-view-mode')
                .first()
                .children('.raven-folder-closed-c')
                .first()
                .removeClass('raven-folder-closed-c')
                .addClass('raven-folder-closed-b');

            var level = $elem.find('div')
                .first()
                .children('.raven-tree-dot-line-extension')
                .length;

            $elem.prevAll('li')
                .each(function() {
                    // change tree-dot-line from normal to breadcrumb in all previous items
                    $(this).find('.raven-tree-dot-line')
                        .first()
                        .removeClass('raven-tree-dot-line-normal')
                        .addClass('raven-tree-dot-line-breadcrumb');

                    // for all "li" children of each previous item,
                    // find the tree-dot-line-extension at a given level
                    // and change it from normal to selected
                    $(this).find('li')
                        .each(function() {
                            $(this).find('.raven-tree-dot-line-extension:eq(' + level + ')')
                                .removeClass('raven-tree-dot-line-extension-normal')
                                .addClass('raven-tree-dot-line-extension-selected');
                        });
                });
        },

        _expandTreeDotLineFolder: function($elem) {
            // change tree-dot-line from normal to selected
            $elem.find('.raven-tree-dot-line')
                .first()
                .removeClass('raven-tree-dot-line-normal')
                .addClass('raven-tree-dot-line-selected');

            // change folders-view-mode from open-c to open-b
            $elem.find('.raven-folders-view-mode')
                .first()
                .children('.raven-folder-open-c')
                .first()
                .removeClass('raven-folder-open-c')
                .addClass('raven-folder-open-b');

            // change folders-view-mode from closed-c to open-b
            $elem.find('.raven-folders-view-mode')
                .first()
                .children('.raven-folder-closed-c')
                .first()
                .removeClass('raven-folder-closed-c')
                .addClass('raven-folder-open-b');

            var level = $elem.find('div')
                .first()
                .children('.raven-tree-dot-line-extension')
                .length;

            $elem.prevAll('li')
                .each(function() {

                    // change tree-dot-line from normal to breadcrumb in all previous items
                    $(this).find('.raven-tree-dot-line')
                        .first()
                        .removeClass('raven-tree-dot-line-normal')
                        .addClass('raven-tree-dot-line-breadcrumb');

                    // for all "li" children of each previous item,
                    // find the tree-dot-line-extension at a given level
                    // and change it from normal to selected
                    $(this).find('li')
                        .each(function() {
                            $(this).find('.raven-tree-dot-line-extension:eq(' + level + ')')
                                .removeClass('raven-tree-dot-line-extension-normal')
                                .addClass('raven-tree-dot-line-extension-selected');
                        });
                });
        },

        unhighlightOutOfPathFolders: function(treeElements) {
            this.$el.find('.raven-folder-open-b, .raven-folder-closed-b').closest('li')
                .each(function() {
                    var contains = false;
                    var openFolder = $(this);

                    $(treeElements).each(function() {
                        if (openFolder.attr('id') === $(this).attr('id')) {
                            // tree contains the current item
                            contains = true;
                        }
                    });

                    // item is out of the tree and should be unhighlighted
                    if (!contains) {
                        // change folders-view-mode from open-b to open-c
                        $(this).find('.raven-folder-open-b')
                            .first()
                            .removeClass('raven-folder-open-b')
                            .addClass('raven-folder-open-c');

                        // change folders-view-mode from closed-b to closed-c
                        $(this).find('.raven-folder-closed-b')
                            .first()
                            .removeClass('raven-folder-closed-b')
                            .addClass('raven-folder-closed-c');
                    }
                });
        },

        /****************************************************************
         *                        EVENT HANDLERS
         ****************************************************************/

        createFolder: function () {
            if (!this.isEditable) {
                return;
            }

            var $elem = this.$el.find('.raven-activeFolder');
            $elem.removeClass('raven-activeFolder');

            this._enableActions();

            // clean the tree dot line of the tree
            this._clearTreeDotLineStructure();

            this._removeActiveFolderStyle();

            var content;
            if ($elem.length > 0
                && $elem.first().attr('id') !== this.ALL_TESTS
                && $elem.first().attr('id') !== this.ALL_ORPHAN_TESTS
                && $elem.first().attr('id') !== this.FOLDER_ROOT) {

                var level = $elem.find('div')
                    .first()
                    .children('.raven-tree-dot-line-extension')
                    .length + 1;

                content = RAVEN.Folder.createTemporaryFolder({'level': level, 'parentId': $elem.first().attr('id')});

                $elem.find('ul.raven-folder-tree-level')
                    .first()
                    .append(content)
                    .show();
            } else {
                content = RAVEN.Folder.createTemporaryFolder({'level': 0, 'parentId': -1});

                this.$el.find('#raven-test-repository-tree').find('ul.raven-folder-tree-level')
                    .first()
                    .append(content)
                    .show();
            }

            var $temporaryItemFolder = this.$el.find('.temporary-item-folder');

            this.addActiveFolderStyle($temporaryItemFolder);

            // unhighlight folder-open-b that aren't element parents
            this.unhighlightOutOfPathFolders($temporaryItemFolder.find('.raven-wrapper').parents('li'));

            this._confirmCreateFolder($temporaryItemFolder);

            this._initTreeDotLineStructure();

            // update the dot line of the tree
            this.updateTreeDotLineStructure($temporaryItemFolder.find('.raven-wrapper'), this.EXPAND);

            this.$el.trigger('folderCreateEvt');

            this.lastActiveFolder = this.TEMPORARY;
        },

        _confirmCreateFolder: function($elem) {
            var folderName = $elem.find('.raven-folder-name-input')
                .first()
                .val();

            var parentId = $elem.attr('data-parent-id');

            var query = '?name=' + folderName;
            query += '&entityKey=' + this.entityKey;
            parentId !== null && parentId !== undefined && parentId !== '-1' ? query += '&parentId=' + parentId : '';

            if (folderName !== null && folderName !== '') {
                this.$el.trigger('blockElementEvt');

                var self = this;
                $.ajax({
                    type: "POST",
                    url: contextPath + '/rest/raven/1.0/' + self.repositoryPath + '/create' + query,
                    processData: true,
                    contentType: "application/json",
                    error: function(qXHR, textStatus, errorThrown) {
                        if (errorThrown && errorThrown.trim() !== '') {
                            errorThrown = errorThrown + ': ' + qXHR.responseText;
                        } else {
                            errorThrown = qXHR.responseText;
                        }

                        self.$el.trigger('messageEvt', [AJS.I18n.getText('raven.test-repository.folders.error-creating-folder-name'), errorThrown]);
                    },
                    success: function(data, textStatus, jqXHR) {
                        $elem.attr('id', data.id)
                            .removeClass('temporary-item-folder');

                        self.$el.find('#span-temporary').attr('id', 'span-' + data.id);
                        self.$el.find('#icon-temporary').attr('id', 'icon-' + data.id);
                        self.$el.find('#ul-temporary').attr('id', 'ul-' + data.id);

                        self._setNameOfElement($elem, data.name);
                        self._setInputText($elem);
                        self.setActiveFolder($elem);
                        self.createdFolderId = data.id;

                        self._messageFlag(AJS.I18n.getText('raven.test-repository.folders.success-creating-folder-name'));
                    },
                    complete: function() {
                        self.$el.trigger('unblockElementEvt');
                    }
                });
            } else {
                $elem.find('.raven-folder-name-input').first().attr('placeholder', AJS.I18n.getText('raven.test-repository.folders.invalid-name'));
            }
        },

        _setNameOfElement: function($elem, name) {
            $elem.find('.raven-folder-name-input').first().val(name);
            $elem.find('.raven-folder-name')
                .first()
                .text(name)
                .attr('title', name);
        },

        _editFolder: function() {
            if (this.isEditable) {
                var $elem = this.$el.find('.raven-activeFolder');

                this._setInputText($elem);
            }
        },

        _setInputText: function($elem) {
            $elem.find('.raven-folder-name')
                .first()
                .hide();

            $elem.find('.raven-folder-name-input')
                .first()
                .show();

            // select input textbox
            var inputTextbox = $elem.find('.raven-folder-name-input').first();
            var text = inputTextbox.val();
            if (text !== null && text !== undefined) {
                inputTextbox._focus()
                    .setSelectionRange(text.length, text.length);
            }
        },

        _confirmEditFolder: function($elem) {
            var folderOldName = $elem.find('.raven-folder-name')
                .first()
                .text();

            var folderNewName = $elem.find('.raven-folder-name-input')
                .first()
                .val();

            if (folderNewName === null || folderNewName === '') {
                $elem.find('.raven-folder-name-input')
                    .first()
                    .attr('placeholder', AJS.I18n.getText('raven.test-repository.folders.invalid-name'));

                return;
            }

            if (folderOldName !== folderNewName) {
                this.$el.trigger('blockElementEvt');

                var self = this;
                $.ajax({
                    type: "PUT",
                    url: contextPath + '/rest/raven/1.0/' + self.repositoryPath + '/rename?id=' + $elem.attr('id') + '&name=' + folderNewName,
                    processData: true,
                    async: true,
                    contentType: "application/json",
                    error: function(qXHR, textStatus, errorThrown) {
                        if (errorThrown && errorThrown.trim() !== '') {
                            errorThrown = errorThrown + ': ' + qXHR.responseText;
                        } else {
                            errorThrown = qXHR.responseText;
                        }

                        self._revertFolderNameChange(element, folderOldName);
                        self.$el.trigger('messageEvt', [AJS.I18n.getText('raven.test-repository.folders.error-updating-folder-name'), errorThrown]);
                    },
                    success: function(data, textStatus, jqXHR) {
                        if (data) {
                            self._setNameOfElement($elem, data.name);

                            $elem.find('.raven-folder-name')
                                .first()
                                .show();

                            $elem.find('.raven-folder-name-input')
                                .first()
                                .hide();

                            self.$el.trigger('folderEditEvt', data.name);

                            self._messageFlag(AJS.I18n.getText('raven.test-repository.folders.success-updating-folder-name', folderOldName, data.name));
                        } else {
                            self.$el.trigger('messageEvt', [AJS.I18n.getText('raven.test-repository.folders.error-updating-folder-name')]);
                        }
                    },
                    complete: function() {
                        self.$el.trigger('unblockElementEvt');
                    }
                });
            } else {
                this._revertFolderNameChange($elem, folderOldName);
            }
        },

        _revertFolderNameChange: function($elem, folderOldName) {
            $elem.find('.raven-folder-name')
                .first()
                .text(folderOldName)
                .attr('title', folderOldName)
                .show();

            $elem.find('.raven-folder-name-input')
                .first()
                .val(folderOldName)
                .hide();
        },

        _expandFolder: function($elem) {
            // unhighlight folder-open-b that aren't element parents
            this.unhighlightOutOfPathFolders($elem.parents('li'));

            // clean the tree dot line of the tree
            this._clearTreeDotLineStructure();

            // show the direct child of the element
            $elem.closest('li')
                .find('ul.raven-folder-tree-level')
                .first()
                .show();

            var updateActiveFolder = true;
            if ($elem.hasClass('raven-folder-closed-b')) {
                updateActiveFolder = false;
            }

            // change element status from closed-c to open-b
            $elem.removeClass('raven-folder-closed-c')
                .addClass('raven-folder-open-b');

            // change element status from closed-b to open-b
            $elem.removeClass('raven-folder-closed-b')
                .addClass('raven-folder-open-b');

            // change element status from open-c to open-b
            $elem.removeClass('raven-folder-open-c')
                .addClass('raven-folder-open-b');

            this._removeActiveFolderStyle();

            this.addActiveFolderStyle($elem.closest('li'));

            if (updateActiveFolder) {
                this.setActiveFolder($elem);
            }

            // update the dot line of the tree
            this.updateTreeDotLineStructure($elem, this.EXPAND);
        },

        _collapseFolder: function($elem) {
            var maxTreeLevel = this.getMaxTreeLevel();

            // hide the children of the element
            $elem.closest('li')
                .find('ul.raven-folder-tree-level')
                .hide();

            if ($elem.hasClass('raven-folder-open-c')) {
                // change element status from open-c to closed-c
                $elem.closest('li')
                    .find('.raven-folder-open-c')
                    .removeClass('raven-folder-open-c')
                    .addClass('raven-folder-closed-c');
            } else {
                // change elements status from closed-b to closed-c
                $elem.closest('li')
                    .find('.raven-folder-closed-b')
                    .removeClass('raven-folder-closed-b')
                    .addClass('raven-folder-closed-c');

                // change elements status from open-c to closed-c
                $elem.closest('li')
                    .find('.raven-folder-open-c')
                    .removeClass('raven-folder-open-c')
                    .addClass('raven-folder-closed-c');

                // change element status from open-b to closed-b
                $elem.closest('li')
                    .find('.raven-folder-open-b')
                    .first()
                    .removeClass('raven-folder-open-b')
                    .addClass('raven-folder-closed-b');

                // change elements status from open-b to closed-c
                $elem.closest('li')
                    .find('.raven-folder-open-b')
                    .removeClass('raven-folder-open-b')
                    .addClass('raven-folder-closed-c');

                this._removeActiveFolderStyle();

                this.addActiveFolderStyle($elem.closest('li'));

                //display content for the active folder
                this.setActiveFolder($elem);
            }

            this.$el.trigger('folderCollapseEvt', [$elem.closest('li').attr('id'), maxTreeLevel]);
        },

        _expandRootFolder: function($elem) {
            // show the direct child of the element
            $elem.closest('li')
                .find('ul.raven-folder-tree-level')
                .first()
                .show();

            // change element status from root-closed to open-a
            $elem.removeClass('raven-folder-root-closed')
                .addClass('raven-folder-open-a');
        },

        _collapseRootFolder: function($elem) {
            var maxTreeLevel = this.getMaxTreeLevel();

            // show the direct child of the element
            $elem.closest('li')
                .find('ul.raven-folder-tree-level')
                .first()
                .hide();

            // change element status from open-a to root-closed
            $elem.removeClass('raven-folder-open-a')
                .addClass('raven-folder-root-closed');

            this.$el.trigger('folderCollapseEvt', [$elem.closest('li').attr('id'), maxTreeLevel]);
        },

        getActiveFolderId: function() {
            return this.$el.find('.raven-activeFolder').attr('id');
        },

        getActiveFolder: function() {
            return this.$el.find('.raven-activeFolder');
        },

        getFolderName: function(folderId) {
            return this.$el.find('#' + folderId).find('.raven-folder-caption').first().text();
        },

        getGlobalFolderName: function() {
            return this.$el.find('.raven-activeFolder').find('.raven-folder-caption').first().text();
        },

        getActiveFolderIcon: function() {
            var id = this.getActiveFolderId();
            var $icon = this.$el.find('#icon-' + id);
            if ($icon.hasClass('raven-folder-open-c')) {
                return 'raven-folder-open-c';
            }

            if ($icon.hasClass('raven-folder-open-b')) {
                return 'raven-folder-open-b';
            }

            if ($icon.hasClass('raven-folder-closed-c')) {
                return 'raven-folder-closed-c';
            }

            if ($icon.hasClass('raven-folder-closed-b')) {
                return 'raven-folder-closed-b';
            }
        },

        setActiveFolder: function($elem) {
            // remove activeFolder from last active li
            this.$el.find('li').removeClass('raven-activeFolder');

            // remove activeFolder to current active li
            $elem.closest('li').addClass('raven-activeFolder');

            this.$el.trigger('folderSelectEvt', $elem.closest('li').attr('id'));
        },

        _deleteFolder: function() {
            var $elem = this.$el.find('.raven-activeFolder');
            var elementId = $elem.attr('id');
            var folderName = this.getFolderName(elementId);

            var self = this;
            if ($elem.length > 0 && elementId !== self.ALL_ORPHAN_TESTS && elementId !== self.ALL_TESTS && elementId !== self.FOLDER_ROOT) {
                var dialog = new JIRA.FormDialog({
                    id: 'raven-delete-folder-dialog',
                    width: 530,
                    height: "250px",
                    content: function(callback) {
                        var title = AJS.I18n.getText('raven.test-repository.delete.confirm.title');
                        var message = AJS.I18n.getText('raven.test-repository.delete.confirm.message', $elem.find('.folder-name').first().text());
                        var confirmButton = AJS.I18n.getText('common.action.delete');
                        var innerHtml = RAVEN.Folder.deleteFoldersDialog({title: title, message: message, confirmButton: confirmButton});

                        callback(innerHtml);
                    },
                    submitHandler: function(e, callback) {
                        e.preventDefault();

                        self.$el.trigger('blockElementEvt', 'DELETE_FOLDER');
                        $.ajax({
                            type: "DELETE",
                            url: contextPath + '/rest/raven/1.0/' + self.repositoryPath + '/delete?id=' + $elem.attr('id'),
                            processData: true,
                            async: true,
                            contentType: "application/json",
                            error: function(qXHR, textStatus, errorThrown) {
                                if (errorThrown && errorThrown.trim() !== '') {
                                    errorThrown = errorThrown + ': ' + qXHR.responseText;
                                } else {
                                    errorThrown = qXHR.responseText;
                                }
                                dialog.hide();
                                self.$el.trigger('messageEvt', [AJS.I18n.getText('raven.test-repository.folders.error-deleting-folder', $elem.attr('id')), errorThrown]);
                            },
                            success: function(data, textStatus, jqXHR) {
                                var parentElement = $elem.parents('li').first();

                                self._removeActiveFolderStyle();

                                if (parentElement.attr('id') === self.FOLDER_ROOT) {
                                    self.addActiveFolderGlobalStyle(parentElement);

                                    // clean the tree dot line of the tree
                                    self._clearTreeDotLineStructure();

                                    self._disableActions();
                                } else {
                                    self.addActiveFolderStyle(parentElement);
                                }

                                self.setActiveFolder(parentElement);

                                self._removeTreeDotLineExtension($elem);

                                self.updateTreeDotLineStructure($elem, self.COLLAPSE);

                                self.$el.trigger('folderSelectEvt', self.getActiveFolderId());

                                $elem.remove();

                                dialog.hide();

                                self.refreshFolderTestCount();

                                self._messageFlag(AJS.I18n.getText('raven.test-repository.folders.success-deleting-folder', folderName));
                            },
                            complete: function() {
                                self.$el.trigger('blockElementEvt', 'DELETE_FOLDER');
                            }
                        });
                    },
                    autoClose: true
                });
                dialog.show();
            }
        },

        refreshFolderTestCount: function() {
            this.$el.trigger('clearMessageEvt');

            var self = this;
            $.ajax({
                type: "GET",
                url: self.model.urlRoot(),
                processData: true,
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() !== '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }

                    self.$el.trigger('messageEvt', [errorThrown]);
                },
                success: function(data, textStatus, jqXHR) {
                    if (!data) {
                        return;
                    }
                    self._updateTestCounts(data);
                }
            });
        },

        _updateTestCounts: function(data) {
            this._folderTreeClimb(data.children);
            this._updateOrphanTestsCount(data.allOrphanTestsCount);
            this._setTotalTestsText(this.$el.find('#all-tests-count'), data.allTestsCount);
            this._updateLeavesCount(data.allTestsCount - data.allOrphanTestsCount);
        },

        _folderTreeClimb: function(rootElements) {
            for (var i = 0; i < rootElements.length; i++) {
                this._setTotalTests(this.$el.find('#' + rootElements[i].id), rootElements[i].testCount, rootElements[i].totalTestCount);
                this._folderTreeClimb(rootElements[i].children);
            }
        },

        _setTotalTestsText: function(element, count) {
            element.text(this._formatTestCountGlobal(count));
        },

        _formatTestCountGlobal: function(testCount) {
            return '(' + testCount + ')';
        },

        _formatTestCount: function(testCount, totalTestCount) {
            return testCount + ' (' + totalTestCount + ')';
        },

        _setTotalTests: function($folder, testCount, totalTestCount) {
            if (!$folder) {
                return;
            }
            var $countSpan = $folder.find('.raven-folder-number-of-tests');
            var $testCount = $folder.find('.totalTestCount');

            if ($testCount) {
                $testCount.text(totalTestCount);
            }

            if (!$countSpan) {
                return;
            }
            $countSpan.text(this._formatTestCount(testCount, totalTestCount));
        },

        _updateOrphanTestsCount: function(orphanCount) {
            this.$el.find('#all-orphan-tests').find('.raven-folder-global-number-of-tests').text("(" + orphanCount + ")");
        },

        _updateLeavesCount: function(leavesCount) {
            var $root = this.$el.find('#folder-root');
            $root.find('.raven-folder-global-number-of-tests').text("(" + leavesCount + ")");
            $root.find('.folder-root-test-counter').text(leavesCount);
        },

        expandAll: function() {
            var $activeFolder = this.$el.find('.raven-activeFolder');

            var self = this;
            if ($activeFolder.attr('id') === self.FOLDER_ROOT ||
                $activeFolder.length === 0 ||
                $activeFolder.parents('#' + self.FOLDER_ROOT).length === 0) {  // self.FOLDER_ROOT).length === 0

                $activeFolder = $('#' + self.FOLDER_ROOT);
                $activeFolder.find('.raven-folder-root-closed')
                    .removeClass('raven-folder-root-closed')
                    .addClass('raven-folder-open-a');
            } else {
                $activeFolder.find('.raven-folder-closed-b')
                    .first()
                    .removeClass('raven-folder-closed-b')
                    .addClass('raven-folder-open-b');
            }

            $activeFolder.find('ul.raven-folder-tree-level').show();
            this.updateTreeDotLineStructure($activeFolder, this.EXPAND_ALL);
        },

        collapseAll: function() {
            var self = this;

            var $elem = this.$el.find('.raven-activeFolder');

            if ($elem.attr('id') !== self.FOLDER_ROOT) {
                $elem.find('ul.raven-folder-tree-level').hide();

                // change all elements with status open-b to closed-b
                $elem.find('.raven-folder-open-b')
                    .first()
                    .removeClass('raven-folder-open-b')
                    .addClass('raven-folder-closed-b');

                // change all elements with status open-c to closed-c
                $elem.find('.raven-folder-open-c')
                    .removeClass('raven-folder-open-c')
                    .addClass('raven-folder-closed-c');
            } else {
                $elem = self.$el;
                $elem.find('ul.raven-folder-tree-level').hide();

                // change all elements with status open-c to closed-c
                $elem.find('.raven-folder-open-c')
                    .removeClass('raven-folder-open-c')
                    .addClass('raven-folder-closed-c');

                // change element status from open-a to root-closed
                $elem.find('.raven-folder-open-a')
                    .removeClass('raven-folder-open-a')
                    .addClass('raven-folder-root-closed');
            }
        },

        _isFolderImg: function(element) {
            return element.hasClass('raven-folder-open-b')
                || element.hasClass('raven-folder-closed-b')
                || element.hasClass('raven-folder-open-c')
                || element.hasClass('raven-folder-closed-c');
        },

        buildCompletePath: function(folderId) {
            var parents = this.$el.find('#' + folderId).parents('li').andSelf();

            var path = this.$el.find(parents[0]).find('.raven-folder-caption').first().text();
            for(var i = 1; i < parents.length; i++) {
                path += '/' + $(parents[i]).find('.raven-folder-caption').first().text();
            }
            return path;
        },

        folderItemClick: function(e, blockFolderStateChange) {
            var self = this;

            e.stopPropagation();

            var $elem = self.$el.find(e.target);

            var isAlreadyActive = false;
            if ($elem.closest('li').hasClass('raven-activeFolder')) {
                isAlreadyActive = true;
            }

            if (blockFolderStateChange || !self._isFolderImg($elem)) {
                $elem = $elem.parents('li').first();
            }

            self._enableActions();
            if ($elem.hasClass('raven-folder-closed-b') || $elem.hasClass('raven-folder-closed-c')) { // Expand Folder
                self._expandFolder($elem);
            } else if ($elem.hasClass('raven-folder-open-b') || $elem.hasClass('raven-folder-open-c')) { // Collapse Folder
                self._collapseFolder($elem);
            } else { // Make Active Folder
                if (!isAlreadyActive) {
                    // unhighlight folder-open-b that aren't element parents
                    self.unhighlightOutOfPathFolders($elem.find('.raven-folders-view-mode').first().parents('li'));

                    // clean the tree dot line of the tree
                    self._clearTreeDotLineStructure();

                    self._removeActiveFolderStyle();

                    self.addActiveFolderStyle($elem);

                    self.setActiveFolder($elem);

                    // update the dot line of the tree
                    self.updateTreeDotLineStructure($elem, self.ACTIVATE);
                }
                self.$el.trigger('folderActiveEvt', $elem.closest('li').attr('id'));
            }
        },

        globalFolderItemClick: function(e, blockFolderStateChange) {
            var self = this;

            e.stopPropagation();

            var $elem = self.$el.find(e.target);

            if(!$elem.hasClass('raven-folder-root-closed') && !$elem.hasClass('raven-folder-open-a')) {
                self.$el.trigger('folderActiveEvt', $elem.closest('li').attr('id'));
            }

            if(!blockFolderStateChange) {
                if ($elem.hasClass('raven-folder-root-closed')) { // Expand Folder
                    self._expandRootFolder($elem);
                } else if ($elem.hasClass('raven-folder-open-a')) { // Collapse Folder
                    self._collapseRootFolder($elem);
                }
            }
            $elem = $elem.parents('li').first();

            // unhighlight folder-open-b that aren't element parents
            self.unhighlightOutOfPathFolders(self.$el.find('li'));

            // clean the tree dot line of the tree
            self._clearTreeDotLineStructure();

            self._removeActiveFolderStyle();

            self.addActiveFolderGlobalStyle($elem);

            // change all elements with status open-b to open-c
            self.$el.find('.raven-folder-open-b')
                .removeClass('raven-folder-open-b')
                .addClass('raven-folder-open-c');

            // change all elements with status closed-b to closed-c
            self.$el.find('.raven-folder-closed-b')
                .removeClass('raven-folder-closed-b')
                .addClass('raven-folder-closed-c');

            self._disableActions();

            self.setActiveFolder($elem);
        },

        _keyupNameChange: function(e) {
            if (this.isEditable) {
                var self = this;

                var $elem = self.$el.find(e.target).closest('li');

                if (e.keyCode === 13) {
                    self._confirmEditFolder($elem);
                } else if (e.keyCode === 27) {
                    var folderName = $elem.find('.raven-folder-name')
                        .first()
                        .text();

                    $elem.find('.raven-folder-name-input')
                        .first()
                        .val(folderName);

                    $elem.find('.raven-folder-name-input')
                        .first()
                        .attr('placeholder', '');
                } else {
                    $elem.find('.raven-folder-name-input')
                        .first()
                        .attr('placeholder', '');
                }
            }
        },

        _blurNameChange: function(e) {
            if (this.isEditable) {
                var self = this;

                self._confirmEditFolder(self.$el.find(e.target).closest('li'));
            }
        },

        _disableEditFolder: function(elementId) {
            this.$el.find('#raven-folder-create').attr('title', '').removeClass('raven-folder-create').addClass('raven-folder-create-disabled');
            this.$el.find('#raven-folder-edit').attr('title', '').removeClass('raven-folder-edit').addClass('raven-folder-edit-disabled');
            this.$el.find('#raven-folder-delete').attr('title', '').removeClass('raven-folder-delete').addClass('raven-folder-delete-disabled');
        },

        _disableActions: function() {
            this.$el.find('#raven-folder-edit').attr('title', '').removeClass('raven-folder-edit').addClass('raven-folder-edit-disabled');
            this.$el.find('#raven-folder-delete').attr('title', '').removeClass('raven-folder-delete').addClass('raven-folder-delete-disabled');
        },

        _enableActions: function() {
            if (this.isEditable) {
                this.$el.find('#raven-folder-edit').attr('title', AJS.I18n.getText('raven.test-repository.edit-folder')).removeClass('raven-folder-edit-disabled').addClass('raven-folder-edit');
                this.$el.find('#raven-folder-delete').attr('title', AJS.I18n.getText('raven.test-repository.delete-folder')).removeClass('raven-folder-delete-disabled').addClass('raven-folder-delete');
            }
        },

        _messageFlag: function(body) {
            require('aui/flag')({
                type: 'success',
                body: body,
                close: 'auto'
            });
        }
    });
})(AJS.$);