/**
 * This method creates a status bar similar to testplan-tests-progressbar.soy,
 * but without text and with a fixed width value so we can account for a 1px margin that separates each bar
 *
 * @param statusBar the testExecutionStatus bean as a JSON
 * @param totalWidth an integer with the total width in px
 * @param clazz the css class to assign to status bars
 * @returns string html with the rendered status
 */
var renderStatusBar = function (statusBar, totalWidth, clazz) {

    /**
     * this is an implementation of the round half to event (aka bankers' rounding)
     * we must use this to ensure that we don't get a rounding error that would create
     * a bar which is longer than the totalWidth given
     * */
    var roundHalfToEven = function (num) {
        var floored = Math.floor(num);
        var diff = num - floored;

        //half to even
        if (Math.abs(diff - 0.5) < Number.EPSILON) {
            return floored % 2 > 0 ? floored : floored + 1;
        }

        //not half, use normal round
        return Math.round(num);
    };

    var displayableStatuses =  statusBar.statuses.filter(function(status) {
        return status.statusCount > 0;
    });

    /**
     * iteratively find the statuses length in pixels.
     * this is required due to rounding error when we move from continuous (%) to discreet (px)
     * we decrease the available working width by 1 px each iteration
    * */
    var attemptCount = 0;
    do{
        var availableWidth = totalWidth - displayableStatuses.length - attemptCount;
        var content = '<ul class="' + clazz + '">';

        displayableStatuses = displayableStatuses.map(function (status) {
            status['length'] = roundHalfToEven(availableWidth * status.statusPercent / 100);
            return status;
        });

        //sum all widths and add 1 px per status for the margin
        var total = displayableStatuses.length + displayableStatuses.reduce(function(carry, element) {return carry + element.length;}, 0);
        attemptCount++;
    } while(total > totalWidth);

    displayableStatuses.forEach(function (status) {
        content += '<li class="status-bar" style="background-color:' + status.color +
            ';width:' + status.length + 'px;' +
            '" title="' + status.name + ': ' + status.statusCount +
            ' (' + status.statusPercent + '%)" data-status-id="' + status.id + '">' + status.name + '</li>'
    });
    content += '</ul>';
    return content;
};