require(['aui/inline-dialog2']);
(function($) {
    RAVEN.namespace("RAVEN.TestRepository");

    /**
     * Test selection mechanics
     * */
    RAVEN.TestRepository.RENDER_MODE_TEST_TYPE = 'testtype';
    RAVEN.TestRepository.RENDER_MODE_TESTRUN_STATUS = 'testrunstatus';

    var lastSelected;
    var lastTarget;
    var runClickEvent = false;

    function unselectFiles() {
        removeElementsClass('raven-selected-file');
    }

    function unselectFolders() {
        removeElementsClass('selected-folder');
    }

    function removeElementsClass(cssClass) {
        $("." + cssClass).each(function() {
            this.classList.remove(cssClass);
        })
    }

    function selectFile(fileId) {
        selectElement(fileId, 'raven-selected-file');
    }

    function selectFolder(folderId) {
        selectElement(folderId, 'selected-folder');
    }

    function selectElement(id, cssClass) {
        $('#' + id).addClass(cssClass);
    }

    function unselectPreviousRangeFiles() {
        unselectPreviousRange('raven-selected-file');
    }

    function unselectPreviousRange(cssClass) {
        var $anchor = $('#' + lastSelected);
        var $selected = $('#' + lastTarget);

        var list = $anchor.nextAll();
        if (jqueryContains(list, $selected)) {
            unselectUntil(list, $selected, cssClass);
        } else {
            list = $selected.nextAll();
            if (jqueryContains(list, $anchor)) {
                unselectUntil(list, $anchor, cssClass);
            }
        }
        $selected.removeClass(cssClass);
    }

    function toggleSelectionFile() {
        toggleSelection('raven-selected-file');
    }

    function toggleSelection(cssClass) {
        if ($('#' + lastSelected).hasClass(cssClass)) {
            $('#' + lastSelected).removeClass(cssClass);
        } else {
            $('#' + lastSelected).addClass(cssClass);
        }
    }

    function multiselectFiles(file) {
        multiselectElements(file, 'raven-selected-file');
    }

    function multiselectElements(selected, selectClass) {
        var $anchor = $('#' + lastSelected);
        var $selected = $('#' + selected);

        var list = $anchor.nextAll();
        if (jqueryContains(list, $selected)) {
            selectUntil(list, $selected, selectClass);
        } else {
            list = $selected.nextAll();
            if (jqueryContains(list, $anchor)) {
                selectUntil(list, $anchor, selectClass);
            }
        }
    }

    function jqueryContains(list, $element) {
        var stopElement = null;
        list.each(function() {
            if ($(this).attr('id') === $element.attr('id')) {
                stopElement = this;
                return false;
            }
            return true;
        });
        return stopElement !== null;
    }

    function selectUntil(list, $end, selectClass) {
        list.each(function() {
            if ($(this).attr('id') === $end.attr('id')) {
                return false; //break
            }
            $(this).addClass(selectClass);
            return true;
        });
    }

    function unselectUntil(list, $end, selectClass) {
        list.each(function() {
            if ($(this).attr('id') === $end.attr('id')) {
                return false; //break
            }
            $(this).removeClass(selectClass);
            return true;
        });
    }

    RAVEN.TestRepository.TestRepositoryWrapperView = Backbone.View.extend({
        KeyCode: {
            DOM_VK_DELETE: 46
        },

        el: '#raven-test-repository-wrapper',

        elFolderTree: '#raven-test-repository-navigation',

        folderView: null,

        TEMPORARY: 'temporary',

        ALL_TESTS_BACKEND_ID: '-1',

        IS_EDITABLE: true,

        FILE: 'file',

        FOLDER: 'folder',

        DRAG_IMAGE_OFFSET: -15,

        HEIGHT_MARGIN: 2, //this is the margin used to prevent an unnecessary vertical scrollbar from appearing

        COLLAPSE: 'collapse',

        EXPAND: 'expand',

        COLLAPSE_ALL: 'collapse_all',

        EXPAND_ALL: 'expand_all',

        ACTIVATE: 'activate',

        MAX_ITEMS_GUI_EFFECTS: 20,

        MAX_TEST_ITEMS_IN_CREATION_FORM: 200,

        /* null value can be used to add a separator to the menu items */
        CONTEXT_MENU_SEPARATOR: null,

        TESTSET: 'TESTSET',

        TESTPLAN: 'TESTPLAN',

        TESTEXECUTION: 'TESTEXECUTION',

        TEST_CONTEXT_MENU_ID: 'create-issue-dialog-by-test-context-menu',

        FOLDER_CONTEXT_MENU_ID: 'create-issue-dialog-by-folder-context-menu',

        ROOT_FOLDER_CONTEXT_MENU_ID: 'create-issue-dialog-by-root-folder-context-menu',

        currentContextMenuId: null,

        replicateFolderStructure: false,

        addTestExecutionToTestPlan: false,

        isIEorEdge: /msie\s|trident\/|edge\//i.test(window.navigator.userAgent) && !!(document.uniqueID || window.MSInputMethodContext),

        folderImageAddress: '/download/resources/com.xpandit.plugins.xray/images/test-repository/folder_closed_a.png',

        testImageAddress: '/download/resources/com.xpandit.plugins.xray/images/test.png',

        testImageDOM: null,

        MIN_FOLDER_DIV_PERCENTAGE: 35,

        MID_FOLDER_DIV_PERCENTAGE: 55,

        MAX_FOLDER_DIV_PERCENTAGE: 75,

        isResizing: null,

        sidebarWidth: null,

        lastActiveFolder: null,

        projectKey: null,

        entityKey: null,

        createdFolderId: null,

        /**
         * drop event handler proxy:
         * - must store the function so it can be set and reset correctly
         * - must proxy the handler so when the event is run we retain access to the view through 'this'
         */
        dropFolderProxy: null,

        dropLeafProxy: null,

        dragOverLeafProxy: null,

        dropFolderRankProxy: null,

        mouseDownProxy: null,

        dragStartProxy: null,

        folderMouseDownProxy: null,

        dragInfo: { mode: '', folderId: '' },
        /* keeps track of what is being dragged since we can't trust event.dataTransfer */

        visibleTree: [],

        lastActiveTestCustomFieldId: null,

        isContextMenuInitialized: false,

        /* filtering related fields */
        customFieldTestTypeId: null,

        requiresExperimentalDialog2: false,

        filtersInitialized: false,

        filterTestType: null,

        filterVersions: null,

        filterPriority: null,

        filterComponents: null,

        savedFilters: null,

        filterIssueKeys: null, //holds the results of the last jql executed

        FILTER_MODE_SIMPLE: 1,
        FILTER_MODE_ADVANCED: 2,

        testsToBeChunked: null,

        pageStart: null,

        pageSize: null,

        currentPath: null, //this stores the current path whenever we move to another folder. it is used to get the path for issue selector jql

        DEFAULT_PAGE_SIZE: 100,

        EMPTY_FOLDER: 0,

        /*
         * Spin types
         */
        LARGE: 'large',

        MEDIUM: 'medium',

        environmentPicker: null,

        repositoryConfig: null,

        filterDialog : null,

        isInitializing: true,

        callback: null,

        dialogLoaded : false, //flag to store the state of the crete test dialog

        events: {
            'click.raven.test-repository #load-all-bt': '_loadAll',
            'click.raven.test-repository #create-in-root': '_createFolder',

            'mousedown.raven.test-repository #raven-test-repository-drag': '_startResizing',
            'mousemove.raven.test-repository': '_resizing',
            'mouseup.raven.test-repository': '_endResizing',
            'dblclick.raven.test-repository #raven-test-repository-drag': '_autoResize',

            'click.raven.test-repository .aui-icon.icon-close': '_resizeTestsSection',

            'click.raven.test-repository .raven-test-filter-inline-dialog': '_showFilterDialog',

            /** Folder Tree Events */
            'messageEvt #raven-test-repository-navigation': 'messageEvt',
            'clearMessageEvt #raven-test-repository-navigation': 'clearMessageEvt',
            'blockElementEvt #raven-test-repository-navigation': 'blockElemEvt',
            'unblockElementEvt #raven-test-repository-navigation': 'unblockElemEvt',
            'folderSelectEvt #raven-test-repository-navigation': 'folderSelectEvt',
            'folderCreateEvt #raven-test-repository-navigation': 'folderCreateEvt',
            'folderEditEvt #raven-test-repository-navigation': 'folderEditEvt'

        },

        initialize: function(options) {
            if (options) {
                _.extend(this, _.pick(options, "repositoryConfig"));
            } else {
                console.error('Error: unable to initialize filters');
                return;
            }

            this.isResizing = false;
            this.sidebarWidth = $('.aui-sidebar-wrapper').width();
            this.projectKey = $('#selected-project-id').val();
            this.entityKey = $('#selected-entity-id').val();
            this.customFieldTestTypeId = $('#custom-field-test-type-id').val();
            this.requiresExperimentalDialog2 = $('#requires-experimental-dialog-2').val();

            //check permissions
            var self = this;
            this.loadEditionPermission()
                .done(function (data) {
                    self.IS_EDITABLE = data;
                    self._clearMessage();
                    self.requestFolders();
                })
                .fail(function (qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() !== '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }

                    AJS.messages.error('#raven-test-repository-tests-header', {
                        title: errorThrown
                    });
                });
        },

        requestFolders: function() {
            if (this.folderView) {
                this.folderView.destroy();
            }

            var $foldersElem = this.$el.find('#raven-test-repository-navigation');

            var self = this;
            $foldersElem.on('folderInitializedEvt', function(event) {
                self.render();
                if (self.callback) {
                    self.callback.run();
                    self.callback = null;
                }

                if(self.isInitializing === true) {
                    self.initializeRepositoryProperties();
                    self.isInitializing = false;
                } else {
                    //refresh folder information
                    self._refreshFolderInfo();
                }
            });

            this.folderView = new RAVEN.Folder.FolderView({
                'el': $foldersElem,
                'mainView': this,
                'repositoryPath': this._getConfigRepositoryPath(),
                'entityKey': this.entityKey,
                'hasAllFolder': true,
                'hasOrphansFolder': true,
                'rootDescription': this._getConfigRootFolderDescription(),
                'isEditable': this._hasPermission(),
                'hasFolderActions': true
            });
        },

        initializeRepositoryProperties: function() {
            this._initializeScrollProperties();
            this._initializeTestsHeader();

            this._initializeFilters();
            this._initializeEnvironmentPicker();
            this._initializeHeaderCells();
            this._initializeKeyboardEvents();
            this._initializeScrollInfoPanel();
            this._preloadImages();
        },

        loadEditionPermission: function() {
            var self = this;
            return $.ajax({
                type: "GET",
                url: contextPath + '/rest/raven/1.0/' + self._getConfigRepositoryPath() + '/hasRoleToEdit?entityKey=' + self.entityKey,
                contentType: "application/json"
            });
        },

        render: function() {
            if (this.isInitializing === true) {
                this.folderView.activateAllFolder();
            }

            this._adjustTreeHeight();
            $('body#jira').css('overflow', 'auto');

            this._initializeWindowResizeEvent();
            this._initializeScrollEvent();

            this._disableDefaultContextMenu();
            this._contextMenuFolderInitialize();
            this._setDndEvents();
        },

        _getConfigRepositoryPath: function() {
            return this.repositoryConfig.repositoryPath;
        },

        _getConfigJqlConstant: function() {
            return this.repositoryConfig.jqlConstant;
        },

        _getConfigAllowDialogProjectFilter: function() {
            return this.repositoryConfig.allowDialogProjectFilter;
        },

        _getConfigAllowDialogFolderFilter: function() {
            return this.repositoryConfig.allowDialogFolderFilter;
        },

        _getConfigAllowContextMenuTestPlan: function() {
            return this.repositoryConfig.allowContextMenuTestPlan;
        },

        _isTestPlanBoard:function() {
           return  !this._getConfigAllowContextMenuTestPlan();
        },

        _getConfigAllowEnvironmentSelect: function() {
            return this.repositoryConfig.allowEnvironmentSelect;
        },

        _getConfigFilterDefinition: function() {
            return this.repositoryConfig.filterDefinition;
        },

        _getConfigFolderContentRenderer: function() {
            return this.repositoryConfig.folderContentRenderer;
        },

        _getConfigRootFolderDescription: function() {
            return this.repositoryConfig.rootFolderDescription;
        },

        _getConfigfolderJQLName: function() {
            return this.repositoryConfig.folderJQLName;
        },

        _getConfigCreateTestFieldFiller: function() {
            return this.repositoryConfig.createTestFieldFiller;
        },

        _initializeScrollProperties: function() {
            this.testsToBeChunked = null;

            this.pageStart = 0;
            this.pageSize = this.DEFAULT_PAGE_SIZE;

            $('.raven-load-all-entries').css('display', 'inline-block');
            $('.raven-info-entries').css('margin-right', '16px');
        },

        _initializeTestsHeader: function() {
            var header = RAVEN.TestRepository.renderTestsHeader();
            $('#raven-test-repository-tests-header').html(header);

            this._resizeTestsSection();
        },

        _genericBlock: function() {
            var $elem = $('#raven-test-repository-wrapper');
            this._blockAndStartSpin($elem, $elem, this.LARGE);
        },

        _genericUnblock: function() {
            var $elem = $('#raven-test-repository-wrapper');
            $elem.unblock();
            $elem.spinStop();
        },

        _blockAndStartSpin: function($blockElem, $spinElem, spinType) {
            this._clearMessage();
            $blockElem.block();
            if ($spinElem) {
                $spinElem.spin(spinType);
            }
        },

        _unblockAndStopSpin: function($blockElem, $spinElem) {
            $blockElem.unblock();
            if ($spinElem) {
                $spinElem.spinStop();
            }
        },

        _adjustTreeHeight: function() {
            var windowHeight = $(window).outerHeight();
            var topNavBar = $('.aui-header').outerHeight();
            var websudoBanner = $('#websudo-banner').outerHeight();
            var pageHeader = $('.aui-page-header-fixed').outerHeight();

            this.$el.height(windowHeight - topNavBar - websudoBanner - pageHeader - this.HEIGHT_MARGIN);
            // set height for the folders tree and adjust the page content
            var structureTreeHeight = this.$el.height() - $('#raven-folders-footer').height();
            $('#raven-test-repository-tree').height(structureTreeHeight - this.HEIGHT_MARGIN);
            $('#footer').hide();
        },

        _initializeWindowResizeEvent: function() {
            var self = this;
            // Event to adjust divs height when resizing
            $(window).resize(function() {
                var windowHeight = $(window).outerHeight();
                var topNavBar = $('.aui-header').outerHeight();
                var pageHeader = $('.aui-page-header-fixed').outerHeight();
                var foldersFooter = $('#raven-folders-footer').height();

                var $testRepositoryWrapper = $('#raven-test-repository-wrapper');
                var $testRepositoryTree = $('#raven-test-repository-tree');

                $testRepositoryWrapper.height(windowHeight - topNavBar - pageHeader - self.HEIGHT_MARGIN);

                // set height for the folders tree and adjust the page content
                var structureTree = $testRepositoryWrapper.height() - foldersFooter;
                $testRepositoryTree.height(structureTree);
                self._resizeTestsSection();
            });
        },

        _resizeTestsSection: function() {
            var repositoryTestsHeight = $('#raven-test-repository-tests').outerHeight(true);
            var repositoryTestsHeaderHeight = $('#raven-test-repository-tests-header').outerHeight(true);

            $('#raven-test-repository-tests-table').height(repositoryTestsHeight - repositoryTestsHeaderHeight - this.HEIGHT_MARGIN);
        },

        _initializeScrollEvent: function() {
            var self = this;
            $('#raven-test-repository-tests-table').scroll(function(e) {
                var repositoryTestsHeight = $('#raven-test-repository-tests').height();
                var testsFileLength = $('.raven-test-file').length;
                var $testFileAuxHeight = $('.raven-test-file-aux');

                if ($testFileAuxHeight.length > 0 && $testFileAuxHeight.position().top < repositoryTestsHeight &&
                    self.pageSize !== -1 &&
                    testsFileLength === self.pageStart + self.pageSize) {
                    self.pageStart += self.pageSize;

                    self._refreshTestsView();
                }
            });
        },

        _loadFolderTests: function(folderId) {
            this.lastActiveFolder = folderId;

            if (folderId === this.folderView.FOLDER_ROOT) {
                this._getRootFolderContent();
            } else {
                var postDetails = this._getTestsPostDetails(folderId);
                this._getTests(postDetails.url, postDetails.errorMessage);
            }
        },

        _getTestsPostDetails: function(folderId, pageStart, pageSize) {
            var url, errorMessage = '';

            var pager = this._validatePager(pageStart, pageSize);

            if (folderId === this.folderView.ALL_TESTS) {
                url = contextPath + '/rest/raven/1.0/' + this._getConfigRepositoryPath() + '/allTests?';
                errorMessage = AJS.I18n.getText('raven.test-repository.folders.error-retrieving-all-tests');
            } else if (folderId === this.folderView.ALL_ORPHAN_TESTS) {
                url = contextPath + '/rest/raven/1.0/' + this._getConfigRepositoryPath() + '/allOrphanTests?';
                errorMessage = AJS.I18n.getText('raven.test-repository.folders.error-retrieving-orphan-tests');
            } else {
                url = contextPath + '/rest/raven/1.0/' + this._getConfigRepositoryPath() + '/tests?folderId=' + folderId + '&';
                errorMessage = AJS.I18n.getText('raven.test-repository.folders.error-retrieving-tests', folderId);
            }

            url +=  'entityKey=' + this.entityKey
                + '&pageStart=' + pager.pageStart
                + '&pageSize=' + pager.pageSize;

            if (this._getConfigAllowEnvironmentSelect()) {
                url += '&environment=' + this._getSelectedEnvironment();
            }

            var jqlFilter = this._getCurrentFilterJQL();
            if (jqlFilter.length > 0) {
                console.log(jqlFilter);
                url += '&jql= ' + encodeURI(jqlFilter);
            }

            return { 'url': url, 'errorMessage': errorMessage };
        },

        _getRootFolderContent: function() {
            var $testRepositoryTestsTable = $('#raven-test-repository-tests-table');

            this._clearMessage();
            this._clearTestSection();

            this._renderFolderInfo({folderInfo: []});

            var rootFolderMsg = RAVEN.TestRepository.renderRootFolder();
            $testRepositoryTestsTable.html(rootFolderMsg);

            this._setScrollInfo(-1);
        },

        _renderFolderInfo: function(folderInfo) {
            var renderer = this._getConfigFolderContentRenderer();
            renderer.renderContent(folderInfo, this.folderView.getGlobalFolderName());
        },

        _loadAll: function() {
            this.pageSize = -1;

            var folderId = this.folderView.getActiveFolderId();
            this.lastActiveFolder = folderId;

            if (folderId === this.folderView.FOLDER_ROOT) {
                this._getRootFolderContent();
            } else {
                var postDetails = this._getTestsPostDetails(folderId);

                this._getAllTests(postDetails.url, postDetails.errorMessage);
            }

            $('.raven-load-all-entries').css('display', 'none');
            $('.raven-info-entries').css('margin-right', '0px');
        },

        _getAllTests: function(url, errorMessage) {
            var self = this;

            self._clearMessage();
            self._clearTestSection();

            var $blockElem = $('#raven-test-repository-wrapper');
            var $spinElem = $('#raven-test-repository-tests');

            self._blockAndStartSpin($blockElem, $spinElem, self.LARGE);
            $.ajax({
                type: "GET",
                url: url,
                processData: true,
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() !== '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }

                    self.message(errorMessage, errorThrown);
                },
                success: function(data, textStatus, jqXHR) {
                    self.testsToBeChunked = data.testIssues;
                    self._processAllTestsAsync(function(tests, firstElemIndex) {
                        var testsTable = self._renderTests(tests);

                        if (firstElemIndex === 0) {
                            var testFileAux = RAVEN.TestRepository.testFileAux();

                            if (data.total > 0) {
                                $('#raven-test-repository-tests-table').append(testsTable).append(testFileAux);
                            } else {
                                var renderedMessage;
                                if (this.hasFilter) {
                                    renderedMessage = RAVEN.TestRepository.renderNoResults();
                                } else {
                                    renderedMessage = RAVEN.TestRepository.renderEmptyFolder();
                                }
                                $('#raven-test-repository-tests-table').append(renderedMessage);
                            }

                            $('.raven-test-file-aux').spin(self.MEDIUM);
                        } else {
                            $('.raven-test-file-aux').before(testsTable);
                        }

                        self._performLastTestsInitActions(data.total);
                    });
                },
                complete: function() {
                    self._unblockAndStopSpin($blockElem, $spinElem);
                    $('.raven-test-file-aux').spinStop();
                }
            });
        },

        _renderTests: function(tests) {
            return RAVEN.TestRepository.renderTests({
                'tests': tests,
                'contextPath': contextPath,
                'isEditable': this._hasPermission(),
                'mode': this._getTestsRenderMode()
            });
        },

        _getTestsRenderMode: function () {
            return this._getConfigAllowEnvironmentSelect() ? RAVEN.TestRepository.RENDER_MODE_TESTRUN_STATUS : RAVEN.TestRepository.RENDER_MODE_TEST_TYPE;
        },

        loadTests: function(folderId) {
            this._initializeScrollProperties();
            this._loadFolderTests(folderId);
        },

        _getSelectedEnvironment: function () {
            if (!this.environmentPicker
                || !this.environmentPicker.selectedItem
                || !this.environmentPicker.selectedItem.id) {
                return '-1';
            }
            return this.environmentPicker.selectedItem.id;
        },

        // load all tests to the dom splitting the tests in chunks
        // * last two args are optional
        _processAllTestsAsync: function(fn, chunk, context) {
            var self = this;

            var firstElemIndex = 0;

            context = context || window;
            chunk = chunk || this.DEFAULT_PAGE_SIZE;

            function doChunk() {
                var array = self.testsToBeChunked;
                if (array === null) {
                    // Folder has changed so the processing of tests asynchronously must be stopped
                    return;
                }

                var chunkArray = array.slice(firstElemIndex, firstElemIndex + chunk);
                // callback called with args (value, index)
                fn.call(context, chunkArray, firstElemIndex);
                firstElemIndex += chunk;

                if (firstElemIndex < array.length) {
                    // set Timeout for async iteration
                    setTimeout(doChunk, 1);
                } else {
                    self.testsToBeChunked = null;
                }
            }
            doChunk();
        },

        _getTests: function(url, errorMessage) {
            var self = this;

            var $blockElem = $('#raven-test-repository-wrapper');
            var $spinElem = $('#raven-test-repository-tests');

            self._clearMessage();
            self._clearTestSection();

            self._blockAndStartSpin($blockElem, $spinElem, self.LARGE);
            $.ajax({
                type: "GET",
                url: url,
                processData: true,
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() !== '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }

                    self.message(errorMessage, errorThrown);
                },
                success: function(data, textStatus, jqXHR) {
                    self._renderFolderInfo(data.folderInfo);
                    var testsTable = RAVEN.TestRepository.renderTestsTable({
                        'tests': data.testIssues,
                        'contextPath': contextPath,
                        'isEditable': self._hasPermission(),
                        'hasFilter': self._isFilterActive(),
                        'mode': self._getTestsRenderMode()
                    });
                    $('#raven-test-repository-tests-table').html(testsTable);

                    self._performLastTestsInitActions(data.total);
                },
                complete: function() {
                    self._unblockAndStopSpin($blockElem, $spinElem);
                }
            });
        },

        _refreshTestsView: function(pageStart, pageSize) {
            var activeFolderId = this.folderView.getActiveFolderId();
            var postDetails = this._getTestsPostDetails(activeFolderId, pageStart, pageSize);

            this._appendTests(postDetails.url, postDetails.errorMessage, pageStart, pageSize);
        },

        _appendTests: function(url, errorMessage, pageStart, pageSize) {
            var self = this;
            var $blockElem = $('#raven-test-repository-wrapper');
            var $spinElem = $('.raven-test-file-aux');

            self._contextMenuTestInitialize();
            self._blockAndStartSpin($blockElem, $spinElem, self.MEDIUM);

            var pager = self._validatePager(pageStart, pageSize);

            $.ajax({
                type: "GET",
                url: url,
                processData: true,
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() !== '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }

                    self.message(errorMessage, errorThrown);
                },
                success: function(data, textStatus, jqXHR) {
                    var testsTable = self._renderTests(data.testIssues);
                    $spinElem.before(testsTable);

                    self._performLastTestsInitActions(data.total, pager.pageStart, pager.pageSize);
                },
                complete: function() {
                    self._unblockAndStopSpin($blockElem, $spinElem);
                }
            });
        },

        _performLastTestsInitActions: function(total, pageStart, pageSize) {
            this._initTooltips();

            this._contextMenuTestInitialize();

            this._setDndEvents();

            this._setScrollInfo(total, pageStart, pageSize);

            this._resizeTestsSection();
        },

        _setScrollInfo: function(total, pageStart, pageSize) {
            var $infoEntries = $(".raven-info-entries");
            var $loadAllEntries = $('.raven-load-all-entries');
            var entriesMsg;

            pager = this._validatePager(pageStart, pageSize);

            if (total === -1) {
                entriesMsg = AJS.I18n.getText('common.label.showing-records', this.EMPTY_FOLDER, this.EMPTY_FOLDER);
                $infoEntries.css('margin-right', '0px');
                $loadAllEntries.css('display', 'none');
            } else if (pager.pageStart + pager.pageSize >= total || pager.pageSize === -1) {
                entriesMsg = AJS.I18n.getText('common.label.showing-records', total, total);
                $infoEntries.css('margin-right', '0px');
                $loadAllEntries.css('display', 'none');
            } else {
                entriesMsg = AJS.I18n.getText('common.label.showing-records', pager.pageStart + pager.pageSize, total);
            }
            $infoEntries.html(entriesMsg);
        },

        _validatePager: function(pageStart, pageSize) {
            if (!pageStart) {
                pageStart = this.pageStart;
            }

            if (!pageSize) {
                pageSize = this.pageSize;
            }

            return { 'pageStart': pageStart, 'pageSize': pageSize };
        },

        _initTooltips: function() {
            AJS.$('.raven-test-components').tooltip();
            AJS.$('.raven-test-type').tooltip();
            AJS.$('.raven-test-labels').tooltip();
            AJS.$('.raven-test-workflow').tooltip();
            AJS.$('.raven-test-priority').tooltip();
        },

        _removeTestsFromList: function(testIds, targetFolder) {
            var $active = this.folderView.getActiveFolder();
            if ($active) {
                var currentId = $active.attr('id');
                if (currentId === targetFolder
                    || currentId === this.folderView.ALL_TESTS
                    || currentId === this.folderView.FOLDER_ROOT) {
                    return;
                }
            }

            if (testIds.length > this.MAX_ITEMS_GUI_EFFECTS) {
                for (var i = 0; i < testIds.length; i++) {
                    $('#' + testIds[i]).remove();
                }
            } else {
                for (var i = 0; i < testIds.length; i++) {
                    $('#' + testIds[i]).hide('slow', function() { $(this).remove(); });
                }
            }

            if ($('.raven-test-file').size() === testIds.length || $('.raven-test-file').size() === 0) {
                var renderedMessage;
                if (this._isFilterActive()) {
                    renderedMessage = RAVEN.TestRepository.renderNoResults();
                } else {
                    renderedMessage = RAVEN.TestRepository.renderEmptyFolder();
                }
                this.$el.find('#raven-test-repository-tests-table').append(renderedMessage);
            }
        },

        _clearTestSection: function() {
            $('#raven-test-repository-tests-table').empty(); //todo may need to also clear path
        },

        _createFolder: function() {
            this.folderView.createFolder();
        },

        message: function(title, message, type) {
            type = (type === undefined || type === "") ? "error" : type;

            if (!message && title) {
                message = title;
                title = null;
            }

            if (type === "error") {
                this._clearMessage();
                AJS.messages.error('#raven-test-repository-message-bar', {
                    title: title,
                    body: message
                });
            }

            if (type === "success") {
                this._clearMessage();
                AJS.messages.success('#raven-test-repository-message-bar', {
                    title: title,
                    body: message
                });
            }

            if (type === "info") {
                this._clearMessage();
                AJS.messages.info('#raven-test-repository-message-bar', {
                    title: title,
                    body: message
                });
            }

            if(type === "warning"){
                this._clearMessage();
                AJS.messages.warning('#raven-test-repository-message-bar', {
                    title: title,
                    body: message
                });
            }

            this._resizeTestsSection();
        },

        _clearMessage: function() {
            $('#raven-test-repository-message-bar').empty();
        },

        /****************************************************************
         *                        EVENT HANDLERS
         ****************************************************************/

        _testPlanDialog: function(message, onSubmit) {
            var self = this;

            var dialogLabel = {
                'title': AJS.I18n.getText('raven.test-repository.folders.replicate-folder-title'),
                'message': message,
                'buttonYes': AJS.I18n.getText('raven.test-repository.folders.replicate-folder-button-yes'),
                'buttonNo': AJS.I18n.getText('raven.test-repository.folders.replicate-folder-button-no')
            };

            var dialog = new JIRA.FormDialog({
                id: 'raven-test-plan-dialog',
                width: 530,
                height: "250px",
                content: function(callback) {
                    var innerHtml = RAVEN.TestRepository.testPlanReplicateStructureDialog(dialogLabel);

                    $(document).off('click', '#raven-dialog-form-submit-yes');
                    $(document).off('click', '#raven-dialog-form-submit-no');

                    $(document).on('click', '#raven-dialog-form-submit-yes', function() {
                        self.replicateFolderStructure = true;
                    });

                    $(document).on('click', '#raven-dialog-form-submit-no', function () {
                        self.replicateFolderStructure = false;
                    });

                    callback(innerHtml);
                },
                submitHandler: function(e, callback) {
                    e.preventDefault();
                    dialog.hide();
                    onSubmit();
                },
                autoClose: true
            });

            dialog.show();
        },

        _confirmDialog: function(labels, onSubmit) {
            var dialog = new JIRA.FormDialog({
                id: 'raven-delete-folder-dialog',
                width: 530,
                height: "250px",
                content: function(callback) {
                    var innerHtml = RAVEN.Folder.deleteFoldersDialog({ title: labels.title, message: labels.message, confirmButton: labels.confirmButton });

                    callback(innerHtml);
                },
                submitHandler: function(e, callback) {
                    e.preventDefault();
                    dialog.hide();
                    onSubmit();
                },
                autoClose: true
            });
            dialog.show();
        },

        /****************************************************************
         *                   FOLDER TREE EVENT HANDLERS
         ****************************************************************/

        messageEvt: function(event, title, message, type) {
            this.message(title, message, type);
        },

        clearMessageEvt: function() {
            this._clearMessage();
        },

        blockElemEvt: function(event, type) {
            var $blockElem, $spinElem, spinType;
            var $testRepositoryWrapper = $('#raven-test-repository-wrapper');

            switch (type) {
                case 'DELETE_FOLDER':
                    $blockElem = $testRepositoryWrapper;
                    break;
                default :
                    $blockElem = $spinElem = $testRepositoryWrapper;
                    spinType = this.LARGE;
                    break;

                    this._blockAndStartSpin($blockElem, $spinElem, spinType);
            }
        },

        unblockElemEvt: function(event, type) {
            var $blockElem, $spinElem;
            var $testRepositoryWrapper = $('#raven-test-repository-wrapper');

            switch (type) {
                case 'DELETE_FOLDER':
                    $blockElem = $testRepositoryWrapper;
                    break;
                default :
                    $blockElem = $spinElem = $testRepositoryWrapper;
                    break;

                    this._unblockAndStopSpin($blockElem, $spinElem);
            }
        },

        folderSelectEvt: function(event, folderId) {
            this.loadTests(folderId);
        },

        folderCreateEvt: function() {
            this._setDndEvents();
            this._contextMenuFolderInitialize();
        },

        folderEditEvt: function(event, folderName) {
            $('#path-container').find('.raven-path')
                .last()
                .text(folderName);
        },

        /****************************************************************
         *                      RESIZING FUNCTIONS
         ****************************************************************/

        _startResizing: function() {
            var self = this;

            self.isResizing = true;
            $('html').disableSelection();
        },

        _resizing: function(e) {
            var self = this;

            var $structureContainer = $('#raven-test-repository-container');
            var $repositoryNavigation = $('#raven-test-repository-navigation');
            var $repositoryContent = $('#raven-test-repository-content');

            if (self.isResizing) {
                var offsetRight = $structureContainer.width() - (e.clientX - $structureContainer.offset().left);
                var offsetPercentage = (offsetRight * 100) / $structureContainer.width();

                if (offsetPercentage >= self.MIN_FOLDER_DIV_PERCENTAGE && offsetPercentage <= self.MAX_FOLDER_DIV_PERCENTAGE) {
                    $repositoryNavigation.css('right', offsetPercentage + '%');
                    $repositoryContent.css('width', offsetPercentage + '%');

                    self._resizeTestsSection();
                }
            }
        },

        _endResizing: function(e) {
            var self = this;

            self.isResizing = false;
            $('html').enableSelection();
        },

        _autoResize: function(e) {
            var self = this;

            var $structureContainer = $('#raven-test-repository-container');
            var $repositoryNavigation = $('#raven-test-repository-navigation');
            var $repositoryContent = $('#raven-test-repository-content');

            self.isResizing = false;

            var offsetRight = $structureContainer.width() - (e.clientX - $structureContainer.offset().left);
            var offsetPercentage = (offsetRight * 100) / $structureContainer.width();

            if (offsetPercentage < self.MID_FOLDER_DIV_PERCENTAGE) {
                $repositoryNavigation.css('right', '75%');
                $repositoryContent.css('width', '75%');
            } else {
                $repositoryNavigation.css('right', '35%');
                $repositoryContent.css('width', '35%');
            }

            self._resizeTestsSection();
        },

        /****************************************************************
         *                             D&D
         ****************************************************************/

        _setDndEvents: function() {
            var self = this;
            self._resetDndEvents();

            //set and store the event proxy
            self.dropFolderProxy = $.proxy(self._dropHandler, self);
            self.dropLeafProxy = $.proxy(self._dropHandlerForSorting, self);
            self.dropFolderRankProxy = $.proxy(self._dropFolderRankHandler, self);
            self.mouseDownProxy = $.proxy(function (e) {self._testMouseDownHandler(e, self);}, self);
            self.dragStartProxy = $.proxy( function(e) {self._dragStartEvent(e)}, self);
            self.folderMouseDownProxy = $.proxy(self._folderMouseDownHandler, self);
            self.dragOverLeafProxy = $.proxy(self._dragOverLeafHandler, self);
            //test drag
            $(".selectable").each(function() { self._setTestDragEvents(this); });

            //drop on folders
            $(".droppable-folder").each(function() { self._setDropOnFolderEvents(this); });
            self._setDropOnFolderEvents(document.getElementById(this.folderView.FOLDER_ROOT));

            //sort tests
            $(".sortable-file").each(function() { self._setDropOnLeafEvents(this); });

            //folder rank drop events
            $(".raven-folder-rank").each(function() { self._setFolderRankDropEvents(this); });
            //folder drag
            $(".draggable-folder").each(function() { self._setFolderDragEvents(this); });
        },

        /************ RESET EVENTS ****************
         * */

        _resetDndEvents: function() {
            var self = this;

            $(".selectable").each(function() { self._resetTestDragEvents(this); });
            $(".droppable-folder").each(function() { self._resetTestDropEvents(this); });
            $("#folder-root").each(function() { self._resetTestDropEvents(this); });
            $('.raven-folder-rank').each(function() { self._resetRankDropEvent(this); });
            $(".draggable-folder").each(function() { self._resetFolderDragEvents(this); });
            $(".sortable-file").each(function() { self._resetDropOnLeafEvents(this); });
        },

        _resetTestDragEvents: function(element) {
            element.removeEventListener("dragstart", this.dragStartProxy, false);
            element.removeEventListener("mousedown", this.mouseDownProxy, false);
            element.removeEventListener("click", this._testClickHandler, false);
        },

        _resetTestDropEvents: function(element) {
            element.removeEventListener("drop", this.dropFolderProxy, false);
        },

        _resetRankDropEvent: function(element) {
            element.removeEventListener("drop", this.dropFolderRankProxy, false);
        },

        _resetFolderDragEvents: function(element) {
            element.removeEventListener("mousedown", this.folderMouseDownProxy, false);
        },

        _resetDropOnLeafEvents: function(element) {
            element.removeEventListener("drop", this.dropLeafProxy, false);
            element.removeEventListener("dragover", this.dragOverLeafProxy, false);
        },

        /************ DRAG AND CLICK
         * */

        _createDragImage: function(img, caption) {
            var dragCanvas = document.getElementById('drag-canvas');
            if (dragCanvas) {
                dragCanvas.parentNode.removeChild(dragCanvas);
            }

            //create canvas to draw image
            var canvas = document.createElement('canvas');
            canvas.setAttribute("id", "drag-canvas");
            canvas.width = 50;
            canvas.height = 17;
            var context = canvas.getContext('2d');

            //draw caption text
            if (caption) {
                context.fillStyle = '#333333';
                context.font = 'normal 14px Arial, sans-serif';
                context.fillText(caption, 20, 14);
            }

            //draw image
            context.drawImage(img, 0, 0, 16, 16);

            //canvas must be added to the page so it is rendered, so we position it off screen
            canvas.style.position = 'fixed';
            canvas.style.left = '-100%';

            if (document.body.append) {
                document.body.append(canvas);
            }

            return canvas
        },

        _setDragImage: function(img, e, caption) {
            if (this.isIEorEdge) {
                //IE and edge do not support setDragImage by default. this is a workaround
                this._setDragImageMicrosoft(img, e, caption);
                return;
            }

            var canvas = this._createDragImage(img, caption);
            e.dataTransfer.setDragImage(canvas, this.DRAG_IMAGE_OFFSET, this.DRAG_IMAGE_OFFSET);
        },

        _setDragImageMicrosoft: function(img, e, caption) {
            var imageOptions = {
                img: img,
                caption: caption
            };
            e.dataTransfer.setDragImage(imageOptions, this.DRAG_IMAGE_OFFSET, this.DRAG_IMAGE_OFFSET);
        },

        _preloadImages: function() {
            //this preloads the images necessary for drag and drop in IE and Edge
            this.testImageDOM = this._createImageElement(this.testImageAddress);
            if (!this.isIEorEdge) {
                return;
            }
            window.setDragImageIEPreload(this._createImageElement(this.folderImageAddress).src);
            window.setDragImageIEPreload(this._createImageElement(this.testImageAddress).src);
        },

        _setTestDragEvents: function(element) {
            element.addEventListener("dragstart", this.dragStartProxy, false);
            element.addEventListener("mousedown", this.mouseDownProxy, false);
            element.addEventListener("click", this._testClickHandler, false);
        },

        _getDragFileCount: function () {
            return $('.raven-selected-file').length;
        },

        _createImageElement: function(url) {
            var img = document.createElement('img');
            img.src = contextPath + url;
            return img;
        },

        _setFolderDragEvents: function(element) {
            var img = this._createImageElement(this.folderImageAddress);

            var self = this;
            element.addEventListener("dragstart", function (e) {
                e.dataTransfer.setData("Text", self.FOLDER);
                self.dragInfo.mode = self.FOLDER;
                self.dragInfo.folderId = self._getTargetFolderId($(e.target));
                self._setDragImage(img, e, '');
            }, false);

            element.addEventListener("mousedown", this.folderMouseDownProxy, false);
            element.addEventListener("click", this._folderClickHandler, false);
        },


        /**
         * test event state machine
         * */
        _testClickHandler: function(e) {
            //this only runs after the mouse down handler has run before
            if (!runClickEvent) {
                return;
            }

            unselectFiles(); //clear previous test selection
            lastSelected = this.id; //store reference to selected as anchor
        },

        _testMouseDownHandler: function(e, self) {
            self._closeFilters();
            unselectFolders();
            runClickEvent = false;

            if (self._disallowDragMicrosoft(e)) {
                self._stopEvents(e);
                return;
            }

            if (e.shiftKey) {
                //shift does not change the last selected value since it is an anchor that remains across shift+click
                unselectPreviousRangeFiles(); //unselects previous shift selection
                $('#' + lastSelected).addClass('raven-selected-file'); //makes sure the anchor remains selected
                selectFile(e.currentTarget.id); //selects current target
                multiselectFiles(e.currentTarget.id); //selects from anchor to target
                lastTarget = e.currentTarget.id; //store reference to target
            } else if ((e.metaKey && self._isMacOS()) || e.ctrlKey) {
                lastSelected = e.currentTarget.id; //update anchor
                //remove from list if already included
                toggleSelectionFile();
            } else {
                //if mouse down on something already selected, only restart selection after click event occurs
                if ($('#' + e.currentTarget.id).hasClass('raven-selected-file')) {
                    runClickEvent = true;
                    return;
                }
                //normal click/start of new selection
                unselectFiles(); //clear previous selection
                lastSelected = e.currentTarget.id; //store reference to selected as anchor
                selectFile(e.currentTarget.id); //add to selected list
            }
        },

        _dragStartEvent: function(e) {
            if (this._disallowDragMicrosoft(e)) {
                this._stopEvents(e);
                return;
            }
            e.dataTransfer.effectAllowed = 'copyMove';
            e.dataTransfer.setData("Text", this.FILE);
            this.dragInfo.mode = this.FILE;
            this.dragInfo.folderId = '';
            this._setDragImage(this.testImageDOM, e, this._getDragFileCount());
        },

        _disallowDragMicrosoft: function(e) {
            return this.isIEorEdge && $(e.target).hasClass('noiedrag');
        },

        /******** folder event state machine ************/

        _folderClickHandler: function(e) {
            if (!runClickEvent) {
                return;
            }

            unselectFiles(); //clear previous selection
            unselectFolders(); //clear previous selection
            lastSelected = this.id; //store reference to selected as anchor
            selectFolder(this.id); //add to selected list
        },

        _folderMouseDownHandler: function(e) {
            var $folder = $(e.target).closest('.raven-folder-item').first();
            var id = $folder.attr("id");
            this._closeFilters();
            unselectFiles();
            runClickEvent = false;

            //there is no multi selection for folders at this time
            if ($('#' + id).hasClass('selected-folder')) {
                runClickEvent = true;
                return;
            }
            //normal click/start of new selection
            unselectFolders(); //clear previous selection
            lastSelected = id; //store reference to selected as anchor
            selectFolder(id); //add to selected list
        },

        _dragOverLeafHandler: function (e) {
            e.preventDefault();
            var target = $(e.target).closest('.sortable-file');
            var activeFolderId = this.folderView.getActiveFolderId();
            if (target && activeFolderId !== this.folderView.ALL_TESTS
                && activeFolderId !== this.folderView.ALL_ORPHAN_TESTS
                && activeFolderId !== this.folderView.FOLDER_ROOT) {
                target.addClass('raven-dragged-over-file-target');
            }
        },

        /************ DROP ON LEAF EVENTS **********/

        _setDropOnLeafEvents: function(droppable) {
            var self = this;
            droppable.addEventListener("drop", self.dropLeafProxy, false);
            droppable.addEventListener("dragenter", function(e) {
                e.preventDefault();
            }, false);

            droppable.addEventListener("dragover", self.dragOverLeafProxy, false);

            droppable.addEventListener("dragleave", self._dragLeaveOnLeafHandler, false);

            $(droppable).greedy = true;
        },

        _dragLeaveOnLeafHandler: function(e) {
            e.preventDefault();
            var target = $(e.target).closest('.sortable-file');
            if (target) {
                target.removeClass('raven-dragged-over-file-target');
            }
        },

        _isMacOS: function() {
            return navigator.appVersion.indexOf("Mac") !== -1;
        },

        /************ DROP ON FOLDER EVENTS ************/

        _setDropOnFolderEvents: function(droppable) {
            var self = this;
            droppable.addEventListener("drop", self.dropFolderProxy, false);
            droppable.addEventListener("dragenter", function(e) {
                e.preventDefault();
            }, false);

            droppable.addEventListener("dragover", function(e) {
                e.preventDefault();
                var target = $(e.target).closest('.droppable-folder');
                if (target) {
                    target.addClass('raven-dragged-over-folder-target');
                }
            }, false);

            droppable.addEventListener("dragleave", self._dragLeaveOnFolderHandler, false);

            $(droppable).greedy = true;
        },

        _setFolderRankDropEvents: function(droppable) {
            var self = this;
            droppable.addEventListener("drop", self.dropFolderRankProxy, false);
            droppable.addEventListener("dragenter", function(e) {
                e.preventDefault();
            }, false);

            droppable.addEventListener("dragover", function(e) {
                e.preventDefault();
                //self._selectRankOver($(e.target));
                if (self.dragInfo.mode === self.FOLDER) {
                    $(e.target).addClass('raven-dragged-over-rank');
                }
            }, false);

            droppable.addEventListener("dragleave", self._dragLeaveRankHandler, false);

            $(droppable).greedy = true;
        },

        _dragLeaveOnFolderHandler: function(e) {
            e.preventDefault();
            var target = $(e.target).closest('.droppable-folder');
            if (target) {
                target.removeClass('raven-dragged-over-folder-target');
            }
        },

        _getTargetFolderId: function($targetElement) {
            if (!$targetElement) {
                return '-1';
            }

            var $parents = $targetElement.parents('.raven-folder-item');
            if (!$parents) {
                return '-1';
            }
            return $parents.prop('id');
        },

        _getTargetParentFolderId: function($targetElement) {
            if (!$targetElement) {
                return '-1';
            }

            var $parents = $targetElement.parents('.raven-folder-item');
            if (!$parents) {
                return '-1';
            }
            return $parents.first().attr('data-parent-id');
        },

        _getTargetTestId: function($targetElement) {
            if (!$targetElement) {
                return '-1';
            }

            var $parents = $targetElement.parents('.raven-test-file');
            if (!$parents) {
                return '-1';
            }
            return $parents.prop('id');
        },

        _dropHandler: function(e) {
            e.preventDefault();
            e.stopPropagation();

            var self = this;
            var targetFolderId = this._getTargetFolderId($(e.target));

            this._dragLeaveOnFolderHandler(e);

            if (targetFolderId === '-1') {
                return;
            }

            var items;
            if (e.dataTransfer.getData('Text') === this.FILE) {
                if (targetFolderId === this.folderView.FOLDER_ROOT || targetFolderId === self.folderView.getActiveFolderId()) {
                    return;
                }

                items = this._getIdsOfClass('raven-selected-file');
                if (!items.length) {
                    return;
                }

                self._moveTestsRequest(targetFolderId, items);
            } else if (e.dataTransfer.getData('Text') === this.FOLDER) {
                items = this._getIdsOfClass('selected-folder');
                if (!items.length) {
                    return;
                }
                self._moveFoldersRequest(targetFolderId, items);
            }
        },

        _getIdsOfClass: function(cssClass) {
            var items = [];
            $('.' + cssClass).each(function() {
                items.push($(this).attr('id'));
            });
            return items;
        },

        _dropHandlerForSorting: function(e) {
            e.preventDefault();
            e.stopPropagation();
            this._dragLeaveOnLeafHandler(e);

            if (!this._allowFolderRankEdit()) {
                return;
            }

            var targetTestId = this._getTargetTestId($(e.target));

            if (targetTestId === '-1') {
                return;
            }

            if (targetTestId === undefined || targetTestId === null) {
                targetTestId = $(e.target).prop('id');
            }

            //build list of selected items
            var items = [];
            $('.raven-selected-file').each(function() {
                items.push($(this).attr('id'));
            });

            if (!items.length || $.inArray(targetTestId, items) > -1) {
                return;
            }

            this._reRankTestsRequest(targetTestId, items);
        },

        _moveTestsRequest: function(destinationId, testIds) {
            var self = this;

            var $blockElem = $('#raven-test-repository-wrapper');
            var $spinElem = $blockElem;

            self._blockAndStartSpin($blockElem, $spinElem, self.LARGE);
            $.ajax({
                type: "PUT",
                url: contextPath + '/rest/raven/1.0/' + self._getConfigRepositoryPath() + '/moveTests?destinationId=' + destinationId,
                processData: true,
                async: true,
                data: JSON.stringify(testIds),
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() !== '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }

                    self.message(AJS.I18n.getText('raven.test-repository.error-moving-files', destinationId), errorThrown);
                },
                success: function(data, textStatus, jqXHR) {
                    if (jqXHR.status === 304) {
                        self.loadTests(self.lastActiveFolder);
                        return;
                    }
                    //remove items
                    self._removeTestsFromList(testIds, destinationId);

                    self._subRefreshPostTestOperation(testIds.length);

                    self._messageFlag(AJS.I18n.getText('raven.test-repository.tests-moved-to-folder', self.folderView.getFolderName(destinationId)));
                },
                complete: function() {
                    self._unblockAndStopSpin($blockElem, $spinElem);
                }
            });
        },

        _moveCreatedTestToFolder: function(folderId, testId, testKey) {
            var self = this;

            var $blockElem = $('#raven-test-repository-wrapper');
            var $spinElem = $blockElem;

            self._blockAndStartSpin($blockElem, $spinElem, self.LARGE);
            $.ajax({
                type: "PUT",
                url: contextPath + '/rest/raven/1.0/' + self._getConfigRepositoryPath() + '/moveCreatedTestToFolder?entityKey=' + this.entityKey + '&folderId=' + folderId + '&testId=' + testId,
                processData: true,
                async: true,
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() !== '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }
                    self.message(AJS.I18n.getText('raven.test-repository.error-moving-files', folderId), errorThrown);
                },
                success: function(data, textStatus, jqXHR) {
                    self._subRefreshPostTestOperation(1);
                    self.loadTests(folderId);
                    self._messageFlag(AJS.I18n.getText('raven.test-repository.created-test-on-folder', testKey, self.folderView.getFolderName(folderId)));
                },
                complete: function() {
                    self._unblockAndStopSpin($blockElem, $spinElem);
                }
            });
        },

        _subRefreshPostTestOperation: function (length) {
            //refresh test count
            this.folderView.refreshFolderTestCount();

            //refresh Tests View
            this._refreshTestAfterMove(length);

            //refresh folder information
            this._refreshFolderInfo();
        },


        _refreshTestAfterMove: function(testIdsSize) {
            var activeFolderId = this.folderView.getActiveFolderId();
            var startValue = this.pageStart + this.pageSize - testIdsSize;

            if (this.pageSize === -1) {
                this._loadAll();
            } else if (activeFolderId !== this.folderView.ALL_TESTS && activeFolderId !== this.folderView.FOLDER_ROOT) {
                this._refreshTestsView(startValue, testIdsSize);
            }
        },

        _refreshFolderInfo: function() {
            var activeFolderId = this.folderView.getActiveFolderId();
            var self = this;
            var queryParams = '?folderId=' + activeFolderId + '&entityKey=' + self.entityKey;

            if (this._getConfigAllowEnvironmentSelect()) {
                queryParams += '&environment=' + this._getSelectedEnvironment();
            }

            $.ajax({
                type: "GET",
                url: contextPath + '/rest/raven/1.0/' + self._getConfigRepositoryPath() + '/folderInfo' + queryParams,
                processData: true,
                async: true,
                contentType: "application/json",
                success: function(data) {
                    self._renderFolderInfo(data);
                }
            });
        },

        _addTestsRequest: function(destinationId, testKeys) {
            this._genericBlock();
            var self = this;
            $.ajax({
                type: "PUT",
                url: contextPath + '/rest/raven/1.0/' + self._getConfigRepositoryPath() + '/moveTestKeys?entityKey=' + self.entityKey + '&destinationId=' + destinationId,
                processData: true,
                async: true,
                data: JSON.stringify(testKeys),
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() !== '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }

                    self.folderView.refreshFolderTestCount();
                    self._loadFolderTests(self.folderView.getActiveFolderId());
                    self.message(AJS.I18n.getText('raven.test-repository.error-moving-files', destinationId), errorThrown);
                },
                success: function(data, textStatus, jqXHR) {
                    //refresh test count
                    self.folderView.refreshFolderTestCount();
                    self._loadFolderTests(self.folderView.getActiveFolderId());
                    self._messageFlag(AJS.I18n.getText('raven.test-repository.tests-moved-to-folder', self.folderView.getFolderName(destinationId)));
                },
                complete: function() {
                    self._genericUnblock();
                }
            });
        },

        _moveFoldersRequest: function(destinationId, folderIds) {
            var self = this;
            var destination = destinationId === self.folderView.FOLDER_ROOT ? self.ALL_TESTS_BACKEND_ID : destinationId;

            var $blockElem = $('#raven-test-repository-wrapper');
            var $spinElem = $blockElem;

            self._blockAndStartSpin($blockElem, $spinElem, self.LARGE);
            $.ajax({
                type: "PUT",
                url: contextPath + '/rest/raven/1.0/' + self._getConfigRepositoryPath() + '/moveFolders?destination=' + destination + '&entityKey=' + self.entityKey,
                processData: true,
                async: true,
                data: JSON.stringify(folderIds),
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() !== '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }

                    if(folderIds && folderIds.length === 1 && folderIds[0] === destinationId) {
                        // don't display error message
                        return;
                    }

                    //add error message
                    self.message(AJS.I18n.getText('raven.test-repository.error-moving-folders', destinationId), errorThrown);
                    //remove dragged over look
                    $('.raven-dragged-over-folder-target').removeClass('raven-dragged-over-folder-target');
                },
                success: function(data, textStatus, jqXHR) {
                    var treeState = self._buildTreeState();
                    treeState.movedFolder = folderIds;
                    treeState.destination = destination;
                    //fetch fresh tree
                    //send callback function to run after refresh (which is async) is done
                    self.callback = {
                        method: $.proxy(self._reloadPageState, self), //proxy to keep scope this as the view
                        data: treeState,
                        run: function() {
                            this.method(this.data);
                        }
                    };
                    self.requestFolders();
                    self._messageFlag(AJS.I18n.getText('raven.test-repository.folder-moved-to-folder', self.folderView.getFolderName(folderIds), self.folderView.getFolderName(destinationId)));
                },
                complete: function() {
                    self._unblockAndStopSpin($blockElem, $spinElem);
                }
            });
        },

        /************** Rank events *******/

        _dragLeaveRankHandler: function(e) {
            e.preventDefault();
            e.stopPropagation();

            $('.raven-dragged-over-rank').each(function() {
                $(this).removeClass('raven-dragged-over-rank');
            })
        },

        _dropFolderRankHandler: function(e) {
            e.preventDefault();
            e.stopPropagation();

            this._dragLeaveRankHandler(e);

            //check if we are dragging a folder
            if (this.dragInfo.mode !== this.FOLDER) {
                return;
            }

            var targetParentId = this._getTargetParentFolderId($(e.target)); //get parent id of target
            var targetFolderId = this._getTargetFolderId($(e.target)); //id of the folder where it was dropped
            var movedFolderId = this.dragInfo.folderId;

            this._rankFoldersRequest(targetParentId, targetFolderId, movedFolderId);
        },

        _rankFoldersRequest: function(parentId, targetId, folderId) {
            var self = this;
            var parent = parentId === self.folderView.FOLDER_ROOT ? self.ALL_TESTS_BACKEND_ID : parentId;

            var $blockElem = $('#raven-test-repository-wrapper');
            var $spinElem = $blockElem;

            self._blockAndStartSpin($blockElem, $spinElem, self.LARGE);
            $.ajax({
                type: "PUT",
                url: contextPath + '/rest/raven/1.0/' + self._getConfigRepositoryPath() + '/rankFolder?parent=' + parent + '&targetId=' + targetId + '&folderId=' + folderId,
                processData: true,
                async: true,
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() !== '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }

                    //add error message
                    self.message(AJS.I18n.getText('raven.test-repository.error-sorting-folders'), errorThrown);
                    //remove dragged over look
                },
                success: function(data, textStatus, jqXHR) {
                    var treeState = self._buildTreeState();
                    treeState.movedFolder = folderId;
                    //fetch fresh tree
                    //send callback function to run after refresh (which is async) is done
                    self.callback = {
                        method: $.proxy(self._reloadPageState, self), //proxy to keep scope this as the view
                        data: treeState,
                        run: function() {
                            this.method(this.data);
                        }
                    };
                    self.requestFolders();
                },
                complete: function() {
                    self._unblockAndStopSpin($blockElem, $spinElem);
                }
            });
        },

        _reRankTestsRequest: function(targetLeaf, testIds) {
            var self = this;

            var $blockElem = $('#raven-test-repository-wrapper');
            var $spinElem = $blockElem;
            var folderId = this.folderView.getActiveFolderId();
            self._blockAndStartSpin($blockElem, $spinElem, self.LARGE);
            $.ajax({
                type: "PUT",
                url: contextPath + '/rest/raven/1.0/' + self._getConfigRepositoryPath() + '/sortTests?folderId=' + folderId + '&targetLeaf=' + targetLeaf,
                processData: true,
                async: true,
                data: JSON.stringify(testIds),
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() !== '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }

                    self.message(AJS.I18n.getText('raven.test-repository.error-sorting-files', targetLeaf), errorThrown);
                },
                success: function(data, textStatus, jqXHR) {
                    //remove items
                    self._reRankTests(testIds, targetLeaf);
                },
                complete: function() {
                    self._unblockAndStopSpin($blockElem, $spinElem);
                }
            });
        },

        _reRankTests: function(testIds, targetLeaf) {
            if (targetLeaf === '-1') {
                if ($('.raven-test-file').length > 1) {
                    $.each(testIds, function(index, value) {
                        $('#' + value + '.raven-test-file')
                            .detach()
                            .insertAfter($('.raven-test-file').last());
                    });
                }
            } else {
                var targetLeafElement = $('#' + targetLeaf + '.raven-test-file');

                $.each(testIds, function(index, value) {
                    $('#' + value + '.raven-test-file')
                        .detach()
                        .insertBefore(targetLeafElement);
                });
            }
        },

        /********* Reload management ************/

        _reloadPageState: function(treeState) {
            //reload visible rows -> this basically expands the folders that were expanded before we reloaded
            this._loadVisibleRows(treeState.visibleRows);

            if (!treeState.activeFolderId) {
                return;
            }

            var selectedElement = $('#' + treeState.activeFolderId);
            //update path line. there is no path to the 'All' folder
            if (treeState.activeFolderId !== this.folderView.FOLDER_ROOT) {
                this.folderView.updateTreeDotLineStructure(selectedElement, this.ACTIVATE);
            }
            this.folderView.unhighlightOutOfPathFolders(selectedElement.parents('li'));

            //set active folder
            this._reloadActiveFolder(treeState.activeFolderId, treeState.activeFolderIcon);

            //set open folder icons while maintaining their highlighting
            this._loadExpandedIcons(treeState.expandedRows);

            $('#' + treeState.destination).find('ul').first().show();

            $('#raven-test-repository-tree').scrollTop(treeState.scrollTop);
        },

        _reloadActiveFolder: function(id, icon) {
            var self = this;

            var activeElement = $('#' + id);
            activeElement.addClass('raven-activeFolder');
            if (id === self.folderView.ALL_TESTS
                || id === self.folderView.ALL_ORPHAN_TESTS
                || id === self.folderView.FOLDER_ROOT) {
                self.folderView.addActiveFolderGlobalStyle(activeElement);
            } else {
                self.folderView.addActiveFolderStyle(activeElement);
            }

            var $icon = $('#icon-' + id);
            if ($icon) {
                $icon.removeClass('raven-folder-closed-c');
                $icon.removeClass('raven-folder-closed-b');
                $icon.addClass(icon);
            }
        },

        _loadVisibleRows: function(visibleRows) {
            //'shows' the visible uls
            var rows = this._getTreeRows();
            rows.each(function() {
                var indexOfId = visibleRows.indexOf($(this).attr('id'));
                if (indexOfId >= 0) {
                    $(this).show();
                }
            });
        },

        _loadExpandedIcons: function(expandedRows) {
            expandedRows.forEach(function(item, index) {
                var $iconElement = $('#icon-' + item);
                if (!$iconElement) {
                    return;
                }
                if ($iconElement.hasClass('raven-folder-closed-c')) {
                    $iconElement.removeClass('raven-folder-closed-c');
                    $iconElement.addClass('raven-folder-open-c');
                } else if ($iconElement.hasClass('raven-folder-closed-b')) {
                    $iconElement.removeClass('raven-folder-closed-b');
                    $iconElement.addClass('raven-folder-open-b');
                }
            });
        },

        _buildTreeState: function() {
            var visibleRows = this._getVisibleTreeRows();
            var visibleTree = [];
            var id, i;
            for (i = 0; i < visibleRows.length; i++) {
                id = this._getRowId(visibleRows[i]);
                if (id) {
                    visibleTree.push(id);
                }
            }

            var expandedRows = [];
            var cropLength = 'ul-'.length;
            for (i = 0; i < visibleRows.length; i++) {
                id = this._getRowId(visibleRows[i]);
                if (!id) {
                    continue;
                }
                id = id.substr(cropLength);
                var $icon = $('#icon-' + id);
                if ($icon.hasClass('raven-folder-open-b') || $icon.hasClass('raven-folder-open-c')) {
                    expandedRows.push(id);
                }
            }

            var scrollTop = $('#raven-test-repository-tree').scrollTop();
            return {
                activeFolderIcon: this.folderView.getActiveFolderIcon(),
                activeFolderId: this.folderView.getActiveFolderId(),
                visibleRows: visibleTree,
                expandedRows: expandedRows,
                scrollTop: scrollTop
            };
        },

        _getTreeRows: function() {
            return $('.raven-folder-tree-level');
        },

        _getVisibleTreeRows: function() {
            return this._getTreeRows().filter(function() {
                return $(this).css('display') !== 'none';
            });
        },

        _getRowId: function(row) {
            return $(row).attr('id');
        },

        _getActiveFolderId: function() {
            return $('.activeFolder').attr('id');
        },

        _initializeScrollInfoPanel: function() {
            var self = this;

            var scrollInfoPanel = RAVEN.TestRepository.renderScrollInfoPanel({ 'requiresExperimentalDialog2': self.requiresExperimentalDialog2 });

            $('#raven-scroll-info-panel').html(scrollInfoPanel);
        },

        /****************************************************************
         *                   FILTERS
         ****************************************************************/
        _initializeFilters: function() {
            if (this.filterDialog !== null) {
                return;
            }

            var self = this;
            this.filterDialog = new RAVEN.Filters.FilterDialog({
                groups: self._getConfigFilterDefinition().groups,
                colCount: 3,
                filters: self._getConfigFilterDefinition().filters,
                projectKey: self.projectKey,
                requiresExperimentalDialog2: self.requiresExperimentalDialog2,
                domContainer: '#raven-search-container',
                customFieldTestTypeId: self.customFieldTestTypeId,
                applyCallback: $.proxy(self._applyDialog, self),
                jqlConstant: self._getConfigJqlConstant()
            });
        },

        _initializeEnvironmentPicker: function() {
            if (!this._getConfigAllowEnvironmentSelect()) {
                return;
            }

            var self = this;
            $.ajax({
                type: "GET",
                url: contextPath + '/rest/raven/1.0/testenvironment/listWithDefaults',
                processData: true,
                contentType: "application/json",
                success: function(data, textStatus, jqXHR) {
                    var pickerHTML = RAVEN.TestRepository.renderEnvironmentPicker({
                        'environments': data
                    });

                    $('#raven-environment-cell').html(pickerHTML);

                    var $picker  = $('#raven-environment-picker');
                    var options = $.map($picker.find('option'), self._selectOptionToSubtleOption);
                    var selectedOption =  $.map($picker.find('option:selected'), self._selectOptionToSubtleOption);
                    var subtlePicker = new RAVEN.Field.SubtleSingleSelect(options, selectedOption[0], $('#raven-environment-cell'));
                    subtlePicker.render();

                    self.environmentPicker = subtlePicker;
                    $(subtlePicker).on('selectedItemChanged', function(e) {
                        self._loadFolderTests(self.folderView.getActiveFolderId());
                    });
                },
            });
        },

        _selectOptionToSubtleOption: function(option) {
            return {
                'id': option.value,
                'name': option.title,
            };
        },

        _initializeHeaderCells: function() {
            if (!this._getConfigAllowEnvironmentSelect()) {
                $('#raven-environment-cell').hide();
            }
        },

        _showFilterDialog: function() {
            $('.raven-test-filter-inline-dialog').show();
        },

        _getCurrentFilterJQL: function() {
            if (this.filterDialog === null) {
                return '';
            }
            return this.filterDialog.getCurrentJQL()
        },

        _applyDialog: function() {
            this._initializeScrollProperties();
            this._loadFolderTests(this.folderView.getActiveFolderId());
        },

        _closeFilters: function () {
            if (this.filterDialog === null) {
                return;
            }
            this.filterDialog.closeFilters();
        },

        /****************************************************************
         *                   Context Menu Functions
         ****************************************************************/

        /**
         * Initialize the context menu listeners for supported Folder items only.
         *
         * @private
         */
        _contextMenuFolderInitialize: function() {
            var folder = $('.raven-folders-view-mode');
            var rootFolder = $('#raven-global-folder-root');

            folder.off('contextmenu');
            rootFolder.off('contextmenu');

            // 1. Init menu actions
            this._contextMenuInitActions();

            var self = this;
            // 2. Sets the right click as left click behavior.
            folder.contextmenu(function(e) {
                self.currentContextMenuId = self.FOLDER_CONTEXT_MENU_ID;
                self.folderView.folderItemClick(e, true);
            });

            rootFolder.contextmenu(function(e) {
                self.currentContextMenuId = self.ROOT_FOLDER_CONTEXT_MENU_ID;
                self.folderView.globalFolderItemClick(e, true);
            });

            // 3. Sets the context-menu items
            var rootFolderItems = [
                contextMenuItemFactory(AJS.I18n.getText('raven.test-repository.expand-all-folder'), $.proxy(self.folderView.expandAll, self.folderView), alwaysEnabled),
                contextMenuItemFactory(AJS.I18n.getText('raven.test-repository.collapse-all-folder'), $.proxy(self.folderView.collapseAll, self.folderView), alwaysEnabled),
                self.CONTEXT_MENU_SEPARATOR,
                contextMenuItemFactory(AJS.I18n.getText('raven.test-repository.create-folder-test-set'), $.proxy(self._testSetAction, self)),
                contextMenuItemFactory(AJS.I18n.getText('raven.test-repository.add-tests-test-set'), $.proxy(self._associateTestSetAction, self)),
                self.CONTEXT_MENU_SEPARATOR
            ];

            if (this._getConfigAllowContextMenuTestPlan()) {
                rootFolderItems = rootFolderItems.concat([
                    contextMenuItemFactory(AJS.I18n.getText('raven.test-repository.create-folder-test-plan'), $.proxy(self._testPlanActionDialog, self)),
                    contextMenuItemFactory(AJS.I18n.getText('raven.test-repository.add-tests-test-plan'), $.proxy(self._associateTestPlanFolderActionDialog, self)),
                    self.CONTEXT_MENU_SEPARATOR
                ]);
            }

            rootFolderItems = rootFolderItems.concat([
                contextMenuItemFactory(AJS.I18n.getText('raven.test-repository.create-folder-test-exec'), $.proxy(self._testExecutionAction, self)),
                contextMenuItemFactory(AJS.I18n.getText('raven.test-repository.add-tests-test-exec'), $.proxy(self._associateTestExecAction, self))
            ]);
            rootFolder.contextPopup({items: rootFolderItems});

            var viewActionItems = [
                contextMenuItemFactory(AJS.I18n.getText('raven.test-repository.expand-all-folder'), $.proxy(self.folderView.expandAll, self.folderView), alwaysEnabled),
                contextMenuItemFactory(AJS.I18n.getText('raven.test-repository.collapse-all-folder'), $.proxy(self.folderView.collapseAll, self.folderView), alwaysEnabled),
                self.CONTEXT_MENU_SEPARATOR
            ];

            var actionItemsForFolders = [];

            actionItemsForFolders = actionItemsForFolders.concat(viewActionItems);

            if (this._hasPermission()) {

                var addTestsLabel = AJS.I18n.getText('common.action.add-tests');

                if(this._isTestPlanBoard()) {
                    addTestsLabel = AJS.I18n.getText('common.action.move-tests');
                }

                actionItemsForFolders = actionItemsForFolders.concat([
                    contextMenuItemFactory(AJS.I18n.getText('common.action.create-test-here'), $.proxy(self._createTestsAction, self), $.proxy(self._isCreateTestsAllowed, self)),
                    contextMenuItemFactory(addTestsLabel, $.proxy(self._addTestsAction, self), $.proxy(self._isAddTestsAllowed, self)),
                    self.CONTEXT_MENU_SEPARATOR
                ]);
            }

            actionItemsForFolders = actionItemsForFolders.concat([
                contextMenuItemFactory(AJS.I18n.getText('raven.test-repository.create-folder-test-set'), $.proxy(self._testSetAction, self)),
                contextMenuItemFactory(AJS.I18n.getText('raven.test-repository.add-tests-test-set'), $.proxy(self._associateTestSetAction, self))
            ]);

            if (this._getConfigAllowContextMenuTestPlan()) {
                actionItemsForFolders = actionItemsForFolders.concat([
                    self.CONTEXT_MENU_SEPARATOR,
                    contextMenuItemFactory(AJS.I18n.getText('raven.test-repository.create-folder-test-plan'), $.proxy(self._testPlanActionDialog, self)),
                    contextMenuItemFactory(AJS.I18n.getText('raven.test-repository.add-tests-test-plan'), $.proxy(self._associateTestPlanFolderActionDialog, self)),
                ]);
            }

            actionItemsForFolders = actionItemsForFolders.concat([
                self.CONTEXT_MENU_SEPARATOR,
                contextMenuItemFactory(AJS.I18n.getText('raven.test-repository.create-folder-test-exec'), $.proxy(self._testExecutionAction, self)),
                contextMenuItemFactory(AJS.I18n.getText('raven.test-repository.add-tests-test-exec'), $.proxy(self._associateTestExecAction, self))
            ]);

            folder.contextPopup({
                items: actionItemsForFolders,
                disabledMessage: AJS.I18n.getText('raven.test-repository.no-test-associated')
            });

            function contextMenuItemFactory(label, action, enable) {
                function enabledAction() {
                    if (enable) {
                        return enable();
                    } else if (action) {
                        return isFolderActionEnabled();
                    }
                    return false;
                }

                return {
                    label: label,
                    action: action,
                    isEnabled: enabledAction
                };
            }

            function alwaysEnabled() {
                return true;
            }

            function isFolderActionEnabled() {
                return getActiveFolderTestCount() > 0;
            }

            function getActiveFolderTestCount() {
                var $textCount;

                if (this.currentContextMenuId === this.ROOT_FOLDER_CONTEXT_MENU_ID) {
                    $textCount = $('#folder-root')
                        .find('.folder-root-test-counter')
                        .first();
                } else {
                    $textCount = $('#' + this.lastActiveFolder)
                        .find('.totalTestCount')
                        .first();
                }

                if ($textCount) {
                    return $textCount.text().trim();
                }

                return 0;
            }
        },

        /**
         * Initialize the context menu listeners for Test items.
         *
         * @private
         */
        _contextMenuTestInitialize: function() {
            var self = this;

            var testFile = $('.raven-test-file');

            testFile.off('contextmenu');

            testFile.contextmenu(function(e) {
                self.currentContextMenuId = self.TEST_CONTEXT_MENU_ID;
                self._stopEvents(e);
            });

            var itemList = [];
            if (this._hasPermission()) {
                itemList = itemList.concat([
                    contextMenuTestsItemFactory(AJS.I18n.getText('raven.test-repository.remove-tests.title'), $.proxy(self._deleteTests, self), $.proxy(self._isRemoveTestAllowed, self)),
                    self.CONTEXT_MENU_SEPARATOR
                ]);
            }

            itemList = itemList.concat([
                contextMenuTestsItemFactory(AJS.I18n.getText('raven.test-repository.create-folder-test-set'), $.proxy(self._testSetAction, self)),
                contextMenuTestsItemFactory(AJS.I18n.getText('raven.test-repository.add-tests-test-set'), $.proxy(self._associateTestSetAction, self)),
                self.CONTEXT_MENU_SEPARATOR
            ]);

            if (this._getConfigAllowContextMenuTestPlan()) {
                itemList = itemList.concat([
                    contextMenuTestsItemFactory(AJS.I18n.getText('raven.test-repository.create-folder-test-plan'), $.proxy(self._testPlanTestActionDialog, self)),
                    contextMenuTestsItemFactory(AJS.I18n.getText('raven.test-repository.add-tests-test-plan'), $.proxy(self._associateTestPlanActionDialog, self)),
                    self.CONTEXT_MENU_SEPARATOR,
                ]);
            }

            itemList = itemList.concat([
                contextMenuTestsItemFactory(AJS.I18n.getText('raven.test-repository.create-folder-test-exec'), $.proxy(self._testExecutionAction, self)),
                contextMenuTestsItemFactory(AJS.I18n.getText('raven.test-repository.add-tests-test-exec'), $.proxy(self._associateTestExecAction, self))
            ]);

            testFile.contextPopup({
                items: itemList
            });

            function contextMenuTestsItemFactory(label, action, enabled) {
                function enabledAction() {
                    if (enabled) {
                        return enabled();
                    }
                    return true;
                }

                return {
                    label: label,
                    action: action,
                    isEnabled: enabledAction
                }
            }
        },

        _isAddTestsAllowed: function() {
            var activeFolderId = this.folderView.getActiveFolderId();
            return activeFolderId !== this.folderView.ALL_TESTS
                && activeFolderId !== this.folderView.ALL_ORPHAN_TESTS
                && activeFolderId !== this.folderView.FOLDER_ROOT;
        },

        _isCreateTestsAllowed: function() {
            return this._isAddTestsAllowed();
        },

        _isRemoveTestAllowed: function() {
            var activeFolderId = $('.raven-activeFolder').attr('id');
            if (!this._hasPermission()) {
                return false;
            }

            if (activeFolderId === this.folderView.ALL_TESTS
                || activeFolderId === this.folderView.ALL_ORPHAN_TESTS
                || activeFolderId === this.folderView.FOLDER_ROOT) {
                return false;
            }

            var selectedTests = this._getSelectedTests();
            return selectedTests.length > 0;
        },

        _addTestsAction: function() {
            var activeFolderId = this.folderView.getActiveFolderId();
            var folderName = this.folderView.getFolderName(activeFolderId);
            var allowedFilters = new Map();

            var projectKey = "";
            //set project filter only if we are in Test Repository
            if (!this._isTestPlanBoard()) {
                allowedFilters.set(RAVEN.Dialogs.FILTER_PROJECT, this._getConfigAllowDialogProjectFilter());
                projectKey = this.projectKey;
            }


            var testIssueTypeId = $('#test-issue-type-id').val();

            var labels = RAVEN.Dialogs.getDefaultTestLabels(folderName);

            //In case we are in a TestPlan
            if(this._isTestPlanBoard()){
                labels.title = AJS.I18n.getText('common.action.move-tests.to', folderName);
            }

            var self = this;
            new RAVEN.Dialogs.createIssueSelectorAdvancedDialog(
                'TEST',
                self._buildIssueSelectorJQL(),
                testIssueTypeId,
                this.projectKey, // this has to be done this way because in the issue-selector we are using the issueKey - project part of string - to get the project
                labels,
                function(issues) {
                    if (issues && issues.length > 0) {
                        self._addTestsRequest(activeFolderId, issues);
                    }
                },
                null,
                null,
                false,
                allowedFilters,
                projectKey
            );
        },

        _createTestsAction: function() {
            var testIssueTypeId = $('#test-issue-type-id').val();
            var createForm = JIRA.Forms.createCreateIssueForm({
                issueType: testIssueTypeId
            });
            var dialog = createForm.asDialog({
                id: 'create-issue-dialog'
            });

            var self = this;
            createForm.bind("sessionComplete", function (e, data) {
                self.dialogLoaded = false;
                if (data && data.length > 0) {
                    var issue = data[data.length - 1];
                    if (issue && issue.createdIssueDetails && issue.createdIssueDetails.id) {
                        var folderId = self.folderView.getActiveFolderId();
                        self._moveCreatedTestToFolder(folderId, issue.createdIssueDetails.id, issue.issueKey);
                    } else {
                        console.error('Cannot find ID of created Issue');
                    }
                }
            });
            dialog.show();


            JIRA.bind(JIRA.Events.NEW_CONTENT_ADDED, function(e, context, reason) {
                context.find('.qf-create-another').hide();
                
                if (self.dialogLoaded) {
                    return;
                }
                self.dialogLoaded = true;
                self._getConfigCreateTestFieldFiller().apply(self, [context]);
            });

        },

        _buildIssueSelectorJQL: function() {
            var  jqlConstant = this._getConfigJqlConstant();

            var pathStr = '';
            $('#path-container')
                .find('.raven-path')
                .slice(1)
                .each(function () {
                    pathStr += $(this).text() + '/';
                });

            if (pathStr.length === 0) {
                //sanity check, should not happen
                console.error("no path is defined");
                return jqlConstant;
            }

            var jql = '';
            if (jqlConstant.length > 0) {
                jql = jqlConstant + ' and '
            }
            return jql + 'issue not in ' + this._getConfigfolderJQLName() + '("' + this.entityKey + '", "' + pathStr + '")';
        },

        _testSetAction: function() {
            //make sure we set the testExec flag to false
            this.addTestExecutionToTestPlan = false;
            this.lastActiveTestCustomFieldId = $('#test-set-tests-custom-field-id').val();
            this._contextMenuCreateIssueForm($('#test-set-issue-type-id').val());
        },

        _testPlanTestActionDialog: function () {
            var activeFolderId = this.folderView.getActiveFolderId();
            this.replicateFolderStructure = false;

            if (activeFolderId !== this.folderView.ALL_TESTS && activeFolderId !== this.folderView.ALL_ORPHAN_TESTS) {
                this._testPlanDialog(AJS.I18n.getText('raven.test-repository.folders.replicate-folder-by-tests-message'),
                    $.proxy(this._testPlanAction, this));
            } else {
                this._testPlanAction();
            }
        },

        _testPlanActionDialog: function () {
            this._testPlanDialog(AJS.I18n.getText('raven.test-repository.folders.replicate-folder-message'),
                $.proxy(this._testPlanAction, this));
        },

        _testPlanAction: function () {
            this.lastActiveTestCustomFieldId = $('#test-plan-tests-custom-field-id').val();
            this._contextMenuCreateIssueForm($('#test-plan-issue-type-id').val());
        },

        _testExecutionAction: function() {
            this.addTestExecutionToTestPlan = true;
            this.lastActiveTestCustomFieldId = $('#test-exec-tests-custom-field-id').val();
            this._contextMenuCreateIssueForm($('#test-exec-issue-type-id').val());
        },

        _associateTestSetAction: function() {
            var labels = {};
            labels.label = AJS.I18n.getText('dialogs.issueselector.testset.label');
            labels.title = AJS.I18n.getText('raven.test-repository.add-tests-test-set');
            labels.description = AJS.I18n.getText('dialogs.issueselector.testset.description');
            labels.enter = AJS.I18n.getText('dialogs.issueselector.testset.enter');

            this._associateIssueService(labels, this.TESTSET, $('#test-set-issue-type-id').val());
        },

        _associateTestPlanFolderActionDialog: function() {
            this._testPlanDialog(AJS.I18n.getText('raven.test-repository.folders.replicate-folder-message'), $.proxy(this._associateTestPlanAction, this))
        },

        _associateTestPlanActionDialog: function() {
            var activeFolderId = this.folderView.getActiveFolderId();

            if (activeFolderId !== this.folderView.ALL_TESTS && activeFolderId !== this.folderView.ALL_ORPHAN_TESTS) {
                this._testPlanDialog(AJS.I18n.getText('raven.test-repository.folders.replicate-folder-by-tests-message'), $.proxy(this._associateTestPlanAction, this));
            } else {
                this.replicateFolderStructure = false;
                this._associateTestPlanAction();
            }
        },

        _associateTestPlanAction: function() {
            var labels = {};
            labels.label = AJS.I18n.getText('dialogs.issueselector.testplan.label');
            labels.title = AJS.I18n.getText('raven.test-repository.add-tests-test-plan');
            labels.description = AJS.I18n.getText('dialogs.issueselector.testplan.description');
            labels.enter = AJS.I18n.getText('dialogs.issueselector.testplan.enter');

            this._associateIssueService(labels, this.TESTPLAN, $('#test-plan-issue-type-id').val());
        },

        _associateTestExecAction: function() {
            var labels = {};
            labels.label = AJS.I18n.getText('dialogs.issueselector.testexec.label');
            labels.title = AJS.I18n.getText('raven.test-repository.add-tests-test-exec');
            labels.description = AJS.I18n.getText('dialogs.issueselector.testexec.description');
            labels.enter = AJS.I18n.getText('dialogs.issueselector.testexec.enter');

            this._associateIssueService(labels, this.TESTEXECUTION, $('#test-exec-issue-type-id').val());
        },

        _associateIssueService: function(labels, issueType, issueTypeId) {
            var self = this;
            var jql = 'key != -1';
            var projectKey = self.projectKey + "-";

            new RAVEN.Dialogs.createIssueSelectorAdvancedDialog(issueType, jql, issueTypeId, projectKey, labels, function(issues) {
                if (issues && issues.length > 0) {
                    self._associateTestsToIssues(issues, issueType);
                }
            }, null);
        },

        _contextMenuCreateIssueForm: function(issueType) {
            var self = this;

            var createForm = JIRA.Forms.createCreateIssueForm({
                issueType: issueType
            });

            var dialog = createForm.asDialog({
                id: 'create-issue-dialog'
            });

            createForm.bind('sessionComplete', function (ev, issues) {
                if (self._isOnFolderItem()) {
                    self.folderView.setActiveFolder($('#' + self.lastActiveFolder));
                }

                var issueKeys = [];
                issues.forEach(function (issue) {
                    issueKeys.push(issue.issueKey);
                });

                //In case we are create a TestExecution From a TestPlan
                if(self._isTestPlanBoard() && self.addTestExecutionToTestPlan) {
                    self._associateTestPlanToTestExecution(issueKeys);
                }

                self._replicateFolderStructureToTestPlan(issueKeys);
            });

            // It is necessary increase the JIRA default timeout to be able save folders with a large numbers of tests.
            AJS.$.ajaxSetup({
                timeout: 120000 // Milliseconds
            });


            dialog.show();

            // Class to indicates that this a popup to Create Issue from folder context menu
            $("#create-issue-dialog").addClass('raven-create-issue-from-context-menu');
        },

        _associateTestPlanToTestExecution: function(issueKeys) {
            //reset flag to create association to Test Execution
            this.addTestExecutionToTestPlan = false;

            var self = this;

                var url = '/rest/raven/1.0/' + self._getConfigRepositoryPath() + '/associate/testPlanExecution/' + self.entityKey;
                $.ajax({
                    type: "POST",
                    url: url,
                    processData: true,
                    async: false,
                    data: JSON.stringify(issueKeys),
                    contentType: "application/json",
                    error: function (qXHR, textStatus, errorThrown) {
                        if (errorThrown && errorThrown.trim() !== '') {
                            errorThrown = errorThrown + ': ' + qXHR.responseText;
                        } else {
                            errorThrown = qXHR.responseText;
                        }

                        self.message(AJS.I18n.getText('raven.test-repository.add-tests-error-msg'), errorThrown);
                    },
                    success: function () {
                        self._clearMessage();
                    }
                });
        },

        _replicateFolderStructureToTestPlan: function (issueKeys) {
            var activeFolderId = this.folderView.getActiveFolderId();

            if (!this.replicateFolderStructure || activeFolderId === this.folderView.ALL_ORPHAN_TESTS || activeFolderId === this.folderView.ALL_TESTS) {
                return;
            }

            var self = this;
            var url = contextPath;
            var jsonBody = JSON.stringify([""]);

            if (self.currentContextMenuId === self.TEST_CONTEXT_MENU_ID) {
                var tests = [];

                $('.raven-selected-file').each(function () {
                    tests.push($(this).attr('test-key'));
                });

                jsonBody = JSON.stringify({targetIssueKeys: issueKeys, testKeys: tests});
                url += ('/rest/raven/1.0/' + this._getConfigRepositoryPath() + '/replicate/tests/issueType/' + this.TESTPLAN + '/entityKey/' + self.entityKey);

            } else if (self.currentContextMenuId === self.ROOT_FOLDER_CONTEXT_MENU_ID) {
                url += ('/rest/raven/1.0/' + this._getConfigRepositoryPath() + '/replicate/folderRoot/entityKey/' + self.entityKey + '/issueType/' + this.TESTPLAN);
                jsonBody = JSON.stringify(issueKeys);

            } else if (self.currentContextMenuId === self.FOLDER_CONTEXT_MENU_ID) {
                url += ('/rest/raven/1.0/' + this._getConfigRepositoryPath() + '/replicate/folder/' + self.lastActiveFolder + '/issueType/' + this.TESTPLAN);
                jsonBody = JSON.stringify(issueKeys);
            }

            $.ajax({
                type: "POST",
                url: url,
                processData: true,
                async: true,
                data: jsonBody,
                contentType: "application/json",
                error: function (qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() !== '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }

                    if(textStatus === "timeout"){
                        self.message('',
                            AJS.I18n.getText('raven.test-repository.folders.error-replicate-folder-complexity',self._createMessageLinks(issueKeys)),
                            "warning");
                    }
                    else{
                        self.message(errorThrown);
                    }
                },
                success: function () {
                    self._clearMessage();
                    self._messageFlag(AJS.I18n.getText('raven.test-repository.replication-success-msg', self._createMessageLinks(issueKeys)));
                }
            });

            this.replicateFolderStructure = false;
        },

        _isOnFolderItem: function(popup) {
            var id = this.currentContextMenuId;

            if (popup) {
                id = this.currentContextMenuId;
            }

            return id === this.FOLDER_CONTEXT_MENU_ID ||
                   id === this.ROOT_FOLDER_CONTEXT_MENU_ID;
        },

        _contextMenuInitActions: function() {
            var self = this;

            if (!self.isContextMenuInitialized) {
                self.isContextMenuInitialized = true;

                $(document).bind("dialogContentReady", function(event, dialog) {
                    var popup = dialog.$popup[0];

                    //we only perform action if this has come from context menu actions
                    if(popup.classList.contains('raven-create-issue-from-context-menu')){

                        // If we are creating a Test Exec from a TestPlan
                        // we copy Fix Versions
                        if(self.addTestExecutionToTestPlan && self._isTestPlanBoard() && $("#testPlanIssueVersions").val()) {

                            var fixVersions = JSON.parse($("#testPlanIssueVersions").val());

                            if(fixVersions) {
                                $(fixVersions).each(function(index, jsonObj) {
                                    AJS.$('select[id="fixVersions"] option').each(function() {
                                        //If the name of version is the same then we choose it.
                                        //this is only using names because identical versions in differents projects have not the same ID
                                        if (jQuery.trim($(this).text()) == jsonObj.name) {
                                            $(popup).find("#fixVersions").trigger('selectOption', [{ value: $(this).val(), title: jsonObj.name, label: jsonObj.name }]);
                                        }
                                    });

                                });
                            }
                        }

                        if (self.currentContextMenuId === self.TEST_CONTEXT_MENU_ID) {
                            var $field = $(popup).find("#" + self.lastActiveTestCustomFieldId);
                            var testKeys = [];

                            $('.raven-selected-file').each(function() {
                                testKeys.push($(this).attr('test-key'));
                            });

                            if (testKeys.length > self.MAX_TEST_ITEMS_IN_CREATION_FORM) {
                                var $token = $(popup).find('.raven-test-keys-token');
                                if ($token) {
                                    $token.attr('value', $token.attr('value') + testKeys.join());
                                }
                            }

                            for (var i = 0; i < testKeys.length; i++) {
                                var testKey = testKeys[i];
                                $field.trigger('selectOption', [{ label: testKey, value: testKey, title: testKey }]);
                                if (i > self.MAX_TEST_ITEMS_IN_CREATION_FORM) {
                                    break;
                                }
                            }

                            self.lastActiveTestCustomFieldId = null;
                        } else if (self._isOnFolderItem(popup)) {
                            self._contextMenuGetTests(popup);
                        }
                    }
                });
            }
        },

        /**
         * Used on context-menu Creation actions
         * @param popup The Current JIRA creation pop-up
         * @private
         */
        _contextMenuGetTests: function(popup) {
            var self = this;

            var $blockElem = $('#raven-test-repository-wrapper');

            self._blockAndStartSpin($blockElem);

            self._clearMessage();

            var url = contextPath;

            if (self.currentContextMenuId === self.ROOT_FOLDER_CONTEXT_MENU_ID) {
                url += '/rest/raven/1.0/' + this._getConfigRepositoryPath() + '/folderRoot/tests?entityKey=' + self.entityKey;
            } else {
                url += '/rest/raven/1.0/' + this._getConfigRepositoryPath() + '/folderWithDescendants/tests?folderId=' + self.lastActiveFolder;
            }

            var cancel = $(popup).find('.cancel');

            cancel.on('click', function() {
                if (self._isOnFolderItem()) {
                    self.folderView.setActiveFolder($('#' + self.lastActiveFolder));
                }
                self.lastActiveTestCustomFieldId = null;
            });

            $.ajax({
                type: "GET",
                url: url,
                processData: true,
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() !== '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }
                    self.message(AJS.I18n.getText('raven.test-repository.add-tests-error-msg'), errorThrown);
                },
                success: function(data, textStatus, jqXHR) {
                    var $field = $(popup).find("#" + self.lastActiveTestCustomFieldId);

                    if (data.length > self.MAX_TEST_ITEMS_IN_CREATION_FORM) {
                        var $testKeys = $(popup).find('.raven-test-keys-token');
                        if ($testKeys) {
                            $testKeys.attr('value', $testKeys.attr('value') + data.join());
                        }
                    }

                    for (var i = 0; i < data.length; i++) {
                        var testKey = data[i];
                        $field.trigger('selectOption', [{ label: testKey, value: testKey, title: testKey }]);
                        if (i > self.MAX_TEST_ITEMS_IN_CREATION_FORM) {
                            break;
                        }
                    }
                },
                complete: function() {
                    self._unblockAndStopSpin($blockElem);
                    self.lastActiveTestCustomFieldId = null;
                }
            });
        },

        _initializeKeyboardEvents: function() {
            var self = this;
            $(document).keyup(function(e) {
                if (self.KeyCode.DOM_VK_DELETE === e.which) {
                    self._keyDeleteHandler(e)
                }
            });
        },

        _keyDeleteHandler: function() {
            if (this._isRemoveTestAllowed()) {
                this._deleteTests();
            }
        },

        _deleteTests: function() {
            var labels = {
                title: AJS.I18n.getText('raven.test-repository.remove-tests.title'),
                message: AJS.I18n.getText('raven.test-repository.remove-tests.description'),
                confirmButton: AJS.I18n.getText('raven.test-repository.remove-tests.title')
            };
            this._confirmDialog(labels, $.proxy(this._removeSelectedTests, this))
        },

        _removeSelectedTests: function() {
            var items = this._getSelectedTests();
            this._removeTestsRequest(items);
        },

        _getSelectedTests: function() {
            var items = [];
            $('.raven-selected-file').each(function() {
                items.push($(this).attr('id'));
            });
            return items;
        },

        _removeTestsRequest: function(testIds) {
            var self = this;

            var $blockElem = $('#raven-test-repository-wrapper');
            self._blockAndStartSpin($blockElem);
            $.ajax({
                type: "DELETE",
                url: contextPath + '/rest/raven/1.0/' + self._getConfigRepositoryPath() + '/removeTests?entityKey=' + self.entityKey,
                processData: true,
                async: true,
                data: JSON.stringify(testIds),
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() !== '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }

                    self.message(AJS.I18n.getText('raven.test-repository.add-tests-error-msg'), errorThrown);
                },
                success: function(data, textStatus, jqXHR) {
                    self._removeTestsFromList(testIds, self.ALL_ORPHAN_TESTS);
                    self._subRefreshPostTestOperation(testIds.length);
                    self._messageFlag(AJS.I18n.getText('raven.test-repository.remove-tests-from-folder', self.folderView.getFolderName(self.lastActiveFolder)));
                },
                complete: function() {
                    self._unblockAndStopSpin($blockElem);
                }
            });
        },

        /**
         * Disable the context menu for unsupported items.
         *
         * @private
         */
        _disableDefaultContextMenu: function() {
            var self = this;

            $('#raven-test-repository-wrapper').contextmenu(function(event) {
                self._stopEvents(event);
            });
        },

        _stopEvents: function(event) {
            if (event.preventDefault) {
                event.preventDefault();
            }

            if (event.stopPropagation) {
                event.stopPropagation();
            }
        },

        _associateTestsToIssues: function (issueKeys, issueType) {
            var self = this;
            var url = contextPath;
            var jsonBody = JSON.stringify([""]);

            if (self.currentContextMenuId === self.TEST_CONTEXT_MENU_ID) {
                var tests = [];

                $('.raven-selected-file').each(function () {
                    tests.push($(this).attr('test-key'));
                });

                jsonBody = JSON.stringify({targetIssueKeys: issueKeys, testKeys: tests});
                url += ('/rest/raven/1.0/' + this._getConfigRepositoryPath() +'/associate/tests/issueType/' + issueType + '/entityKey/' + self.entityKey);

            } else if (self.currentContextMenuId === self.ROOT_FOLDER_CONTEXT_MENU_ID) {
                url += ('/rest/raven/1.0/' + this._getConfigRepositoryPath() + '/associate/folderRoot/entityKey/' + self.entityKey + '/issueType/' + issueType);
                jsonBody = JSON.stringify(issueKeys);

            } else if (self.currentContextMenuId === self.FOLDER_CONTEXT_MENU_ID) {
                url += ('/rest/raven/1.0/' + this._getConfigRepositoryPath() + '/associate/folder/' + self.lastActiveFolder + '/issueType/' + issueType);
                jsonBody = JSON.stringify(issueKeys);
            }

            $.ajax({
                type: "POST",
                url: url,
                processData: true,
                async: true,
                data: jsonBody,
                contentType: "application/json",
                error: function(qXHR, textStatus, errorThrown) {
                    if (errorThrown && errorThrown.trim() !== '') {
                        errorThrown = errorThrown + ': ' + qXHR.responseText;
                    } else {
                        errorThrown = qXHR.responseText;
                    }

                    if(textStatus === "timeout"){
                        self.message('',
                            AJS.I18n.getText('raven.test-repository.folders.error-add-tests-complexity',self._createMessageLinks(issueKeys)),
                            "warning");
                    }else{
                        self.message(AJS.I18n.getText('raven.test-repository.add-tests-error-msg'), errorThrown);
                    }

                },
                success: function() {
                    self._clearMessage();
                    self._messageFlag(AJS.I18n.getText('raven.test-repository.add-tests-success-msg', self._createMessageLinks(issueKeys)));
                    self._replicateFolderStructureToTestPlan(issueKeys, issueType);
                }
            });
        },

        _createMessageLinks: function(issueKeys) {
            var htmlBody = [];

            for (var i = 0; i < issueKeys.length; i++) {
                var issueKey = issueKeys[i];
                htmlBody.push('<span class="raven-test-key"><a href="' + contextPath + '/browse/' + issueKey + '" title="' + issueKey + '">' + issueKey + '</a></span>');
                if (i > 5) {
                    break;
                }
            }

            var issuesAsString = htmlBody.join();

            if (i > 5 && issueKeys.length !== 5) {
                issuesAsString += ',...';
            } else {
                issuesAsString += '.';
            }

            return issuesAsString;
        },

        _messageFlag: function(body) {
            require('aui/flag')({
                type: 'success',
                body: body,
                close: 'auto'
            });
        },

        _allowFolderRankEdit: function() {
            return this._hasPermission() &&
                !this._isFilterActive() &&
                this.dragInfo.mode === this.FILE &&
                this._isRegularFolder(this.folderView.getActiveFolderId());
        },

        _isRegularFolder: function(folderId) {
            return folderId !== this.folderView.ALL_TESTS &&
                folderId !== this.folderView.ALL_ORPHAN_TESTS &&
                folderId !== this.folderView.FOLDER_ROOT;
        },

        _isFilterActive: function() {
            if (this.filterDialog === null) {
               return false;
            }
            return this.filterDialog.isFilterActive();
        },

        _hasPermission: function() {
            return this.IS_EDITABLE;
        }
    });
})(AJS.$);